/*
 * Copyright (C) 2001-2011 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/*
 * Automatic Directory Listing Search
 * Henrik Engstrm, henrikengstrom at home se
 */

#include "stdafx.h"
#include "Resource.h"
#include "../client/DCPlusPlus.h"
#include "../client/Client.h"
#include "ADLSearchFrame.h"
#include "ADLSProperties.h"
#include "FlatTabCtrl.h"

int ADLSearchFrame::columnIndexes[] = { COLUMN_ACTIVE_SEARCH_STRING, COLUMN_SOURCE_TYPE, COLUMN_DEST_DIR, COLUMN_MIN_FILE_SIZE, COLUMN_MAX_FILE_SIZE };
int ADLSearchFrame::columnSizes[] = { 120, 90, 90, 90, 90 };
static ResourceManager::Strings columnNames[] = { ResourceManager::ACTIVE_SEARCH_STRING, ResourceManager::SOURCE_TYPE, ResourceManager::DESTINATION, ResourceManager::SIZE_MIN,
    ResourceManager::MAX_SIZE
};
ATOM ADLSearchFrame::m_aAtom = 0;
UINT ADLSearchFrame::m_uiResourceID = IDR_ADLSEARCH;

ADLSearchFrame::ADLSearchFrame() : closed(false), contextMenu(NULL) {
    RegisterWindowClass(m_aAtom, ADLSearchFrameWndProc, _T("ADLSearchFrame"), m_uiResourceID);
}

ADLSearchFrame::~ADLSearchFrame() {
	if(contextMenu != NULL) {
		::DestroyMenu(contextMenu);
	}
}

LRESULT CALLBACK ADLSearchFrame::ADLSearchFrameWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
	ADLSearchFrame * pParent;

	if(uMsg == WM_NCCREATE) {
		pParent = (ADLSearchFrame *)((LPMDICREATESTRUCT)((LPCREATESTRUCT)lParam)->lpCreateParams)->lParam;
		SetWindowLongPtr(hWnd, GWLP_USERDATA, (LONG_PTR)pParent);

		pParent->m_hWnd = hWnd;
	} else {
		pParent = (ADLSearchFrame *)GetWindowLongPtr(hWnd, GWLP_USERDATA);
		if(pParent == NULL) {
			return ::DefMDIChildProc(hWnd, uMsg, wParam, lParam);
		}
	}

	return pParent->ADLSearchFrameProc(hWnd, uMsg, wParam, lParam);
}

LRESULT ADLSearchFrame::ADLSearchFrameProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    switch(uMsg) {
        case WM_CREATE:
            onCreate();
            return 0;
        case WM_CLOSE:
            if(onClose() == true) {
                return 0;
            }
            break;
        case WM_CTLCOLOREDIT:
        case WM_CTLCOLORSTATIC: {
            LRESULT lResult = onCtlColor(wParam, lParam);
            if(lResult != 0) {
                return lResult;
            }
            break;
        }
        case WM_CONTEXTMENU:
            if(onContextMenu(wParam, lParam) == true) {
                return 0;
            }
            break;
        case WM_SIZE:
            if(wParam != SIZE_MINIMIZED) {
                UpdateLayout();
            }
            break;
        case WM_REFRESH_SETTINGS:
            onRefreshSettings();
            return 0;
        case WM_COMMAND:
            switch(LOWORD(wParam)) {
                case IDC_ADD:
                    onAdd();
                    return 0;
                case IDC_EDIT:
                    onEdit();
                    return 0;
                case IDC_REMOVE:
                    onRemove();
                    return 0;
                case IDC_HELP_FAQ:
                    onHelp();
                    return 0;
                case IDC_MOVE_UP:
                    onMoveUp();
                    return 0;
                case IDC_MOVE_DOWN:
                    onMoveDown();
                    return 0;
            }

            break;
        case WM_NOTIFY:
            if(((LPNMHDR)lParam)->idFrom == IDC_ADLLIST) {
                switch(((LPNMHDR)lParam)->code) {
                    case NM_DBLCLK:
                    	if(onDoubleClickList((LPNMHDR)lParam) == true) {
                    		return 0;
                        }
                        break;
                    case LVN_ITEMCHANGED:
                    	onItemChanged((LPNMHDR)lParam);
                    	return 0;
                    case LVN_KEYDOWN:
                    	if(onKeyDown((LPNMHDR)lParam) == true) {
                    		return 0;
                        }
                        break;
                }
            }
            break;
        case WM_NCDESTROY:
            delete this;
            return ::DefMDIChildProc(hWnd, uMsg, wParam, lParam);
    }

    LRESULT lResult = 0;
	if(MDIWindowProc(hWnd, uMsg, wParam, lParam, lResult) == true) {
		return lResult;
    }

	return ::DefMDIChildProc(hWnd, uMsg, wParam, lParam);
}

// Frame creation
void ADLSearchFrame::onCreate() {
    WinUtil::tabCtrl->addTab(m_hWnd, RGB(0, 0, 0), IDR_ADLSEARCH);
    created = true;

	// Create list control
	ctrlList.Create(m_hWnd, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | WS_HSCROLL | WS_VSCROLL | LVS_REPORT | LVS_SHOWSELALWAYS, WS_EX_CLIENTEDGE, IDC_ADLLIST,
        LVS_EX_LABELTIP | LVS_EX_HEADERDRAGDROP | LVS_EX_CHECKBOXES | LVS_EX_FULLROWSELECT);

	// Create listview columns
	WinUtil::splitTokens(columnIndexes, SETTING(ADLSEARCHFRAME_ORDER), COLUMN_LAST);
	WinUtil::splitTokens(columnSizes, SETTING(ADLSEARCHFRAME_WIDTHS), COLUMN_LAST);

	for(uint8_t ui8i = 0; ui8i < (sizeof(columnSizes) / sizeof(columnSizes[0])); ui8i++) {
		ListViewInsertColumn(ctrlList.m_hWnd, ui8i, CTSTRING_I(columnNames[ui8i]), LVCFMT_LEFT, columnSizes[ui8i], ui8i);
	}

	::SendMessage(ctrlList.m_hWnd, LVM_SETCOLUMNORDERARRAY, COLUMN_LAST, (LPARAM)columnIndexes);

	// Create buttons
	hWndAdd = CreateWindowEx(0, WC_BUTTON, CTSTRING(NEW), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | BS_PUSHBUTTON, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
        m_hWnd, (HMENU)IDC_ADD, g_hInstance, NULL);
	::SendMessage(hWndAdd, WM_SETFONT, (WPARAM)WinUtil::font, MAKELPARAM(TRUE, 0));

	hWndEdit = CreateWindowEx(0, WC_BUTTON, CTSTRING(PROPERTIES), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | BS_PUSHBUTTON, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
        m_hWnd, (HMENU)IDC_EDIT, g_hInstance, NULL);
	::SendMessage(hWndEdit, WM_SETFONT, (WPARAM)WinUtil::font, MAKELPARAM(TRUE, 0));

	hWndRemove = CreateWindowEx(0, WC_BUTTON, CTSTRING(REMOVE), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | BS_PUSHBUTTON, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
        m_hWnd, (HMENU)IDC_REMOVE, g_hInstance, NULL);
	::SendMessage(hWndRemove, WM_SETFONT, (WPARAM)WinUtil::font, MAKELPARAM(TRUE, 0));

	hWndMoveUp = CreateWindowEx(0, WC_BUTTON, CTSTRING(MOVE_UP), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | BS_PUSHBUTTON, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
        m_hWnd, (HMENU)IDC_MOVE_UP, g_hInstance, NULL);
	::SendMessage(hWndMoveUp, WM_SETFONT, (WPARAM)WinUtil::font, MAKELPARAM(TRUE, 0));

	hWndMoveDown = CreateWindowEx(0, WC_BUTTON, CTSTRING(MOVE_DOWN), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | BS_PUSHBUTTON, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
        m_hWnd, (HMENU)IDC_MOVE_DOWN, g_hInstance, NULL);
	::SendMessage(hWndMoveDown, WM_SETFONT, (WPARAM)WinUtil::font, MAKELPARAM(TRUE, 0));

	hWndHelp = CreateWindowEx(0, WC_BUTTON, CTSTRING(MENU_HELP), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | BS_PUSHBUTTON, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
        m_hWnd, (HMENU)IDC_HELP_FAQ, g_hInstance, NULL);
	::SendMessage(hWndHelp, WM_SETFONT, (WPARAM)WinUtil::font, MAKELPARAM(TRUE, 0));

	// Create context menu
	contextMenu = ::CreatePopupMenu();
	::AppendMenu(contextMenu, MF_STRING, IDC_ADD, CTSTRING(NEW));
	::AppendMenu(contextMenu, MF_STRING, IDC_REMOVE, CTSTRING(REMOVE));
	::AppendMenu(contextMenu, MF_STRING, IDC_EDIT, CTSTRING(PROPERTIES));

	SettingsManager::getInstance()->addListener(this);
	// Load all searches
	LoadAll();
}

// Close window
bool ADLSearchFrame::onClose() {
	if(!closed) {
		ADLSearchManager::getInstance()->Save();
		SettingsManager::getInstance()->removeListener(this);
		closed = true;
		::PostMessage(m_hWnd, WM_CLOSE, 0, 0);
		return true;
	} else {
		WinUtil::saveHeaderOrder(ctrlList.m_hWnd, SettingsManager::ADLSEARCHFRAME_ORDER,
			SettingsManager::ADLSEARCHFRAME_WIDTHS, COLUMN_LAST, columnIndexes, columnSizes);

		return false;
	}
}

// Recalculate frame control layout
void ADLSearchFrame::UpdateLayout() {
	RECT rect;
	::GetClientRect(m_hWnd, &rect);

	// Position list control
	RECT rc = rect;
	rc.top += 2;
	rc.bottom -= 28;
	::MoveWindow(ctrlList.m_hWnd, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, TRUE);

	// Position buttons
	const long bwidth = 90;
	const long bspace = 10;
	rc = rect;
	rc.bottom -= 2;
	rc.top = rc.bottom - 22;

	rc.left = 2;
	rc.right = rc.left + bwidth;
	::MoveWindow(hWndAdd, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, TRUE);

	rc.left += bwidth + 2;
	rc.right = rc.left + bwidth;
	::MoveWindow(hWndEdit, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, TRUE);

	rc.left += bwidth + 2;
	rc.right = rc.left + bwidth;
	::MoveWindow(hWndRemove, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, TRUE);

	rc.left += bspace;

	rc.left += bwidth + 2;
	rc.right = rc.left + bwidth;
	::MoveWindow(hWndMoveUp, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, TRUE);

	rc.left += bwidth + 2;
	rc.right = rc.left + bwidth;
	::MoveWindow(hWndMoveDown, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, TRUE);

	rc.left += bspace;

	rc.left += bwidth + 2;
	rc.right = rc.left + bwidth;
	::MoveWindow(hWndHelp, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, TRUE);
}

// Keyboard shortcuts
bool ADLSearchFrame::onKeyDown(LPNMHDR pnmh) {
	NMLVKEYDOWN* kd = (NMLVKEYDOWN*) pnmh;

	switch(kd->wVKey) {
        case VK_INSERT:
            ::PostMessage(m_hWnd, WM_COMMAND, IDC_ADD, 0);
            return true;
        case VK_DELETE:
            ::PostMessage(m_hWnd, WM_COMMAND, IDC_REMOVE, 0);
            return true;
        case VK_RETURN:
            ::PostMessage(m_hWnd, WM_COMMAND, IDC_EDIT, 0);
            return true;
        default:
            break;
	}

	return false;
}

bool ADLSearchFrame::onContextMenu(WPARAM wParam, LPARAM lParam) {
	if((HWND)wParam != ctrlList.m_hWnd || (int)::SendMessage(ctrlList.m_hWnd, LVM_GETNEXTITEM, (WPARAM)-1, LVNI_SELECTED) == -1) {
        return false;
    }

    int iX = GET_X_LPARAM(lParam),
        iY = GET_Y_LPARAM(lParam);

	if(iX == -1 && iY == -1) {
        ListViewGetContextMenuPos(ctrlList.m_hWnd, iX, iY);
	}

	::TrackPopupMenuEx(contextMenu, TPM_LEFTALIGN | TPM_RIGHTBUTTON, iX, iY, m_hWnd, NULL);

	return true;
}

// Add new search
void ADLSearchFrame::onAdd() {
	// Invoke edit dialog with fresh search
	ADLSearch search;
	ADLSProperties dlg(&search);
	if(dlg.DoModal(m_hWnd) == IDOK) {
		// Add new search to the end or if selected, just before
		ADLSearchManager::SearchCollection& collection = ADLSearchManager::getInstance()->collection;

		int i = (int)::SendMessage(ctrlList.m_hWnd, LVM_GETNEXTITEM, (WPARAM)-1, LVNI_SELECTED);
		if(i < 0) {
			// Add to end
			collection.push_back(search);
			i = (int)(collection.size() - 1);
		} else {
			// Add before selection
			collection.insert(collection.begin() + i, search);
		}

		// Update list control
		int j = i;
		while(j < (int)collection.size()) {
			UpdateSearch(j++);
		}

		::SendMessage(ctrlList.m_hWnd, LVM_ENSUREVISIBLE, i, FALSE);
	}
}

// Edit existing search
void ADLSearchFrame::onEdit() {
	// Get selection info
	int i = (int)::SendMessage(ctrlList.m_hWnd, LVM_GETNEXTITEM, (WPARAM)-1, LVNI_SELECTED);
	if(i < 0) {
		// Nothing selected
		return;
	}

	// Edit existing
	ADLSearchManager::SearchCollection& collection = ADLSearchManager::getInstance()->collection;
	ADLSearch search = collection[i];

	// Invoke dialog with selected search
	ADLSProperties dlg(&search);
	if(dlg.DoModal(m_hWnd) == IDOK) {
		// Update search collection
		collection[i] = search;

		// Update list control
		UpdateSearch(i);
	}
}

// Remove searches
void ADLSearchFrame::onRemove() {
	ADLSearchManager::SearchCollection& collection = ADLSearchManager::getInstance()->collection;

	// Loop over all selected items
	int i;
	while((i = (int)::SendMessage(ctrlList.m_hWnd, LVM_GETNEXTITEM, (WPARAM)-1, LVNI_SELECTED)) >= 0) {
		collection.erase(collection.begin() + i);
		::SendMessage(ctrlList.m_hWnd, LVM_DELETEITEM, i, 0);
	}
}

// Help
void ADLSearchFrame::onHelp() {
	TCHAR message[] =
		_T("ADLSearch is a tool for fast searching of directory listings downloaded from users. \n")
		_T("Create a new ADLSearch entering 'avi' as search string for example. When you \n")
		_T("download a directory listing from a user, all avi-files will be placed in a special folder \n")
		_T("called <<<ADLSearch>>> for easy finding. It is almost the same as using the standard \n")
		_T("'Find' multiple times in a directory listing. \n")
		_T("\n")
		_T("Special options: \n")
		_T("- 'Active' check box selects if the search is used or not. \n")
		_T("- 'Source Type' can be the following options; 'Filename' matches search against filename, \n")
		_T("   'Directory' matches against current subdirectory and places the whole structure in the \n")
		_T("   special folder, 'Full Path' matches against whole directory + filename. \n")
		_T("- 'Destination Directory' selects the special output folder for a search. Multiple folders \n")
		_T("   with different names can exist simultaneously. \n")
		_T("- 'Min/Max Size' sets file size limits. This is not used for 'Directory' type searches. \n")
		_T("- 'Move Up'/'Move Down' can be used to organize the list of searches. \n")
		_T("\n")
		_T("There is a new option in the context menu (right-click) for directory listings. It is called \n")
		_T("'Go to directory' and can be used to jump to the original location of the file or directory. \n")
		_T("\n")
		_T("Extra features:\n")
		_T("\n")
		_T("1) If you use %y.%m.%d in a search string it will be replaced by todays date. Switch \n")
		_T("place on y/m/d, or leave any of them out to alter the substitution. If you use %[nick] \n")
		_T("it will be replaced by the nick of the user you download the directory listing from. \n")
		_T("\n")
		_T("2) If you name a destination directory 'discard', it will not be shown in the total result. \n")
		_T("Useful with the extra feature 3) below to remove uninteresting results. \n")
		_T(" \n")
		_T("3) There is a switch called 'Break on first ADLSearch match' in Settings->Advanced'.  \n")
		_T("If enabled, ADLSearch will stop after the first match for a specific file/directory. \n")
		_T("The order in the ADLSearch windows is therefore important. Example: Add a search \n")
		_T("item at the top of the list with string='xxx' and destination='discard'. It will catch \n")
		_T("many pornographic files and they will not be included in any following search results. \n");

	::MessageBox(m_hWnd, message, _T("ADLSearch brief description"), MB_OK);
}

// Move selected entries up one step
void ADLSearchFrame::onMoveUp() {
	ADLSearchManager::SearchCollection& collection = ADLSearchManager::getInstance()->collection;

	// Get selection
	vector<int> sel;
	int i = -1;
	while((i = (int)::SendMessage(ctrlList.m_hWnd, LVM_GETNEXTITEM, (WPARAM)-1, LVNI_SELECTED)) >= 0) {
		sel.push_back(i);
	}

	if(sel.size() < 1) {
		return;
	}

	// Find out where to insert
	int i0 = sel[0];
	if(i0 > 0) {
		i0 = i0 - 1;
	}

	// Backup selected searches
	ADLSearchManager::SearchCollection backup;
	for(i = 0; i < (int)sel.size(); ++i) {
		backup.push_back(collection[sel[i]]);
	}

	// Erase selected searches
	for(i = (int)(sel.size() - 1); i >= 0; --i) {
		collection.erase(collection.begin() + sel[i]);
	}

	// Insert (grouped together)
	for(i = 0; i < (int)sel.size(); ++i) {
		collection.insert(collection.begin() + i0 + i, backup[i]);
	}

	// Update UI
	LoadAll();

	// Restore selection
	for(i = 0; i < (int)sel.size(); ++i) {
		ListView_SetItemState(ctrlList.m_hWnd, i0 + i, LVNI_SELECTED, LVNI_SELECTED);
	}
}

// Move selected entries down one step
void ADLSearchFrame::onMoveDown() {
	ADLSearchManager::SearchCollection& collection = ADLSearchManager::getInstance()->collection;

	// Get selection
	vector<int> sel;
	int i = -1;
	while((i = (int)::SendMessage(ctrlList.m_hWnd, LVM_GETNEXTITEM, (WPARAM)-1, LVNI_SELECTED)) >= 0) {
		sel.push_back(i);
	}

	if(sel.size() < 1) {
		return;
	}

	// Find out where to insert
	int i0 = sel[sel.size() - 1] + 2;
	if(i0 > (int)collection.size()) {
		i0 = (int)collection.size();
	}

	// Backup selected searches
	ADLSearchManager::SearchCollection backup;
	for(i = 0; i < (int)sel.size(); ++i) {
		backup.push_back(collection[sel[i]]);
	}

	// Erase selected searches
	for(i = (int)sel.size() - 1; i >= 0; --i) {
		collection.erase(collection.begin() + sel[i]);
		if(i < i0)
		{
			i0--;
		}
	}

	// Insert (grouped together)
	for(i = 0; i < (int)sel.size(); ++i) {
		collection.insert(collection.begin() + i0 + i, backup[i]);
	}

	// Update UI
	LoadAll();

	// Restore selection
	for(i = 0; i < (int)sel.size(); ++i) {
		ListView_SetItemState(ctrlList.m_hWnd, i0 + i, LVNI_SELECTED, LVNI_SELECTED);
	}

	::SendMessage(ctrlList.m_hWnd, LVM_ENSUREVISIBLE, i0, FALSE);
}

// Clicked 'Active' check box
void ADLSearchFrame::onItemChanged(LPNMHDR pnmh) {
	NMITEMACTIVATE* item = (NMITEMACTIVATE*)pnmh;

    UINT uiState = (UINT)::SendMessage(ctrlList.m_hWnd, LVM_GETITEMSTATE, item->iItem, LVIS_SELECTED);

	::EnableWindow(hWndEdit, uiState);
	::EnableWindow(hWndRemove, uiState);
	::EnableWindow(hWndMoveUp, uiState);
	::EnableWindow(hWndMoveDown, uiState);

	if((item->uChanged & LVIF_STATE) == 0 || (item->uOldState & INDEXTOSTATEIMAGEMASK(0xf)) == 0 ||
        (item->uNewState & INDEXTOSTATEIMAGEMASK(0xf)) == 0) {
		return;
    }

	if(item->iItem >= 0) {
		// Set new active status check box
		ADLSearchManager::SearchCollection& collection = ADLSearchManager::getInstance()->collection;
		ADLSearch& search = collection[item->iItem];
		search.isActive = ((((UINT)::SendMessage(ctrlList.m_hWnd, LVM_GETITEMSTATE, item->iItem, LVIS_STATEIMAGEMASK)>> 12) - 1) != 0);
	}
}

// Double-click on list control
bool ADLSearchFrame::onDoubleClickList(LPNMHDR pnmh) {
	NMITEMACTIVATE* item = (NMITEMACTIVATE*)pnmh;

	if(item->iItem >= 0) {
		// Treat as onEdit command
		::PostMessage(m_hWnd, WM_COMMAND, IDC_EDIT, 0);
		return true;
	} else if(item->iItem == -1) {
		::PostMessage(m_hWnd, WM_COMMAND, IDC_ADD, 0);
		return true;
	}

	return false;
}

// Load all searches from manager
void ADLSearchFrame::LoadAll() {
	// Clear current contents
	::SendMessage(ctrlList.m_hWnd, LVM_DELETEALLITEMS, 0, 0);

	// Load all searches
	ADLSearchManager::SearchCollection& collection = ADLSearchManager::getInstance()->collection;
	for(unsigned long l = 0; l < collection.size(); l++) {
		UpdateSearch(l, FALSE);
	}
}

// Update a specific search item
void ADLSearchFrame::UpdateSearch(int index, BOOL doDelete) {
	ADLSearchManager::SearchCollection& collection = ADLSearchManager::getInstance()->collection;

	// Check args
	if(index >= (int)collection.size()) {
		return;
	}

	ADLSearch& search = collection[index];

	// Delete from list control
	if(doDelete) {
		::SendMessage(ctrlList.m_hWnd, LVM_DELETEITEM, index, 0);
	}

	// Generate values
	TStringList line;
	tstring fs;
	line.push_back(Text::toT(search.searchString));
	line.push_back(search.SourceTypeToDisplayString(search.sourceType));
	line.push_back(Text::toT(search.destDir));

	fs = _T("");
	if(search.minFileSize >= 0) {
		fs = Text::toT(Util::toString(search.minFileSize));
		fs += _T(" ");
		fs += search.SizeTypeToDisplayString(search.typeFileSize);
	}
	line.push_back(fs);

	fs = _T("");
	if(search.maxFileSize >= 0) {
		fs = Text::toT(Util::toString(search.maxFileSize));
		fs += _T(" ");
		fs += search.SizeTypeToDisplayString(search.typeFileSize);
	}
	line.push_back(fs);

	// Insert in list control
	ctrlList.insert(index, line);

	// Update 'Active' check box
	ListView_SetItemState(ctrlList.m_hWnd, index, INDEXTOSTATEIMAGEMASK(search.isActive? 2 : 1), LVIS_STATEIMAGEMASK);
}

void ADLSearchFrame::on(SettingsManagerListener::Save, SimpleXML& /*xml*/) throw() {
	::PostMessage(m_hWnd, WM_REFRESH_SETTINGS, 0, 0);
}

void ADLSearchFrame::onRefreshSettings() {
	bool refresh = false;

	if((COLORREF)::SendMessage(ctrlList.m_hWnd, LVM_GETBKCOLOR, 0, 0) != WinUtil::bgColor) {
		::SendMessage(ctrlList.m_hWnd, LVM_SETBKCOLOR, 0, WinUtil::bgColor);
		::SendMessage(ctrlList.m_hWnd, LVM_SETTEXTBKCOLOR, 0, WinUtil::bgColor);
		refresh = true;
	}
	if((COLORREF)::SendMessage(ctrlList.m_hWnd, LVM_GETTEXTCOLOR, 0, 0) != WinUtil::textColor) {
		::SendMessage(ctrlList.m_hWnd, LVM_SETTEXTCOLOR, 0, WinUtil::textColor);
		refresh = true;
	}

	if(refresh == true) {
		::RedrawWindow(m_hWnd, NULL, NULL, RDW_ERASE | RDW_INVALIDATE | RDW_UPDATENOW | RDW_ALLCHILDREN);
	}
}

/*
 * Some parts Copyright (C) 2002-2011 Petr Kozelka, PPK at czdc dot org
 * Licensed under GNU General Public License version 3.
 */
