/*
 * CzDC - client for Direct Connect peer to peer network.

 * Copyright (C) 2002-2011 Petr Kozelka, PPK at CzDC dot org

 * CzDC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3
 * as published by the Free Software Foundation.

 * CzDC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with CzDC. If not, see <http://www.gnu.org/licenses/>.
 */

//----------------------------------------------------------------------------------------------------------------------------------------------------
#include "stdafx.h"
//----------------------------------------------------------------------------------------------------------------------------------------------------
#include "../client/DCPlusPlus.h"
#include "../client/version.h"
//----------------------------------------------------------------------------------------------------------------------------------------------------
#include "Resource.h"
#include "AboutDlg.h"
#include "WinUtil.h"
#include "WinApiUtil.h"
//----------------------------------------------------------------------------------------------------------------------------------------------------

INT_PTR CALLBACK AboutDlg::AboutDlgProc(HWND hWndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam){
	AboutDlg * pParent;

	if(uMsg == WM_INITDIALOG) {
		pParent = (AboutDlg *)lParam;
		SetWindowLongPtr(hWndDlg, GWLP_USERDATA, (LONG_PTR)pParent);

		pParent->m_hWnd = hWndDlg;
	} else {
		pParent = (AboutDlg *)GetWindowLongPtr(hWndDlg, GWLP_USERDATA);
		if(pParent == NULL) {
			return FALSE;
		}
	}

    SetWindowLongPtr(hWndDlg, DWLP_MSGRESULT, 0);

	return pParent->AboutProc(hWndDlg, uMsg, wParam, lParam);
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

LRESULT AboutDlg::AboutProc(HWND /*hWnd*/, UINT uMsg, WPARAM wParam, LPARAM /*lParam*/) {
    switch(uMsg) {
        case WM_INITDIALOG:
            OnInitDialog();
            return TRUE;
        case WM_VERSIONDATA:
            onVersionData(wParam);
            return TRUE;
        case WM_SPEAKER:
            onSpeaker(wParam);
            return TRUE;
        case WM_COMMAND:
            if(LOWORD(wParam) == IDOK || LOWORD(wParam) == IDCANCEL) {
                OnCloseCmd(LOWORD(wParam));
                return TRUE;
            }

            break;
    }

	return FALSE;
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void AboutDlg::OnInitDialog() {
	::SetDlgItemText(m_hWnd, IDC_VERSION, _T(CzDC_VERSION_STRING));
	::SetDlgItemText(m_hWnd, IDC_ABOUT_LINK, _T("http://www.czdc.org"));

	::SetDlgItemText(m_hWnd, IDC_ABOUT_INFO, _T("Created by Petr \"PPK\" Kozelka\n\nCzDC is DC++ modification licenced under GNU GPL version 3"));

	::SendMessage(::GetDlgItem(m_hWnd, IDC_THANKS), EM_FMTLINES, TRUE, 0);
	::SendMessage(::GetDlgItem(m_hWnd, IDC_THANKS), EM_SETSEL, ::GetWindowTextLength(::GetDlgItem(m_hWnd, IDC_THANKS)), 
		::GetWindowTextLength(::GetDlgItem(m_hWnd, IDC_THANKS)));
	::SendMessage(::GetDlgItem(m_hWnd, IDC_THANKS), EM_REPLACESEL, (WPARAM)FALSE, (LPARAM)thanks);

	::SetDlgItemText(m_hWnd, IDC_TTH, WinUtil::tth.c_str());
	::SetDlgItemText(m_hWnd, IDC_LATEST, CTSTRING(DOWNLOADING));
		
	char buf[256];
	sprintf_s(buf, 256, "Uptime: %s", WinUtil::formatTime(SETTING(TOTAL_UPTIME)).c_str());
	::SetDlgItemText(m_hWnd, IDC_TOTAL_UPTIME, Text::toT(buf).c_str());
	sprintf_s(buf, 256, "Upload: %s, Download: %s", Util::formatBytes(SETTING(TOTAL_UPLOAD)).c_str(), Util::formatBytes(SETTING(TOTAL_DOWNLOAD)).c_str());
	::SetDlgItemText(m_hWnd, IDC_TOTALS, Text::toT(buf).c_str());
	if(SETTING(TOTAL_DOWNLOAD) > 0) {
		sprintf_s(buf, 256, "Ratio (up/down): %.2f", ((double)SETTING(TOTAL_UPLOAD)) / ((double)SETTING(TOTAL_DOWNLOAD)));
		::SetDlgItemText(m_hWnd, IDC_RATIO, Text::toT(buf).c_str());
	}
	sprintf_s(buf, 256, "Uptime: %s", WinUtil::formatTime(Util::getUptime()).c_str());
	::SetDlgItemText(m_hWnd, IDC_UPTIME, Text::toT(buf).c_str());
	sprintf_s(buf, 256, "Upload: %s, Download: %s", Util::formatBytes(Socket::getTotalUp()).c_str(), Util::formatBytes(Socket::getTotalDown()).c_str());
	::SetDlgItemText(m_hWnd, IDC_ACTUALS, Text::toT(buf).c_str());
	if(Socket::getTotalDown() > 0) {
		sprintf_s(buf, 256, "Ratio (up/down): %.2f", ((double)Socket::getTotalUp()) / ((double)Socket::getTotalDown()));
		::SetDlgItemText(m_hWnd, IDC_ACTUAL_RATIO, Text::toT(buf).c_str());
	}
	CenterWindow(m_hWnd, ::GetParent(m_hWnd));
	c.addListener(this);
	c.downloadFile("http://www.czdc.org/version.xml");
	TimerManager::getInstance()->addListener(this);
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void AboutDlg::onVersionData(WPARAM wParam) {
	tstring* x = (tstring*)wParam;
	::SetDlgItemText(m_hWnd, IDC_LATEST, x->c_str());
	delete x;
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void AboutDlg::OnCloseCmd(WORD wID) {
	TimerManager::getInstance()->removeListener(this);
	::EndDialog(m_hWnd, wID);
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void AboutDlg::on(HttpConnectionListener::Complete, HttpConnection* conn, const string&) throw() {
	if(!downBuf.empty()) {
		SimpleXML xml;
		xml.fromXML(downBuf);
		if(xml.findChild("Version")) {
			string version = xml.getChildAttrib("ActualVersion");
			version += "[" + xml.getChildAttrib("SubVersion") + "]";
			tstring* x = new tstring(Text::toT(version));
			::PostMessage(m_hWnd, WM_VERSIONDATA, (WPARAM)x, 0);
		}
	}
	conn->removeListener(this);
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void AboutDlg::on(HttpConnectionListener::Failed, HttpConnection* conn, const string& aLine) throw() {
	tstring* x = new tstring(Text::toT(aLine));
	::PostMessage(m_hWnd, WM_VERSIONDATA, (WPARAM)x, 0);
	conn->removeListener(this);
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void AboutDlg::onSpeaker(WPARAM wParam) {
    static char buf[256];
	if(wParam == STATS_SECOND) {
    	sprintf_s(buf, 256, "Uptime: %s", WinUtil::formatTime(Util::getUptime()).c_str());
		::SetDlgItemText(m_hWnd, IDC_UPTIME, Text::toT(buf).c_str());
    	sprintf_s(buf, 256, "Upload: %s, Download: %s", Util::formatBytes(Socket::getTotalUp()).c_str(), Util::formatBytes(Socket::getTotalDown()).c_str());
		::SetDlgItemText(m_hWnd, IDC_ACTUALS, Text::toT(buf).c_str());
	} else if(wParam == STATS_MINUTE) {
    	if(Socket::getTotalDown() > 0) {
    		sprintf_s(buf, 256, "Ratio (up/down): %.2f", ((double)Socket::getTotalUp()) / ((double)Socket::getTotalDown()));
			::SetDlgItemText(m_hWnd, IDC_ACTUAL_RATIO, Text::toT(buf).c_str());
    	}
    	sprintf_s(buf, 256, "Uptime: %s", WinUtil::formatTime(SETTING(TOTAL_UPTIME)).c_str());
		::SetDlgItemText(m_hWnd, IDC_TOTAL_UPTIME, Text::toT(buf).c_str());
    	sprintf_s(buf, 256, "Upload: %s, Download: %s", Util::formatBytes(SETTING(TOTAL_UPLOAD)).c_str(), Util::formatBytes(SETTING(TOTAL_DOWNLOAD)).c_str());
		::SetDlgItemText(m_hWnd, IDC_TOTALS, Text::toT(buf).c_str());
    	if(SETTING(TOTAL_DOWNLOAD) > 0) {
    		sprintf_s(buf, 256, "Ratio (up/down): %.2f", ((double)SETTING(TOTAL_UPLOAD)) / ((double)SETTING(TOTAL_DOWNLOAD)));
			::SetDlgItemText(m_hWnd, IDC_RATIO, Text::toT(buf).c_str());
    	}
	}
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void AboutDlg::DoModal(HWND hWndParent) {
	::DialogBoxParam(g_hInstance, MAKEINTRESOURCE(IDD_ABOUTBOX), hWndParent, AboutDlgProc, (LPARAM)this);
}
//----------------------------------------------------------------------------------------------------------------------------------------------------
