/*
 * CzDC - client for Direct Connect peer to peer network.

 * Copyright (C) 2002-2011 Petr Kozelka, PPK at CzDC dot org

 * CzDC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3
 * as published by the Free Software Foundation.

 * CzDC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with CzDC. If not, see <http://www.gnu.org/licenses/>.
 */

//--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
#include "stdafx.h"
//--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
#include "../client/DCPlusPlus.h"
//--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
#include "Resource.h"
//--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
#include "BasicSplitter.h"
//--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
#include "WinUtil.h"
//--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

BasicSplitter::BasicSplitter() : iSplitterPos(0), iProportionalPos(0), bVertical(true), bUpdateProportionalPos(true), bSplitterDisabled(false) {
	::SetRectEmpty(&rcSplitter);
	hCursor = WinUtil::hVerticalCursor;
}
//--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

bool BasicSplitter::BasicSplitterProc(UINT uMsg, WPARAM wParam, LPARAM lParam) {
    switch(uMsg) {
        case WM_MOUSEMOVE:
            return OnMouseMove(wParam, lParam);
        case WM_LBUTTONDOWN:
            OnLButtonDown(lParam);
            return false;
        case WM_LBUTTONUP:
            OnLButtonUp();
            return false;
    }

    return false;
}
//--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

bool BasicSplitter::OnMouseMove(WPARAM wParam, LPARAM lParam) {
	int ixPos = GET_X_LPARAM(lParam);
	int iyPos = GET_Y_LPARAM(lParam);

	if((wParam & MK_LBUTTON) && ::GetCapture() == GetWindowHandle()) {
		int iNewSplitPos = (bVertical == true ? (ixPos - rcSplitter.left) : (iyPos - rcSplitter.top));

		if(iSplitterPos != iNewSplitPos) {
			SetSplitterPosition(iNewSplitPos, true);
		}

		return true;
	} else {
		if(IsCursorOverSplitter(ixPos, iyPos) == true) {
			::SetCursor(hCursor);
        }

		return false;
	}
}
//--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

void BasicSplitter::OnLButtonDown(LPARAM lParam) {
	int ixPos = GET_X_LPARAM(lParam);
	int iyPos = GET_Y_LPARAM(lParam);

	if(IsCursorOverSplitter(ixPos, iyPos) == true) {
		::SetCapture(GetWindowHandle());
		::SetCursor(hCursor);
	}
}
//--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

void BasicSplitter::OnLButtonUp() {
	::ReleaseCapture();
}
//--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

void BasicSplitter::SetSplitterRect(const LPRECT &lpRect) {
	rcSplitter = *lpRect;

    int iTotal = (bVertical == true? (rcSplitter.right - rcSplitter.left - 4) : (rcSplitter.bottom - rcSplitter.top - 4));

    if(iTotal > 0) {
        bUpdateProportionalPos = false;

    	int iNewPos = (iProportionalPos * iTotal) / 10000;

       	SetSplitterPosition(iNewPos, false);
    }

	UpdateSplitterParts();
}
//--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

void BasicSplitter::SetSplitterPosition(int iPos, const bool &bUpdate/* = true*/) {
    if(iPos < 10) {
        iPos = 10;
    } else {
        int iMaxPos = (bVertical == true ? (rcSplitter.right - rcSplitter.left) : (rcSplitter.bottom - rcSplitter.top)) - 10;
        if(iPos > iMaxPos) {
            iPos = iMaxPos;
        }
    }

	bool bPosChanged = (iSplitterPos != iPos);

	iSplitterPos = iPos;

    if(bUpdateProportionalPos == true) {
        int iTotal = (bVertical == true ? (rcSplitter.right - rcSplitter.left - 4) : (rcSplitter.bottom - rcSplitter.top - 4));

        if(iTotal > 0) {
        	iProportionalPos = (iSplitterPos * 10000) / iTotal;
        } else {
        	iProportionalPos = 0;
        }
    } else {
    	bUpdateProportionalPos = true;
    }

	if(bUpdate == true && bPosChanged == true) {
		UpdateSplitterParts();
    }
}
//--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

bool BasicSplitter::IsCursorOverSplitter(const int &iX, const int &iY) {
	if(bSplitterDisabled == true) {
		return false;
    }

	if(iX == -1 || iY == -1 || iX < rcSplitter.left || iX > rcSplitter.right || iY < rcSplitter.top || iY > rcSplitter.bottom) {
		return false;
    }

    if(bVertical == true) {
        if(iX > (rcSplitter.left + iSplitterPos - 2) && iX < (rcSplitter.left + iSplitterPos + 2)) {
            return true;
        }
    } else {
        if(iY > (rcSplitter.top + iSplitterPos - 2) && iY < (rcSplitter.top + iSplitterPos + 2)) {
            return true;
        }
    }

	return false;
}
//--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

void BasicSplitter::SetSplitterHorCursor() {
    bVertical = false;
    hCursor = WinUtil::hHorizontalCursor;
}
//--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
