/*
 * Copyright (C) 2001-2011 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "stdafx.h"
#include "../client/DCPlusPlus.h"

#include "Resource.h"

#include "DirectoryListingFrm.h"
#include "ShellContextMenu.h"
#include "WinUtil.h"
#include "LineDlg.h"
#include "FlatTabCtrl.h"

#include "../client/File.h"
#include "../client/QueueManager.h"
#include "../client/ShareManager.h"
#include "../client/StringTokenizer.h"
#include "../client/ADLSearch.h"
#include "../client/MerkleTree.h"
#include "../client/User.h"
#include "../client/ClientManager.h"

ATOM DirectoryListingFrame::m_aAtom = 0;
UINT DirectoryListingFrame::m_uiResourceID = IDR_DIRECTORY;

LRESULT CALLBACK DirectoryListingFrame::DirectoryListingFrameWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
	DirectoryListingFrame * pParent;

	if(uMsg == WM_NCCREATE) {
		pParent = (DirectoryListingFrame *)((LPMDICREATESTRUCT)((LPCREATESTRUCT)lParam)->lpCreateParams)->lParam;
		SetWindowLongPtr(hWnd, GWLP_USERDATA, (LONG_PTR)pParent);

		pParent->m_hWnd = hWnd;
	} else {
		pParent = (DirectoryListingFrame *)GetWindowLongPtr(hWnd, GWLP_USERDATA);
		if(pParent == NULL) {
			return ::DefMDIChildProc(hWnd, uMsg, wParam, lParam);
		}
	}

	return pParent->DirectoryListingFrameProc(hWnd, uMsg, wParam, lParam);
}

LRESULT DirectoryListingFrame::DirectoryListingFrameProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    switch(uMsg) {
        case WM_CREATE:
            OnCreate();
            return 0;
        case WM_CONTEXTMENU:
            if(onContextMenu(wParam, lParam) == true) {
                return 0;
            }
            break;
        case WM_CLOSE:
            if(onClose() == true) {
                return 0;
            }
            break;
        case WM_SETFOCUS:
            ::SetFocus(ctrlList.m_hWnd);
            break;
        case WM_SIZE:
            if(wParam != SIZE_MINIMIZED) {
                UpdateLayout();
            }
            break;
        case WM_REFRESH_SETTINGS:
            onRefreshSettings();
            return 0;
        case WM_COMMAND:
            switch(LOWORD(wParam)) {
                case IDC_DOWNLOAD:
                    downloadList(Text::toT(SETTING(DOWNLOAD_DIRECTORY)));
                    return 0;
                case IDC_DOWNLOADDIR:
                    onDownloadDir();
                    return 0;
                case IDC_DOWNLOADDIRTO:
                    onDownloadDirTo();
                    return 0;
                case IDC_DOWNLOADTO:
                    onDownloadTo();
                    return 0;
                case IDC_GO_TO_DIRECTORY:
                    onGoToDirectory();
                    return 0;
                case IDC_VIEW_AS_TEXT:
                    downloadList(Text::toT(Util::getTempPath()), true);
                    return 0;
                case IDC_COPY_LINK:
                case IDC_COPY_TTH:
                case IDC_COPY_NICK:
                case IDC_COPY_FILENAME:
                case IDC_COPY_SIZE:
                    onCopy(LOWORD(wParam));
                    return 0;
                case IDC_ADD_TO_FAVORITES:
                    onAddToFavorites();
                    return 0;
                case IDC_SEARCH_ALTERNATES:
                    onSearchAlternates();
                    return 0;
            }

            if(LOWORD(wParam) >= IDC_DOWNLOAD_TARGET && LOWORD(wParam) <= IDC_DOWNLOAD_TARGET + targets.size() + WinUtil::lastDirs.size()) {
                onDownloadTarget(LOWORD(wParam));
                return 0;
            } else if(LOWORD(wParam) >= IDC_DOWNLOAD_TARGET_DIR && LOWORD(wParam) <= IDC_DOWNLOAD_TARGET_DIR + WinUtil::lastDirs.size()) {
                onDownloadTargetDir(LOWORD(wParam));
                return 0;
            } else if(LOWORD(wParam) >= IDC_DOWNLOAD_FAVORITE_DIRS && LOWORD(wParam) <= IDC_DOWNLOAD_FAVORITE_DIRS + FavoriteManager::getInstance()->getFavoriteDirs().size()) {
                onDownloadFavoriteDirs(LOWORD(wParam));
                return 0;
            } else if(LOWORD(wParam) >= IDC_DOWNLOAD_WHOLE_FAVORITE_DIRS && LOWORD(wParam) <= IDC_DOWNLOAD_WHOLE_FAVORITE_DIRS + FavoriteManager::getInstance()->getFavoriteDirs().size()) {
                onDownloadWholeFavoriteDirs(LOWORD(wParam));
                return 0;
            } else if(LOWORD(wParam) >= IDC_PRIORITY_PAUSED && LOWORD(wParam) <= IDC_PRIORITY_HIGHEST) {
                onDownloadWithPrio(LOWORD(wParam));
                return 0;
            } else if(LOWORD(wParam) >= IDC_PRIORITY_PAUSED+90 && LOWORD(wParam) <= IDC_PRIORITY_HIGHEST+90) {
                onDownloadDirWithPrio(LOWORD(wParam));
                return 0;
            }

            break;
        case WM_NOTIFY:
            if(((LPNMHDR)lParam)->idFrom == IDC_FILES) {
                switch(((LPNMHDR)lParam)->code) {
                    case LVN_GETDISPINFO: {
                        ctrlList.onGetDispInfo((LPNMHDR)lParam);
                        return 0;
                    }
                    case LVN_COLUMNCLICK: {
                        ctrlList.onColumnClick((LPNMHDR)lParam);
                        return 0;
                    }
                    case LVN_KEYDOWN:
                        if(onKeyDown((LPNMHDR)lParam) == true) {
                        	return 0;
                        }
                        break;
                    case NM_DBLCLK:
                        onDoubleClickFiles((LPNMHDR)lParam);
                        return 0;
                    case LVN_ITEMCHANGED:
                        updateStatus();
                        return 0;
                }
            } else if(((LPNMHDR)lParam)->idFrom == IDC_DIRECTORIES) {
                if(((LPNMHDR)lParam)->code == TVN_KEYDOWN) {
                    if(onKeyDownDirs((LPNMHDR)lParam) == true) {
                        return 0;
                    }
                } else if(((LPNMHDR)lParam)->code == TVN_SELCHANGED) {
                    onSelChangedDirectories((LPNMHDR)lParam);
                    return 0;
                }
            }

            break;
        case WM_NCDESTROY:
            delete this;
            return ::DefMDIChildProc(hWnd, uMsg, wParam, lParam);
    }

    if(UCHandlerProc(uMsg, wParam) == true) {
    	return 0;
    }

	LRESULT lResult = 0;
	if(MDIWindowProc(hWnd, uMsg, wParam, lParam, lResult) == true) {
    	return lResult;
    }

    if(uMsg == WM_MOUSEACTIVATE) {
        lResult = ::DefMDIChildProc(m_hWnd, uMsg, wParam, lParam);
    }

    if(BasicSplitterProc(uMsg, wParam, lParam) == true) {
    	return lResult;
    }

	return ::DefMDIChildProc(hWnd, uMsg, wParam, lParam);
}

LRESULT CALLBACK DirectoryListingFrame::TreeWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    if(uMsg == WM_XBUTTONUP) {
		DirectoryListingFrame * pParent = (DirectoryListingFrame *)::GetWindowLongPtr(hWnd, GWLP_USERDATA);
		if(pParent != NULL) {
            if(pParent->onXButtonUp(wParam) == true) {
                return TRUE;
            }
        }
    }

    return ::CallWindowProc(OldTreeViewProc, hWnd, uMsg, wParam, lParam);
}

LRESULT CALLBACK DirectoryListingFrame::StatusWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    if(uMsg == WM_COMMAND) {
        switch(LOWORD(wParam)) {
            case IDC_FIND: {
                DirectoryListingFrame * pParent = (DirectoryListingFrame *)::GetWindowLongPtr(hWnd, GWLP_USERDATA);
                if(pParent != NULL) {
                    pParent->onFind();
                }

                break;
            }
            case IDC_NEXT: {
                DirectoryListingFrame * pParent = (DirectoryListingFrame *)::GetWindowLongPtr(hWnd, GWLP_USERDATA);
                if(pParent != NULL) {
                    pParent->onNext();
                }

                break;
            }
            case IDC_MATCH_QUEUE: {
                DirectoryListingFrame * pParent = (DirectoryListingFrame *)::GetWindowLongPtr(hWnd, GWLP_USERDATA);
                if(pParent != NULL) {
                    pParent->onMatchQueue();
                }

                break;
            }
            case IDC_FILELIST_DIFF: {
                DirectoryListingFrame * pParent = (DirectoryListingFrame *)::GetWindowLongPtr(hWnd, GWLP_USERDATA);
                if(pParent != NULL) {
                    pParent->onListDiff();
                }

                break;
            }
        }
    }

    return ::CallWindowProc(OldStatusBarProc, hWnd, uMsg, wParam, lParam);
}

int DirectoryListingFrame::columnIndexes[] = { COLUMN_FILENAME, COLUMN_TYPE, COLUMN_SIZE, COLUMN_EXACT_SIZE, COLUMN_TTH };
int DirectoryListingFrame::columnSizes[] = { 300, 60, 100, 100, 200 };

static ResourceManager::Strings columnNames[] = { ResourceManager::FILE, ResourceManager::TYPE, ResourceManager::SIZE, ResourceManager::EXACT_SIZE, ResourceManager::TTH_ROOT };
DirectoryListingFrame::FrameMap DirectoryListingFrame::frames;

DirectoryListingFrame::UserMap DirectoryListingFrame::lists;

void DirectoryListingFrame::openWindow(const tstring& aFile, const UserPtr& aUser, int64_t aSpeed) {
	UserIter i = lists.find(aUser);
	if(i != lists.end()) {
		if(!BOOLSETTING(POPUNDER_FILELIST)) {
			i->second->speed = aSpeed;
			::SendMessage(WinUtil::mdiClient, WM_MDIACTIVATE, (WPARAM)i->second->m_hWnd, 0);
		}
	} else {
		DirectoryListingFrame* frame = new DirectoryListingFrame(aUser, aSpeed);
		if(BOOLSETTING(POPUNDER_FILELIST)) {
			WinUtil::hiddenCreateEx(frame);
		} else {
			frame->CreateEx(frame, m_aAtom, m_uiResourceID);
		}
		frame->loadFile(aFile);
		frames.insert( FramePair( frame->m_hWnd, frame ) );
	}
}

void DirectoryListingFrame::openWindow(const UserPtr& aUser, const string& txt, int64_t aSpeed) {
	UserIter i = lists.find(aUser);
	if(i != lists.end()) {
		i->second->speed = aSpeed;
		i->second->loadXML(txt);
	} else {
		DirectoryListingFrame* frame = new DirectoryListingFrame(aUser, aSpeed);
		if(BOOLSETTING(POPUNDER_FILELIST)) {
			WinUtil::hiddenCreateEx(frame);
		} else {
			frame->CreateEx(frame, m_aAtom, m_uiResourceID);
		}
		frame->loadXML(txt);
		frames.insert( FramePair( frame->m_hWnd, frame ) );
	}
}

DirectoryListingFrame::DirectoryListingFrame(const UserPtr& aUser, int64_t aSpeed) :
	historyIndex(0), treeRoot(NULL), skipHits(0), files(0), speed(aSpeed), updating(false),
    dl(new DirectoryListing(aUser)), searching(false), targetMenu(NULL),
    targetDirMenu(NULL), fileMenu(NULL), directoryMenu(NULL), copyMenu(NULL) {
	lists.insert(make_pair(aUser, this));

    RegisterWindowClass(m_aAtom, DirectoryListingFrameWndProc, _T("DirectoryListingFrame"), m_uiResourceID);
}

DirectoryListingFrame::~DirectoryListingFrame() {
	dcassert(lists.find(dl->getUser()) != lists.end());
	lists.erase(dl->getUser());

    if(fileMenu != NULL) {
        ::DestroyMenu(fileMenu);
    }

    if(directoryMenu != NULL) {
        ::DestroyMenu(directoryMenu);
    }
}

void DirectoryListingFrame::loadFile(const tstring& name) {
	try {
		dl->loadFile(Text::fromT(name));
		ADLSearchManager::getInstance()->matchListing(*dl);
		refreshTree(Text::toT(WinUtil::getInitialDir(dl->getUser())));
	} catch(const Exception& e) {
		error = WinUtil::getNicks(dl->getUser()) + Text::toT(": " + e.getError());
	}

	initStatus();
}

void DirectoryListingFrame::loadXML(const string& txt) {
	try {
		refreshTree(Text::toT(Util::toNmdcFile(dl->loadXML(txt, true))));
	} catch(const Exception& e) {
		error = WinUtil::getNicks(dl->getUser()) + Text::toT(": " + e.getError());
	}

	initStatus();
}

void DirectoryListingFrame::OnCreate() {
    WinUtil::tabCtrl->addTab(m_hWnd, RGB(255, 0, 255), IDR_DIRECTORY);
    created = true;

    m_hWndStatusBar = CreateWindowEx(0, STATUSCLASSNAME, _T(""), WS_CHILD | WS_VISIBLE | WS_CLIPCHILDREN | WS_CLIPSIBLINGS | SBARS_SIZEGRIP, 0, 0, 0, 0, m_hWnd, 0, g_hInstance, NULL);
    ::SetWindowLongPtr(m_hWndStatusBar, GWLP_USERDATA, (LONG_PTR)this);
    OldStatusBarProc = (WNDPROC)::SetWindowLongPtr(m_hWndStatusBar, GWLP_WNDPROC, (LONG_PTR)StatusWndProc);

    hWndTree = CreateWindowEx(WS_EX_CLIENTEDGE, WC_TREEVIEW, _T(""), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | TVS_HASBUTTONS | TVS_LINESATROOT | TVS_HASLINES | TVS_SHOWSELALWAYS |
        TVS_DISABLEDRAGDROP, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, m_hWnd, (HMENU)IDC_DIRECTORIES, g_hInstance, NULL);
    ::SetWindowLongPtr(hWndTree, GWLP_USERDATA, (LONG_PTR)this);
    OldTreeViewProc = (WNDPROC)::SetWindowLongPtr(hWndTree, GWLP_WNDPROC, (LONG_PTR)TreeWndProc);

	ctrlList.Create(m_hWnd, WS_CHILD | WS_VISIBLE | WS_CLIPCHILDREN | WS_CLIPSIBLINGS | LVS_REPORT | LVS_SHOWSELALWAYS | LVS_SHAREIMAGELISTS, WS_EX_CLIENTEDGE, IDC_FILES,
        LVS_EX_LABELTIP | LVS_EX_HEADERDRAGDROP | LVS_EX_FULLROWSELECT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT);
	
	::SendMessage(hWndTree, TVM_SETBKCOLOR, 0, (LPARAM)WinUtil::bgColor);
	::SendMessage(hWndTree, TVM_SETTEXTCOLOR, 0, (LPARAM)WinUtil::textColor);

	WinUtil::splitTokens(columnIndexes, SETTING(DIRECTORLISTINGFRAME_ORDER), COLUMN_LAST);
	WinUtil::splitTokens(columnSizes, SETTING(DIRECTORLISTINGFRAME_WIDTHS), COLUMN_LAST);

	for(uint8_t ui8i = 0; ui8i < (sizeof(columnSizes) / sizeof(columnSizes[0])); ui8i++) {
		int fmt = ((ui8i == COLUMN_SIZE) || (ui8i == COLUMN_EXACT_SIZE) || (ui8i == COLUMN_TYPE)) ? LVCFMT_RIGHT : LVCFMT_LEFT;
		ctrlList.InsertColumn(ui8i, CTSTRING_I(columnNames[ui8i]), fmt, columnSizes[ui8i], ui8i);
	}
	ctrlList.setColumnOrderArray(COLUMN_LAST, columnIndexes);

	ctrlList.setSortColumn(COLUMN_FILENAME);

	::SendMessage(hWndTree, TVM_SETIMAGELIST, (WPARAM)TVSIL_NORMAL, (LPARAM)WinUtil::fileImages);
	::SendMessage(ctrlList.m_hWnd, LVM_SETIMAGELIST, LVSIL_SMALL, (LPARAM)WinUtil::fileImages);

	hWndFind = CreateWindowEx(0, WC_BUTTON, CTSTRING(FIND), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | BS_PUSHBUTTON, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
        m_hWndStatusBar, (HMENU)IDC_FIND, g_hInstance, NULL);
	::SendMessage(hWndFind, WM_SETFONT, (WPARAM)WinUtil::systemFont, MAKELPARAM(TRUE, 0));

	hWndFindNext = CreateWindowEx(0, WC_BUTTON, CTSTRING(NEXT), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | BS_PUSHBUTTON, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
        m_hWndStatusBar, (HMENU)IDC_NEXT, g_hInstance, NULL);
	::SendMessage(hWndFindNext, WM_SETFONT, (WPARAM)WinUtil::systemFont, MAKELPARAM(TRUE, 0));

	hWndMatchQueue = CreateWindowEx(0, WC_BUTTON, CTSTRING(MATCH_QUEUE), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | BS_PUSHBUTTON, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
        m_hWndStatusBar, (HMENU)IDC_MATCH_QUEUE, g_hInstance, NULL);
	::SendMessage(hWndMatchQueue, WM_SETFONT, (WPARAM)WinUtil::systemFont, MAKELPARAM(TRUE, 0));

	hWndListDiff = CreateWindowEx(0, WC_BUTTON, CTSTRING(FILE_LIST_DIFF), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | BS_PUSHBUTTON, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
        m_hWndStatusBar, (HMENU)IDC_FILELIST_DIFF, g_hInstance, NULL);
	::SendMessage(hWndListDiff, WM_SETFONT, (WPARAM)WinUtil::systemFont, MAKELPARAM(TRUE, 0));

	iProportionalPos = 2500;

	string nick = ClientManager::getInstance()->getNicks(dl->getUser()->getCID())[0];
	treeRoot = TreeInsertItem(hWndTree, TVIF_IMAGE | TVIF_SELECTEDIMAGE | TVIF_TEXT | TVIF_PARAM, Text::toT(nick).c_str(), WinUtil::getDirIconIndex(),
        WinUtil::getDirIconIndex(), 0, 0, (LPARAM)dl->getRoot(), NULL, TVI_SORT);

	memset(statusSizes, 0, sizeof(statusSizes));
	statusSizes[STATUS_FILE_LIST_DIFF] = WinUtil::getTextWidth(TSTRING(FILE_LIST_DIFF), m_hWnd) + 8;
	statusSizes[STATUS_MATCH_QUEUE] = WinUtil::getTextWidth(TSTRING(MATCH_QUEUE), m_hWnd) + 8;
	statusSizes[STATUS_FIND] = WinUtil::getTextWidth(TSTRING(FIND), m_hWnd) + 8;
	statusSizes[STATUS_NEXT] = WinUtil::getTextWidth(TSTRING(NEXT), m_hWnd) + 8;

	::SendMessage(m_hWndStatusBar, SB_SETPARTS, STATUS_LAST, (LPARAM)statusSizes);

    targetMenu = ::CreatePopupMenu();

	copyMenu = ::CreatePopupMenu();
	::AppendMenu(copyMenu, MF_STRING, IDC_COPY_NICK, CTSTRING(COPY_NICK));
	::AppendMenu(copyMenu, MF_STRING, IDC_COPY_FILENAME, CTSTRING(FILENAME));
	::AppendMenu(copyMenu, MF_STRING, IDC_COPY_SIZE, CTSTRING(SIZE));
	::AppendMenu(copyMenu, MF_STRING, IDC_COPY_TTH, CTSTRING(TTH_ROOT));
	::AppendMenu(copyMenu, MF_STRING, IDC_COPY_LINK, CTSTRING(COPY_MAGNET_LINK));

	fileMenu = ::CreatePopupMenu();
    ::AppendMenu(fileMenu, MF_STRING, IDC_DOWNLOAD, CTSTRING(DOWNLOAD));
	::AppendMenu(fileMenu, MF_POPUP, (UINT_PTR)targetMenu, CTSTRING(DOWNLOAD_TO));

    WinUtil::AddPrioritySubMenu(fileMenu, TSTRING(DOWNLOAD_WITH_PRIORITY));

	::AppendMenu(fileMenu, MF_STRING, IDC_VIEW_AS_TEXT, CTSTRING(VIEW_AS_TEXT));
	::AppendMenu(fileMenu, MF_STRING, IDC_SEARCH_ALTERNATES, CTSTRING(SEARCH_FOR_ALTERNATES));
	::AppendMenu(fileMenu, MF_SEPARATOR, 0, NULL);
	::AppendMenu(fileMenu, MF_STRING, IDC_ADD_TO_FAVORITES, CTSTRING(ADD_TO_FAVORITES));
	::AppendMenu(fileMenu, MF_SEPARATOR, 0, NULL);
	::AppendMenu(fileMenu, MF_POPUP, (UINT_PTR)copyMenu, CTSTRING(COPY));
    ::SetMenuDefaultItem(fileMenu, IDC_DOWNLOAD, 0);

    targetDirMenu = ::CreatePopupMenu();

    HMENU priorityDirMenu = ::CreatePopupMenu();
    ::AppendMenu(priorityDirMenu, MF_STRING, IDC_PRIORITY_PAUSED+90, CTSTRING(PAUSED));
	::AppendMenu(priorityDirMenu, MF_STRING, IDC_PRIORITY_LOWEST+90, CTSTRING(LOWEST));
	::AppendMenu(priorityDirMenu, MF_STRING, IDC_PRIORITY_LOW+90, CTSTRING(LOW));
	::AppendMenu(priorityDirMenu, MF_STRING, IDC_PRIORITY_NORMAL+90, CTSTRING(NORMAL));
	::AppendMenu(priorityDirMenu, MF_STRING, IDC_PRIORITY_HIGH+90, CTSTRING(HIGH));
	::AppendMenu(priorityDirMenu, MF_STRING, IDC_PRIORITY_HIGHEST+90, CTSTRING(HIGHEST));

    directoryMenu = ::CreatePopupMenu();
    ::AppendMenu(directoryMenu, MF_STRING, IDC_DOWNLOADDIR, CTSTRING(DOWNLOAD));
	::AppendMenu(directoryMenu, MF_POPUP, (UINT_PTR)targetDirMenu, CTSTRING(DOWNLOAD_TO));
	::AppendMenu(directoryMenu, MF_POPUP, (UINT_PTR)priorityDirMenu, CTSTRING(DOWNLOAD_WITH_PRIORITY));
	::AppendMenu(directoryMenu, MF_SEPARATOR, 0, NULL);
	::AppendMenu(directoryMenu, MF_STRING, IDC_ADD_TO_FAVORITES, CTSTRING(ADD_TO_FAVORITES));

	SettingsManager::getInstance()->addListener(this);
	closed = false;

	setWindowTitle();
}

void DirectoryListingFrame::updateTree(DirectoryListing::Directory* aTree, HTREEITEM aParent) {
	for(DirectoryListing::Directory::Iter i = aTree->directories.begin(); i != aTree->directories.end(); ++i) {
		tstring name;
		if(dl->getUtf8()) {
			name = Text::toT((*i)->getName());
		} else {
			name = Text::toT(Text::acpToUtf8((*i)->getName()));
		}
		int index = WinUtil::getDirIconIndex();
		HTREEITEM ht = TreeInsertItem(hWndTree, TVIF_IMAGE | TVIF_SELECTEDIMAGE | TVIF_TEXT | TVIF_PARAM, name.c_str(), index, index, 0, 0,
            (LPARAM)*i, aParent, TVI_SORT);
		if((*i)->getAdls()) {
            TreeSetItem(hWndTree, ht, TVIF_STATE, NULL, 0, 0, TVIS_BOLD, TVIS_BOLD, NULL);
		}
		updateTree(*i, ht);
	}
}
void DirectoryListingFrame::refreshTree(const tstring& root) {
	::SendMessage(hWndTree, WM_SETREDRAW, (WPARAM)FALSE, 0);

	HTREEITEM ht = findItem(treeRoot, root);
	if(ht == NULL) {
		ht = treeRoot;
	}

	DirectoryListing::Directory* d = (DirectoryListing::Directory*)TreeGetItemData(hWndTree, ht);

	HTREEITEM next = NULL;
	while((next = (HTREEITEM)::SendMessage(hWndTree, TVM_GETNEXTITEM, TVGN_CHILD, (LPARAM)ht)) != NULL) {
		::SendMessage(hWndTree, TVM_DELETEITEM, 0, (LPARAM)next);
	}

	updateTree(d, ht);
	
	::SendMessage(hWndTree, TVM_EXPAND, TVE_EXPAND, (LPARAM)treeRoot);

	int index = WinUtil::getDirIconIndex();
	TreeSetItem(hWndTree, ht, TVIF_IMAGE | TVIF_SELECTEDIMAGE, NULL, index, index, 0, 0, NULL);

	::SendMessage(hWndTree, TVM_SELECTITEM, TVGN_CARET, (LPARAM)NULL);
	selectItem(root);

	::SendMessage(hWndTree, WM_SETREDRAW, (WPARAM)TRUE, 0);
}

void DirectoryListingFrame::updateStatus() {
	if(!searching && !updating && ::IsWindow(m_hWndStatusBar)) {
		int cnt = (UINT)::SendMessage(ctrlList.m_hWnd, LVM_GETSELECTEDCOUNT, 0, 0);
		int64_t total = 0;
		if(cnt == 0) {
			cnt = (int)::SendMessage(ctrlList.m_hWnd, LVM_GETITEMCOUNT, 0, 0);
			total = ctrlList.forEachT(ItemInfo::TotalSize()).total;
		} else {
			total = ctrlList.forEachSelectedT(ItemInfo::TotalSize()).total;
		}

		tstring tmp = Text::toT(STRING(ITEMS) + ": " + Util::toString(cnt));
		bool u = false;

		int w = WinUtil::getTextWidth(tmp, m_hWndStatusBar);
		if(statusSizes[STATUS_SELECTED_FILES] < w) {
			statusSizes[STATUS_SELECTED_FILES] = w;
			u = true;
		}
		::SendMessage(m_hWndStatusBar, SB_SETTEXT, (STATUS_SELECTED_FILES | 0), (LPARAM)tmp.c_str());

		tmp = Text::toT(STRING(SIZE) + ": " + Util::formatBytes(total));
		w = WinUtil::getTextWidth(tmp, m_hWndStatusBar);
		if(statusSizes[STATUS_SELECTED_SIZE] < w) {
			statusSizes[STATUS_SELECTED_SIZE] = w;
			u = true;
		}
		::SendMessage(m_hWndStatusBar, SB_SETTEXT, (STATUS_SELECTED_SIZE | 0), (LPARAM)tmp.c_str());

		if(u) {
			UpdateLayout(true);
        }
	}
}

void DirectoryListingFrame::initStatus() {
	files = dl->getTotalFileCount();
	size = Util::formatBytes(dl->getTotalSize());

	tstring tmp = Text::toT(STRING(FILES) + ": " + Util::toString((uint32_t)dl->getTotalFileCount(true)));
	statusSizes[STATUS_TOTAL_FILES] = WinUtil::getTextWidth(tmp, m_hWnd);
	::SendMessage(m_hWndStatusBar, SB_SETTEXT, (STATUS_TOTAL_FILES | 0), (LPARAM)tmp.c_str());

	tmp = Text::toT(STRING(SIZE) + ": " + Util::formatBytes(dl->getTotalSize(true)));
	statusSizes[STATUS_TOTAL_SIZE] = WinUtil::getTextWidth(tmp, m_hWnd);
	::SendMessage(m_hWndStatusBar, SB_SETTEXT, (STATUS_TOTAL_SIZE | 0), (LPARAM)tmp.c_str());

	tmp = Text::toT(STRING(SPEED) + ": " + Util::formatBytes(speed) + "/s");
	statusSizes[STATUS_SPEED] = WinUtil::getTextWidth(tmp, m_hWnd);
	::SendMessage(m_hWndStatusBar, SB_SETTEXT, (STATUS_SPEED | 0), (LPARAM)tmp.c_str());

	UpdateLayout(false);
}

void DirectoryListingFrame::onSelChangedDirectories(LPNMHDR pnmh) {
	NMTREEVIEW* p = (NMTREEVIEW*) pnmh;

	if(p->itemNew.state & TVIS_SELECTED) {
		DirectoryListing::Directory* d = (DirectoryListing::Directory*)p->itemNew.lParam;
		changeDir(d, TRUE);
		addHistory(dl->getPath(d));
	}
}


void DirectoryListingFrame::addHistory(const string& name) {
	history.erase(history.begin() + historyIndex, history.end());
	while(history.size() > 25)
		history.pop_front();
	history.push_back(name);
	historyIndex = history.size();
}

void DirectoryListingFrame::changeDir(DirectoryListing::Directory* d, BOOL enableRedraw) {
	::SendMessage(ctrlList.m_hWnd, WM_SETREDRAW, (WPARAM)FALSE, 0);
	updating = true;
	clearList();

	for(DirectoryListing::Directory::Iter i = d->directories.begin(); i != d->directories.end(); ++i) {
		ctrlList.insertItem((int)::SendMessage(ctrlList.m_hWnd, LVM_GETITEMCOUNT, 0, 0), new ItemInfo(*i, dl->getUtf8()), WinUtil::getDirIconIndex());
	}
	for(DirectoryListing::File::Iter j = d->files.begin(); j != d->files.end(); ++j) {
		ItemInfo* ii = new ItemInfo(*j, dl->getUtf8());
		ctrlList.insertItem((int)::SendMessage(ctrlList.m_hWnd, LVM_GETITEMCOUNT, 0, 0), ii, WinUtil::getIconIndex(ii->getFileName()));
	}
	ctrlList.resort();
	::SendMessage(ctrlList.m_hWnd, WM_SETREDRAW, (WPARAM)enableRedraw, 0);
	updating = false;
	updateStatus();
}

void DirectoryListingFrame::up() {
	HTREEITEM t = (HTREEITEM)::SendMessage(hWndTree, TVM_GETNEXTITEM, TVGN_CARET, 0);

	if(t == NULL) {
		return;
    }

	t = (HTREEITEM)::SendMessage(hWndTree, TVM_GETNEXTITEM, TVGN_PARENT, (LPARAM)t);

	if(t == NULL) {
		return;
    }

	::SendMessage(hWndTree, TVM_SELECTITEM, TVGN_CARET, (LPARAM)t);
}

void DirectoryListingFrame::back() {
	if(history.size() > 1 && historyIndex > 1) {
		size_t n = min(historyIndex, history.size()) - 1;
		deque<string> tmp = history;
		selectItem(Text::toT(history[n - 1]));
		historyIndex = n;
		history = tmp;
	}
}
void DirectoryListingFrame::forward() {
	if(history.size() > 1 && historyIndex < history.size()) {
		size_t n = min(historyIndex, history.size() - 1);
		deque<string> tmp = history;
		selectItem(Text::toT(history[n]));
		historyIndex = n + 1;
		history = tmp;
	}
}

void DirectoryListingFrame::onDoubleClickFiles(LPNMHDR pnmh) {
	NMITEMACTIVATE* item = (NMITEMACTIVATE*) pnmh;

	HTREEITEM t = (HTREEITEM)::SendMessage(hWndTree, TVM_GETNEXTITEM, TVGN_CARET, 0);
	if(t != NULL && item->iItem != -1) {
		ItemInfo* ii = ctrlList.getItemData(item->iItem);

		if(ii->type == ItemInfo::FILE) {
			try {
				dl->download(ii->file, SETTING(DOWNLOAD_DIRECTORY) + Text::fromT(ii->getFileName()), false, WinUtil::isShift());
			} catch(const Exception& e) {
				::SendMessage(m_hWndStatusBar, SB_SETTEXT, (STATUS_TEXT | 0), (LPARAM)Text::toT(e.getError()).c_str());
			}
		} else {
			HTREEITEM ht = (HTREEITEM)::SendMessage(hWndTree, TVM_GETNEXTITEM, TVGN_CHILD, (LPARAM)t);
			while(ht != NULL) {
				if((DirectoryListing::Directory*)TreeGetItemData(hWndTree, ht) == ii->dir) {
					::SendMessage(hWndTree, TVM_SELECTITEM, TVGN_CARET, (LPARAM)ht);
					break;
				}
				ht = (HTREEITEM)::SendMessage(hWndTree, TVM_GETNEXTITEM, TVGN_NEXT, (LPARAM)ht);
			}
		}
	}
}

void DirectoryListingFrame::onDownloadDir() {
	HTREEITEM t = (HTREEITEM)::SendMessage(hWndTree, TVM_GETNEXTITEM, TVGN_CARET, 0);
	if(t != NULL) {
		DirectoryListing::Directory* dir = (DirectoryListing::Directory*)TreeGetItemData(hWndTree, t);
		try {
			dl->download(dir, SETTING(DOWNLOAD_DIRECTORY), WinUtil::isShift());
		} catch(const Exception& e) {
			::SendMessage(m_hWndStatusBar, SB_SETTEXT, (STATUS_TEXT | 0), (LPARAM)Text::toT(e.getError()).c_str());
		}
	}
}

void DirectoryListingFrame::onDownloadDirWithPrio(WORD wID) {
	HTREEITEM t = (HTREEITEM)::SendMessage(hWndTree, TVM_GETNEXTITEM, TVGN_CARET, 0);
	if(t != NULL) {
		DirectoryListing::Directory* dir = (DirectoryListing::Directory*)TreeGetItemData(hWndTree, t);

		QueueItem::Priority p;
		switch(wID-90) {
			case IDC_PRIORITY_PAUSED:
                p = QueueItem::PAUSED;
                break;
			case IDC_PRIORITY_LOWEST:
                p = QueueItem::LOWEST;
                break;
			case IDC_PRIORITY_LOW:
                p = QueueItem::LOW;
                break;
			case IDC_PRIORITY_NORMAL:
                p = QueueItem::NORMAL;
                break;
			case IDC_PRIORITY_HIGH:
                p = QueueItem::HIGH;
                break;
			case IDC_PRIORITY_HIGHEST:
                p = QueueItem::HIGHEST;
                break;
			default:
                p = QueueItem::DEFAULT;
                break;
		}

		try {
			dl->download(dir, SETTING(DOWNLOAD_DIRECTORY), WinUtil::isShift(), p);
		} catch(const Exception& e) {
			::SendMessage(m_hWndStatusBar, SB_SETTEXT, (STATUS_TEXT | 0), (LPARAM)Text::toT(e.getError()).c_str());
		}
	}
}

void DirectoryListingFrame::onDownloadDirTo() {
	HTREEITEM t = (HTREEITEM)::SendMessage(hWndTree, TVM_GETNEXTITEM, TVGN_CARET, 0);
	if(t != NULL) {
		DirectoryListing::Directory* dir = (DirectoryListing::Directory*)TreeGetItemData(hWndTree, t);
		tstring target = Text::toT(SETTING(DOWNLOAD_DIRECTORY));
		if(WinUtil::browseDirectory(target, m_hWnd)) {
			WinUtil::addLastDir(target);

			try {
				dl->download(dir, Text::fromT(target), WinUtil::isShift());
			} catch(const Exception& e) {
				::SendMessage(m_hWndStatusBar, SB_SETTEXT, (STATUS_TEXT | 0), (LPARAM)Text::toT(e.getError()).c_str());
			}
		}
	}
}

void DirectoryListingFrame::downloadList(const tstring& aTarget, bool view /* = false */, QueueItem::Priority prio /* = QueueItem::Priority::DEFAULT */) {
	int i = -1;
	while( (i = (int)::SendMessage(ctrlList.m_hWnd, LVM_GETNEXTITEM, i, LVNI_SELECTED)) != -1) {
		ItemInfo* ii = ctrlList.getItemData(i);
		tstring target = aTarget.empty() ? Text::toT(SETTING(DOWNLOAD_DIRECTORY)) : aTarget;

		try {
			if(ii->type == ItemInfo::FILE) {
				if(view) {
					File::deleteFile(Text::fromT(target) + Util::validateFileName(ii->file->getName()));
				}
				dl->download(ii->file, Text::fromT(target + ii->getFileName()), view, WinUtil::isShift(), prio);
			} else if(!view) {
				dl->download(ii->dir, Text::fromT(target), WinUtil::isShift(), prio);
			}
		} catch(const Exception& e) {
			::SendMessage(m_hWndStatusBar, SB_SETTEXT, (STATUS_TEXT | 0), (LPARAM)Text::toT(e.getError()).c_str());
		}
	}
}

void DirectoryListingFrame::onDownloadWithPrio(WORD wID) {
	QueueItem::Priority p;

	switch(wID) {
		case IDC_PRIORITY_PAUSED:
            p = QueueItem::PAUSED;
            break;
		case IDC_PRIORITY_LOWEST:
            p = QueueItem::LOWEST;
            break;
		case IDC_PRIORITY_LOW:
            p = QueueItem::LOW;
            break;
		case IDC_PRIORITY_NORMAL:
            p = QueueItem::NORMAL;
            break;
		case IDC_PRIORITY_HIGH:
            p = QueueItem::HIGH;
            break;
		case IDC_PRIORITY_HIGHEST:
            p = QueueItem::HIGHEST;
            break;
		default:
            p = QueueItem::DEFAULT;
            break;
	}

	downloadList(Text::toT(SETTING(DOWNLOAD_DIRECTORY)), false, p);
}

void DirectoryListingFrame::onDownloadTo() {
	if((UINT)::SendMessage(ctrlList.m_hWnd, LVM_GETSELECTEDCOUNT, 0, 0) == 1) {
		ItemInfo* ii = ctrlList.getItemData((int)::SendMessage(ctrlList.m_hWnd, LVM_GETNEXTITEM, (WPARAM)-1, LVNI_SELECTED));

		try {
			if(ii->type == ItemInfo::FILE) {
				tstring target = Text::toT(SETTING(DOWNLOAD_DIRECTORY)) + ii->getFileName();
				if(WinUtil::browseFile(target, m_hWnd)) {
					WinUtil::addLastDir(Util::getFilePath(target));
					dl->download(ii->file, Text::fromT(target), false, WinUtil::isShift());
				}
			} else {
				tstring target = Text::toT(SETTING(DOWNLOAD_DIRECTORY));
				if(WinUtil::browseDirectory(target, m_hWnd)) {
					WinUtil::addLastDir(target);
					dl->download(ii->dir, Text::fromT(target), WinUtil::isShift());
				}
			}
		} catch(const Exception& e) {
			::SendMessage(m_hWndStatusBar, SB_SETTEXT, (STATUS_TEXT | 0), (LPARAM)Text::toT(e.getError()).c_str());
		}
	} else {
		tstring target = Text::toT(SETTING(DOWNLOAD_DIRECTORY));
		if(WinUtil::browseDirectory(target, m_hWnd)) {
			WinUtil::addLastDir(target);
			downloadList(target);
		}
	}
}

void DirectoryListingFrame::onAddToFavorites() {
	UserPtr pUser = dl->getUser();
	if(pUser != (User*)NULL) {
		FavoriteManager::getInstance()->addFavoriteUser(pUser);
    }
}

void DirectoryListingFrame::onMatchQueue() {
	int x = QueueManager::getInstance()->matchListing(*dl);
	TCHAR * buf = new TCHAR[STRING(MATCHED_FILES).length() + 32];
	_stprintf_s(buf, STRING(MATCHED_FILES).length() + 32, CTSTRING(MATCHED_FILES), x);
	::SendMessage(m_hWndStatusBar, SB_SETTEXT, (STATUS_TEXT | 0), (LPARAM)buf);
	delete[] buf;
}

void DirectoryListingFrame::onListDiff() {
	tstring file;
	if(WinUtil::browseFile(file, m_hWnd, false, Text::toT(Util::getListPath()), _T("File Lists\0*.xml.bz2\0All Files\0*.*\0"))) {
		DirectoryListing dirList(dl->getUser());
		dirList.loadFile(Text::fromT(file));
		dl->getRoot()->filterList(dirList);
		refreshTree(Util::emptyStringT);
		initStatus();
		updateStatus();
	}
}

void DirectoryListingFrame::onGoToDirectory() {
	if((UINT)::SendMessage(ctrlList.m_hWnd, LVM_GETSELECTEDCOUNT, 0, 0) != 1) {
		return;
    }

	tstring fullPath;
	ItemInfo* ii = ctrlList.getItemData((int)::SendMessage(ctrlList.m_hWnd, LVM_GETNEXTITEM, (WPARAM)-1, LVNI_SELECTED));
	if(ii->type == ItemInfo::FILE) {
		if(!ii->file->getAdls()) {
			return;
        }
		DirectoryListing::Directory* pd = ii->file->getParent();
		while(pd != NULL && pd != dl->getRoot()) {
			fullPath = Text::toT(pd->getName()) + _T("\\") + fullPath;
			pd = pd->getParent();
		}
	} else if(ii->type == ItemInfo::DIRECTORY) {
		if(!(ii->dir->getAdls() && ii->dir->getParent() != dl->getRoot())) {
			return;
        }
		fullPath = Text::toT(((DirectoryListing::AdlDirectory*)ii->dir)->getFullPath());
	}

	selectItem(fullPath);
}

HTREEITEM DirectoryListingFrame::findItem(HTREEITEM ht, const tstring& name) {
	string::size_type i = name.find('\\');
	if(i == string::npos)
		return ht;

	for(HTREEITEM child = (HTREEITEM)::SendMessage(hWndTree, TVM_GETNEXTITEM, TVGN_CHILD, (LPARAM)ht);
        child != NULL; child = (HTREEITEM)::SendMessage(hWndTree, TVM_GETNEXTITEM, TVGN_NEXT, (LPARAM)child)) {
		DirectoryListing::Directory* d = (DirectoryListing::Directory*)TreeGetItemData(hWndTree, child);
		if(Text::toT(d->getName()) == name.substr(0, i)) {
			return findItem(child, name.substr(i+1));
		}
	}
	return NULL;
}

void DirectoryListingFrame::selectItem(const tstring& name) {
	HTREEITEM ht = findItem(treeRoot, name);
	if(ht != NULL) {
		::SendMessage(hWndTree, TVM_ENSUREVISIBLE, 0, (LPARAM)ht);
		::SendMessage(hWndTree, TVM_SELECTITEM, TVGN_CARET, (LPARAM)ht);
	}
}

bool DirectoryListingFrame::onContextMenu(WPARAM wParam, LPARAM lParam) {
    ::RemoveMenu(fileMenu, IDC_GO_TO_DIRECTORY, MF_BYCOMMAND);

	if((HWND)wParam == ctrlList.m_hWnd && (UINT)::SendMessage(ctrlList.m_hWnd, LVM_GETSELECTEDCOUNT, 0, 0) > 0) {
        int iX = GET_X_LPARAM(lParam),
            iY = GET_Y_LPARAM(lParam);
		
		if(iX == -1 && iY == -1) {
            ListViewGetContextMenuPos(ctrlList.m_hWnd, iX, iY);
		}

		int n = 0;

		ItemInfo* ii = ctrlList.getItemData((int)::SendMessage(ctrlList.m_hWnd, LVM_GETNEXTITEM, (WPARAM)-1, LVNI_SELECTED));

		while(::GetMenuItemCount(targetMenu) > 0) {
            ::RemoveMenu(targetMenu, 0, MF_BYPOSITION);
		}

		if((UINT)::SendMessage(ctrlList.m_hWnd, LVM_GETSELECTEDCOUNT, 0, 0) == 1 && ii->type == ItemInfo::FILE) {
			if(BOOLSETTING(SHOW_SHELL_MENU) && (dl->getUser() == ClientManager::getInstance()->getMe())) {
				string path;
				try {
					path = ShareManager::getInstance()->toReal(Util::toAdcFile(dl->getPath(ii->file) + ii->file->getName()));
				} catch(const ShareException&) {
                    // ...
                }

				if(!path.empty() && (File::getSize(path) != -1)) {
					CShellContextMenu shellMenu;
					shellMenu.SetPath(Text::toT(path));
					shellMenu.ShowContextMenu(m_hWnd, iX, iY);
					return true;
				}
			}

			//Append Favorite download dirs
			StringPairList spl = FavoriteManager::getInstance()->getFavoriteDirs();
			if(spl.size() > 0) {
				for(StringPairIter i = spl.begin(); i != spl.end(); i++) {
                    ::AppendMenu(targetMenu, MF_STRING, IDC_DOWNLOAD_FAVORITE_DIRS + n, Text::toT(i->second).c_str());
					n++;
				}
				::AppendMenu(targetMenu, MF_SEPARATOR, 0, NULL);
			}

			::AppendMenu(targetMenu, MF_STRING, IDC_DOWNLOADTO, CTSTRING(BROWSE));

			targets.clear();
			if(ii->file->getTTH() != NULL) {
				QueueManager::getInstance()->getTargetsByRoot(targets, *ii->file->getTTH());
			} else {
				QueueManager::getInstance()->getTargetsBySize(targets, ii->file->getSize(), Util::getFileExt(ii->file->getName()));
			}

			n = 0;

			if(targets.size() > 0) {
				::AppendMenu(targetMenu, MF_SEPARATOR, 0, NULL);
				for(StringIter i = targets.begin(); i != targets.end(); ++i) {
                    ::AppendMenu(targetMenu, MF_STRING, IDC_DOWNLOAD_TARGET + (++n), Text::toT(*i).c_str());
				}
			}
			if(ii->file->getTTH()) {
                ::EnableMenuItem(copyMenu, IDC_COPY_TTH, MF_BYCOMMAND | MFS_ENABLED);
				::EnableMenuItem(copyMenu, IDC_COPY_LINK, MF_BYCOMMAND | MFS_ENABLED);
				::EnableMenuItem(fileMenu, IDC_SEARCH_ALTERNATES, MF_BYCOMMAND | MFS_ENABLED);
			} else {
                ::EnableMenuItem(copyMenu, IDC_COPY_TTH, MF_BYCOMMAND | MFS_DISABLED);
				::EnableMenuItem(copyMenu, IDC_COPY_LINK, MF_BYCOMMAND | MFS_DISABLED);
				::EnableMenuItem(fileMenu, IDC_SEARCH_ALTERNATES, MF_BYCOMMAND | MFS_DISABLED);
			}				
			if(WinUtil::lastDirs.size() > 0) {
				::AppendMenu(targetMenu, MF_SEPARATOR, 0, NULL);
				for(TStringIter i = WinUtil::lastDirs.begin(); i != WinUtil::lastDirs.end(); ++i) {
                    ::AppendMenu(targetMenu, MF_STRING, IDC_DOWNLOAD_TARGET + (++n), i->c_str());
				}
			}
			if(ii->file->getAdls())	{
                ::AppendMenu(fileMenu, MF_STRING, IDC_GO_TO_DIRECTORY, CTSTRING(GO_TO_DIRECTORY));
			}

            ::EnableMenuItem(fileMenu, (UINT)copyMenu, MF_BYCOMMAND | MFS_ENABLED);

			StringList userhubs = ClientManager::getInstance()->getHubs(dl->getUser()->getCID());
			if(userhubs.size() > 0) {
				prepareMenu(fileMenu, UserCommand::CONTEXT_FILELIST, userhubs[0], ClientManager::getInstance()->isOp(ClientManager::getInstance()->getMe(), userhubs[0]));
			} else {
				prepareMenu(fileMenu, UserCommand::CONTEXT_FILELIST, "", false);
			}

            ::TrackPopupMenuEx(fileMenu, TPM_LEFTALIGN | TPM_RIGHTBUTTON, iX, iY, m_hWnd, NULL);

			cleanMenu(fileMenu);
		} else {
            ::EnableMenuItem(fileMenu, (UINT)copyMenu, MF_BYCOMMAND | MFS_DISABLED);
			::EnableMenuItem(fileMenu, IDC_SEARCH_ALTERNATES, MF_BYCOMMAND | MFS_DISABLED);

			//Append Favorite download dirs
			StringPairList spl = FavoriteManager::getInstance()->getFavoriteDirs();
			if (spl.size() > 0) {
				for(StringPairIter i = spl.begin(); i != spl.end(); i++) {
                    ::AppendMenu(targetMenu, MF_STRING, IDC_DOWNLOAD_FAVORITE_DIRS + n, Text::toT(i->second).c_str());
					n++;
				}
				::AppendMenu(targetMenu, MF_SEPARATOR, 0, NULL);
			}

			::AppendMenu(targetMenu, MF_STRING, IDC_DOWNLOADTO, CTSTRING(BROWSE));

            n = 0;

			if(WinUtil::lastDirs.size() > 0) {
				::AppendMenu(targetMenu, MF_SEPARATOR, 0, NULL);
				for(TStringIter i = WinUtil::lastDirs.begin(); i != WinUtil::lastDirs.end(); ++i) {
                    ::AppendMenu(targetMenu, MF_STRING, IDC_DOWNLOAD_TARGET + (++n), i->c_str());
				}
			}
			if(ii->dir->getAdls() && ii->dir->getParent() != dl->getRoot()) {
                ::AppendMenu(fileMenu, MF_STRING, IDC_GO_TO_DIRECTORY, CTSTRING(GO_TO_DIRECTORY));
			}

			StringList userhubs = ClientManager::getInstance()->getHubs(dl->getUser()->getCID());
			if(userhubs.size() > 0) {
				prepareMenu(fileMenu, UserCommand::CONTEXT_FILELIST, userhubs[0], ClientManager::getInstance()->isOp(ClientManager::getInstance()->getMe(), userhubs[0]));
			} else {
				prepareMenu(fileMenu, UserCommand::CONTEXT_FILELIST, "", false);
			}

            ::TrackPopupMenuEx(fileMenu, TPM_LEFTALIGN | TPM_RIGHTBUTTON, iX, iY, m_hWnd, NULL);

			cleanMenu(fileMenu);
		}

		return true;
	} else if((HWND)wParam == hWndTree && (HTREEITEM)::SendMessage(hWndTree, TVM_GETNEXTITEM, TVGN_CARET, 0) != NULL) {
        int iX = GET_X_LPARAM(lParam),
            iY = GET_Y_LPARAM(lParam);

        TreeGetContextMenuPos(hWndTree, iX, iY);

		while(::GetMenuItemCount(targetDirMenu) > 0) {
			::RemoveMenu(targetDirMenu, 0, MF_BYPOSITION);
		}

		int n = 0;
		//Append Favorite download dirs
		StringPairList spl = FavoriteManager::getInstance()->getFavoriteDirs();
		if (spl.size() > 0) {
			for(StringPairIter i = spl.begin(); i != spl.end(); i++) {
                ::AppendMenu(targetDirMenu, MF_STRING, IDC_DOWNLOAD_WHOLE_FAVORITE_DIRS + n, Text::toT(i->second).c_str());
				n++;
			}
			::AppendMenu(targetDirMenu, MF_SEPARATOR, 0, NULL);
		}

		::AppendMenu(targetDirMenu, MF_STRING, IDC_DOWNLOADDIRTO, CTSTRING(BROWSE));

		if(WinUtil::lastDirs.size() > 0) {
            n = 0;

			::AppendMenu(targetDirMenu, MF_SEPARATOR, 0, NULL);

			for(TStringIter i = WinUtil::lastDirs.begin(); i != WinUtil::lastDirs.end(); ++i) {
                ::AppendMenu(targetDirMenu, MF_STRING, IDC_DOWNLOAD_TARGET_DIR + (++n), i->c_str());
			}
		}

        ::TrackPopupMenuEx(directoryMenu, TPM_LEFTALIGN | TPM_RIGHTBUTTON, iX, iY, m_hWnd, NULL);

		return true;
	}

	return false;
}

bool DirectoryListingFrame::onXButtonUp(WPARAM wParam) {
	if(GET_XBUTTON_WPARAM(wParam) & XBUTTON1) {
		back();
		return true;
	} else if(GET_XBUTTON_WPARAM(wParam) & XBUTTON2) {
		forward();
		return true;
	}

	return false;
}

void DirectoryListingFrame::onDownloadTarget(WORD wID) {
	int newId = wID - IDC_DOWNLOAD_TARGET - 1;
	dcassert(newId >= 0);

	if((UINT)::SendMessage(ctrlList.m_hWnd, LVM_GETSELECTEDCOUNT, 0, 0) == 1) {
		ItemInfo* ii = ctrlList.getItemData((int)::SendMessage(ctrlList.m_hWnd, LVM_GETNEXTITEM, (WPARAM)-1, LVNI_SELECTED));

		if(ii->type == ItemInfo::FILE) {
			if(newId < (int)targets.size()) {
				try {
					dl->download(ii->file, targets[newId], false, WinUtil::isShift());
				} catch(const Exception& e) {
					::SendMessage(m_hWndStatusBar, SB_SETTEXT, (STATUS_TEXT | 0), (LPARAM)Text::toT(e.getError()).c_str());
				}
			} else {
				newId -= (int)targets.size();
				dcassert(newId < (int)WinUtil::lastDirs.size());
				downloadList(WinUtil::lastDirs[newId]);
			}
		} else {
			dcassert(newId < (int)WinUtil::lastDirs.size());
			downloadList(WinUtil::lastDirs[newId]);
		}
	} else if((UINT)::SendMessage(ctrlList.m_hWnd, LVM_GETSELECTEDCOUNT, 0, 0) > 1) {
		dcassert(newId < (int)WinUtil::lastDirs.size());
		downloadList(WinUtil::lastDirs[newId]);
	}
}

void DirectoryListingFrame::onDownloadTargetDir(WORD wID) {
	int newId = wID - IDC_DOWNLOAD_TARGET_DIR - 1;
	dcassert(newId >= 0);

	HTREEITEM t = (HTREEITEM)::SendMessage(hWndTree, TVM_GETNEXTITEM, TVGN_CARET, 0);
	if(t != NULL) {
		DirectoryListing::Directory* dir = (DirectoryListing::Directory*)TreeGetItemData(hWndTree, t);
		string target = SETTING(DOWNLOAD_DIRECTORY);
		try {
			dcassert(newId < (int)WinUtil::lastDirs.size());
			dl->download(dir, Text::fromT(WinUtil::lastDirs[newId]), WinUtil::isShift());
		} catch(const Exception& e) {
			::SendMessage(m_hWndStatusBar, SB_SETTEXT, (STATUS_TEXT | 0), (LPARAM)Text::toT(e.getError()).c_str());
		}
	}
}

void DirectoryListingFrame::onDownloadFavoriteDirs(WORD wID) {
	int newId = wID - IDC_DOWNLOAD_FAVORITE_DIRS;
	dcassert(newId >= 0);
	StringPairList spl = FavoriteManager::getInstance()->getFavoriteDirs();

	if((UINT)::SendMessage(ctrlList.m_hWnd, LVM_GETSELECTEDCOUNT, 0, 0) == 1) {
		ItemInfo* ii = ctrlList.getItemData((int)::SendMessage(ctrlList.m_hWnd, LVM_GETNEXTITEM, (WPARAM)-1, LVNI_SELECTED));

		if(ii->type == ItemInfo::FILE) {
			if(newId < (int)targets.size()) {
				try {
					dl->download(ii->file, targets[newId], false, WinUtil::isShift());
				} catch(const Exception& e) {
					::SendMessage(m_hWndStatusBar, SB_SETTEXT, (STATUS_TEXT | 0), (LPARAM)Text::toT(e.getError()).c_str());
				}
			} else {
				newId -= (int)targets.size();
				dcassert(newId < (int)spl.size());
				downloadList(Text::toT(spl[newId].first));
			}
		} else {
			dcassert(newId < (int)spl.size());
			downloadList(Text::toT(spl[newId].first));
		}
	} else if((UINT)::SendMessage(ctrlList.m_hWnd, LVM_GETSELECTEDCOUNT, 0, 0) > 1) {
		dcassert(newId < (int)spl.size());
		downloadList(Text::toT(spl[newId].first));
	}
}

void DirectoryListingFrame::onDownloadWholeFavoriteDirs(WORD wID) {
	int newId = wID - IDC_DOWNLOAD_WHOLE_FAVORITE_DIRS;
	dcassert(newId >= 0);

	HTREEITEM t = (HTREEITEM)::SendMessage(hWndTree, TVM_GETNEXTITEM, TVGN_CARET, 0);
	if(t != NULL) {
		DirectoryListing::Directory* dir = (DirectoryListing::Directory*)TreeGetItemData(hWndTree, t);
		string target = SETTING(DOWNLOAD_DIRECTORY);
		try {
			StringPairList spl = FavoriteManager::getInstance()->getFavoriteDirs();
			dcassert(newId < (int)spl.size());
			dl->download(dir, spl[newId].first, WinUtil::isShift());
		} catch(const Exception& e) {
			::SendMessage(m_hWndStatusBar, SB_SETTEXT, (STATUS_TEXT | 0), (LPARAM)Text::toT(e.getError()).c_str());
		}
	}
}

bool DirectoryListingFrame::onKeyDown(LPNMHDR pnmh) {
	NMLVKEYDOWN* kd = (NMLVKEYDOWN*)pnmh;

	if(kd->wVKey == VK_BACK) {
		up();
		return true;
	} else if(kd->wVKey == VK_TAB) {
		onTab();
		return true;
	} else if(kd->wVKey == VK_LEFT && WinUtil::isAlt()) {
		back();
		return true;
	} else if(kd->wVKey == VK_RIGHT && WinUtil::isAlt()) {
		forward();
		return true;
	} else if(kd->wVKey == VK_RETURN) {
		if((UINT)::SendMessage(ctrlList.m_hWnd, LVM_GETSELECTEDCOUNT, 0, 0) == 1) {
			ItemInfo* ii = ctrlList.getItemData((int)::SendMessage(ctrlList.m_hWnd, LVM_GETNEXTITEM, (WPARAM)-1, LVNI_SELECTED));
			if(ii->type == ItemInfo::DIRECTORY) {
				HTREEITEM ht = (HTREEITEM)::SendMessage(hWndTree, TVM_GETNEXTITEM, TVGN_CHILD,
                    (LPARAM)(HTREEITEM)::SendMessage(hWndTree, TVM_GETNEXTITEM, TVGN_CARET, 0));
				while(ht != NULL) {
					if((DirectoryListing::Directory*)TreeGetItemData(hWndTree, ht) == ii->dir) {
						::SendMessage(hWndTree, TVM_SELECTITEM, TVGN_CARET, (LPARAM)ht);
						break;
					}
					ht = (HTREEITEM)::SendMessage(hWndTree, TVM_GETNEXTITEM, TVGN_NEXT, (LPARAM)ht);
				}
			} else {
				downloadList(Text::toT(SETTING(DOWNLOAD_DIRECTORY)));
			}
		} else {
			downloadList(Text::toT(SETTING(DOWNLOAD_DIRECTORY)));
		}

		return true;
	}

	return false;
}

void DirectoryListingFrame::UpdateLayout(bool bResizeStatusBar /* = true */) {
	RECT rect;
	::GetClientRect(m_hWnd, &rect);

	UpdateStatusBar(rect, bResizeStatusBar);

	if(::IsWindow(m_hWndStatusBar)) {
		RECT sr;
		int w[STATUS_LAST];
		::GetClientRect(m_hWndStatusBar, &sr);
		w[STATUS_DUMMY-1] = sr.right - 16;
		for(int i = STATUS_DUMMY - 2; i >= 0; --i) {
			w[i] = max(w[i+1] - statusSizes[i+1], 0);
		}

		::SendMessage(m_hWndStatusBar, SB_SETPARTS, STATUS_LAST, (LPARAM)w);
		::SendMessage(m_hWndStatusBar, SB_GETRECT, 0, (LPARAM)&sr);

		sr.left = w[STATUS_FILE_LIST_DIFF - 1];
		sr.right = w[STATUS_FILE_LIST_DIFF];
		::MoveWindow(hWndListDiff, sr.left, sr.top, sr.right - sr.left, sr.bottom - sr.top, TRUE);

		sr.left = w[STATUS_MATCH_QUEUE - 1];
		sr.right = w[STATUS_MATCH_QUEUE];
		::MoveWindow(hWndMatchQueue, sr.left, sr.top, sr.right - sr.left, sr.bottom - sr.top, TRUE);

		sr.left = w[STATUS_FIND - 1];
		sr.right = w[STATUS_FIND];
		::MoveWindow(hWndFind, sr.left, sr.top, sr.right - sr.left, sr.bottom - sr.top, TRUE);

		sr.left = w[STATUS_NEXT - 1];
		sr.right = w[STATUS_NEXT];
		::MoveWindow(hWndFindNext, sr.left, sr.top, sr.right - sr.left, sr.bottom - sr.top, TRUE);
	}

	SetSplitterRect(&rect);
}

HTREEITEM DirectoryListingFrame::findFile(const StringSearch& str, HTREEITEM root,
										  int &foundFile, int &skipHits)
{
	// Check dir name for match
	DirectoryListing::Directory* dir = (DirectoryListing::Directory*)TreeGetItemData(hWndTree, root);
	if(str.match(dir->getName()))
	{
		if(skipHits == 0)
		{
			foundFile = -1;
			return root;
		}
		else
			skipHits--;
	}

	// Force list pane to contain files of current dir
	changeDir(dir, FALSE);

	// Check file names in list pane
	for(int i = 0 ; i < (int)::SendMessage(ctrlList.m_hWnd, LVM_GETITEMCOUNT, 0, 0); i++)
	{
		ItemInfo* ii = ctrlList.getItemData(i);
		if(ii->type == ItemInfo::FILE)
		{
			if(str.match(ii->file->getName()))
			{
				if(skipHits == 0)
				{
					foundFile = i;
					return root;
				}
				else
					skipHits--;
			}
		}
	}

	dcdebug("looking for directories...\n");
	// Check subdirs recursively
	HTREEITEM item = (HTREEITEM)::SendMessage(hWndTree, TVM_GETNEXTITEM, TVGN_CHILD, (LPARAM)root);
	while(item != NULL)
	{
		HTREEITEM srch = findFile(str, item, foundFile, skipHits);
		if(srch)
			return srch;
		else
			item = (HTREEITEM)::SendMessage(hWndTree, TVM_GETNEXTITEM, TVGN_NEXT, (LPARAM)item);
	}

	return 0;
}

void DirectoryListingFrame::findFile(bool findNext)
{
	if(!findNext) {
		// Prompt for substring to find
		LineDlg dlg;
		dlg.title = TSTRING(SEARCH_FOR_FILE);
		dlg.description = TSTRING(ENTER_SEARCH_STRING);
		dlg.line = Util::emptyStringT;

		if(dlg.DoModal(m_hWnd) != IDOK)
			return;

		findStr = Text::fromT(dlg.line);
		if(!dl->getUtf8())
			findStr = Text::utf8ToAcp(findStr);
		skipHits = 0;
	} else {
		skipHits++;
	}

	if(findStr.empty())
		return;

	// Do a search
	int foundFile = -1, skipHitsTmp = skipHits;
	HTREEITEM const oldDir = (HTREEITEM)::SendMessage(hWndTree, TVM_GETNEXTITEM, TVGN_CARET, 0);
	HTREEITEM const foundDir = findFile(StringSearch(findStr), (HTREEITEM)::SendMessage(hWndTree, TVM_GETNEXTITEM, TVGN_ROOT, 0),
        foundFile, skipHitsTmp);
	::SendMessage(hWndTree, WM_SETREDRAW, (WPARAM)TRUE, 0);

	if(foundDir) {
		// Highlight the directory tree and list if the parent dir/a matched dir was found
		if(foundFile >= 0) {
			// SelectItem won't update the list if SetRedraw was set to FALSE and then
			// to TRUE and the item selected is the same as the last one... workaround:
			if(oldDir == foundDir) {
                ::SendMessage(hWndTree, TVM_SELECTITEM, TVGN_CARET, (LPARAM)NULL);
            }

			::SendMessage(hWndTree, TVM_SELECTITEM, TVGN_CARET, (LPARAM)foundDir);
		} else {
			// Got a dir; select its parent directory in the tree if there is one
			HTREEITEM parentItem = (HTREEITEM)::SendMessage(hWndTree, TVM_GETNEXTITEM, TVGN_PARENT, (LPARAM)foundDir);
			if(parentItem) {
				// Go to parent file list
				::SendMessage(hWndTree, TVM_SELECTITEM, TVGN_CARET, (LPARAM)parentItem);

				// Locate the dir in the file list
				DirectoryListing::Directory* dir = (DirectoryListing::Directory*)TreeGetItemData(hWndTree, foundDir);

				foundFile = ctrlList.findItem(Text::toT(dir->getName()), -1, false);
			} else {
				// If no parent exists, just the dir tree item and skip the list highlighting
				::SendMessage(hWndTree, TVM_SELECTITEM, TVGN_CARET, (LPARAM)foundDir);
			}
		}

		// Remove prev. selection from file list
		if((UINT)::SendMessage(ctrlList.m_hWnd, LVM_GETSELECTEDCOUNT, 0, 0) > 0) {
			for(int i = 0; i < (int)::SendMessage(ctrlList.m_hWnd, LVM_GETITEMCOUNT, 0, 0); i++) {
				ListView_SetItemState(ctrlList.m_hWnd, i, 0, LVIS_SELECTED);
			}
		}

		// Highlight and focus the dir/file if possible
		if(foundFile >= 0) {
			::SetFocus(ctrlList.m_hWnd);
			::SendMessage(ctrlList.m_hWnd, LVM_ENSUREVISIBLE, foundFile, FALSE);
			ListView_SetItemState(ctrlList.m_hWnd, foundFile, LVIS_SELECTED | LVIS_FOCUSED, (UINT)-1);
		} else {
			::SetFocus(hWndTree);
	}
	} else {
		::SendMessage(hWndTree, TVM_SELECTITEM, TVGN_CARET, (LPARAM)oldDir);
		::MessageBox(m_hWnd, CTSTRING(NO_MATCHES), CTSTRING(SEARCH_FOR_FILE), MB_OK);
	}
}

void DirectoryListingFrame::runUserCommand(UserCommand& uc) {
	if(!WinUtil::getUCParams(m_hWnd, uc, ucLineParams))
		return;

	StringMap ucParams = ucLineParams;

	set<UserPtr> nicks;

	int sel = -1;
	while((sel = (int)::SendMessage(ctrlList.m_hWnd, LVM_GETNEXTITEM, sel, LVNI_SELECTED)) != -1) {
		ItemInfo* ii = ctrlList.getItemData(sel);
		if(uc.getType() == UserCommand::TYPE_RAW_ONCE) {
			if(nicks.find(dl->getUser()) != nicks.end())
				continue;
			nicks.insert(dl->getUser());
		}
		if(!dl->getUser()->isOnline())
			return;
		ucParams["fileTR"] = "NONE";
		if(ii->type == ItemInfo::FILE) {
			ucParams["type"] = "File";
			ucParams["fileFN"] = dl->getPath(ii->file) + ii->file->getName();
			ucParams["fileSI"] = Util::toString(ii->file->getSize());
			ucParams["fileSIshort"] = Util::formatBytes(ii->file->getSize());
			TTHValue *hash = ii->file->getTTH();
			if(hash != NULL) {
				ucParams["fileTR"] = hash->toBase32();
			}
		} else {
			ucParams["type"] = "Directory";
			ucParams["fileFN"] = dl->getPath(ii->dir) + ii->dir->getName();
			ucParams["fileSI"] = Util::toString(ii->dir->getTotalSize());
			ucParams["fileSIshort"] = Util::formatBytes(ii->dir->getTotalSize());
		}

		// compatibility with 0.674 and earlier
		ucParams["file"] = ucParams["fileFN"];
		ucParams["filesize"] = ucParams["fileSI"];
		ucParams["filesizeshort"] = ucParams["fileSIshort"];
		ucParams["tth"] = ucParams["fileTR"];

		StringMap tmp = ucParams;
		UserPtr tmpPtr = dl->getUser();
		ClientManager::getInstance()->userCommand(dl->getUser(), uc, tmp, true);
	}
}

void DirectoryListingFrame::onCopy(WORD wID) {
	if((UINT)::SendMessage(ctrlList.m_hWnd, LVM_GETSELECTEDCOUNT, 0, 0) == 1) {
		ItemInfo* ii = ctrlList.getSelectedItem();
		if(ii->type != ItemInfo::FILE) {
			return;
        }

        string sCopy;

		switch (wID) {
			case IDC_COPY_NICK:
				sCopy = Text::fromT(WinUtil::getNicks(dl->getUser()));
				break;
			case IDC_COPY_FILENAME:
				sCopy = Util::getFileName(ii->file->getName());
				break;
			case IDC_COPY_SIZE:
				sCopy = Util::formatBytes(ii->file->getSize());
				break;
			case IDC_COPY_LINK:
				if(ii != NULL && ii->type == ItemInfo::FILE && ii->file->getTTH() != NULL) {
					WinUtil::copyMagnet(ii->file->getTTH(), ii->file->getSize(), Text::toT(ii->file->getName()));
                    return;
				}
				break;
			case IDC_COPY_TTH:
				if(ii != NULL && ii->type == ItemInfo::FILE && ii->file->getTTH() != NULL) {
					sCopy = ii->file->getTTH()->toBase32();
                }
				break;
			default:
				return;
        }

		if(sCopy.empty() == false) {
			WinUtil::setClipboard(Text::toT(sCopy));
        }
	}
}

void DirectoryListingFrame::onSearchAlternates() {
	if((UINT)::SendMessage(ctrlList.m_hWnd, LVM_GETSELECTEDCOUNT, 0, 0) == 1) {
		ItemInfo* ii = ctrlList.getSelectedItem();
		if(ii != NULL && ii->type == ItemInfo::FILE && ii->file->getTTH() != NULL) {
			WinUtil::searchHash(ii->file->getTTH());
        }
	} 
}

bool DirectoryListingFrame::onClose() {
	if(!closed) {
		WinUtil::saveHeaderOrder(ctrlList.m_hWnd, SettingsManager::DIRECTORLISTINGFRAME_ORDER, SettingsManager::DIRECTORLISTINGFRAME_WIDTHS, COLUMN_LAST, columnIndexes, columnSizes);
		SettingsManager::getInstance()->removeListener(this);
		::SendMessage(ctrlList.m_hWnd, WM_SETREDRAW, (WPARAM)FALSE, 0);
		clearList();
		frames.erase(m_hWnd);
		closed = true;
		::PostMessage(m_hWnd, WM_CLOSE, 0, 0);
		return true;
	} else {
		return false;
	}
}

void DirectoryListingFrame::on(SettingsManagerListener::Save, SimpleXML& /*xml*/) throw() {
	::PostMessage(m_hWnd, WM_REFRESH_SETTINGS, 0, 0);
}

void DirectoryListingFrame::onRefreshSettings() {
	bool refresh = false;

	if((COLORREF)::SendMessage(ctrlList.m_hWnd, LVM_GETBKCOLOR, 0, 0) != WinUtil::bgColor) {
		::SendMessage(ctrlList.m_hWnd, LVM_SETBKCOLOR, 0, WinUtil::bgColor);
		::SendMessage(ctrlList.m_hWnd, LVM_SETTEXTBKCOLOR, 0, WinUtil::bgColor);
		::SendMessage(hWndTree, TVM_SETBKCOLOR, 0, (LPARAM)WinUtil::bgColor);
		refresh = true;
	}
	if((COLORREF)::SendMessage(ctrlList.m_hWnd, LVM_GETTEXTCOLOR, 0, 0) != WinUtil::textColor) {
		::SendMessage(ctrlList.m_hWnd, LVM_SETTEXTCOLOR, 0, WinUtil::textColor);
		::SendMessage(hWndTree, TVM_SETTEXTCOLOR, 0, (LPARAM)WinUtil::textColor);
		refresh = true;
	}

	if(refresh == true) {
		::RedrawWindow(m_hWnd, NULL, NULL, RDW_ERASE | RDW_INVALIDATE | RDW_UPDATENOW | RDW_ALLCHILDREN);
	}
}

void DirectoryListingFrame::closeAll() {
    for(FrameIter i = frames.begin(); i != frames.end(); ++i) {
		::PostMessage(i->second->m_hWnd, WM_CLOSE, 0, 0);
	}
}

const TCHAR* DirectoryListingFrame::ItemInfo::getText(const int &col, TCHAR* buf) const {
    if(type == FILE) {
        switch(col) {
            case COLUMN_FILENAME:
            	return filename.c_str();
			case COLUMN_TYPE: {
    			tstring ext = Util::getFileExt(filename);

                if(ext.size() > 0 && ext[0] == '.') {
        			ext.erase(0, 1);
                }

            	_tcsncpy_s(buf, 260, ext.c_str(), _TRUNCATE);
            	return buf;
			}
            case COLUMN_SIZE:
                _tcsncpy_s(buf, 260, Text::toT(Util::formatBytes(file->getSize())).c_str(), _TRUNCATE);
                return buf;
            case COLUMN_EXACT_SIZE:
                _tcsncpy_s(buf, 260, Text::toT(Util::formatExactSize(file->getSize())).c_str(), _TRUNCATE);
                return buf;
            case COLUMN_TTH:
                if(file->getTTH() != NULL) {
                    _tcsncpy_s(buf, 260, Text::toT(file->getTTH()->toBase32()).c_str(), _TRUNCATE);
                    return buf;
                } else {
                    return Util::emptyStringT.c_str();
                }
            default:
                return Util::emptyStringT.c_str();
        }
    } else {
        switch(col) {
            case COLUMN_FILENAME:
                _tcsncpy_s(buf, 260, filename.c_str(), _TRUNCATE);
            	return buf;
            case COLUMN_SIZE:
                _tcsncpy_s(buf, 260, Text::toT(Util::formatBytes(dir->getTotalSize())).c_str(), _TRUNCATE);
                return buf;
            case COLUMN_EXACT_SIZE:
                _tcsncpy_s(buf, 260, Text::toT(Util::formatExactSize(dir->getTotalSize())).c_str(), _TRUNCATE);
                return buf;
            default:
                return Util::emptyStringT.c_str();
		}
    }
}

HWND DirectoryListingFrame::GetWindowHandle() {
    return m_hWnd;
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void DirectoryListingFrame::UpdateSplitterParts() {
    ::SetWindowPos(hWndTree, NULL, rcSplitter.left, rcSplitter.top, iSplitterPos-2, rcSplitter.bottom-rcSplitter.top, SWP_NOZORDER);
    ::SetWindowPos(ctrlList.m_hWnd, NULL, rcSplitter.left+(iSplitterPos+2), rcSplitter.top, (rcSplitter.right-rcSplitter.left)-(iSplitterPos+2), rcSplitter.bottom-rcSplitter.top,
        SWP_NOZORDER);
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

/*
 * Some parts Copyright (C) 2002-2011 Petr Kozelka, PPK at czdc dot org
 * Licensed under GNU General Public License version 3.
 */
