/*
 * Copyright (C) 2001-2011 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "stdafx.h"
#include "../client/DCPlusPlus.h"
#include "Resource.h"

#include "ExListViewCtrl.h"

LRESULT CALLBACK ExListViewCtrl::ExListViewProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
	ExListViewCtrl * pParent = (ExListViewCtrl *)::GetWindowLongPtr(hWnd, GWLP_USERDATA);
    if(pParent == NULL) { // should never happen, but ... :-D
        return ::CallWindowProc(OldListViewProc, hWnd, uMsg, wParam, lParam);
    }

    if(uMsg == WM_CHAR) {
		if(pParent->onChar() == true) {
			return 0;
        }
    }

	return ::CallWindowProc(OldListViewProc, hWnd, uMsg, wParam, lParam);
}

// TODO: make sure that moved items maintain their selection state
int ExListViewCtrl::moveItem(int oldPos, int newPos) {
	TCHAR buf[512];
	LVITEM lvi;
	lvi.iItem = oldPos;
	lvi.iSubItem = 0;
	lvi.mask = LVIF_IMAGE | LVIF_INDENT | LVIF_PARAM | LVIF_STATE;
	::SendMessage(m_hWnd, LVM_GETITEM, 0, (LPARAM)&lvi);
	TStringList l;

	for(int j = 0; j < (int)::SendMessage((HWND)::SendMessage(m_hWnd, LVM_GETHEADER, 0, 0), HDM_GETITEMCOUNT, 0, 0); j++) {
		ListViewGetItemText(m_hWnd, oldPos, j, buf, 512);
		l.push_back(buf);
	}

	::SendMessage(m_hWnd, WM_SETREDRAW, (WPARAM)FALSE, 0);

	if(oldPos < newPos) {
		lvi.iItem = newPos + 1;
	} else {
		lvi.iItem = newPos;
	}
	int i = (int)::SendMessage(m_hWnd, LVM_INSERTITEM, 0, (LPARAM)&lvi);
	int m = 0;
	for(TStringIter k = l.begin(); k != l.end(); ++k, m++) {
		ListViewSetItem(m_hWnd, i, m, LVIF_TEXT, k->c_str(), 0);
	}

	SendMessage(m_hWnd, LVM_ENSUREVISIBLE, i, FALSE);

	if(i < oldPos) {
		::SendMessage(m_hWnd, LVM_DELETEITEM, oldPos + 1, 0);
	} else {
		::SendMessage(m_hWnd, LVM_DELETEITEM, oldPos, 0);
	}

	::SendMessage(m_hWnd, WM_SETREDRAW, (WPARAM)TRUE, 0);

	RECT rc;

	rc.left = LVIR_BOUNDS;
	SendMessage(m_hWnd, LVM_GETITEMRECT, (WPARAM)i, (LPARAM)&rc);

	::InvalidateRect(m_hWnd, &rc, TRUE);

	return i;
}

int ExListViewCtrl::insert(TStringList& aList, int iImage, LPARAM lParam) {
	TCHAR buf[128];
	int loc = 0;
	int count = (int)::SendMessage(m_hWnd, LVM_GETITEMCOUNT, 0, 0);

	LVITEM a;
	a.mask = LVIF_IMAGE | LVIF_INDENT | LVIF_PARAM | LVIF_STATE | LVIF_TEXT;
	a.iItem = -1;
	a.iSubItem = sortColumn;
	a.iImage = iImage;
	a.state = 0;
	a.stateMask = 0;
	a.lParam = lParam;
	a.iIndent = 0;
	a.pszText = const_cast<TCHAR*>(sortColumn == -1 ? aList[0].c_str() : aList[sortColumn].c_str());
	a.cchTextMax = (int)(sortColumn == -1 ? aList[0].size() : aList[sortColumn].size());

	if(sortColumn == -1) {
		loc = count;
	} else if(count == 0) {
		loc = 0;
	} else {

		tstring& b = aList[sortColumn];
		int c = _tstoi(b.c_str());
		double f = _tstof(b.c_str());
		LPARAM data = NULL;
		int low = 0;
		int high = count-1;
		int comp = 0;
		while(low <= high)
		{
			loc = (low + high)/2;

			// This is a trick, so that if fun() returns something bigger than one, use the
			// internal default sort functions
			comp = sortType;
			if(comp == SORT_FUNC) {
				data = ListViewGetItem(m_hWnd, loc);
				comp = fun(lParam, data);
			}

			if(comp == SORT_STRING) {
				ListViewGetItemText(m_hWnd, loc, sortColumn, buf, 128);
				comp = compare(b, tstring(buf));
			} else if(comp == SORT_STRING_NOCASE) {
				ListViewGetItemText(m_hWnd, loc, sortColumn, buf, 128);
				comp = Util::stricmp(b.c_str(), buf);
			} else if(comp == SORT_INT) {
				ListViewGetItemText(m_hWnd, loc, sortColumn, buf, 128);
				comp = compare(c, _tstoi(buf));
			} else if(comp == SORT_FLOAT) {
				ListViewGetItemText(m_hWnd, loc, sortColumn, buf, 128);
				comp = compare(f, _tstof(buf));
			}

			if(!ascending)
				comp = -comp;

			if(comp < 0) {
				high = loc - 1;
			} else if(comp > 0) {
				low = loc + 1;
			} else {
				break;
			}
		}

		comp = sortType;
		if(comp == SORT_FUNC) {
			comp = fun(lParam, data);
		}
		if(comp == SORT_STRING) {
			comp = compare(b, tstring(buf));
		} else if(comp == SORT_STRING_NOCASE) {
			comp = Util::stricmp(b.c_str(), buf);
		} else if(comp == SORT_INT) {
			comp = compare(c, _tstoi(buf));
		} else if(comp == SORT_FLOAT) {
			comp = compare(f, _tstof(buf));
		}

		if(!ascending)
			comp = -comp;

		if(comp > 0)
			loc++;
	}
	dcassert(loc >= 0 && loc <= GetItemCount());
	a.iItem = loc;
	a.iSubItem = 0;
	int i = (int)::SendMessage(m_hWnd, LVM_INSERTITEM, 0, (LPARAM)&a);
	int k = 0;
	for(TStringIter j = aList.begin(); j != aList.end(); ++j, k++) {
		ListViewSetItem(m_hWnd, i, k, LVIF_TEXT, j->c_str(), 0);
	}
	return loc;
}

int ExListViewCtrl::insert(int nItem, TStringList& aList, int iImage, LPARAM lParam) {

	dcassert(aList.size() > 0);

	int i = insert(nItem, aList[0], iImage, lParam);

	int k = 0;
	for(TStringIter j = aList.begin(); j != aList.end(); ++j, k++) {
		ListViewSetItem(m_hWnd, i, k, LVIF_TEXT, j->c_str(), 0);
	}
	return i;
}

void ExListViewCtrl::Create(HWND hWndParent, DWORD dwStyle, DWORD dwExStyle, UINT uiID, DWORD dwExListViewStyle) {
	m_hWnd = ::CreateWindowEx(dwExStyle, WC_LISTVIEW, _T(""), dwStyle, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, hWndParent, (HMENU)uiID, g_hInstance, NULL);

    ::SendMessage(m_hWnd, LVM_SETEXTENDEDLISTVIEWSTYLE, 0, dwExListViewStyle);
	::SendMessage(m_hWnd, WM_SETFONT, (WPARAM)WinUtil::font, MAKELPARAM(TRUE, 0));
	::SendMessage(m_hWnd, LVM_SETBKCOLOR, 0, WinUtil::bgColor);
	::SendMessage(m_hWnd, LVM_SETTEXTBKCOLOR, 0, WinUtil::bgColor);
	::SendMessage(m_hWnd, LVM_SETTEXTCOLOR, 0, WinUtil::textColor);

    ::SetWindowLongPtr(m_hWnd, GWLP_USERDATA, (LONG_PTR)this);
    OldListViewProc = (WNDPROC)::SetWindowLongPtr(m_hWnd, GWLP_WNDPROC, (LONG_PTR)ExListViewProc);
}

/*
 * Some parts Copyright (C) 2002-2011 Petr Kozelka, PPK at czdc dot org
 * Licensed under GNU General Public License version 3.
 */
