/*
 * Copyright (C) 2001-2011 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "stdafx.h"
#include "../client/DCPlusPlus.h"
#include "Resource.h"

#include "GeneralPage.h"
#include "../client/SettingsManager.h"
#include "../client/Socket.h"
#include "WinUtil.h"

GeneralPage * pGeneralPage = NULL;

GeneralPage::GeneralPage(SettingsManager *s) : PropPage(s, IDD_GENERALPAGE) {
    pGeneralPage = this;
	SetTitle(CTSTRING(SETTINGS_GENERAL));
	m_psp.pfnDlgProc = (DLGPROC)GeneralPageProc;
}

GeneralPage::~GeneralPage() {
    pGeneralPage = NULL;
}

BOOL CALLBACK GeneralPage::GeneralPageProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    if(uMsg == WM_INITDIALOG) {
        pGeneralPage->m_hWnd = hWnd;
        pGeneralPage->onInitDialog();
        return TRUE;
    } else if(uMsg == WM_COMMAND) {
        switch(LOWORD(wParam)) {
            case IDC_ACTIVE:
            case IDC_PASSIVE:
            case IDC_SOCKS5:
                pGeneralPage->onClickedActive();
                return TRUE;
            case IDC_NICK:
            case IDC_EMAIL:
            case IDC_DESCRIPTION:
                if(HIWORD(wParam) == EN_CHANGE) {
                    pGeneralPage->onTextChanged(LOWORD(wParam), (HWND)lParam);
                    return TRUE;
                }

                break;
        }
    }

	return FALSE;
}

PropPage::TextItem GeneralPage::texts[] = {
	{ IDC_SETTINGS_PERSONAL_INFORMATION, ResourceManager::SETTINGS_PERSONAL_INFORMATION },
	{ IDC_SETTINGS_NICK, ResourceManager::NICK },
	{ IDC_SETTINGS_EMAIL, ResourceManager::EMAIL },
	{ IDC_SETTINGS_DESCRIPTION, ResourceManager::DESCRIPTION },
	{ IDC_SETTINGS_CONNECTION_TYPE, ResourceManager::SETTINGS_CONNECTION_TYPE },
	{ IDC_SETTINGS_CONNECTION_SETTINGS, ResourceManager::SETTINGS_CONNECTION_SETTINGS },
	{ IDC_ACTIVE, ResourceManager::ACTIVE },
	{ IDC_PASSIVE, ResourceManager::SETTINGS_PASSIVE },
	{ IDC_SOCKS5, ResourceManager::SETTINGS_SOCKS5 }, 
	{ IDC_SETTINGS_IP, ResourceManager::SETTINGS_IP },
	{ IDC_SETTINGS_PORT, ResourceManager::SETTINGS_PORT },
	{ IDC_SETTINGS_UDP_PORT, ResourceManager::SETTINGS_UDP_PORT },
	{ IDC_SETTINGS_SOCKS5_IP, ResourceManager::SETTINGS_SOCKS5_IP },
	{ IDC_SETTINGS_SOCKS5_PORT, ResourceManager::SETTINGS_SOCKS5_PORT },
	{ IDC_SETTINGS_SOCKS5_USERNAME, ResourceManager::SETTINGS_SOCKS5_USERNAME },
	{ IDC_SETTINGS_SOCKS5_PASSWORD, ResourceManager::PASSWORD },
	{ IDC_SOCKS_RESOLVE, ResourceManager::SETTINGS_SOCKS5_RESOLVE },
	{ IDC_SETTINGS_BIND_ADDRESS, ResourceManager::SETTINGS_BIND_ADDRESS },
	{ IDC_SETTINGS_UPLOAD_SPEED, ResourceManager::SETTINGS_UPLOAD_SPEED },
	{ IDC_SETTINGS_MEGABITS, ResourceManager::MEGABITS },
	{ IDC_USE_UPNP, ResourceManager::SETTINGS_USE_UPNP }, 
	{ IDC_USE_UPLOAD_SPEED, ResourceManager::SETTINGS_USE_UPLOAD_SPEED }, 
	{ IDC_OVERRIDE, ResourceManager::SETTINGS_OVERRIDE },
	{ 0, ResourceManager::SETTINGS_AUTO_AWAY }
};

PropPage::Item GeneralPage::items[] = {
	{ IDC_NICK,		SettingsManager::NICK,			PropPage::T_STR },
	{ IDC_EMAIL,		SettingsManager::EMAIL,			PropPage::T_STR },
	{ IDC_DESCRIPTION,	SettingsManager::DESCRIPTION,		PropPage::T_STR },
	{ IDC_CONNECTION,	SettingsManager::CONNECTION,		PropPage::T_STR },
	{ IDC_SERVER,		SettingsManager::EXTERNAL_IP,		PropPage::T_STR },
	{ IDC_PORT,		SettingsManager::TCP_PORT,		PropPage::T_INT },
	{ IDC_UDP_PORT,		SettingsManager::UDP_PORT,		PropPage::T_INT },
	{ IDC_SOCKS_SERVER, 	SettingsManager::SOCKS_SERVER,		PropPage::T_STR },
	{ IDC_SOCKS_PORT,	SettingsManager::SOCKS_PORT,		PropPage::T_INT },
	{ IDC_SOCKS_USER,	SettingsManager::SOCKS_USER,		PropPage::T_STR },
	{ IDC_SOCKS_PASSWORD, 	SettingsManager::SOCKS_PASSWORD,	PropPage::T_STR },
	{ IDC_SOCKS_RESOLVE,	SettingsManager::SOCKS_RESOLVE, 	PropPage::T_BOOL },
	{ IDC_BIND_ADDRESS, 	SettingsManager::BIND_ADDRESS, 		PropPage::T_STR },
	{ IDC_UPLOAD_SPEED,	SettingsManager::UPLOAD_SPEED,		PropPage::T_STR }, 
	{ IDC_USE_UPNP,		SettingsManager::SETTINGS_USE_UPNP,	PropPage::T_BOOL }, 
	{ IDC_USE_UPLOAD_SPEED,	SettingsManager::USE_UPLOAD_SPEED,	PropPage::T_BOOL },
	{ IDC_OVERRIDE,		SettingsManager::NO_IP_OVERRIDE, PropPage::T_BOOL },
	{ 0, 0, PropPage::T_END }
};

void GeneralPage::write() {
	TCHAR tmp[1024];
	::GetDlgItemText(m_hWnd, IDC_SOCKS_SERVER, tmp, 1024);
	tstring x = tmp;
	tstring::size_type i;

	while((i = x.find(' ')) != string::npos) {
		x.erase(i, 1);
	}
	::SetDlgItemText(m_hWnd, IDC_SOCKS_SERVER, x.c_str());

	::GetDlgItemText(m_hWnd, IDC_SERVER, tmp, 1024);
	x = tmp;

	while((i = x.find(' ')) != string::npos) {
		x.erase(i, 1);
	}

	::SetDlgItemText(m_hWnd, IDC_SERVER, x.c_str());
	
	PropPage::write(m_hWnd, items);

	// Set connection active/passive
	int ct = -1;
	if(::IsDlgButtonChecked(m_hWnd, IDC_ACTIVE))
		ct = SettingsManager::CONNECTION_ACTIVE;
	else if(::IsDlgButtonChecked(m_hWnd, IDC_PASSIVE))
		ct = SettingsManager::CONNECTION_PASSIVE;
	else if(::IsDlgButtonChecked(m_hWnd, IDC_SOCKS5))
		ct = SettingsManager::CONNECTION_SOCKS5;

	if(SETTING(CONNECTION_TYPE) != ct) {
		settings->set(SettingsManager::CONNECTION_TYPE, ct);
		Socket::socksUpdated();
	}
		
	settings->set(SettingsManager::CONNECTION, SettingsManager::connectionSpeeds[(int)::SendMessage(::GetDlgItem(m_hWnd, IDC_CONNECTION), CB_GETCURSEL, 0, 0)]);
}

void GeneralPage::onInitDialog() {
	PropPage::translate(m_hWnd, texts);

	::SendMessage(::GetDlgItem(m_hWnd, IDC_CONNECTION), CBEM_SETIMAGELIST, 0, (LPARAM)WinUtil::userImages);
	
	int q = 0;
	for(StringIter i = SettingsManager::connectionSpeeds.begin(); i != SettingsManager::connectionSpeeds.end(); ++i) {
		COMBOBOXEXITEM cbitem = {CBEIF_TEXT|CBEIF_IMAGE|CBEIF_SELECTEDIMAGE};
		tstring conn = Text::toT((*i));
		cbitem.pszText = const_cast<TCHAR*>(conn.c_str());		
		int w = 0;
		switch(q) {
			case 0: w = 1; break;
			case 1: w = 2; break;
			case 2: w = 3; break;
			case 3: w = 3; break;
			case 4: w = 5; break;
			case 5: w = 4; break;
			case 6: w = 6; break;
			case 7: w = 7; break;
		}
		
		cbitem.iItem = q++;
		cbitem.iImage = w;
		cbitem.iSelectedImage = w;
		::SendMessage(::GetDlgItem(m_hWnd, IDC_CONNECTION), CBEM_INSERTITEM, 0, (LPARAM)&cbitem);
	}

	int const connType = settings->get(SettingsManager::CONNECTION_TYPE);
	if(connType == SettingsManager::CONNECTION_ACTIVE)
		::CheckRadioButton(m_hWnd, IDC_ACTIVE, IDC_SOCKS5, IDC_ACTIVE);
	else if(connType == SettingsManager::CONNECTION_PASSIVE)
		::CheckRadioButton(m_hWnd, IDC_ACTIVE, IDC_SOCKS5, IDC_PASSIVE);
	else if(connType == SettingsManager::CONNECTION_SOCKS5)
		::CheckRadioButton(m_hWnd, IDC_ACTIVE, IDC_SOCKS5, IDC_SOCKS5);

	for(StringIter i = SettingsManager::QuackSpeeds.begin(); i != SettingsManager::QuackSpeeds.end(); ++i) {
		::SendMessage(::GetDlgItem(m_hWnd, IDC_UPLOAD_SPEED), CB_ADDSTRING, 0, (LPARAM)Text::toT(*i).c_str());
	}

	PropPage::read(m_hWnd, items);

	for(int m = 0; m < 8; m++) {
		if(SettingsManager::connectionSpeeds[m] == SETTING(CONNECTION)) {
			::SendMessage(::GetDlgItem(m_hWnd, IDC_CONNECTION), CB_SETCURSEL, m, 0);
			break;
		}
	}

	::SendMessage(::GetDlgItem(m_hWnd, IDC_UPLOAD_SPEED), CB_SETCURSEL, ::SendMessage(::GetDlgItem(m_hWnd, IDC_UPLOAD_SPEED), 
		CB_FINDSTRING, 0, (LPARAM)Text::toT(SETTING(UPLOAD_SPEED)).c_str()), 0);
	::SendMessage(::GetDlgItem(m_hWnd, IDC_NICK), EM_LIMITTEXT, 35, 0);
	::SendMessage(::GetDlgItem(m_hWnd, IDC_DESCRIPTION), EM_LIMITTEXT, 50, 0);
	::SendMessage(::GetDlgItem(m_hWnd, IDC_SETTINGS_EMAIL), EM_LIMITTEXT, 64, 0);
	::SendMessage(::GetDlgItem(m_hWnd, IDC_SOCKS_SERVER), EM_LIMITTEXT, 250, 0);
	::SendMessage(::GetDlgItem(m_hWnd, IDC_SOCKS_PORT), EM_LIMITTEXT, 5, 0);
	::SendMessage(::GetDlgItem(m_hWnd, IDC_SOCKS_USER), EM_LIMITTEXT, 250, 0);
	::SendMessage(::GetDlgItem(m_hWnd, IDC_SOCKS_PASSWORD), EM_LIMITTEXT, 250, 0);

	fixControls();
}

void GeneralPage::fixControls() {
	BOOL checked = ::IsDlgButtonChecked(m_hWnd, IDC_ACTIVE);
	::EnableWindow(::GetDlgItem(m_hWnd, IDC_SERVER), checked);
	::EnableWindow(::GetDlgItem(m_hWnd, IDC_PORT), checked);
	::EnableWindow(::GetDlgItem(m_hWnd, IDC_UDP_PORT), checked);
	::EnableWindow(::GetDlgItem(m_hWnd, IDC_USE_UPNP), checked);
	::EnableWindow(::GetDlgItem(m_hWnd, IDC_BIND_ADDRESS), checked);
	::EnableWindow(::GetDlgItem(m_hWnd, IDC_OVERRIDE), checked);

	checked = ::IsDlgButtonChecked(m_hWnd, IDC_SOCKS5);
	::EnableWindow(::GetDlgItem(m_hWnd, IDC_SOCKS_SERVER), checked);
	::EnableWindow(::GetDlgItem(m_hWnd, IDC_SOCKS_PORT), checked);
	::EnableWindow(::GetDlgItem(m_hWnd, IDC_SOCKS_USER), checked);
	::EnableWindow(::GetDlgItem(m_hWnd, IDC_SOCKS_PASSWORD), checked);
	::EnableWindow(::GetDlgItem(m_hWnd, IDC_SOCKS_RESOLVE), checked);

}

void GeneralPage::onClickedActive() {
	fixControls();
}

void GeneralPage::onTextChanged(WORD wID, HWND hWndCtl) {
	TCHAR buf[SETTINGS_BUF_LEN];

	::GetDlgItemText(m_hWnd, wID, buf, SETTINGS_BUF_LEN);
	tstring old = buf;

	// Strip '$', '|', '<', '>' and ' ' from text
	TCHAR *b = buf, *f = buf, c;
	while((c = *b++) != 0){
		if(c != '$' && c != '|' && (wID == IDC_DESCRIPTION || c != ' ') && ( (wID != IDC_NICK && wID != IDC_DESCRIPTION && wID != IDC_SETTINGS_EMAIL) || (c != '<' && c != '>') ) )
			*f++ = c;
	}

	*f = '\0';

	if(old != buf) {
		// Something changed; update window text without changing cursor pos
		int start, end;
		
		::SendMessage(hWndCtl, EM_GETSEL, (WPARAM)&start, (LPARAM)&end);

		::SetWindowText(hWndCtl, buf);
		if(start > 0) start--;
		if(end > 0) end--;

		::SendMessage(hWndCtl, EM_SETSEL, start, end);
		::SendMessage(hWndCtl, EM_SCROLLCARET, 0, 0);
	}
}

/*
 * Some parts Copyright (C) 2002-2011 Petr Kozelka, PPK at czdc dot org
 * Licensed under GNU General Public License version 3.
 */
