/*
 * Copyright (C) 2001-2011 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "stdafx.h"
#include "../client/DCPlusPlus.h"
#include "Resource.h"

#include "HashProgressDlg.h"

#include "../client/ResourceManager.h"
#include "WinApiUtil.h"

INT_PTR CALLBACK HashProgressDlg::HashProgressDlgProc(HWND hWndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam){
    HashProgressDlg * pParent;

    if(uMsg == WM_INITDIALOG) {
        pParent = (HashProgressDlg *)lParam;
        SetWindowLongPtr(hWndDlg, GWLP_USERDATA, (LONG_PTR)pParent);

        pParent->m_hWnd = hWndDlg;
    } else {
        pParent = (HashProgressDlg *)GetWindowLongPtr(hWndDlg, GWLP_USERDATA);
        if(pParent == NULL) {
            return FALSE;
        }
    }

    SetWindowLongPtr(hWndDlg, DWLP_MSGRESULT, 0);

    return pParent->HashProgressProc(hWndDlg, uMsg, wParam, lParam);
}

LRESULT HashProgressDlg::HashProgressProc(HWND /*hWnd*/, UINT uMsg, WPARAM wParam, LPARAM /*lParam*/) {
    switch(uMsg) {
        case WM_INITDIALOG:
            OnInitDialog();
            return TRUE;
        case WM_TIMER:
            onTimer();
            return TRUE;
        case WM_DESTROY:
            onDestroy();
            return FALSE;
        case WM_COMMAND:
            if(LOWORD(wParam) == IDOK || LOWORD(wParam) == IDCANCEL) {
                OnCloseCmd(LOWORD(wParam));
                return TRUE;
            }
            break;
    }

    return FALSE;
}

void HashProgressDlg::OnInitDialog() {
	// Translate static strings
	::SetWindowText(m_hWnd, CTSTRING(HASH_PROGRESS));
	::SetDlgItemText(m_hWnd, IDOK, CTSTRING(HASH_PROGRESS_BACKGROUND));
	::SetDlgItemText(m_hWnd, IDC_STATISTICS, CTSTRING(HASH_PROGRESS_STATS));
	::SetDlgItemText(m_hWnd, IDC_HASH_INDEXING, CTSTRING(HASH_PROGRESS_TEXT));

	string tmp;
	startTime = GET_TICK();
	HashManager::getInstance()->getStats(tmp, startBytes, startFiles);

	::SendMessage(::GetDlgItem(m_hWnd, IDC_HASH_PROGRESS), PBM_SETRANGE, 0, MAKELPARAM(0, 10000));
	updateStats();

	HashManager::getInstance()->setPriority(Thread::NORMAL);

	::SetTimer(m_hWnd, 1, 1000, NULL);

	// Center dialog
	CenterWindow(m_hWnd, ::GetParent(m_hWnd));
}

void HashProgressDlg::onTimer() {
	updateStats();
}

void HashProgressDlg::onDestroy() {
	HashManager::getInstance()->setPriority(Thread::IDLE);
	::KillTimer(m_hWnd, 1);
}

void HashProgressDlg::updateStats() {
	string file;
	int64_t bytes = 0;
	size_t files = 0;
	uint64_t tick = GET_TICK();

	HashManager::getInstance()->getStats(file, bytes, files);
	if(bytes > startBytes) {
		startBytes = bytes;
    }

	if(files > startFiles) {
		startFiles = files;
    }

	if(autoClose && files == 0) {
		::PostMessage(m_hWnd, WM_CLOSE, 0, 0);
		return;
	}

	double diff = (double)(tick - startTime);
	if(diff < 1000 || files == 0 || bytes == 0) {
		::SetDlgItemText(m_hWnd, IDC_FILES_PER_HOUR, Text::toT("-.-- " + STRING(FILES_PER_HOUR) + ", " + Util::toString((uint32_t)files) + " " + STRING(FILES_LEFT)).c_str());
		::SetDlgItemText(m_hWnd, IDC_HASH_SPEED, Text::toT("-.-- B/s, " + Util::formatBytes(bytes) + " " + STRING(LEFT)).c_str());
		::SetDlgItemText(m_hWnd, IDC_TIME_LEFT, Text::toT("-:--:-- " + STRING(LEFT)).c_str());
		::SendMessage(::GetDlgItem(m_hWnd, IDC_HASH_PROGRESS), PBM_SETPOS, 0, 0);
	} else {
		double filestat = (((double)(startFiles - files)) * 60 * 60 * 1000) / diff;
		double speedStat = (((double)(startBytes - bytes)) * 1000) / diff;

		::SetDlgItemText(m_hWnd, IDC_FILES_PER_HOUR, Text::toT(Util::toString(filestat) + " " + STRING(FILES_PER_HOUR) + ", " + Util::toString((uint32_t)files) + " " + STRING(FILES_LEFT)).c_str());
		::SetDlgItemText(m_hWnd, IDC_HASH_SPEED, Text::toT(Util::formatBytes((int64_t)speedStat) + "/s, " + Util::formatBytes(bytes) + " " + STRING(LEFT)).c_str());

		if(filestat == 0 || speedStat == 0) {
			::SetDlgItemText(m_hWnd, IDC_TIME_LEFT, Text::toT("-:--:-- " + STRING(LEFT)).c_str());
		} else {
			double fs = files * 60 * 60 / filestat;
			double ss = bytes / speedStat;

			::SetDlgItemText(m_hWnd, IDC_TIME_LEFT, Text::toT(Util::formatSeconds((int64_t)(fs + ss) / 2) + " " + STRING(LEFT)).c_str());
		}
	}

	if(files == 0) {
		::SetDlgItemText(m_hWnd, IDC_CURRENT_FILE, CTSTRING(DONE));
	} else {
		::SetDlgItemText(m_hWnd, IDC_CURRENT_FILE, Text::toT(file).c_str());
	}

	if(startFiles == 0 || startBytes == 0) {
		::SendMessage(::GetDlgItem(m_hWnd, IDC_HASH_PROGRESS), PBM_SETPOS, 0, 0);
	} else {
		::SendMessage(::GetDlgItem(m_hWnd, IDC_HASH_PROGRESS), PBM_SETPOS, (int)(10000 * ((0.5 * (startFiles - files)/startFiles) + 0.5 * (startBytes - bytes) / startBytes)), 0);
	}
}

void HashProgressDlg::OnCloseCmd(WORD wID) {
	::EndDialog(m_hWnd, wID);
}

void HashProgressDlg::DoModal(HWND hWndParent) {
	::DialogBoxParam(g_hInstance, MAKEINTRESOURCE(IDD_HASH_PROGRESS), hWndParent, HashProgressDlgProc, (LPARAM)this);
}

/*
 * Some parts Copyright (C) 2002-2011 Petr Kozelka, PPK at czdc dot org
 * Licensed under GNU General Public License version 3.
 */
