/*
 * Copyright (C) 2001-2011 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "stdafx.h"
#include "../client/DCPlusPlus.h"
#include "Resource.h"

#include "HubFrame.h"
#include "LineDlg.h"
#include "SearchFrame.h"
#include "PrivateFrame.h"
#include "CountryFlags.h"

#include "../client/QueueManager.h"
#include "../client/ShareManager.h"
#include "../client/Util.h"
#include "../client/FavoriteManager.h"
#include "../client/LogManager.h"
#include "../client/SettingsManager.h"
#include "../client/ConnectionManager.h"
#include "../client/SearchManager.h"

HubFrame::FrameMap HubFrame::frames;

int HubFrame::columnSizes[] = { 100, 75, 75, 75, 100, 75, 100, 40, 40, 40, 40, 40, 40, 50 };
int HubFrame::columnIndexes[] = { COLUMN_NICK, COLUMN_SHARED, COLUMN_EXACT_SHARED, COLUMN_DESCRIPTION, COLUMN_TAG, COLUMN_CONNECTION, COLUMN_EMAIL, COLUMN_CLIENTID, COLUMN_VERSION,
    COLUMN_MODE, COLUMN_HUBS, COLUMN_SLOTS, COLUMN_LIMITER, COLUMN_COUNTRY };
static ResourceManager::Strings columnNames[] = { ResourceManager::NICK, ResourceManager::SHARED, ResourceManager::EXACT_SHARED, ResourceManager::DESCRIPTION, ResourceManager::TAG,
    ResourceManager::CONNECTION, ResourceManager::EMAIL, ResourceManager::CLIENTID, ResourceManager::VERSION, ResourceManager::MODE, ResourceManager::HUBS, ResourceManager::SLOTS,
    ResourceManager::LIMIT, ResourceManager::COUNTRY
};

ATOM HubFrame::m_aAtom = 0;
UINT HubFrame::m_uiResourceID = IDR_HUB;

tstring sSelectedURL = Util::emptyStringT;
long lURLBegin = 0;
long lURLEnd = 0;

HubFrame::HubFrame(const tstring& aServer, int chatusersplit, bool userliststate,
    string scolumsorder, string scolumswidth, string scolumsvisible) :
	waitingForPW(false), extraSort(false), server(aServer), closed(false),
	updateUsers(false), curCommandPosition(0), inTabComplete(false),
	timeStamps(BOOLSETTING(TIME_STAMPS)), hubchatusersplit(chatusersplit),
	sColumsOrder(scolumsorder), sColumsWidth(scolumswidth),
	sColumsVisible(scolumsvisible), tabMenu(NULL) {
	client = ClientManager::getInstance()->getClient(Text::fromT(aServer));
	client->addListener(this);
	if(FavoriteManager::getInstance()->getFavoriteHubEntry(Text::fromT(server)) != NULL) {
		ShowUserList = userliststate;
	} else {
		ShowUserList = BOOLSETTING(GET_USER_INFO);
	}

    RegisterWindowClass(m_aAtom, HubFrameWndProc, _T("HubFrame"), m_uiResourceID);
}

HubFrame::~HubFrame() {
	ClientManager::getInstance()->putClient(client);

	dcassert(frames.find(server) != frames.end());
	dcassert(frames[server] == this);
	frames.erase(server);

	clearTaskList();

    if(tabMenu != NULL) {
        ::DestroyMenu(tabMenu);
    }
}

LRESULT CALLBACK HubFrame::HubFrameWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
	HubFrame * pParent;

	if(uMsg == WM_NCCREATE) {
		pParent = (HubFrame *)((LPMDICREATESTRUCT)((LPCREATESTRUCT)lParam)->lpCreateParams)->lParam;
		SetWindowLongPtr(hWnd, GWLP_USERDATA, (LONG_PTR)pParent);

		pParent->m_hWnd = hWnd;
	} else {
		pParent = (HubFrame *)GetWindowLongPtr(hWnd, GWLP_USERDATA);
		if(pParent == NULL) {
			return ::DefMDIChildProc(hWnd, uMsg, wParam, lParam);
		}
	}

	return pParent->HubFrameProc(hWnd, uMsg, wParam, lParam);
}

LRESULT HubFrame::HubFrameProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    switch(uMsg) {
        case WM_CLOSE:
            if(onClose() == true) {
                return 0;
            }
            break;
        case WM_SETFOCUS:
            onSetFocus();
            break;
        case WM_CREATE:
            OnCreate();
            return 0;
        case WM_FORWARDMSG:
            OnForwardMsg(lParam);
            return 0;
        case WM_SPEAKER:
            onSpeaker();
            return 0;
        case WM_CONTEXTMENU:
            if(onContextMenu(wParam, lParam) == true) {
                return 0;
            }
            break;
        case WM_CTLCOLOREDIT:
        case WM_CTLCOLORSTATIC:
            return onCtlColor(wParam);
        case FTM_CONTEXTMENU:
            onTabContextMenu(lParam);
            return TRUE;
        case WM_REFRESH_SETTINGS:
            onRefreshSettings();
            return 0;
        case WM_SIZE:
            onSize(wParam, lParam);
            break;
        case WM_COMMAND:
            switch(LOWORD(wParam)) {
                case ID_FILE_RECONNECT:
                    onFileReconnect();
                    return 0;
                case IDC_FOLLOW:
                    onFollow();
                    return 0;
                case IDC_SEND_MESSAGE:
                    onEnter();
                    return 0;
                case IDC_ADD_AS_FAVORITE:
                    addAsFavorite();
                    return 0;
                case IDC_COPY_NICK:
                case IDC_COPY_EXACT_SHARE:
                case IDC_COPY_TAG:
                case IDC_COPY_DESCRIPTION:
                case IDC_COPY_EMAIL_ADDRESS:
                case IDC_COPY_IP:
                    onCopyUserInfo(LOWORD(wParam));
                    return 0;
                case IDC_CLOSE_WINDOW:
                    ::PostMessage(m_hWnd, WM_CLOSE, 0, 0);
                    return 0;
                case ID_EDIT_COPY:
                    onEditCopy();
                    return 0;
                case ID_EDIT_SELECT_ALL:
                    onEditSelectAll();
                    return 0;
                case ID_EDIT_CLEAR_ALL:
                    onEditClearAll();
                    return 0;
                case IDC_COPY_ACTUAL_LINE:
                    onCopyActualLine();
                    return 0;
                case IDC_SELECT_USER:
                    onSelectUser();
                    return 0;
                case IDC_PUBLIC_MESSAGE:
                    onPublicMessage();
                    return 0;
                case IDC_AUTOSCROLL_CHAT:
                    onAutoScrollChat();
                    return 0;
                case IDC_COPY_URL:
                    onCopyURL();
                    return 0;
                case IDC_OPEN_HUB_LOG:
                    onOpenHubLog();
                    return 0;
                case IDC_OPEN_USER_LOG:
                    onOpenUserLog();
                    return 0;
                case IDC_WHOIS_IP:
                    onWhoisIP();
                    return 0;
                case IDC_IGNORE:
                    onIgnore();
                    return 0;
                case IDC_UNIGNORE:
                    onUnignore();
                    return 0;
                case IDC_COPY_HUB:
                    WinUtil::setClipboard(Text::toT(client->getHubName()));
                    return 0;
                case IDC_COPY_LINK:
                    WinUtil::setClipboard(Text::toT(client->getHubUrl()));
                    return 0;
                case IDC_GETLIST:
                    onGetList();
                    return 0;
                case IDC_MATCH_QUEUE:
                    onMatchQueue();
                    return 0;
                case IDC_PRIVATEMESSAGE:
                    onPrivateMessage();
                    return 0;
                case IDC_ADD_TO_FAVORITES:
                    onAddToFavorites();
                    return 0;
                case IDC_GRANTSLOT:
                    onGrantSlot();
                    return 0;
                case IDC_GRANTSLOT_HOUR:
                    onGrantSlotHour();
                    return 0;
                case IDC_GRANTSLOT_DAY:
                    onGrantSlotDay();
                    return 0;
                case IDC_GRANTSLOT_WEEK:
                    onGrantSlotWeek();
                    return 0;
                case IDC_UNGRANTSLOT:
                    onUnGrantSlot();
                    return 0;
                case IDC_SEARCH_DC:
                case IDC_SEARCH_GOOGLE:
                case IDC_SEARCH_YAHOO:
                case IDC_SEARCH_IMDB:
                case IDC_SEARCH_CSFD:
                case IDC_SEARCH_WIKI:
                    onSearch(LOWORD(wParam));
                    return 0;
            }

            break;
        case WM_NOTIFY:
            if(((LPNMHDR)lParam)->idFrom == IDC_USERS) {
                switch(((LPNMHDR)lParam)->code) {
                    case LVN_GETDISPINFO: {
                        ctrlUsers.onGetDispInfo((LPNMHDR)lParam);
                        return 0;
                    }
                    case LVN_COLUMNCLICK: {
                        ctrlUsers.onColumnClick((LPNMHDR)lParam);
                        return 0;
                    }
                    case LVN_GETINFOTIP: {
                        ctrlUsers.onInfoTip((LPNMHDR)lParam);
                        return 0;
                    }
                    case NM_DBLCLK:
                        onDoubleClickUsers((LPNMHDR)lParam);
                        return 0;
                    case LVN_KEYDOWN:
                        if(onKeyDownUsers((LPNMHDR)lParam) == true) {
                            return 0;
                        }
                        break;
                    case NM_RETURN:
                        onEnterUsers();
                        return 0;
                    case NM_CUSTOMDRAW: {
                        bool bHandled = true;
                        LRESULT lResult = onCustomDraw((LPNMHDR)lParam, bHandled);
                        if(bHandled == true) {
                            return lResult;
                        }
                        break;
                    }
                }
            } else if(((LPNMHDR)lParam)->idFrom == IDC_CLIENT && ((LPNMHDR)lParam)->code == EN_LINK) {
                onClientEnLink((LPNMHDR)lParam);
                return 0;
            } else if(((LPNMHDR)lParam)->code == TTN_GETDISPINFO) {
                onGetToolTip((LPNMHDR)lParam);
                return 0;
            }

            break;
        case WM_NCDESTROY:
            delete this;
            return ::DefMDIChildProc(hWnd, uMsg, wParam, lParam);
    }

	if(UCHandlerProc(uMsg, wParam) == true) {
		return 0;
    }

	LRESULT lResult = 0;
	if(MDIWindowProc(hWnd, uMsg, wParam, lParam, lResult) == true) {
		return lResult;
    }

    if(uMsg == WM_MOUSEACTIVATE) {
        lResult = ::DefMDIChildProc(m_hWnd, uMsg, wParam, lParam);
    }

    if(BasicSplitterProc(uMsg, wParam, lParam) == true) {
		return lResult;
    }

	return ::DefMDIChildProc(hWnd, uMsg, wParam, lParam);
}

LRESULT CALLBACK HubFrame::ShowUsersWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    if(uMsg == BM_SETCHECK) {
        HubFrame * pParent = (HubFrame *)::GetWindowLongPtr(hWnd, GWLP_USERDATA);
        if(pParent != NULL) {
            pParent->onShowUsers(wParam);
        }
    }

    return ::CallWindowProc(OldButtonProc, hWnd, uMsg, wParam, lParam);
}

LRESULT CALLBACK HubFrame::EditWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    if(uMsg == WM_CHAR || uMsg == WM_KEYDOWN || uMsg == WM_KEYUP) {
        HubFrame * pParent = (HubFrame *)::GetWindowLongPtr(hWnd, GWLP_USERDATA);
        if(pParent != NULL) {
            if(pParent->onChar(uMsg, wParam) == true) {
                return 0;
            }
        }
    }

    return ::CallWindowProc(OldEditProc, hWnd, uMsg, wParam, lParam);
}

LRESULT CALLBACK HubFrame::RichEditWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    if(uMsg == WM_CHAR || uMsg == WM_KEYDOWN || uMsg == WM_KEYUP) {
        HubFrame * pParent = (HubFrame *)::GetWindowLongPtr(hWnd, GWLP_USERDATA);
        if(pParent != NULL) {
            if(pParent->onChar(uMsg, wParam) == true) {
                return 0;
            }
        }
    } else if(uMsg == WM_LBUTTONDBLCLK) {
        HubFrame * pParent = (HubFrame *)::GetWindowLongPtr(hWnd, GWLP_USERDATA);
        if(pParent != NULL) {
            if(pParent->onLButton(wParam, lParam) == true) {
                return 0;
            }
        }
    }

    return ::CallWindowProc(OldRichEditProc, hWnd, uMsg, wParam, lParam);
}

LRESULT CALLBACK HubFrame::FilterWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    HubFrame * pParent = (HubFrame *)::GetWindowLongPtr(hWnd, GWLP_USERDATA);
    if(pParent == NULL) { // should never happen, but ... :-D
        return 0;
    }

    if(uMsg == WM_CTLCOLORLISTBOX) {
        return pParent->onCtlColor(wParam);
    } else if(uMsg == WM_CHAR) {
        if(pParent->onFilterChar(wParam) == true) {
            return 0;
        }
    }

    if(hWnd == pParent->hWndFilter) {
        return ::CallWindowProc(OldEditProc, hWnd, uMsg, wParam, lParam);
    } else if(hWnd == pParent->hWndFilterSel) {
        return ::CallWindowProc(OldComboBoxProc, hWnd, uMsg, wParam, lParam);
    }

    return 0;
}

void HubFrame::OnCreate() {
    WinUtil::tabCtrl->addTab(m_hWnd, RGB(255, 0, 0), IDR_HUB, IDR_HUB_OFF);
    created = true;

    m_hWndStatusBar = CreateWindowEx(0, STATUSCLASSNAME, _T(""), WS_CHILD | WS_VISIBLE | WS_CLIPCHILDREN | WS_CLIPSIBLINGS | SBARS_SIZEGRIP, 0, 0, 0, 0, m_hWnd, 0, g_hInstance, NULL);

	ctrlClient.m_hWnd = CreateWindowEx(WS_EX_CLIENTEDGE, RICHEDIT_CLASS, _T(""), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | WS_VSCROLL | ES_MULTILINE | ES_NOHIDESEL |
        ES_READONLY, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, m_hWnd, (HMENU)IDC_CLIENT, g_hInstance, NULL);

	::SendMessage(ctrlClient.m_hWnd, EM_EXLIMITTEXT, 0, 0);
	::SendMessage(ctrlClient.m_hWnd, EM_SETLANGOPTIONS, 0, 0);
	::SendMessage(ctrlClient.m_hWnd, WM_SETFONT, (WPARAM)WinUtil::font, MAKELPARAM(TRUE, 0));
    ::SetWindowLongPtr(ctrlClient.m_hWnd, GWLP_USERDATA, (LONG_PTR)this);
    OldRichEditProc = (WNDPROC)::SetWindowLongPtr(ctrlClient.m_hWnd, GWLP_WNDPROC, (LONG_PTR)RichEditWndProc);

	::SendMessage(ctrlClient.m_hWnd, EM_AUTOURLDETECT, FALSE, 0);
	::SendMessage(ctrlClient.m_hWnd, EM_SETEVENTMASK, 0, (DWORD)::SendMessage(ctrlClient.m_hWnd, EM_GETEVENTMASK, 0, 0) | ENM_LINK);
	ctrlClient.SetClient(client);

    hWndMessage = CreateWindowEx(WS_EX_CLIENTEDGE, WC_EDIT, _T(""), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | ES_AUTOVSCROLL | ES_MULTILINE,
        CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, m_hWnd, 0, g_hInstance, NULL);
	::SendMessage(hWndMessage, WM_SETFONT, (WPARAM)WinUtil::font, MAKELPARAM(TRUE, 0));
	::SendMessage(hWndMessage, EM_SETLIMITTEXT, 9999, 0);
    ::SetWindowLongPtr(hWndMessage, GWLP_USERDATA, (LONG_PTR)this);
    OldEditProc = (WNDPROC)::SetWindowLongPtr(hWndMessage, GWLP_WNDPROC, (LONG_PTR)EditWndProc);

    hWndFilter = CreateWindowEx(WS_EX_CLIENTEDGE, WC_EDIT, _T(""), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | ES_AUTOHSCROLL , CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
        m_hWnd, 0, g_hInstance, NULL);
    ::SendMessage(hWndFilter, WM_SETFONT, (WPARAM)WinUtil::font, MAKELPARAM(TRUE, 0));
    ::SetWindowLongPtr(hWndFilter, GWLP_USERDATA, (LONG_PTR)this);
    OldEditProc = (WNDPROC)::SetWindowLongPtr(hWndFilter, GWLP_WNDPROC, (LONG_PTR)FilterWndProc);

    hWndFilterSel = CreateWindowEx(0, WC_COMBOBOX, _T(""), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_HSCROLL | WS_VSCROLL | CBS_DROPDOWNLIST,
        CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, m_hWnd, 0, g_hInstance, NULL);
    ::SendMessage(hWndFilterSel, WM_SETFONT, (WPARAM)WinUtil::font, MAKELPARAM(TRUE, 0));
    ::SetWindowLongPtr(hWndFilterSel, GWLP_USERDATA, (LONG_PTR)this);
    OldComboBoxProc = (WNDPROC)::SetWindowLongPtr(hWndFilterSel, GWLP_WNDPROC, (LONG_PTR)FilterWndProc);

	ctrlUsers.Create(m_hWnd, WS_CHILD | WS_VISIBLE | WS_CLIPCHILDREN | WS_CLIPSIBLINGS | WS_HSCROLL | WS_VSCROLL | LVS_REPORT | LVS_SHOWSELALWAYS | LVS_SHAREIMAGELISTS, WS_EX_CLIENTEDGE,
        IDC_USERS, LVS_EX_LABELTIP | LVS_EX_HEADERDRAGDROP | LVS_EX_FULLROWSELECT | LVS_EX_INFOTIP, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT);

    iProportionalPos = (hubchatusersplit == 0 ? 7500 : hubchatusersplit);

    hWndShowUsers = CreateWindowEx(0, WC_BUTTON, _T(""), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | BS_AUTOCHECKBOX, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
        m_hWndStatusBar, 0, g_hInstance, NULL);
	::SendMessage(hWndShowUsers, WM_SETFONT, (WPARAM)WinUtil::systemFont, MAKELPARAM(TRUE, 0));
    ::SetWindowLongPtr(hWndShowUsers, GWLP_USERDATA, (LONG_PTR)this);
    OldButtonProc = (WNDPROC)::SetWindowLongPtr(hWndShowUsers, GWLP_WNDPROC, (LONG_PTR)ShowUsersWndProc);
    ::SendMessage(hWndShowUsers, BM_SETCHECK, (ShowUserList == true ? BST_CHECKED : BST_UNCHECKED), 0);

    bool bColumsFromFavorite = false;
    if(sColumsOrder != Util::emptyString && sColumsWidth != Util::emptyString && sColumsVisible != Util::emptyString) {
        bColumsFromFavorite = true;
    }

    if(bColumsFromFavorite == false) {
		WinUtil::splitTokens(columnIndexes, SETTING(HUBFRAME_ORDER), COLUMN_LAST);
		WinUtil::splitTokens(columnSizes, SETTING(HUBFRAME_WIDTHS), COLUMN_LAST);                           
    } else {
		WinUtil::splitTokens(columnIndexes, sColumsOrder, COLUMN_LAST);
		WinUtil::splitTokens(columnSizes, sColumsWidth, COLUMN_LAST);
	}

	for(uint8_t ui8i = 0; ui8i < (sizeof(columnSizes) / sizeof(columnSizes[0])); ui8i++) {
		int fmt = (ui8i == COLUMN_SHARED || ui8i == COLUMN_EXACT_SHARED) ? LVCFMT_RIGHT : LVCFMT_LEFT;
		ctrlUsers.InsertColumn(ui8i, CTSTRING_I(columnNames[ui8i]), fmt, columnSizes[ui8i], ui8i);
	}

	ctrlUsers.setColumnOrderArray(COLUMN_LAST, columnIndexes);
    if(bColumsFromFavorite == true) {
	    ctrlUsers.setVisible(sColumsVisible);
    } else {
	    ctrlUsers.setVisible(SETTING(HUBFRAME_VISIBLE));
    }

	::SendMessage(ctrlClient.m_hWnd, EM_SETBKGNDCOLOR, 0, WinUtil::bgColor);

	ctrlUsers.setSortColumn(COLUMN_NICK);

	::SendMessage(ctrlUsers.m_hWnd, LVM_SETIMAGELIST, LVSIL_SMALL, (LPARAM)WinUtil::userImages);

	TOOLINFO ti;
    ToolInfoInit(ti, TTF_SUBCLASS, m_hWndStatusBar, 0, NULL, LPSTR_TEXTCALLBACK);

	hWndLastLines = CreateWindowEx(WS_EX_TOPMOST, TOOLTIPS_CLASS, _T(""), WS_POPUP | TTS_NOPREFIX | TTS_ALWAYSTIP, CW_USEDEFAULT, CW_USEDEFAULT,
        CW_USEDEFAULT, CW_USEDEFAULT, m_hWndStatusBar, 0, g_hInstance, NULL);
	::SetWindowPos(hWndLastLines, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_NOACTIVATE);
	::SendMessage(hWndLastLines, TTM_ADDTOOL, 0, (LPARAM)&ti);
	::SendMessage(hWndLastLines, TTM_SETDELAYTIME, TTDT_AUTOPOP, MAKELPARAM(30000, 0));

	tabMenu = ::CreatePopupMenu();

	if(BOOLSETTING(LOG_MAIN_CHAT)) {
        ::AppendMenu(tabMenu, MF_STRING, IDC_OPEN_HUB_LOG, CTSTRING(OPEN_HUB_LOG));
		::AppendMenu(tabMenu, MF_SEPARATOR, 0, NULL);
	}

    ::AppendMenu(tabMenu, MF_STRING, IDC_ADD_AS_FAVORITE, CTSTRING(ADD_TO_FAVORITES));
	::AppendMenu(tabMenu, MF_STRING, ID_FILE_RECONNECT, CTSTRING(MENU_RECONNECT));

	HMENU tabcopyMenu = ::CreatePopupMenu();
	::AppendMenu(tabcopyMenu, MF_STRING, IDC_COPY_HUB, CTSTRING(HUB_NAME));
	::AppendMenu(tabcopyMenu, MF_STRING, IDC_COPY_LINK, CTSTRING(HUB_ADDRESS));

	::AppendMenu(tabMenu, MF_POPUP, (UINT_PTR)tabcopyMenu, CTSTRING(COPY));

	showJoins = BOOLSETTING(SHOW_JOINS);
	favShowJoins = BOOLSETTING(FAV_SHOW_JOINS);

	for(int i = 0; i < COLUMN_LAST; i++) {
		::SendMessage(hWndFilterSel, CB_ADDSTRING, 0, (LPARAM)CTSTRING_I(columnNames[i]));
	}

	::SendMessage(hWndFilterSel, CB_SETCURSEL, 0, 0);

	client->connect();

	FavoriteManager::getInstance()->addListener(this);
	TimerManager::getInstance()->addListener(this);
	SettingsManager::getInstance()->addListener(this);
}

void HubFrame::openWindow(const tstring& aServer, 
	int windowposx, int windowposy, int windowsizex, int windowsizey, int windowtype, int chatusersplit, bool userliststate, 
	string sColumsOrder, string sColumsWidth, string sColumsVisible) {
	FrameIter i = frames.find(aServer);
	if(i == frames.end()) {
		HubFrame* frm = new HubFrame(aServer, chatusersplit, userliststate, sColumsOrder, sColumsWidth, sColumsVisible);
		frames[aServer] = frm;

		int nCmdShow = SW_SHOWDEFAULT;
		RECT rc = { 0 };

		rc.left = windowposx;
		rc.top = windowposy;
		rc.right = rc.left + windowsizex;
		rc.bottom = rc.top + windowsizey;
		if((rc.left < 0 ) || (rc.top < 0) || (rc.right - rc.left < 10) || ((rc.bottom - rc.top) < 10)) {
			rc.left = CW_USEDEFAULT;
			rc.top = CW_USEDEFAULT;
			rc.right = CW_USEDEFAULT;
			rc.bottom = CW_USEDEFAULT;
		}

		frm->CreateEx(frm, m_aAtom, m_uiResourceID, &rc);
		if(windowtype) {
			::ShowWindow(frm->m_hWnd, ((nCmdShow == SW_SHOWDEFAULT) || (nCmdShow == SW_SHOWNORMAL)) ? windowtype : nCmdShow);
		}
	} else {
		if(::IsIconic(i->second->m_hWnd))
			::ShowWindow(i->second->m_hWnd, SW_RESTORE);
		::SendMessage(WinUtil::mdiClient, WM_MDIACTIVATE, (WPARAM)i->second->m_hWnd, 0);
	}
}

void HubFrame::OnForwardMsg(LPARAM lParam) {
	LPMSG pMsg = (LPMSG)lParam;
	if((pMsg->message >= WM_MOUSEFIRST) && (pMsg->message <= WM_MOUSELAST)) {
		::SendMessage(hWndLastLines, TTM_RELAYEVENT, 0, (LPARAM)pMsg);
    }
}

void HubFrame::onEnter() {
	if(::GetWindowTextLength(hWndMessage) > 0) {
        int iMsgLen = ::GetWindowTextLength(hWndMessage);
		TCHAR * msg = new TCHAR[iMsgLen+1];
		::GetWindowText(hWndMessage, msg, iMsgLen+1);
		tstring s(msg, iMsgLen);
		delete []msg;

		// save command in history, reset current buffer pointer to the newest command
		curCommandPosition = prevCommands.size();		//this places it one position beyond a legal subscript
		if(!curCommandPosition || curCommandPosition > 0 && prevCommands[curCommandPosition - 1] != s) {
			++curCommandPosition;
			prevCommands.push_back(s);
		}
		currentCommand = _T("");

		// Special command
		if(s[0] == _T('/')) {
			tstring m = s;
			tstring param;
			tstring message;
			tstring status;

			if(BOOLSETTING(CLIENT_COMMANDS)) {
				addClientLine(_T("Client command: ") + s, WinUtil::m_ChatTextSystem, true);
			}

			if(WinUtil::checkCommand(s, param, message, status)) {
				if(!message.empty()) {
					client->hubMessage(Text::fromT(message));
				}
				if(!status.empty()) {
					addClientLine(status, WinUtil::m_ChatTextSystem, true);
				}
			} else if(Util::stricmp(s.c_str(), _T("join"))==0) {
				if(!param.empty()) {
					redirect = param;
					if(BOOLSETTING(JOIN_OPEN_NEW_WINDOW)) {
						HubFrame::openWindow(param);
					} else {
						onFollow();
					}
				} else {
					addClientLine(TSTRING(SPECIFY_SERVER), WinUtil::m_ChatTextSystem, true);
				}
			} else if((Util::stricmp(s.c_str(), _T("clear")) == 0) || (Util::stricmp(s.c_str(), _T("cls")) == 0)) {
				ctrlClient.ClearChat();
			} else if(Util::stricmp(s.c_str(), _T("ts")) == 0) {
				timeStamps = !timeStamps;
				if(timeStamps) {
					addClientLine(TSTRING(TIMESTAMPS_ENABLED), WinUtil::m_ChatTextSystem, true);
				} else {
					addClientLine(TSTRING(TIMESTAMPS_DISABLED), WinUtil::m_ChatTextSystem, true);
				}
			} else if((Util::stricmp(s.c_str(), _T("password")) == 0) && waitingForPW) {
				client->setPassword(Text::fromT(param));
				client->password(Text::fromT(param));
				waitingForPW = false;
			} else if( Util::stricmp(s.c_str(), _T("showjoins")) == 0 ) {
				showJoins = !showJoins;
				if(showJoins) {
					addClientLine(TSTRING(JOIN_SHOWING_ON), WinUtil::m_ChatTextSystem, true);
				} else {
					addClientLine(TSTRING(JOIN_SHOWING_OFF), WinUtil::m_ChatTextSystem, true);
				}
			} else if( Util::stricmp(s.c_str(), _T("favshowjoins")) == 0 ) {
				favShowJoins = !favShowJoins;
				if(favShowJoins) {
					addClientLine(TSTRING(FAV_JOIN_SHOWING_ON), WinUtil::m_ChatTextSystem, true);
				} else {
					addClientLine(TSTRING(FAV_JOIN_SHOWING_OFF), WinUtil::m_ChatTextSystem, true);
				}
			} else if(Util::stricmp(s.c_str(), _T("close")) == 0) {
				::PostMessage(m_hWnd, WM_CLOSE, 0, 0);
			} else if(Util::stricmp(s.c_str(), _T("userlist")) == 0) {
				::SendMessage(hWndShowUsers, BM_SETCHECK, (ShowUserList == true ? BST_UNCHECKED : BST_CHECKED), 0);
			} else if(Util::stricmp(s.c_str(), _T("connection")) == 0) {
				addClientLine(Text::toT((STRING(IP) + client->getLocalIp() + ", " + 
				STRING(PORT) +
				Util::toString(ConnectionManager::getInstance()->getPort()) + "/" + 
				Util::toString(SearchManager::getInstance()->getPort()))), 
				WinUtil::m_ChatTextSystem, true);
			} else if((Util::stricmp(s.c_str(), _T("favorite")) == 0) || (Util::stricmp(s.c_str(), _T("fav")) == 0)) {
				addAsFavorite();
			} else if((Util::stricmp(s.c_str(), _T("removefavorite")) == 0) || (Util::stricmp(s.c_str(), _T("removefav")) == 0)) {
				removeFavoriteHub();
			} else if(Util::stricmp(s.c_str(), _T("getlist")) == 0){
				if(param.empty() == false){
					UserInfo* ui = findUser(param);
					if(ui != NULL) {
						ui->getList();
					} else {
                        OnlineUser* ou = client->findUser(Text::fromT(param));
                        if(ou != NULL) {
                        	try {
                        		QueueManager::getInstance()->addList(ou->getUser(), QueueItem::FLAG_CLIENT_VIEW);
                        	} catch(const Exception& e) {
                        		LogManager::getInstance()->message(e.getError());
                        	}
                        }
                    }
				}
			} else if(Util::stricmp(s.c_str(), _T("extraslots")) == 0) {
				int j = Util::toInt(Text::fromT(param));
				if(j > 0) {
					SettingsManager::getInstance()->set(SettingsManager::EXTRA_SLOTS, j);
					addClientLine(TSTRING(EXTRA_SLOTS_SET), WinUtil::m_ChatTextSystem, true);
				} else {
					addClientLine(TSTRING(INVALID_NUMBER_OF_SLOTS), WinUtil::m_ChatTextSystem, true);
				}
			} else if(Util::stricmp(s.c_str(), _T("smallfilesize")) == 0) {
				int j = Util::toInt(Text::fromT(param));
				if(j >= 64) {
					SettingsManager::getInstance()->set(SettingsManager::SMALL_FILE_SIZE, j);
					addClientLine(TSTRING(SMALL_FILE_SIZE_SET), WinUtil::m_ChatTextSystem, true);
				} else {
					addClientLine(TSTRING(INVALID_SIZE), WinUtil::m_ChatTextSystem, true);
				}
			} else if(Util::stricmp(s.c_str(), _T("savequeue")) == 0) {
				QueueManager::getInstance()->saveQueue();
				addClientLine(_T("Queue saved."), WinUtil::m_ChatTextSystem, true);
			} else if(Util::stricmp(s.c_str(), _T("whois")) == 0) {
				WinUtil::openLink(_T("http://www.ripe.net/perl/whois?form_type=simple&full_query_string=&searchtext=") + Text::toT(Util::encodeURI(Text::fromT(param))));
			} else if(Util::stricmp(s.c_str(), _T("log")) == 0) {
				StringMap params;
				params["hubNI"] = client->getHubName();
				params["hubURL"] = client->getHubUrl();
				params["myNI"] = client->getMyNick();
				if(param.empty()) {
					WinUtil::openFile(Text::toT(Util::validateFileName(SETTING(LOG_DIRECTORY) + Util::formatParams(SETTING(LOG_FILE_MAIN_CHAT), params, true))));
				} else if(Util::stricmp(param.c_str(), _T("status")) == 0) {
					WinUtil::openFile(Text::toT(Util::validateFileName(SETTING(LOG_DIRECTORY) + Util::formatParams(SETTING(LOG_FILE_STATUS), params, true))));
				}
			} else if(Util::stricmp(s.c_str(), _T("help")) == 0) {
				addSystemLine(_T("*** ") + WinUtil::commands + _T(", /smallfilesize #, /extraslots #, /savequeue, /join <hub-ip>, /clear, /ts, /showjoins, /favshowjoins, /close, /userlist, /connection, /favorite, /pm <user> [message], /getlist <user>, /log <status, system, downloads, uploads>, /winamp, /whois [IP], /favoritenick, /favoritepassword, /removefavorite"), WinUtil::m_ChatTextSystem);
			} else if(Util::stricmp(s.c_str(), _T("pm")) == 0) {
				string::size_type j = param.find(_T(' '));
				if(j != string::npos) {
					tstring nick = param.substr(0, j);
					UserInfo* ui = findUser(nick);
					if(ui != NULL) {
						if(param.size() > j + 1)
							PrivateFrame::openWindow(ui->user, param.substr(j+1));
						else
							PrivateFrame::openWindow(ui->user);
					} else {
                        OnlineUser* ou = client->findUser(Text::fromT(nick));
                        if(ou != NULL) {
    						if(param.size() > j + 1)
    							PrivateFrame::openWindow(ou->getUser(), param.substr(j+1));
    						else
    							PrivateFrame::openWindow(ou->getUser());
                        }
                    }
				} else if(!param.empty()) {
					UserInfo* ui = findUser(param);
					if(ui != NULL) {
						PrivateFrame::openWindow(ui->user);
					} else {
                        OnlineUser* ou = client->findUser(Text::fromT(param));
                        if(ou != NULL) {
    						PrivateFrame::openWindow(ou->getUser());
                        }
                    }
				}
			} else if(Util::stricmp(s.c_str(), _T("me")) == 0) {
				client->hubMessage(Text::fromT(m));
			} else if((Util::stricmp(s.c_str(), _T("favoritenick")) == 0) || (Util::stricmp(s.c_str(), _T("favnick")) == 0)) {
				FavoriteHubEntry* hub = FavoriteManager::getInstance()->getFavoriteHubEntry(Text::fromT(server));
				if(hub) {
					if(!param.empty()) {
						hub->setNick(Text::fromT(param));
					} else {
						hub->setNick(client->getMyNick());
					}
					FavoriteManager::getInstance()->save();
					addClientLine(TSTRING(NICK_FOR_FAVORITE), WinUtil::m_ChatTextSystem, true);
				} else {
					addClientLine(TSTRING(HUB_NOT_FAVORITE), WinUtil::m_ChatTextSystem, true);
				}
			} else if((Util::stricmp(s.c_str(), _T("favoritepassword")) == 0) || (Util::stricmp(s.c_str(), _T("favpass")) == 0)) {
				FavoriteHubEntry* hub = FavoriteManager::getInstance()->getFavoriteHubEntry(Text::fromT(server));
				if(hub) {
					if(!param.empty()) {
						hub->setPassword(Text::fromT(param));
					} else {
						hub->setPassword(client->getPassword());
					}
					FavoriteManager::getInstance()->save();
					addClientLine(TSTRING(PASS_FOR_FAVORITE), WinUtil::m_ChatTextSystem, true);
				} else {
					addClientLine(TSTRING(HUB_NOT_FAVORITE), WinUtil::m_ChatTextSystem, true);
				}
			} else if((Util::stricmp(s.c_str(), _T("stat")) == 0) || (Util::stricmp(s.c_str(), _T("stats")) == 0)) {
			    if(client->isOp())
					client->hubMessage(WinUtil::generateStats());
			    else
					addSystemLine(Text::toT(WinUtil::generateStats()), WinUtil::m_ChatTextSystem);
			} else {
				if (BOOLSETTING(SEND_UNKNOWN_COMMANDS)) {
					client->hubMessage(Text::fromT(m));
				} else {
					addClientLine(TSTRING(UNKNOWN_COMMAND) + s, WinUtil::m_ChatTextSystem, true);
				}
			}

			::SetWindowText(hWndMessage, _T(""));
		} else if(waitingForPW) {
			addClientLine(TSTRING(DONT_REMOVE_SLASH_PASSWORD), WinUtil::m_ChatTextSystem, true);
			::SetWindowText(hWndMessage, _T("/password "));
			::SetFocus(hWndMessage);
            ::SendMessage(hWndMessage, EM_SETSEL, 10, 10);
			::SendMessage(hWndMessage, EM_SCROLLCARET, 0, 0);
		} else {
			if(BOOLSETTING(SERVER_COMMANDS)) {
				if(s[0] == '!' || s[0] == '+' || s[0] == '-')
					addClientLine(_T("Server command: ") + s, WinUtil::m_ChatTextSystem, true);
			}

			client->hubMessage(Text::fromT(s));
			::SetWindowText(hWndMessage, _T(""));
		}
	} else {
		MessageBeep(MB_ICONEXCLAMATION);
	}
}

void HubFrame::addAsFavorite() {
	FavoriteHubEntry* Hub = FavoriteManager::getInstance()->getFavoriteHubEntry(client->getHubUrl());
	if(!Hub) {
		FavoriteHubEntry aEntry;
		TCHAR buf[256];
		::GetWindowText(m_hWnd, buf, 255);
		aEntry.setServer(Text::fromT(server));
		aEntry.setName(Text::fromT(buf));
		aEntry.setDescription(Text::fromT(buf));
		aEntry.setConnect(false);
		aEntry.setNick(client->getMyNick());
		aEntry.setPassword(client->getPassword());
		aEntry.setConnect(false);
		FavoriteManager::getInstance()->addFavorite(aEntry);
		addClientLine(TSTRING(FAVORITE_HUB_ADDED), WinUtil::m_ChatTextSystem, true);
	} else {
		addClientLine(TSTRING(FAVORITE_HUB_ALREADY_EXISTS), WinUtil::m_ChatTextSystem, true);
	}
}

void HubFrame::removeFavoriteHub() {
	FavoriteHubEntry* removeHub = FavoriteManager::getInstance()->getFavoriteHubEntry(client->getHubUrl());
	if(removeHub) {
		FavoriteManager::getInstance()->removeFavorite(removeHub);
		addClientLine(TSTRING(FAVORITE_HUB_REMOVED), WinUtil::m_ChatTextSystem, true);
	} else {
		addClientLine(TSTRING(FAVORITE_HUB_DOES_NOT_EXIST), WinUtil::m_ChatTextSystem, true);
	}
}

void HubFrame::onCopyUserInfo(WORD wID) {
    if(client->isConnected() == false || ChatCtrl::sSelectedUser.empty() == true) {
        return;
    }

	UserInfo* ui = findUser(ChatCtrl::sSelectedUser);
	if(ui != NULL) {
		switch(wID) {
			case IDC_COPY_NICK:
				WinUtil::setClipboard(ui->getText(COLUMN_NICK, NULL));
				return;
			case IDC_COPY_EXACT_SHARE:
				WinUtil::setClipboard(ui->getText(COLUMN_EXACT_SHARED, NULL));
				return;
			case IDC_COPY_DESCRIPTION:
				WinUtil::setClipboard(ui->getText(COLUMN_DESCRIPTION, NULL));
				return;
			case IDC_COPY_TAG:
				WinUtil::setClipboard(ui->getText(COLUMN_TAG, NULL));
				return;
			case IDC_COPY_EMAIL_ADDRESS:
				WinUtil::setClipboard(ui->getText(COLUMN_EMAIL, NULL));
				return;
			case IDC_COPY_IP: {
                const TCHAR * sIP = ui->getText(COLUMN_COUNTRY, NULL);
                if(sIP[0] == _T('\0') || _istdigit(sIP[0])) {
                    WinUtil::setClipboard(sIP);
                } else {
                    WinUtil::setClipboard(sIP+3);
                }
				return;
			}
			default:
				return;
		}
    }

	OnlineUser* ou = client->findUser(Text::fromT(ChatCtrl::sSelectedUser));
	if(ou != NULL) {
		switch(wID) {
			case IDC_COPY_NICK:
				WinUtil::setClipboard(Text::toT(ou->getIdentity().getNick()));
				return;
			case IDC_COPY_EXACT_SHARE:
				WinUtil::setClipboard(Text::toT(Util::formatExactSize(ou->getIdentity().getBytesShared())));
				return;
			case IDC_COPY_DESCRIPTION:
				WinUtil::setClipboard(Text::toT(TO_UTF8(ou->getIdentity().getDescription())));
				return;
			case IDC_COPY_TAG:
				WinUtil::setClipboard(Text::toT(TO_UTF8(ou->getIdentity().getTag())));
				return;
			case IDC_COPY_EMAIL_ADDRESS:
				WinUtil::setClipboard(Text::toT(TO_UTF8(ou->getIdentity().getEmail())));
				return;
			case IDC_COPY_IP:
				WinUtil::setClipboard(Text::toT(TO_UTF8(ou->getIdentity().getIp())));
				return;
			default:
				return;
		}
    }
}

void HubFrame::onDoubleClickUsers(LPNMHDR pnmh) {
	NMITEMACTIVATE* item = (NMITEMACTIVATE*)pnmh;

	if(item->iItem == -1) {
        return;
    }

	switch(SETTING(USERLIST_DBLCLICK)) {
		case 0:
            ctrlUsers.getItemData(item->iItem)->getList();
            break;
		case 1: {
            tstring sUser = Text::toT(ctrlUsers.getItemData(item->iItem)->getNick()) + _T(": ");

            ::SendMessage(hWndMessage, EM_SETSEL, ::GetWindowTextLength(hWndMessage), ::GetWindowTextLength(hWndMessage));
            ::SendMessage(hWndMessage, EM_SCROLLCARET, 0, 0);
            ::SendMessage(hWndMessage, EM_REPLACESEL, (WPARAM)FALSE, (LPARAM)sUser.c_str());

            ::SetFocus(hWndMessage);
            break;
		}
		case 2:
            if(ctrlUsers.getItemData(item->iItem)->user != ClientManager::getInstance()->getMe()) {
                ctrlUsers.getItemData(item->iItem)->pm();
            }
            break;
		case 3:
            ctrlUsers.getItemData(item->iItem)->matchQueue();
            break;
		case 4:
            ctrlUsers.getItemData(item->iItem)->grant();
            break;
		case 5:
            ctrlUsers.getItemData(item->iItem)->addFav();
            break;
	}
}

void HubFrame::onEditCopy() {
	if(GetFocus() == ctrlClient.m_hWnd) {
		::SendMessage(ctrlClient.m_hWnd, WM_COPY, 0, 0);
	}
}

void HubFrame::onEditSelectAll() {
	if(GetFocus() == ctrlClient.m_hWnd) {
		ctrlClient.SetSelAll();
	}
}

void HubFrame::onEditClearAll() {
	if(GetFocus() == ctrlClient.m_hWnd) {
		ctrlClient.ClearChat();
	}
}

bool HubFrame::updateUser(const UserTask& u, bool &bResort) {
	UserMapIter i = userMap.find(u.user);

	if(i == userMap.end()) {
        if(ShowUserList == false) {
            userMap.insert(make_pair(u.user, (UserInfo*)NULL));
            return true;
        }
		UserInfo* ui = new UserInfo(u, client->isOp(), false);
		bResort = ui->update(u.identity, ctrlUsers.getSortColumn(), client->isOp()) || bResort;
		userMap.insert(make_pair(u.user, ui));
		if(filterstring.empty() || filterUser(u.identity) == true) {
			ctrlUsers.insertItem(ui, WinUtil::getImage(u.identity));
		}
		return true;
	} else {
        if(ShowUserList == false) {
            return false;
        }
		if(filterstring.empty() || filterUser(u.identity) == true) {
			UserInfo* ui = i->second;
			if(i->second != NULL) {
                bResort = ui->update(u.identity, ctrlUsers.getSortColumn(), client->isOp()) || bResort;
    			int pos = ctrlUsers.findItem(ui);
    			if(pos != -1) {
    				ctrlUsers.updateItem(pos);
					ListViewSetItem(ctrlUsers.m_hWnd, pos, 0, LVIF_IMAGE, NULL, WinUtil::getImage(u.identity));
    			}
            } else {
        		ui = new UserInfo(u, client->isOp(), false);
        		bResort = ui->update(u.identity, ctrlUsers.getSortColumn(), client->isOp()) || bResort;
        		i->second = ui;
        		if(filterstring.empty() || filterUser(u.identity) == true) {
        			ctrlUsers.insertItem(ui, WinUtil::getImage(u.identity));
        		}
            }
		} else {
            if(i->second != NULL) {
    			int pos = ctrlUsers.findItem(i->second);
    			if(pos != -1) {
					ListView_SetItemState(ctrlUsers.m_hWnd, pos, 0, LVIS_SELECTED);
					::SendMessage(ctrlUsers.m_hWnd, LVM_DELETEITEM, pos, 0);
    			}
            } else {
        		i->second = new UserInfo(u, client->isOp());
            }
		}
		return false;
	}
}

void HubFrame::removeUser(const UserPtr& aUser) {
	UserMapIter i = userMap.find(aUser);

	if(i == userMap.end()) {
		return;
	}

	if(i->second != NULL) {
        UserInfo* ui = i->second;
		int pos = ctrlUsers.findItem(ui);
		if(pos != -1) {
			ListView_SetItemState(ctrlUsers.m_hWnd, pos, 0, LVIS_SELECTED);
			::SendMessage(ctrlUsers.m_hWnd, LVM_DELETEITEM, pos, 0);
		}
		delete ui;
    }

	userMap.erase(i);
}

UserInfo* HubFrame::findUser(const tstring& nick) {
	for(UserMapIter i = userMap.begin(); i != userMap.end(); ++i) {
		if(i->second != NULL && _tcscmp(i->second->getText(COLUMN_NICK, NULL), nick.c_str()) == NULL)
			return i->second;
	}
	return 0;
}

void HubFrame::onSpeaker() {
	TaskQueue::List t;
	tasks.get(t);

	if(t.size() > 2) {
		::SendMessage(ctrlUsers.m_hWnd, WM_SETREDRAW, (WPARAM)FALSE, 0);
	}

	bool bResort = false;

	for(TaskQueue::Iter i = t.begin(); i != t.end(); ++i) {
		switch(i->first) {
			case UPDATE_USER: {
                UserTask * ut = (UserTask *)i->second;
				updateUser(*ut, bResort);
				delete ut;
				break;
			}
        	case UPDATE_USER_JOIN: {
                UserTask * ut = (UserTask *)i->second;
        		if(updateUser(*ut, bResort)) {
        			if(showJoins || (favShowJoins && FavoriteManager::getInstance()->isFavoriteUser(ut->user))) {
        				addSystemLine(_T("*** ") + TSTRING(JOINS) + Text::toT(ut->identity.getNick()), WinUtil::m_ChatTextSystem);
        			}
        			if(client->isOp() && BOOLSETTING(SHOW_SHARE_CHEAT)) {
        				int64_t bytesSharedInt64 = ut->identity.getBytesShared();
        				if(bytesSharedInt64 > 0) {
        					string bytesShared = Util::toString(bytesSharedInt64);
        					bool divided = (bytesSharedInt64 % (1024*1024) == 0);
        					bool samenumbers = false;
        					static const char* sSameNumbers[] = { "000000", "111111", "222222", "333333", "444444", "555555", "666666", "777777", "888888", "999999" };
        					for(int i = 0; i < 10; i++) {
        						if(strstr(bytesShared.c_str(), sSameNumbers[i]) != 0) {
        							samenumbers = true;
        							break;
        						}
        					}
        					char buf[512];
        					if(samenumbers && divided) {
        						sprintf_s(buf, 512, "*** User %s %s %s bytes", ut->identity.getNick().c_str(), (STRING(SAMENUMBERS_DIVIDED)).c_str(), bytesShared.c_str());
        					} else if(samenumbers) {
        						sprintf_s(buf, 512, "*** User %s %s %s bytes", ut->identity.getNick().c_str(), (STRING(MANY_SAMENUMBERS)).c_str(), bytesShared.c_str());
        					} else if(divided) {
        						sprintf_s(buf, 512, "*** User %s %s %s bytes", ut->identity.getNick().c_str(), (STRING(DIVIDED_MB)).c_str(), bytesShared.c_str());
        					}
        					if(samenumbers || divided) {
        						addSystemLine(Text::toT(buf), WinUtil::m_ChatTextSystem);
        					}
        				}
        			}
        		}
        		delete ut;
        		break;
        	}
        	case REMOVE_USER: {
                UserTask * ut = (UserTask *)i->second;
                removeUser(ut->user);
        		  
        		if(showJoins || (favShowJoins && FavoriteManager::getInstance()->isFavoriteUser(ut->user))) {
        			addSystemLine(Text::toT("*** " + STRING(PARTS) + ut->identity.getNick()), WinUtil::m_ChatTextSystem);
        		}
        		delete ut;
        		break;
    		}
    		case CONNECTED: {
	    		addClientLine(TSTRING(CONNECTED), WinUtil::m_ChatTextServer, true);
	    		setTabColor(RGB(0, 255, 0));
	    		unsetIconState();
	    		break;
	    	}    
	        case DISCONNECTED: {
	    		clearUserList();
	    		setTabColor(RGB(255, 0, 0));
	    		setIconState();
	    		break;
	    	}
	        case ADD_CHAT_LINE: {
	        	MsgTask * mt = (MsgTask *)i->second;
	            if(mt->bNoId == true || !FavoriteManager::getInstance()->isIgnoredUser(mt->identity.getUser())) {
	                addLine(mt->identity, Text::toT(mt->msg), WinUtil::m_ChatTextGeneral);
	            }
	            delete mt;
	    		break;
	    	}
	        case ADD_STATUS_LINE: {
                StringTask * st = (StringTask *)i->second;
	        	addClientLine(Text::toT(st->str), WinUtil::m_ChatTextServer, true);
	        	delete st;
	    		break;
	    	}
	        case SET_WINDOW_TITLE: {
                StringTask * st = (StringTask *)i->second;
				::SetWindowText(m_hWnd, Text::toT(st->str).c_str());
				delete st;
	    		break;
	    	}
	        case STATS: {
            	size_t userCount = userMap.size();	
            	size_t ShownUsers = (int)::SendMessage(ctrlUsers.m_hWnd, LVM_GETITEMCOUNT, 0, 0);
            	string sStatus;
				if(ShowUserList == true) {
                	if((UINT)::SendMessage(ctrlUsers.m_hWnd, LVM_GETSELECTEDCOUNT, 0, 0) > 1) {
                        sStatus += Util::toString((UINT)::SendMessage(ctrlUsers.m_hWnd, LVM_GETSELECTEDCOUNT, 0, 0)) + "/";
                        ::SendMessage(m_hWndStatusBar, SB_SETTEXT, (2 | 0),
                            (LPARAM)Text::toT(Util::formatBytes(ctrlUsers.forEachSelectedT(CountAvailable()).available) + "/" + 
                            Util::formatBytes(CountUserMapAvailable())).c_str());
                    } else {
                        ::SendMessage(m_hWndStatusBar, SB_SETTEXT, (2 | 0), (LPARAM)Text::toT(Util::formatBytes(CountUserMapAvailable())).c_str());
                    }
                	if(filterstring.empty() == false && userCount != ShownUsers) {
                        sStatus += Util::toString((uint32_t)ShownUsers) + "/";
                	}
                } else {
					::SendMessage(m_hWndStatusBar, SB_SETTEXT, (2 | 0), (LPARAM)Text::toT(Util::formatBytes(client->getAvailable())).c_str());
                }
            	::SendMessage(m_hWndStatusBar, SB_SETTEXT, (1 | 0), (LPARAM)Text::toT(sStatus + Util::toString((uint32_t)userCount) + " " + STRING(HUB_USERS)).c_str());
	    		break;
	    	}
	        case GET_PASSWORD: {
	    		if(client->getPassword().size() > 0) {
	    			client->password(client->getPassword());
	    			addClientLine(TSTRING(STORED_PASSWORD_SENT), WinUtil::m_ChatTextSystem, true);
	    		} else {
					if(!BOOLSETTING(PROMPT_PASSWORD)) {
	    				::SetWindowText(hWndMessage, _T("/password "));
	    				::SetFocus(hWndMessage);

                        ::SendMessage(hWndMessage, EM_SETSEL, 10, 10);
            			::SendMessage(hWndMessage, EM_SCROLLCARET, 0, 0);

	    				waitingForPW = true;
					} else {
						LineDlg linePwd;
						linePwd.title = CTSTRING(ENTER_PASSWORD);
						linePwd.description = CTSTRING(ENTER_PASSWORD);
						linePwd.password = true;
						if(linePwd.DoModal(m_hWnd) == IDOK) {
							client->setPassword(Text::fromT(linePwd.line));
							client->password(Text::fromT(linePwd.line));
							waitingForPW = false;
						} else {
							client->disconnect(true);
						}
					}
	    		}
	    		break;
	    	}
	        case PRIVATE_MESSAGE: {
	        	PMTask * pt = (PMTask *)i->second;
	    		if(pt->bNoId == true || !FavoriteManager::getInstance()->isIgnoredUser(pt->fromidentity.getUser())) {
					if(pt->fromidentity.getUser()->isOnline()) {
	    				if((BOOLSETTING(POPUP_PMS) && PrivateFrame::getPmCount() < (size_t)SETTING(MAX_PMS)) || PrivateFrame::isOpen(pt->fromidentity.getUser())) {
	    					PrivateFrame::gotMessage(pt->fromidentity, pt->byidentity, Text::toT(pt->msg));
	    				} else {
							addSystemLine(TSTRING(PRIVATE_MESSAGE_FROM) + Text::toT(pt->fromidentity.getNick()) + _T(": ") + Text::toT(pt->msg), WinUtil::m_ChatTextPrivate);
	    				}
	    				if(BOOLSETTING(MINIMIZE_TRAY) && (pt->fromidentity.getConnection().empty() == false || pt->fromidentity.getBytesShared() != 0)) {
							::PostMessage(MainFrame::pMainFrame->m_hWnd, WM_SPEAKER, 7, NULL);
	    				}
	    			} else {
	    				if(BOOLSETTING(IGNORE_OFFLINE)) {
	    					addClientLine(TSTRING(IGNORED_MESSAGE) + Text::toT(pt->msg), WinUtil::m_ChatTextPrivate, false);
	    				} else if(BOOLSETTING(POPUP_OFFLINE) && PrivateFrame::getPmCount() < (size_t)SETTING(MAX_PMS)) {
	    					PrivateFrame::gotMessage(pt->fromidentity, pt->byidentity, Text::toT(pt->msg));
	    				} else {
	    					addSystemLine(TSTRING(PRIVATE_MESSAGE_FROM) + Text::toT(pt->fromidentity.getNick()) + _T(": ") + Text::toT(pt->msg), WinUtil::m_ChatTextPrivate);
	    				}
					}
	    		}
	    		delete pt;
	    		break;
	    	}
	        case KICK_MSG: {
                MsgTask * mt = (MsgTask *)i->second;
	            if(BOOLSETTING(FILTER_MESSAGES) == true || (BOOLSETTING(FILTER_KICK_MESSAGES_NO_OP) == true && client->isOp() == false)) {
	                addClientLine(Text::toT(mt->msg), WinUtil::m_ChatTextSystem, false, true);
	            } else {
	        		addLine(mt->identity, Text::toT(mt->msg), WinUtil::m_ChatTextServer, false);
	            }
	            delete mt;
	            break;
	    	}
	    	case PRIVATE_MESSAGE_NICK: {
	    		PMNickTask * pnt = (PMNickTask *)i->second;
	    		if(pnt->bNoId == true || !FavoriteManager::getInstance()->isIgnoredUser(pnt->byidentity.getUser())) {
	    			if(BOOLSETTING(IGNORE_OFFLINE)) {
	    				addClientLine(TSTRING(IGNORED_MESSAGE) + Text::toT(pnt->msg), WinUtil::m_ChatTextPrivate, false);
	    			} else if(BOOLSETTING(POPUP_OFFLINE)) {
						Identity id = Identity(ClientManager::getInstance()->getUser(pnt->nick, client->getHubUrl()));
						PrivateFrame::gotMessage(id, pnt->byidentity, Text::toT(pnt->msg));
	    			} else {
	    				addSystemLine(TSTRING(PRIVATE_MESSAGE_FROM) + Text::toT(pnt->nick) + _T(": ") + Text::toT(pnt->msg), WinUtil::m_ChatTextPrivate);
					}
	    		}
	    		delete pnt;
	    		break;
	    	}
			case RESORT_USERLIST:
				bResort = true;
				break;
	    	default:
	            break;
    	}
	}

	if(bResort) {
    	ctrlUsers.resort();
    }

	if(t.size() > 2) {
		::SendMessage(ctrlUsers.m_hWnd, WM_SETREDRAW, (WPARAM)TRUE, 0);
    }
}

void HubFrame::UpdateLayout(bool bResizeStatusBar /* = true */) {
	RECT rect;
	::GetClientRect(m_hWnd, &rect);

	UpdateStatusBar(rect, bResizeStatusBar);

	if(::IsWindow(m_hWndStatusBar)) {
		RECT sr;
		int w[4];
		::GetClientRect(m_hWndStatusBar, &sr);

		int tmp = (sr.right - sr.left) > 332 ? 232 : (((sr.right - sr.left) > 132) ? (sr.right - sr.left) - 100 : 32);

		w[0] = sr.right - tmp;
		w[1] = w[0] + (tmp-32)/2;
		w[2] = w[0] + (tmp-32);
		w[3] = w[2] + 16;

		::SendMessage(m_hWndStatusBar, SB_SETPARTS, 4, (LPARAM)w);

		::SendMessage(hWndLastLines, TTM_SETMAXTIPWIDTH, 0, w[0]);

		// Strange, can't get the correct width of the last field...
		::SendMessage(m_hWndStatusBar, SB_GETRECT, 2, (LPARAM)&sr);
		sr.left = sr.right + 2;
		sr.right = sr.left + 16;
		::MoveWindow(hWndShowUsers, sr.left, sr.top, sr.right - sr.left, sr.bottom - sr.top, TRUE);
	}

	SetSplitterRect(&rect);
}

bool HubFrame::onClose() {
	if(!closed) {
		if(MainFrame::closing == true || BOOLSETTING(CONFIRM_HUBEXIT) == false ||
			::MessageBox(m_hWnd, CTSTRING(CONFIRM_HUBCLOSE), _T(CzDC_VERSION_STRING), MB_YESNO | MB_ICONQUESTION | MB_DEFBUTTON2) == IDYES) {
			RecentHubEntry* r = FavoriteManager::getInstance()->getRecentHubEntry(Text::fromT(server));
			if(r != NULL) {
				TCHAR buf[256];
				::GetWindowText(m_hWnd, buf, 255);
				r->setName(Text::fromT(buf));
				r->setUsers(Util::toString((uint32_t)client->getUserCount()));
				r->setShared(Util::toString(client->getAvailable()));
				FavoriteManager::getInstance()->updateRecent(r);
			}
			TimerManager::getInstance()->removeListener(this);
			FavoriteManager::getInstance()->removeListener(this);
			SettingsManager::getInstance()->removeListener(this);
			client->removeListener(this);
			client->disconnect(true);

			closed = true;
			::PostMessage(m_hWnd, WM_CLOSE, 0, 0);
		}

		return true;
	} else {
		SettingsManager::getInstance()->set(SettingsManager::GET_USER_INFO, ShowUserList);
		FavoriteManager::getInstance()->removeUserCommand(Text::fromT(server));

		clearUserList();
		clearTaskList();

		FavoriteHubEntry* hub = FavoriteManager::getInstance()->getFavoriteHubEntry(Text::fromT(server));
		if(hub != NULL) {
			WINDOWPLACEMENT wp;
			wp.length = sizeof(wp);
			::GetWindowPlacement(m_hWnd, &wp);

			RECT rc;
			::GetWindowRect(m_hWnd, &rc);
			RECT rcmdiClient;
			::GetWindowRect(WinUtil::mdiClient, &rcmdiClient);
			if(wp.showCmd == SW_SHOW || wp.showCmd == SW_SHOWNORMAL) {
				hub->setWindowPosX(rc.left - (rcmdiClient.left + 2));
				hub->setWindowPosY(rc.top - (rcmdiClient.top + 2));
				hub->setWindowSizeX((rc.right - rc.left));
				hub->setWindowSizeY((rc.bottom - rc.top));
			}
			if(wp.showCmd == SW_SHOWNORMAL || wp.showCmd == SW_SHOW || wp.showCmd == SW_SHOWMAXIMIZED || wp.showCmd == SW_MAXIMIZE)
				hub->setWindowType((int)wp.showCmd);
			hub->setChatUserSplit(iProportionalPos);
			hub->setUserListState(ShowUserList);
			ctrlUsers.saveFavoriteHeaderOrder(hub);
			FavoriteManager::getInstance()->save();
		} else {
			ctrlUsers.saveHeaderOrder(SettingsManager::HUBFRAME_ORDER, SettingsManager::HUBFRAME_WIDTHS,
				SettingsManager::HUBFRAME_VISIBLE);
		}

		return false;
	}
}

void HubFrame::clearUserList() {
	::SendMessage(ctrlUsers.m_hWnd, LVM_DELETEALLITEMS, 0, 0);
	for(UserMapIter i = userMap.begin(); i != userMap.end(); ++i) {
		delete i->second;
	}
	userMap.clear();
}

void HubFrame::clearTaskList() {
	TaskQueue::List t;
	tasks.get(t);

	for(TaskQueue::Iter i = t.begin(); i != t.end(); ++i) {
		switch(i->first) {
			case UPDATE_USER:
            case UPDATE_USER_JOIN:
            case REMOVE_USER: {
                UserTask * ut = (UserTask *)i->second;
				delete ut;
				break;
			}
	        case ADD_CHAT_LINE:
            case KICK_MSG: {
	        	MsgTask * mt = (MsgTask *)i->second;
	            delete mt;
	    		break;
	    	}
	        case ADD_STATUS_LINE:
            case SET_WINDOW_TITLE: {
                StringTask * st = (StringTask *)i->second;
	        	delete st;
	    		break;
	    	}
	        case PRIVATE_MESSAGE: {
	        	PMTask * pt = (PMTask *)i->second;
	    		delete pt;
	    		break;
	    	}
	    	case PRIVATE_MESSAGE_NICK: {
	    		PMNickTask * pnt = (PMNickTask *)i->second;
	    		delete pnt;
	    		break;
	    	}
	    	default:
	            break;
    	}
    }
}

bool HubFrame::onLButton(WPARAM wParam, LPARAM lParam) {
	POINT pt = { GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam) };

	ChatCtrl::sSelectedUser = Util::emptyStringT;

	int iBegin = 0, iEnd = 0;
	if(ctrlClient.HitNick(pt, ChatCtrl::sSelectedUser, iBegin, iEnd) == false) {
		return false;
	}

	ctrlClient.SetSel(iBegin, iEnd);
	::InvalidateRect(ctrlClient.m_hWnd, NULL, TRUE);

	UserInfo* ui = findUser(ChatCtrl::sSelectedUser);
	if(ui != NULL) {
		if (wParam & MK_CONTROL) { // MK_CONTROL = 0x0008
			PrivateFrame::openWindow(ui->user);
		} else if (wParam & MK_SHIFT) {
			try {
				QueueManager::getInstance()->addList(ui->user, QueueItem::FLAG_CLIENT_VIEW);
			} catch(const Exception& e) {
				addClientLine(Text::toT(e.getError()), WinUtil::m_ChatTextSystem, true);
			}
		} else {
			switch(SETTING(CHAT_DBLCLICK)) {
				case 0: {
                    int items = (int)::SendMessage(ctrlUsers.m_hWnd, LVM_GETITEMCOUNT, 0, 0);
                    int pos = -1;
					::SendMessage(ctrlUsers.m_hWnd, WM_SETREDRAW, (WPARAM)FALSE, 0);
					for(int i = 0; i < items; ++i) {
						if(ctrlUsers.getItemData(i) == ui) {
                            pos = i;
						}
						ListView_SetItemState(ctrlUsers.m_hWnd, i, (i == pos) ? LVIS_SELECTED | LVIS_FOCUSED : 0, LVIS_SELECTED | LVIS_FOCUSED);
					}
					::SendMessage(ctrlUsers.m_hWnd, WM_SETREDRAW, (WPARAM)TRUE, 0);
					::SendMessage(ctrlUsers.m_hWnd, LVM_ENSUREVISIBLE, pos, FALSE);

					break;
				}
				case 1: {
                    tstring sUser = tstring(ui->getText(COLUMN_NICK, NULL)) + _T(": ");

                    ::SendMessage(hWndMessage, EM_SETSEL, ::GetWindowTextLength(hWndMessage), ::GetWindowTextLength(hWndMessage));
                    ::SendMessage(hWndMessage, EM_SCROLLCARET, 0, 0);
                    ::SendMessage(hWndMessage, EM_REPLACESEL, (WPARAM)FALSE, (LPARAM)sUser.c_str());

					::SetFocus(hWndMessage);

					break;
				}
				case 2:
					if(ui->user != ClientManager::getInstance()->getMe()) {
                        ui->pm();
                    }

                    break;
				case 3:
					ui->getList();

					break;
				case 4:
					ui->matchQueue();

					break;
				case 5:
					ui->grant();

					break;
				case 6:
					ui->addFav();

					break;
			}
		}

		return true;
	}

    OnlineUser* ou = client->findUser(Text::fromT(ChatCtrl::sSelectedUser));
	if(ou != NULL) {
		if(wParam & MK_CONTROL) { // MK_CONTROL = 0x0008
			PrivateFrame::openWindow(ou->getUser());
		} else if (wParam & MK_SHIFT) {
			try {
				QueueManager::getInstance()->addList(ou->getUser(), QueueItem::FLAG_CLIENT_VIEW);
			} catch(const Exception& e) {
				addClientLine(Text::toT(e.getError()), WinUtil::m_ChatTextSystem, true);
			}
		} else {
			switch(SETTING(CHAT_DBLCLICK)) {
				case 1: {
                    tstring sUser = ChatCtrl::sSelectedUser + _T(": ");

                    ::SendMessage(hWndMessage, EM_SETSEL, ::GetWindowTextLength(hWndMessage), ::GetWindowTextLength(hWndMessage));
                    ::SendMessage(hWndMessage, EM_SCROLLCARET, 0, 0);
                    ::SendMessage(hWndMessage, EM_REPLACESEL, (WPARAM)FALSE, (LPARAM)sUser.c_str());

					::SetFocus(hWndMessage);

					break;
				}
				case 2:
					if(ou->getUser() != ClientManager::getInstance()->getMe()) {
                        PrivateFrame::openWindow(ou->getUser());
                    }

                    break;
				case 3:
                    try {
                    	QueueManager::getInstance()->addList(ou->getUser(), QueueItem::FLAG_CLIENT_VIEW);
                    } catch(const Exception& e) {
                    	LogManager::getInstance()->message(e.getError());
                    }

					break;
				case 4:
                    try {
                    	QueueManager::getInstance()->addList(ou->getUser(), QueueItem::FLAG_MATCH_QUEUE);
                    } catch(const Exception& e) {
                    	LogManager::getInstance()->message(e.getError());
                    }

					break;
				case 5:
					UploadManager::getInstance()->reserveSlot(ou->getUser());

					break;
				case 6:
					FavoriteManager::getInstance()->addFavoriteUser(ou->getUser());

					break;
			}
		}

		return true;
	}

	return false;
}

void HubFrame::addLine(const Identity& i, const tstring& aLine, CHARFORMAT2& cf, bool bUseEmo/* = true*/) {
	ctrlClient.AdjustTextSize();

	if(BOOLSETTING(LOG_MAIN_CHAT)) {
		StringMap params;
		params["message"] = Text::fromT(aLine);
		client->getHubIdentity().getParams(params, "hub", false);
		params["hubURL"] = client->getHubUrl();
		client->getMyIdentity().getParams(params, "my", true);
		LOG(LogManager::CHAT, params);
	}
	
	ctrlClient.AppendText(i, Text::toT(client->getMyNick()), timeStamps ? Text::toT("[" + Util::getShortTimeString() + "] ") : _T(""), 
		(BOOLSETTING(IP_IN_CHAT) && client->isOp()) ? Text::toT(TO_UTF8(i.getIp())): Util::emptyStringT, aLine, cf, bUseEmo);

	if (BOOLSETTING(BOLD_HUB)) {
		setDirty();
	}
}

void HubFrame::onTabContextMenu(LPARAM lParam) {
	tabMenuShown = true;
	ipMenuShown = false;

	prepareMenu(tabMenu, ::UserCommand::CONTEXT_HUB, client->getHubUrl(), client->isOp());

    bool delseparator = false;

	if(!(::GetMenuState(tabMenu, ::GetMenuItemCount(tabMenu)-1, MF_BYPOSITION) & MF_SEPARATOR)) {	
		::AppendMenu(tabMenu, MF_SEPARATOR, 0, NULL);
		delseparator = true;
	}

    ::AppendMenu(tabMenu, MF_STRING, IDC_CLOSE_WINDOW, CTSTRING(CLOSE));

    ::TrackPopupMenuEx(tabMenu, TPM_LEFTALIGN | (BOOLSETTING(TABS_ON_TOP) == false ? TPM_BOTTOMALIGN : TPM_TOPALIGN) | TPM_RIGHTBUTTON,
        GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam), m_hWnd, NULL);

	if(delseparator == true) {
        ::RemoveMenu(tabMenu, ::GetMenuItemCount(tabMenu)-1, MF_BYPOSITION);
	}

	::RemoveMenu(tabMenu, ::GetMenuItemCount(tabMenu)-1, MF_BYPOSITION);

	cleanMenu(tabMenu);
}

LRESULT HubFrame::onCtlColor(WPARAM wParam) {
	HDC hDC = (HDC)wParam;
	::SetBkColor(hDC, WinUtil::bgColor);
	::SetTextColor(hDC, WinUtil::textColor);

	return (LRESULT)WinUtil::bgBrush;
}

bool HubFrame::onContextMenu(WPARAM wParam, LPARAM lParam) {
	tabMenuShown = false;
	ipMenuShown = false;

	ChatCtrl::sSelectedUser = Util::emptyStringT;

    int iX = GET_X_LPARAM(lParam),
        iY = GET_Y_LPARAM(lParam);

    {
        RECT rc;
        ::GetWindowRect((HWND)::SendMessage(ctrlUsers.m_hWnd, LVM_GETHEADER, 0, 0), &rc);

        POINT pt = { iX, iY };

        if(ShowUserList == true && ::PtInRect(&rc, pt)) {
            ctrlUsers.showMenu(pt);
            return true;
        }
    }

	if((HWND)wParam == ctrlUsers.m_hWnd && ShowUserList == true) {
        UINT uiSelectedCount = (UINT)::SendMessage(ctrlUsers.m_hWnd, LVM_GETSELECTEDCOUNT, 0, 0);

        if(uiSelectedCount == 0) {
            return false;
        }

		if(iX == -1 && iY == -1) {
			ListViewGetContextMenuPos(ctrlUsers.m_hWnd, iX, iY);
		}

		if(uiSelectedCount == 1) {
			int i = (int)::SendMessage(ctrlUsers.m_hWnd, LVM_GETNEXTITEM, (WPARAM)-1, LVNI_SELECTED);
			if(i != -1) {
				ChatCtrl::sSelectedUser = Text::toT(((UserInfo*)ctrlUsers.getItemData(i))->getNick());
			}
		}

        HMENU Mnu = ::CreatePopupMenu();

		PreparePopupMenu(ctrlUsers.m_hWnd, ChatCtrl::sSelectedUser, Mnu);
		prepareMenu(Mnu, ::UserCommand::CONTEXT_CHAT, client->getHubUrl(), client->isOp());
		::TrackPopupMenuEx(Mnu, TPM_LEFTALIGN | TPM_RIGHTBUTTON, iX, iY, m_hWnd, NULL);

		::DestroyMenu(Mnu);

		return true;
	}

	if((HWND)wParam == ctrlClient.m_hWnd) {
		if(iX == -1 && iY == -1) {
			ctrlClient.GetContextMenuPos(iX, iY);
		}

		POINT pt = { iX, iY };
		::ScreenToClient(ctrlClient.m_hWnd, &pt);
		ctrlClient.OnRButtonDown(pt);

		bool bHitURL = ctrlClient.HitURL();
		if(bHitURL == false) {
			sSelectedURL = _T("");
        }

        HMENU Mnu = ::CreatePopupMenu();

		PreparePopupMenu(ctrlClient.m_hWnd, ChatCtrl::sSelectedUser, Mnu);

		if(ChatCtrl::sSelectedUser.empty() == true) {
            ::TrackPopupMenuEx(Mnu, TPM_LEFTALIGN | TPM_RIGHTBUTTON, iX, iY, m_hWnd, NULL);
		} else {
			prepareMenu(Mnu, ::UserCommand::CONTEXT_CHAT, client->getHubUrl(), client->isOp());
			if(!(::GetMenuState(Mnu, ::GetMenuItemCount(Mnu)-1, MF_BYPOSITION) & MF_SEPARATOR)) {
                ::AppendMenu(Mnu, MF_SEPARATOR, 0, NULL);	
			}
			::AppendMenu(Mnu, MF_STRING, ID_EDIT_CLEAR_ALL, CTSTRING(CLEAR));
			::TrackPopupMenuEx(Mnu, TPM_LEFTALIGN | TPM_RIGHTBUTTON, iX, iY, m_hWnd, NULL);
		}

		::DestroyMenu(Mnu);

		return true;
	}

	return false;
}

void HubFrame::runUserCommand(::UserCommand& uc) {
	if(!WinUtil::getUCParams(m_hWnd, uc, ucLineParams))
		return;

	StringMap ucParams = ucLineParams;

	client->getMyIdentity().getParams(ucParams, "my", true);
	client->getHubIdentity().getParams(ucParams, "hub", false);

	if(tabMenuShown == true) {
		client->escapeParams(ucParams);
		if(BOOLSETTING(USER_COMMANDS)) {
			addClientLine(_T("User command: ") + Text::toT(Util::formatParams(uc.getCommand(), ucParams, false)), 
				WinUtil::m_ChatTextSystem, true);
		}

		client->sendUserCmd(Util::formatParams(uc.getCommand(), ucParams, false));
	} else if(ipMenuShown == true) {
		if(ChatCtrl::sSelectedIP.empty() == false) {
			StringMap tmp = ucParams;
			tmp["userI4"] = Text::fromT(ChatCtrl::sSelectedIP);
			client->escapeParams(tmp);

			if(BOOLSETTING(USER_COMMANDS)) {
				addClientLine(_T("User command: ") + Text::toT(Util::formatParams(uc.getCommand(), tmp, false)), 
					WinUtil::m_ChatTextSystem, true);
			}

            client->sendUserCmd(Util::formatParams(uc.getCommand(), tmp, false)); 
        }
    } else {
		if(ChatCtrl::sSelectedUser.empty() == false) {
			int sel = ctrlUsers.findItem(ChatCtrl::sSelectedUser);
			if(sel != -1) { 
				UserInfo* u = ctrlUsers.getItemData(sel);
				if(u->user->isOnline()) {
					StringMap tmp = ucParams;
					u->getIdentity().getParams(tmp, "user", true);
					client->escapeParams(tmp); 
					if(BOOLSETTING(USER_COMMANDS)) {
						addClientLine(_T("User command: ") + Text::toT(Util::formatParams(uc.getCommand(), tmp, false)), 
							WinUtil::m_ChatTextSystem, true);
					}

					client->sendUserCmd(Util::formatParams(uc.getCommand(), tmp, false));
				}

				return;
			}

			OnlineUser* ou = client->findUser(Text::fromT(ChatCtrl::sSelectedUser));				
    		if(ou != NULL) {
				if(ou->getUser()->isOnline()) {
					StringMap tmp = ucParams;
					ou->getIdentity().getParams(tmp, "user", true);
					client->escapeParams(tmp); 
					if(BOOLSETTING(USER_COMMANDS)) {
						addClientLine(_T("User command: ") + Text::toT(Util::formatParams(uc.getCommand(), tmp, false)), 
							WinUtil::m_ChatTextSystem, true);
					}

					client->sendUserCmd(Util::formatParams(uc.getCommand(), tmp, false));
				}
            }
		} else {
			int sel = -1;
			while((sel = (int)::SendMessage(ctrlUsers.m_hWnd, LVM_GETNEXTITEM, sel, LVNI_SELECTED)) != -1) {
				UserInfo* u = ctrlUsers.getItemData(sel);
				if(u->user->isOnline()) {
					StringMap tmp = ucParams;
					u->getIdentity().getParams(tmp, "user", true);
					client->escapeParams(tmp);
					if(BOOLSETTING(USER_COMMANDS)) {
						addClientLine(_T("User command: ") + Text::toT(Util::formatParams(uc.getCommand(), tmp, false)), 
							WinUtil::m_ChatTextSystem, true);
					}

					client->sendUserCmd(Util::formatParams(uc.getCommand(), tmp, false));
				}
			}
		}
	}

	return;
}

void HubFrame::onTab() {
	HWND focus = GetFocus();
	if((focus == hWndMessage) && !WinUtil::isShift()) {
		AutoArray<TCHAR> buf(::GetWindowTextLength(hWndMessage)+1);
		::GetWindowText(hWndMessage, buf, ::GetWindowTextLength(hWndMessage)+1);
		tstring text(buf, ::GetWindowTextLength(hWndMessage));
		string::size_type textStart = text.find_last_of(_T(" \n\t"));

		if(complete.empty()) {
			if(textStart != string::npos) {
				complete = text.substr(textStart + 1);
			} else {
				complete = text;
			}
			if(complete.empty()) {
				// Still empty, no text entered...
				::SetFocus(ctrlUsers.m_hWnd);
				return;
			}
			int y = (int)::SendMessage(ctrlUsers.m_hWnd, LVM_GETITEMCOUNT, 0, 0);

			for(int x = 0; x < y; ++x) {
				ListView_SetItemState(ctrlUsers.m_hWnd, x, 0, LVNI_FOCUSED | LVNI_SELECTED);
			}
		}

		if(textStart == string::npos)
			textStart = 0;
		else
			textStart++;

		if(inTabComplete) {
			// Already pressed tab once. Output nick candidate list.
			tstring nicks;
			for(StringList::const_iterator i = client->tabCompleteNicks.begin(); i < client->tabCompleteNicks.end(); i+=2)
				nicks.append(Text::toT(*i + " "));

			addClientLine(nicks, WinUtil::m_ChatTextSystem, true);
			inTabComplete = false;
		} else {
			// First tab. Maximally extend proposed nick.
			tstring nick = Text::toT(client->scanNickPrefix(Text::fromT(complete)));
			if(client->tabCompleteNicks.empty())
                return;

			// Maybe it found a unique match. If userlist showing, highlight.
			if(ShowUserList && client->tabCompleteNicks.size() == 2) {
				int i = ctrlUsers.findItem(Text::toT(client->tabCompleteNicks[1]));
				ListView_SetItemState(ctrlUsers.m_hWnd, i, LVNI_FOCUSED | LVNI_SELECTED, LVNI_FOCUSED | LVNI_SELECTED);
				::SendMessage(ctrlUsers.m_hWnd, LVM_ENSUREVISIBLE, i, FALSE);
			}

			::SendMessage(hWndMessage, EM_SETSEL, (int)textStart, ::GetWindowTextLength(hWndMessage));
			// no shift, use partial nick when appropriate
			if(GetAsyncKeyState(VK_SHIFT) & 0x8000) {
				::SendMessage(hWndMessage, EM_REPLACESEL, (WPARAM)FALSE, (LPARAM)nick.c_str());
			} else {
                ::SendMessage(hWndMessage, EM_REPLACESEL, (WPARAM)FALSE, (LPARAM)Text::toT(client->stripNick(Text::fromT(nick))).c_str());
			}

			inTabComplete = true;
		}
	}

	if(focus == ctrlClient.m_hWnd) {
		::SetFocus(hWndMessage);
	} else if(focus == ctrlUsers.m_hWnd) {
		::SetFocus(hWndFilter);
	} else if(focus == hWndFilter) {
		::SetFocus(hWndFilterSel);
	} else if(focus == hWndFilterSel) {
		::SetFocus(ctrlClient.m_hWnd);
	}
}

void HubFrame::onFileReconnect() {
	if(client->isConnected() == false || BOOLSETTING(CONFIRM_HUB_RECONNECT) == false ||
		::MessageBox(m_hWnd, CTSTRING(CONFIRM_HUBRECONNECT), _T(CzDC_VERSION_STRING), MB_YESNO | MB_ICONQUESTION | MB_DEFBUTTON2) == IDYES) {
	    client->reconnect();
	}
}

bool HubFrame::onChar(UINT uMsg, WPARAM wParam) {
	if(GetFocus() != hWndMessage) { // don't handle these keys unless the user is entering a message
		if(uMsg != WM_KEYDOWN) {
			if(wParam == VK_TAB) {
				return true;
			}
		}

		if(wParam == VK_TAB) {
			onTab();
			return true;
		}

		return false;
	}

	if(!complete.empty() && wParam != VK_TAB && uMsg == WM_KEYDOWN) {
		complete.clear();
		inTabComplete = false;
	}

	if(uMsg != WM_KEYDOWN) {
		switch(wParam) {
			case VK_RETURN:
				if( (GetKeyState(VK_CONTROL) & 0x8000) || (GetKeyState(VK_MENU) & 0x8000) ) {
                    return false;
				}

				break;
			case VK_TAB:
				return true;
  			default:
                return false;
		}

        return true;
	}

	switch(wParam) {
		case VK_TAB:
			onTab();
			return true;
		case VK_RETURN:
			if( (GetKeyState(VK_CONTROL) & 0x8000) ||
				(GetKeyState(VK_MENU) & 0x8000) ) {
				return false;
			} else {
				onEnter();
			}

			break;
		case VK_UP:
			if ( (GetKeyState(VK_MENU) & 0x8000) ||	( ((GetKeyState(VK_CONTROL) & 0x8000) == 0) ^ (BOOLSETTING(SETTINGS_USE_CTRL_FOR_LINE_HISTORY) == true) ) ) {
				//scroll up in chat command history
				//currently beyond the last command?
				if (curCommandPosition > 0) {
					//check whether current command needs to be saved
					if (curCommandPosition == prevCommands.size()) {
						auto_ptr<TCHAR> messageContents(new TCHAR[::GetWindowTextLength(hWndMessage)+2]);
						::GetWindowText(hWndMessage, messageContents.get(), ::GetWindowTextLength(hWndMessage)+1);
						currentCommand = tstring(messageContents.get());
					}

					//replace current chat buffer with current command
					::SetWindowText(hWndMessage, prevCommands[--curCommandPosition].c_str());
				}

				// move cursor to end of line
                ::SendMessage(hWndMessage, EM_SETSEL, ::GetWindowTextLength(hWndMessage), ::GetWindowTextLength(hWndMessage));
    			::SendMessage(hWndMessage, EM_SCROLLCARET, 0, 0);
			} else {
				return false;
			}

			break;
		case VK_DOWN:
			if ( (GetKeyState(VK_MENU) & 0x8000) ||	( ((GetKeyState(VK_CONTROL) & 0x8000) == 0) ^ (BOOLSETTING(SETTINGS_USE_CTRL_FOR_LINE_HISTORY) == true) ) ) {
				//scroll down in chat command history

				//currently beyond the last command?
				if (curCommandPosition + 1 < prevCommands.size()) {
					//replace current chat buffer with current command
					::SetWindowText(hWndMessage, prevCommands[++curCommandPosition].c_str());
				} else if (curCommandPosition + 1 == prevCommands.size()) {
					//revert to last saved, unfinished command

					::SetWindowText(hWndMessage, currentCommand.c_str());
					++curCommandPosition;
				}
				// move cursor to end of line
                ::SendMessage(hWndMessage, EM_SETSEL, ::GetWindowTextLength(hWndMessage), ::GetWindowTextLength(hWndMessage));
    			::SendMessage(hWndMessage, EM_SCROLLCARET, 0, 0);
			} else {
				return false;
			}

			break;
		case VK_PRIOR: // page up
			::SendMessage(ctrlClient.m_hWnd, WM_VSCROLL, SB_PAGEUP, NULL);
			break;
		case VK_NEXT: // page down
			::SendMessage(ctrlClient.m_hWnd, WM_VSCROLL, SB_PAGEDOWN, NULL);
			break;
		case VK_HOME:
			if (!prevCommands.empty() && (GetKeyState(VK_CONTROL) & 0x8000) || (GetKeyState(VK_MENU) & 0x8000)) {
				curCommandPosition = 0;

				auto_ptr<TCHAR> messageContents(new TCHAR[::GetWindowTextLength(hWndMessage)+2]);
				::GetWindowText(hWndMessage, messageContents.get(), ::GetWindowTextLength(hWndMessage)+1);
				currentCommand = tstring(messageContents.get());

				::SetWindowText(hWndMessage, prevCommands[curCommandPosition].c_str());
			} else {
				return false;
			}

			break;
		case VK_END:
			if ((GetKeyState(VK_CONTROL) & 0x8000) || (GetKeyState(VK_MENU) & 0x8000)) {
				curCommandPosition = prevCommands.size();

				::SetWindowText(hWndMessage, currentCommand.c_str());
			} else {
				return false;
			}

			break;
		default:
			return false;
	}

	return true;
}

void HubFrame::onShowUsers(WPARAM wParam) {
	if((wParam == BST_CHECKED)) {
		ShowUserList = true;
		client->GetUserList();

        bSplitterDisabled = false;
        ::ShowWindow(ctrlUsers.m_hWnd, SW_SHOW);
        ::EnableWindow(ctrlUsers.m_hWnd, TRUE);
        ::ShowWindow(hWndFilter, SW_SHOW);
        ::EnableWindow(hWndFilter, TRUE);
        ::ShowWindow(hWndFilterSel, SW_SHOW);
        ::EnableWindow(hWndFilterSel, TRUE);
	} else {
		ShowUserList = false;
		::SendMessage(ctrlUsers.m_hWnd, LVM_DELETEALLITEMS, 0, 0);
        for(UserMapIter i = userMap.begin(); i != userMap.end(); ++i) {
            delete i->second;
            i->second = NULL;
        }

        bSplitterDisabled = true;
        ::ShowWindow(ctrlUsers.m_hWnd, SW_HIDE);
        ::EnableWindow(ctrlUsers.m_hWnd, FALSE);
        ::ShowWindow(hWndFilter, SW_HIDE);
        ::EnableWindow(hWndFilter, FALSE);
        ::ShowWindow(hWndFilterSel, SW_HIDE);
        ::EnableWindow(hWndFilterSel, FALSE);
	}

    UpdateSplitterParts();
}

void HubFrame::onFollow() {
	if(!redirect.empty()) {
		if(ClientManager::getInstance()->isConnected(Text::fromT(redirect))) {
			addClientLine(TSTRING(REDIRECT_ALREADY_CONNECTED), WinUtil::m_ChatTextServer, true);
			return;
		}

		dcassert(frames.find(server) != frames.end());
		dcassert(frames[server] == this);
		frames.erase(server);
		server = redirect;
		frames[server] = this;

		// the client is dead, long live the client!
		client->removeListener(this);
		ClientManager::getInstance()->putClient(client);
		clearUserList();
		clearTaskList();
		client = ClientManager::getInstance()->getClient(Text::fromT(server));
		ctrlClient.SetClient(client);

		RecentHubEntry r;
		r.setName("*");
		r.setDescription("***");
		r.setUsers("*");
		r.setShared("*");
		r.setServer(Text::fromT(redirect));
		FavoriteManager::getInstance()->addRecent(r);

		client->addListener(this);
		client->connect();
	}
}

void HubFrame::onEnterUsers() {
	int item = (int)::SendMessage(ctrlUsers.m_hWnd, LVM_GETNEXTITEM, (WPARAM)-1, LVNI_FOCUSED);
	if(item == -1) {
        return;
    }

	try {
		QueueManager::getInstance()->addList((ctrlUsers.getItemData(item))->user, QueueItem::FLAG_CLIENT_VIEW);
	} catch(const Exception& e) {
		addClientLine(Text::toT(e.getError()), WinUtil::m_ChatTextSystem, true);
	}
}

void HubFrame::onGetToolTip(LPNMHDR pnmh) {
	NMTTDISPINFO* nm = (NMTTDISPINFO*)pnmh;
	lastLines.clear();

	for(TStringIter i = lastLinesList.begin(); i != lastLinesList.end(); ++i) {
		lastLines += *i;
		lastLines += _T("\r\n");
	}

	if(lastLines.size() > 2) {
		lastLines.erase(lastLines.size() - 2);
	}

	nm->lpszText = (LPTSTR)lastLines.c_str();
}


void HubFrame::resortUsers() {
	for(FrameIter i = frames.begin(); i != frames.end(); ++i)
		i->second->resortForFavsFirst(true);
}

void HubFrame::closeDisconnected() {
	for(FrameIter i=frames.begin(); i!= frames.end(); ++i) {
		if (!(i->second->client->isConnected())) {
			::PostMessage(i->second->m_hWnd, WM_CLOSE, 0, 0);
		}
	}
}

void HubFrame::on(FavoriteManagerListener::UserAdded, const FavoriteUser& /*aUser*/) throw() {
	resortForFavsFirst();
}
void HubFrame::on(FavoriteManagerListener::UserRemoved, const FavoriteUser& /*aUser*/) throw() {
	resortForFavsFirst();
}

void HubFrame::resortForFavsFirst(bool justDoIt /* = false */) {
	if(justDoIt || BOOLSETTING(SORT_FAVUSERS_FIRST)) {
		speak(RESORT_USERLIST);
	}
}

void HubFrame::on(Second, uint64_t /*aTick*/) throw() {
	if(updateUsers) {
		updateUsers = false;
		if(m_hWnd != NULL) {
            speak(STATS);
        } else {
			::PostMessage(m_hWnd, WM_SPEAKER, 0, 0);
        }
	} else if(m_hWnd != NULL) {
        speak(STATS);
    }
}

void HubFrame::on(Connecting, Client*) throw() {
	if(BOOLSETTING(SEARCH_PASSIVE) && (SETTING(CONNECTION_TYPE) == SettingsManager::CONNECTION_ACTIVE)) {
		addSystemLine(TSTRING(ANTI_PASSIVE_SEARCH), WinUtil::m_ChatTextSystem);
	}
	speak(ADD_STATUS_LINE, STRING(CONNECTING_TO) + client->getHubUrl() + "...");
	speak(SET_WINDOW_TITLE, client->getHubUrl());
}
void HubFrame::on(Connected, Client*) throw() {
	speak(CONNECTED);
}
void HubFrame::on(UserUpdated, Client*, const OnlineUser& user) throw() {
	speak(UPDATE_USER_JOIN, user);
}
void HubFrame::on(UsersUpdated, Client*, const OnlineUser::List& aList) throw() {
	for(OnlineUser::List::const_iterator i = aList.begin(); i != aList.end(); ++i) {
		tasks.add(UPDATE_USER, new UserTask(*(*i)));
	}
	updateUsers = true;
}

void HubFrame::on(ClientListener::UserRemoved, Client*, const OnlineUser& user) throw() {
	speak(REMOVE_USER, user);
}

void HubFrame::on(Redirect, Client*, const string& line) throw() {
	if(ClientManager::getInstance()->isConnected(line)) {
		speak(ADD_STATUS_LINE, STRING(REDIRECT_ALREADY_CONNECTED));
		return;
	}

	redirect = Text::toT(line);
	if(BOOLSETTING(AUTO_FOLLOW)) {
		::PostMessage(m_hWnd, WM_COMMAND, IDC_FOLLOW, 0);
	} else {
		speak(ADD_STATUS_LINE, STRING(PRESS_FOLLOW) + line);
	}
}
void HubFrame::on(Failed, Client*, const string& line) throw() {
	speak(ADD_STATUS_LINE, line);
	speak(DISCONNECTED);
}
void HubFrame::on(GetPassword, Client*) throw() {
	speak(GET_PASSWORD);
}
void HubFrame::on(HubUpdated, Client*) throw() {
	string hubName = client->getHubName();
	if(!client->getHubDescription().empty()) {
		hubName += " - " + client->getHubDescription();
	}
	hubName += " (" + client->getHubUrl() + ")";
	speak(SET_WINDOW_TITLE, hubName);
}
void HubFrame::on(Message, Client*, const OnlineUser& u, const string& line) throw() {
    char *chatline = (char *)line.c_str();
    if(&u == NULL || u.getIdentity().isOp()) {
        char *temp;
        if((temp = strchr(chatline, '\n')) != NULL) {
            temp[0] = NULL;
            if((strstr(chatline, "is kicking") != NULL) && (strstr(chatline, "because:") != NULL) ||
    		  (strstr(chatline, "was kicked by") != NULL)) {
                temp[0] = '\n';
				speak(KICK_MSG, u, Util::toDOS(line));
                return;
            }
            temp[0] = '\n';
    	} else if((strstr(chatline, "is kicking") != NULL) && (strstr(chatline, "because:") != NULL) ||
    		(strstr(chatline, "was kicked by") != NULL)) {
            speak(KICK_MSG, u, Util::toDOS(line));
            return;
    	}
    }

	speak(ADD_CHAT_LINE, u, Util::toDOS(line));
}

void HubFrame::on(PrivateMessage, Client*, const OnlineUser& user, const OnlineUser& by, const string& line) throw() {
	speak(PRIVATE_MESSAGE, user, by, line);
}

void HubFrame::on(PrivateMessageNick, Client*, const string& nick, const OnlineUser& by, const string& line) throw() {
	speak(PRIVATE_MESSAGE_NICK, nick, by, line);
}

void HubFrame::on(NickTaken, Client*) throw() {
	speak(ADD_STATUS_LINE, STRING(NICK_TAKEN));
}
void HubFrame::on(SearchFlood, Client*, const string& line) throw() {
	speak(ADD_STATUS_LINE, STRING(SEARCH_SPAM_FROM) + line);
}

void HubFrame::addClientLine(const tstring& aLine, const CHARFORMAT2& cf, const bool &inChat, bool bNoBold/* = false*/) {
	tstring line = _T("[") + Text::toT(Util::getShortTimeString()) + _T("] ") + aLine;

    if(line.size() > 512) {
        line[512] = NULL;
    }

    ::SendMessage(m_hWndStatusBar, SB_SETTEXT, (0 | 0), (LPARAM)line.c_str());
	while(lastLinesList.size() + 1 > MAX_CLIENT_LINES)
		lastLinesList.erase(lastLinesList.begin());

	lastLinesList.push_back(line.c_str());

	if(BOOLSETTING(BOLD_HUB) == true && bNoBold == false) {
		setDirty();
	}

	if(BOOLSETTING(STATUS_IN_CHAT) && inChat) {
		ctrlClient.AppendStatusText(timeStamps ? Text::toT("[" + Util::getShortTimeString() + "] ") : _T(""), _T("*** ") + aLine, (CHARFORMAT2&)cf);
	}

	if(BOOLSETTING(LOG_STATUS_MESSAGES)) {
		StringMap params;
		client->getHubIdentity().getParams(params, "hub", false);
		params["hubURL"] = client->getHubUrl();
		client->getMyIdentity().getParams(params, "my", true);
		params["message"] = Text::fromT(aLine);
		LOG(LogManager::STATUS, params);
	}
}

void HubFrame::addSystemLine(const tstring& aLine, const CHARFORMAT2& cf) {
    ctrlClient.AppendStatusText(timeStamps ? Text::toT("[" + Util::getShortTimeString() + "] ") : _T(""), aLine, (CHARFORMAT2&)cf);
}

void HubFrame::PreparePopupMenu(HWND hWndCtrl, tstring& sNick, HMENU pMenu) {
	bool bIsChat = (hWndCtrl == ctrlClient.m_hWnd);
	if(sNick.empty() && bIsChat == true) {
		if(ChatCtrl::sSelectedIP.empty() == false) {
			ipMenuShown = true;
			::AppendMenu(pMenu, MF_STRING, IDC_WHOIS_IP, (CTSTRING(WHO_IS) + ChatCtrl::sSelectedIP).c_str());
			::SetMenuDefaultItem(pMenu, IDC_WHOIS_IP, 0);

			prepareMenu(pMenu, ::UserCommand::CONTEXT_IP, client->getHubUrl(), client->isOp());
			if((::GetMenuState(pMenu, ::GetMenuItemCount(pMenu)-1, MF_BYPOSITION) & MF_SEPARATOR) == false) {	
				::AppendMenu(pMenu, MF_SEPARATOR, 0, NULL);
			}
		}
		long lSelBegin, lSelEnd; ctrlClient.GetSel(lSelBegin, lSelEnd);
        if(lSelEnd > lSelBegin) {
            WinUtil::AddSearchSubMenu(pMenu);
            ::AppendMenu(pMenu, MF_SEPARATOR, 0, NULL);
        }
        ::AppendMenu(pMenu, MF_STRING, ID_EDIT_COPY, CTSTRING(COPY));
		::AppendMenu(pMenu, MF_STRING, IDC_COPY_ACTUAL_LINE, CTSTRING(COPY_LINE));
		if(sSelectedURL.empty() == false) {
            ::AppendMenu(pMenu, MF_STRING, IDC_COPY_URL, CTSTRING(COPY_URL));
        }
		::AppendMenu(pMenu, MF_SEPARATOR, 0, NULL);
		::AppendMenu(pMenu, MF_STRING, ID_EDIT_SELECT_ALL, CTSTRING(SELECT_ALL));
		::AppendMenu(pMenu, MF_STRING, ID_EDIT_CLEAR_ALL, CTSTRING(CLEAR));
		::AppendMenu(pMenu, MF_SEPARATOR, 0, NULL);
		::AppendMenu(pMenu, MF_STRING, IDC_AUTOSCROLL_CHAT, CTSTRING(ASCROLL_CHAT));

		if(ctrlClient.GetAutoScroll()) {
            ::CheckMenuItem(pMenu, IDC_AUTOSCROLL_CHAT, MF_BYCOMMAND | MF_CHECKED);
        }
	} else {
		if(sNick.empty() == false) {
		    bool bIsMe = (sNick == Text::toT(client->getMyNick()));
			// Jediny nick
			if(bIsChat) {
				tstring sTmp = TSTRING(USER) + _T(" ") + sNick;
				if(!BOOLSETTING(LOG_PRIVATE_CHAT)) {
                    ::AppendMenu(pMenu, MF_STRING | MF_DISABLED, 0, sTmp.c_str());
				} else {
                    ::AppendMenu(pMenu, MF_STRING, IDC_OPEN_USER_LOG, sTmp.c_str());
				}
				::AppendMenu(pMenu, MF_SEPARATOR, 0, NULL);
			}
			if(bIsChat == false && bIsMe == false && BOOLSETTING(LOG_PRIVATE_CHAT) == true) {
                ::AppendMenu(pMenu, MF_STRING, IDC_OPEN_USER_LOG, CTSTRING(OPEN_USER_LOG));
				::AppendMenu(pMenu, MF_SEPARATOR, 0, NULL);
			}
			if(bIsMe == false) {
                ::AppendMenu(pMenu, MF_STRING, IDC_PUBLIC_MESSAGE, CTSTRING(SEND_PUBLIC_MESSAGE));
				::AppendMenu(pMenu, MF_STRING, IDC_PRIVATEMESSAGE, CTSTRING(SEND_PRIVATE_MESSAGE));
			}
			if(bIsChat == true && ShowUserList == true) {
                ::AppendMenu(pMenu, MF_STRING, IDC_SELECT_USER, CTSTRING(SELECT_USER_LIST));
			}
			if(bIsMe == false) {
			   ::AppendMenu(pMenu, MF_SEPARATOR, 0, NULL);
			}

            HMENU copyMenu = ::CreatePopupMenu();
            ::AppendMenu(copyMenu, MF_STRING, IDC_COPY_NICK, CTSTRING(COPY_NICK));
            ::AppendMenu(copyMenu, MF_STRING, IDC_COPY_EXACT_SHARE, CTSTRING(COPY_EXACT_SHARE));
            ::AppendMenu(copyMenu, MF_STRING, IDC_COPY_DESCRIPTION, CTSTRING(COPY_DESCRIPTION));
            ::AppendMenu(copyMenu, MF_STRING, IDC_COPY_TAG, CTSTRING(COPY_TAG));
            ::AppendMenu(copyMenu, MF_STRING, IDC_COPY_EMAIL_ADDRESS, CTSTRING(COPY_EMAIL_ADDRESS));

        	if(client->isOp() == true) {
                ::AppendMenu(copyMenu, MF_STRING, IDC_COPY_IP, CTSTRING(COPY_IP));
        	}

            ::AppendMenu(pMenu, MF_POPUP, (UINT_PTR)copyMenu, CTSTRING(COPY));

			if(bIsMe == false) {
                WinUtil::AddGrantSubMenu(pMenu);
				::AppendMenu(pMenu, MF_SEPARATOR, 0, NULL);

				if(FavoriteManager::getInstance()->isIgnoredUser(ClientManager::getInstance()->getUser(Text::fromT(sNick), client->getHubUrl())) == false) {
                    ::AppendMenu(pMenu, MF_STRING, IDC_IGNORE, CTSTRING(IGNORE_USER));
				} else {
                    ::AppendMenu(pMenu, MF_STRING, IDC_UNIGNORE, CTSTRING(UNIGNORE_USER));
				}

				::AppendMenu(pMenu, MF_SEPARATOR, 0, NULL);
				::AppendMenu(pMenu, MF_STRING, IDC_GETLIST, CTSTRING(GET_FILE_LIST));
				::AppendMenu(pMenu, MF_STRING, IDC_MATCH_QUEUE, CTSTRING(MATCH_QUEUE));
				::AppendMenu(pMenu, MF_STRING, IDC_ADD_TO_FAVORITES, CTSTRING(ADD_TO_FAVORITES));
			}
		} else {
			// Muze byt vice nicku
			if(bIsChat == false) {
				// Pocet oznacenych
				int iCount = (UINT)::SendMessage(ctrlUsers.m_hWnd, LVM_GETSELECTEDCOUNT, 0, 0);
				TCHAR sTmp[256];
				_stprintf_s(sTmp, 256, _T("%i user(s)"), iCount);
				::AppendMenu(pMenu, MF_STRING | MF_DISABLED, 0, sTmp);
			}
			if((UINT)::SendMessage(ctrlUsers.m_hWnd, LVM_GETSELECTEDCOUNT, 0, 0) <= 25) {
				::AppendMenu(pMenu, MF_SEPARATOR, 0, NULL);
				::AppendMenu(pMenu, MF_STRING, IDC_PUBLIC_MESSAGE, CTSTRING(SEND_PUBLIC_MESSAGE));
			}
			::AppendMenu(pMenu, MF_SEPARATOR, 0, NULL);
			::AppendMenu(pMenu, MF_STRING, IDC_GETLIST, CTSTRING(GET_FILE_LIST));
			::AppendMenu(pMenu, MF_STRING, IDC_MATCH_QUEUE, CTSTRING(MATCH_QUEUE));
			::AppendMenu(pMenu, MF_STRING, IDC_ADD_TO_FAVORITES, CTSTRING(ADD_TO_FAVORITES));
		}
		if(bIsChat) {
			switch(SETTING(CHAT_DBLCLICK)) {
		          case 0:
                        ::SetMenuDefaultItem(pMenu, IDC_SELECT_USER, 0);
						break;
		          case 1:
                        ::SetMenuDefaultItem(pMenu, IDC_PUBLIC_MESSAGE, 0);
						break;
		          case 2:
                        ::SetMenuDefaultItem(pMenu, IDC_PRIVATEMESSAGE, 0);
						break;
		          case 3:
                        ::SetMenuDefaultItem(pMenu, IDC_GETLIST, 0);
						break;
		          case 4:
                        ::SetMenuDefaultItem(pMenu, IDC_MATCH_QUEUE, 0);
						break;
		          case 6:
                        ::SetMenuDefaultItem(pMenu, IDC_ADD_TO_FAVORITES, 0);
						break;
			}
		} else {
			switch(SETTING(USERLIST_DBLCLICK)) {
		          case 0:
						::SetMenuDefaultItem(pMenu, IDC_GETLIST, 0);
						break;
		          case 1:
						::SetMenuDefaultItem(pMenu, IDC_PUBLIC_MESSAGE, 0);
						break;
		          case 2:
						::SetMenuDefaultItem(pMenu, IDC_PRIVATEMESSAGE, 0);
						break;
		          case 3:
						::SetMenuDefaultItem(pMenu, IDC_MATCH_QUEUE, 0);
						break;
		          case 5:
						::SetMenuDefaultItem(pMenu, IDC_ADD_TO_FAVORITES, 0);
						break;
			}
		}
	}
}

void HubFrame::onCopyActualLine() {
	if(GetFocus() == ctrlClient.m_hWnd && ChatCtrl::sSelectedLine.empty() == false) {
		WinUtil::setClipboard(ChatCtrl::sSelectedLine);
	}
}

void HubFrame::onSelectUser() {
	if(ChatCtrl::sSelectedUser.empty() == true) {
		return;
	}

	int pos = ctrlUsers.findItem(ChatCtrl::sSelectedUser);
	if(pos == -1) {
		return;
	}

	int items = (int)::SendMessage(ctrlUsers.m_hWnd, LVM_GETITEMCOUNT, 0, 0);
	::SendMessage(ctrlUsers.m_hWnd, WM_SETREDRAW, (WPARAM)FALSE, 0);
	for(int i = 0; i < items; ++i) {
		ListView_SetItemState(ctrlUsers.m_hWnd, i, (i == pos) ? LVIS_SELECTED | LVIS_FOCUSED : 0, LVIS_SELECTED | LVIS_FOCUSED);
	}
	::SendMessage(ctrlUsers.m_hWnd, WM_SETREDRAW, (WPARAM)TRUE, 0);
	::SendMessage(ctrlUsers.m_hWnd, LVM_ENSUREVISIBLE, pos, FALSE);
}

void HubFrame::onPublicMessage() {
	if(client->isConnected() == false) {
		return;
    }

	tstring sUsers = Util::emptyStringT;

	if(ChatCtrl::sSelectedUser.empty() == false) {
		sUsers = ChatCtrl::sSelectedUser;
	} else {
        int i = -1;
		while((i = (int)::SendMessage(ctrlUsers.m_hWnd, LVM_GETNEXTITEM, i, LVNI_SELECTED)) != -1) {
			if(sUsers.size() > 0) {
  				sUsers += _T(", ");
            }
 
			sUsers += Text::toT(((UserInfo*)ctrlUsers.getItemData(i))->getNick());
		}
	}

	sUsers += _T(": ");

	::SendMessage(hWndMessage, EM_SETSEL, ::GetWindowTextLength(hWndMessage), ::GetWindowTextLength(hWndMessage));
	::SendMessage(hWndMessage, EM_SCROLLCARET, 0, 0);
	::SendMessage(hWndMessage, EM_REPLACESEL, (WPARAM)FALSE, (LPARAM)sUsers.c_str());

	::SetFocus(hWndMessage);
}

void HubFrame::onAutoScrollChat() {
	ctrlClient.SetAutoScroll(!ctrlClient.GetAutoScroll());
}

void HubFrame::onCopyURL() {
	if(sSelectedURL.empty() == false) {
		WinUtil::setClipboard(sSelectedURL);
	}
}

void HubFrame::onClientEnLink(LPNMHDR pnmh) {
	ENLINK* pEL = (ENLINK*)pnmh;

	if(pEL->msg == WM_LBUTTONUP) {
		long lBegin = pEL->chrg.cpMin, lEnd = pEL->chrg.cpMax;

		TCHAR* sURLTemp = new TCHAR[(lEnd - lBegin)+1];
		if(sURLTemp == NULL) {
			return;
		}
    	ctrlClient.GetTextRange(lBegin, lEnd, sURLTemp);
    	tstring sURL(sURLTemp, lEnd - lBegin);
    	WinUtil::openLink(sURL);
		delete[] sURLTemp;
	} else if(pEL->msg == WM_RBUTTONUP) {
		sSelectedURL = _T("");
		long lBegin = pEL->chrg.cpMin, lEnd = pEL->chrg.cpMax;

		TCHAR* sURLTemp = new TCHAR[(lEnd - lBegin)+1];
		if(sURLTemp == NULL) {
			return;
		}
    	ctrlClient.GetTextRange(lBegin, lEnd, sURLTemp);
    	sSelectedURL = sURLTemp;
		delete[] sURLTemp;

		ctrlClient.SetSel(lBegin, lEnd);
		::InvalidateRect(ctrlClient.m_hWnd, NULL, TRUE);
	}
}

void HubFrame::onOpenUserLog() {
	StringMap params;
	UserInfo* ui = NULL;
	if(ChatCtrl::sSelectedUser.empty() == false) {
		int k = ctrlUsers.findItem(ChatCtrl::sSelectedUser);
		if(k != -1) {
            ui = ctrlUsers.getItemData(k);
        }
	} else {
		int i = -1;
		if((i = (int)::SendMessage(ctrlUsers.m_hWnd, LVM_GETNEXTITEM, i, LVNI_SELECTED)) != -1) {
			ui = (UserInfo*)ctrlUsers.getItemData(i);
		}
	}

	if(ui != NULL) {
        params["userNI"] = ui->getNick();
        params["userCID"] = ui->user->getCID().toBase32();
    } else if(ChatCtrl::sSelectedUser.empty() == false) {
        OnlineUser* ou = client->findUser(Text::fromT(ChatCtrl::sSelectedUser));				
    	if(ou != NULL) {
            params["userNI"] = ou->getIdentity().getNick();
            params["userCID"] = ou->getUser()->getCID().toBase32();
        }
    }

	params["hubNI"] = client->getHubName();
	params["myNI"] = client->getMyNick();
	params["hubURL"] = client->getHubUrl();
	tstring file = Text::toT(Util::validateFileName(SETTING(LOG_DIRECTORY) + Util::formatParams(SETTING(LOG_FILE_PRIVATE_CHAT), params, true)));
	if(File::FileExist(Text::fromT(file))) {
		ShellExecute(NULL, NULL, file.c_str(), NULL, NULL, SW_SHOWNORMAL);
	} else {
		::MessageBox(m_hWnd, CTSTRING(NO_LOG_FOR_USER),CTSTRING(NO_LOG_FOR_USER), MB_OK);	  
	}
}

void HubFrame::onOpenHubLog() {
	StringMap params;
	params["hubNI"] = client->getHubName();
	params["hubURL"] = client->getHubUrl();
	params["myNI"] = client->getMyNick(); 
	tstring filename = Text::toT(Util::validateFileName(SETTING(LOG_DIRECTORY) + Util::formatParams(SETTING(LOG_FILE_MAIN_CHAT), params, true)));
	if(File::FileExist(Text::fromT(filename))){
		ShellExecute(NULL, NULL, filename.c_str(), NULL, NULL, SW_SHOWNORMAL);
	} else {
		::MessageBox(m_hWnd, CTSTRING(NO_LOG_FOR_HUB),CTSTRING(NO_LOG_FOR_HUB), MB_OK);	  
	}
}

void HubFrame::onWhoisIP() {
	if(ChatCtrl::sSelectedIP.empty() == false) {
 		WinUtil::openLink(_T("http://www.ripe.net/perl/whois?form_type=simple&full_query_string=&searchtext=") + ChatCtrl::sSelectedIP);
 	}
}

bool HubFrame::onFilterChar(WPARAM wParam) {
	switch(wParam) {
		case VK_RETURN: {
            AutoArray<TCHAR> buf(::GetWindowTextLength(hWndFilter)+1);
			::GetWindowText(hWndFilter, buf, ::GetWindowTextLength(hWndFilter)+1);
			filterstring = Text::fromT(tstring(buf, ::GetWindowTextLength(hWndFilter)));
			filtercolumn = (int)::SendMessage(hWndFilterSel, CB_GETCURSEL, 0, 0);
			updateUserList();

			return false;
        }
		case VK_TAB:
			onTab();

			return true;
		default:
			return false;
	}
}

void HubFrame::updateUserList() {
    ::SendMessage(ctrlUsers.m_hWnd, WM_SETREDRAW, (WPARAM)FALSE, 0);

    ::SendMessage(ctrlUsers.m_hWnd, LVM_DELETEALLITEMS, 0, 0);

    if(filterstring.empty()) {
    	for(UserMap::iterator i = userMap.begin(); i != userMap.end(); ++i){
    		if(i->second != NULL) {
    			i->second->update(i->second->getIdentity(), ctrlUsers.getSortColumn(), client->isOp());
    			ctrlUsers.insertItem(i->second, WinUtil::getImage(i->second->getIdentity()));
    		}
    	}
    } else {
        for(UserMap::iterator i = userMap.begin(); i != userMap.end(); ++i){
        	if(i->second != NULL) {
        		if(filterUser(i->second->getIdentity()) == true) {
        			ctrlUsers.insertItem(i->second, WinUtil::getImage(i->second->getIdentity()));
        		}
        	}
        }
    }
	::SendMessage(ctrlUsers.m_hWnd, WM_SETREDRAW, (WPARAM)TRUE, 0);
} 

bool HubFrame::filterUser(const Identity& id) {
	switch(filtercolumn) {
		case 0:
			//Nick
			if(Util::stristr(id.getNick().c_str(), filterstring.c_str()) != 0) {
				return true;
			}
			break;
		case 1:
			//Shared
			if(Util::stristr(Util::formatBytes(id.getBytesShared()).c_str(), filterstring.c_str()) != 0) {
				return true;
			}
			break;
		case 2:
			//Exact Shared
			if(Util::stristr(Util::toString(id.getBytesShared()).c_str(), filterstring.c_str()) != 0) {
				return true;
			}
			break;
		case 3:
			//Description
			if(Util::stristr((TO_UTF8(id.getDescription())).c_str(), filterstring.c_str()) != 0) {
				return true;
			}
			break;
		case 4:
			//Tag
			if(Util::stristr((TO_UTF8(id.getTag())).c_str(), filterstring.c_str()) != 0) {
				return true;
			}
			break;
		case 5:
			//Connection
			if(Util::stristr((TO_UTF8(id.getConnection())).c_str(), filterstring.c_str()) != 0) {
				return true;
			}
			break;
		case 6:
			//Email
			if(Util::stristr((TO_UTF8(id.getEmail())).c_str(), filterstring.c_str()) != 0) {
				return true;
			}
			break;
		case 7:
			//Client ID
			if(Util::stristr((TO_UTF8(id.getClientID())).c_str(), filterstring.c_str()) != 0) {
				return true;
			}
			break;
		case 8:
			//Version
			if(Util::stristr((TO_UTF8(id.getVersion())).c_str(), filterstring.c_str()) != 0) {
				return true;
			}
			break;
		case 9:
			//Mode
			if(Util::stristr((TO_UTF8(id.getMode())).c_str(), filterstring.c_str()) != 0) {
				return true;
			}
			break;
		case 10:
			//Hubs
			if(Util::stristr((TO_UTF8(id.getHubs())).c_str(), filterstring.c_str()) != 0) {
				return true;
			}
			break;
		case 11:
			//Slots
			if(Util::stristr((TO_UTF8(id.getSlots())).c_str(), filterstring.c_str()) != 0) {
				return true;
			}
			break;
		case 12:
			//Limit
			if(Util::stristr((TO_UTF8(id.getUpload())).c_str(), filterstring.c_str()) != 0) {
				return true;
			}
			break;
		case 13:
			//IP
			if(Util::stristr((TO_UTF8(id.getIp())).c_str(), filterstring.c_str()) != 0) {
				return true;
			}
			break;
		default:
			return false;
			break;
	}
	return false;
}

void HubFrame::on(SettingsManagerListener::Save, SimpleXML& /*xml*/) throw() {
	::PostMessage(m_hWnd, WM_REFRESH_SETTINGS, 0, 0);
}

void HubFrame::onRefreshSettings() {
	if((COLORREF)::SendMessage(ctrlUsers.m_hWnd, LVM_GETBKCOLOR, 0, 0) != WinUtil::bgColor) {
		::SendMessage(ctrlClient.m_hWnd, EM_SETBKGNDCOLOR, 0, WinUtil::bgColor);
		::SendMessage(ctrlUsers.m_hWnd, LVM_SETBKCOLOR, 0, WinUtil::bgColor);
		::SendMessage(ctrlUsers.m_hWnd, LVM_SETTEXTBKCOLOR, 0, WinUtil::bgColor);
	}
	if((COLORREF)::SendMessage(ctrlUsers.m_hWnd, LVM_GETTEXTCOLOR, 0, 0) != WinUtil::textColor) {
		::SendMessage(ctrlUsers.m_hWnd, LVM_SETTEXTCOLOR, 0, WinUtil::textColor);
	}

	::RedrawWindow(m_hWnd, NULL, NULL, RDW_ERASE | RDW_INVALIDATE | RDW_UPDATENOW | RDW_ALLCHILDREN);   
}

LRESULT HubFrame::onCustomDraw(LPNMHDR pnmh, bool& bHandled) {
	LPNMLVCUSTOMDRAW cd = (LPNMLVCUSTOMDRAW)pnmh;

	switch(cd->nmcd.dwDrawStage) {
		case CDDS_PREPAINT:
			return CDRF_NOTIFYITEMDRAW;
		case CDDS_ITEMPREPAINT: {
            if(BOOLSETTING(USERLIST_COLORING) == false) {
        		bHandled = false;
        		return 0;
        	}

		    UserInfo* ui = (UserInfo*)cd->nmcd.lItemlParam; 
			if(FavoriteManager::getInstance()->isFavoriteUser(ui->user)){
				cd->clrText = SETTING(FAVORITE_COLOR);
			} else if(UploadManager::getInstance()->hasReservedSlot(ui->user)) {
				cd->clrText = SETTING(EXTRA_SLOT_COLOR);
			} else if(FavoriteManager::getInstance()->isIgnoredUser(ui->getIdentity().getUser()) == true) {
				cd->clrText = SETTING(IGNORED_COLOR);
			} else if(ui->getIdentity().isSet(Identity::FIREBALL)) {
				cd->clrText = SETTING(FIREBALL_COLOR);
			} else if(ui->getIdentity().isSet(Identity::SERVER)) {
				cd->clrText = SETTING(SERVER_COLOR);
			} else if(ui->getIdentity().isOp()) {
				cd->clrText = SETTING(OP_COLOR);
			} else if(ui->getIdentity().getMode() == "A" && !ui->user->isSet(User::PASSIVE)) {
				cd->clrText = SETTING(ACTIVE_COLOR);
			} else if(ui->getIdentity().getUser()->isSet(User::PASSIVE) || ui->getIdentity().getMode() == "P" || ui->getIdentity().getMode() == "5") {
				cd->clrText = SETTING(PASIVE_COLOR);
			} else {
				cd->clrText = SETTING(NORMAL_COLOR);
			}

			return CDRF_NEWFONT | CDRF_NOTIFYSUBITEMDRAW;
		}
		case CDDS_SUBITEM | CDDS_ITEMPREPAINT: {
            if(ctrlUsers.findColumn(cd->iSubItem) == COLUMN_COUNTRY) {
                RECT rc;
    			UserInfo* ui = (UserInfo*)cd->nmcd.lItemlParam; 
    			if(ui->getIdentity().getIp().empty() == false && BOOLSETTING(GET_USER_COUNTRY) == true) {
					rc.top = cd->iSubItem;
					rc.left = LVIR_BOUNDS;
					::SendMessage(ctrlUsers.m_hWnd, LVM_GETSUBITEMRECT, (int)cd->nmcd.dwItemSpec, (LPARAM)&rc);
    				COLORREF color;
    				if((UINT)::SendMessage(ctrlUsers.m_hWnd, LVM_GETITEMSTATE, (int)cd->nmcd.dwItemSpec, LVIS_SELECTED) & LVIS_SELECTED) {
    					if(ctrlUsers.m_hWnd == ::GetFocus()) {
    						color = GetSysColor(COLOR_HIGHLIGHT);
    						SetBkColor(cd->nmcd.hdc, GetSysColor(COLOR_HIGHLIGHT));
    						SetTextColor(cd->nmcd.hdc, GetSysColor(COLOR_HIGHLIGHTTEXT));
    					} else {
    						color = GetBkColor(cd->nmcd.hdc);
    						SetBkColor(cd->nmcd.hdc, color);
    					}
    				} else {
    					color = WinUtil::bgColor;
    					SetBkColor(cd->nmcd.hdc, WinUtil::bgColor);
    					SetTextColor(cd->nmcd.hdc, cd->clrText);
    				}
    				RECT rc2 = rc;
    				rc2.left += 2;

    				HGDIOBJ oldpen = ::SelectObject(cd->nmcd.hdc, CreatePen(PS_SOLID,0, color));
    				HGDIOBJ oldbr = ::SelectObject(cd->nmcd.hdc, CreateSolidBrush(color));
    				Rectangle(cd->nmcd.hdc,rc.left, rc.top, rc.right, rc.bottom);
    				DeleteObject(::SelectObject(cd->nmcd.hdc, oldpen));
    				DeleteObject(::SelectObject(cd->nmcd.hdc, oldbr));
    				LONG top = rc2.top + ((rc2.bottom - rc2.top) - 15)/2;
    				if((top - rc2.top) < 2) {
    					top = rc2.top + 1;
                    }

    				TCHAR buf[256];
					ListViewGetItemText(ctrlUsers.m_hWnd, (int)cd->nmcd.dwItemSpec, cd->iSubItem, buf, 255);
    				buf[255] = '\0';
    				POINT p = { rc2.left, top };
					ImageList_Draw(CountryFlags::flagImages, CountryFlags::GetFlagImageIndex(Util::getIpCountry(ui->getIdentity().getIp().c_str()).c_str()), 
						cd->nmcd.hdc, p.x, p.y, LVSIL_SMALL);
    				top = rc2.top + ((rc2.bottom - rc2.top) - WinUtil::getTextHeight(cd->nmcd.hdc) - 1)/2;
    				::ExtTextOut(cd->nmcd.hdc, rc2.left + 29, top + 1, ETO_CLIPPED, &rc2, buf, (uint32_t)_tcslen(buf), NULL);
    				return CDRF_SKIPDEFAULT;
    			}
    		}
        }
    	default:
    		return CDRF_DODEFAULT;
	}
}

void HubFrame::onSearch(WORD wID) {
    long lSelBegin, lSelEnd;
    ctrlClient.GetSel(lSelBegin, lSelEnd);

    if(lSelEnd > lSelBegin) { 
        long iLen = lSelEnd - lSelBegin;
        AutoArray<TCHAR> sTemp(iLen+1);
        ctrlClient.GetTextRange(lSelBegin, lSelEnd, sTemp);
        sTemp[iLen] = 0;
        tstring sSearchString(sTemp, iLen);
    	switch (wID) {
    		case IDC_SEARCH_DC:
    			WinUtil::SearchOnDC(sSearchString);
    			break;
    		case IDC_SEARCH_GOOGLE:
    			WinUtil::SearchOnGoogle(sSearchString);
    			break;
    		case IDC_SEARCH_YAHOO:
    			WinUtil::SearchOnYahoo(sSearchString);
    			break;
    		case IDC_SEARCH_IMDB:
    			WinUtil::SearchOnIMDB(sSearchString);
    			break;
    		case IDC_SEARCH_CSFD:
    			WinUtil::SearchOnCSFD(sSearchString);
    			break;
    		default:
    			break;
    	}
    }
}

void HubFrame::onIgnore() {
	if(client->isConnected() == false) {
        return;
    }

	if(ChatCtrl::sSelectedUser.empty() == false) {
		OnlineUser* ou = client->findUser(Text::fromT(ChatCtrl::sSelectedUser));
    	if(ou != NULL) {
            FavoriteManager::getInstance()->addIgnoredUser(ou->getUser());
        }
	} else {
        int i = -1;
		while((i = (int)::SendMessage(ctrlUsers.m_hWnd, LVM_GETNEXTITEM, i, LVNI_SELECTED)) != -1) {
			OnlineUser* ou = client->findUser(((UserInfo*)ctrlUsers.getItemData(i))->getIdentity().getNick());
    		if(ou != NULL) {
				FavoriteManager::getInstance()->addIgnoredUser(ou->getUser());
			}
		}
	}
}

void HubFrame::onUnignore() {
	if(client->isConnected() == false) {
        return;
    }

	if(ChatCtrl::sSelectedUser.empty() == false) {
		OnlineUser* ou = client->findUser(Text::fromT(ChatCtrl::sSelectedUser));
    	if(ou != NULL) {
            FavoriteManager::getInstance()->removeIgnoredUser(ou->getUser());
        }
	} else {
        int i = -1;
		while((i = (int)::SendMessage(ctrlUsers.m_hWnd, LVM_GETNEXTITEM, i, LVNI_SELECTED)) != -1) {
			OnlineUser* ou = client->findUser(((UserInfo*)ctrlUsers.getItemData(i))->getIdentity().getNick());
    		if(ou != NULL) {
				FavoriteManager::getInstance()->removeIgnoredUser(ou->getUser());
			}
		}
	}
}

void HubFrame::onMatchQueue() {
	if(ChatCtrl::sSelectedUser.empty() == false) {
		int i = ctrlUsers.findItem(ChatCtrl::sSelectedUser);
		if(i != -1) {
			ctrlUsers.forEachAtPos(i, &UserInfoBase::matchQueue);
		} else {
            OnlineUser* ou = client->findUser(Text::fromT(ChatCtrl::sSelectedUser));				
    		if(ou != NULL) {
                try {
                	QueueManager::getInstance()->addList(ou->getUser(), QueueItem::FLAG_MATCH_QUEUE);
                } catch(const Exception& e) {
                	LogManager::getInstance()->message(e.getError());
                }
            }
        }
	} else {
		ctrlUsers.forEachSelected(&UserInfoBase::matchQueue);
	}
}

void HubFrame::onGetList() {
	if(ChatCtrl::sSelectedUser.empty() == false) {
		int i = ctrlUsers.findItem(ChatCtrl::sSelectedUser);
		if(i != -1) {
			ctrlUsers.forEachAtPos(i, &UserInfoBase::getList);
		} else {
            OnlineUser* ou = client->findUser(Text::fromT(ChatCtrl::sSelectedUser));				
    		if(ou != NULL) {
                try {
                	QueueManager::getInstance()->addList(ou->getUser(), QueueItem::FLAG_CLIENT_VIEW);
                } catch(const Exception& e) {
                	LogManager::getInstance()->message(e.getError());
                }
            }
        }
	} else {
		ctrlUsers.forEachSelected(&UserInfoBase::getList);
	}
}

void HubFrame::onAddToFavorites() {
	if(ChatCtrl::sSelectedUser.empty() == false) {
		int i = ctrlUsers.findItem(ChatCtrl::sSelectedUser);
		if(i != -1) {
			ctrlUsers.forEachAtPos(i, &UserInfoBase::addFav);
		} else {
            OnlineUser* ou = client->findUser(Text::fromT(ChatCtrl::sSelectedUser));				
    		if(ou != NULL) {
                FavoriteManager::getInstance()->addFavoriteUser(ou->getUser());
            }
        }
	} else {
		ctrlUsers.forEachSelected(&UserInfoBase::addFav);
	}
}

void HubFrame::onPrivateMessage() {
	if(ChatCtrl::sSelectedUser.empty() == false) {
		int i = ctrlUsers.findItem(ChatCtrl::sSelectedUser);
		if(i != -1) {
			ctrlUsers.forEachAtPos(i, &UserInfoBase::pm);
		} else {
            OnlineUser* ou = client->findUser(Text::fromT(ChatCtrl::sSelectedUser));				
    		if(ou != NULL) {
                PrivateFrame::openWindow(ou->getUser());
            }
        }
	} else {
		ctrlUsers.forEachSelected(&UserInfoBase::pm);
	}
}

void HubFrame::onGrantSlot() {
	if(ChatCtrl::sSelectedUser.empty() == false) {
		int i = ctrlUsers.findItem(ChatCtrl::sSelectedUser);
		if(i != -1) {
			ctrlUsers.forEachAtPos(i, &UserInfoBase::grant);
		} else {
            OnlineUser* ou = client->findUser(Text::fromT(ChatCtrl::sSelectedUser));				
    		if(ou != NULL) {
                UploadManager::getInstance()->reserveSlot(ou->getUser());
            }
        }
	} else {
		ctrlUsers.forEachSelected(&UserInfoBase::grant);
	}
}

void HubFrame::onGrantSlotHour() {
	if(ChatCtrl::sSelectedUser.empty() == false) {
		int i = ctrlUsers.findItem(ChatCtrl::sSelectedUser);
		if(i != -1) {
			ctrlUsers.forEachAtPos(i, &UserInfoBase::grantSlotHour);
		} else {
            OnlineUser* ou = client->findUser(Text::fromT(ChatCtrl::sSelectedUser));				
    		if(ou != NULL) {
                UploadManager::getInstance()->reserveSlotHour(ou->getUser());
            }
        }
	} else {
		ctrlUsers.forEachSelected(&UserInfoBase::grantSlotHour);
	}
}

void HubFrame::onGrantSlotDay() {
	if(ChatCtrl::sSelectedUser.empty() == false) {
		int i = ctrlUsers.findItem(ChatCtrl::sSelectedUser);
		if(i != -1) {
			ctrlUsers.forEachAtPos(i, &UserInfoBase::grantSlotDay);
		} else {
            OnlineUser* ou = client->findUser(Text::fromT(ChatCtrl::sSelectedUser));				
    		if(ou != NULL) {
                UploadManager::getInstance()->reserveSlotDay(ou->getUser());
            }
        }
	} else {
		ctrlUsers.forEachSelected(&UserInfoBase::grantSlotDay);
	}
}

void HubFrame::onGrantSlotWeek() {
	if(ChatCtrl::sSelectedUser.empty() == false) {
		int i = ctrlUsers.findItem(ChatCtrl::sSelectedUser);
		if(i != -1) {
			ctrlUsers.forEachAtPos(i, &UserInfoBase::grantSlotWeek);
		} else {
            OnlineUser* ou = client->findUser(Text::fromT(ChatCtrl::sSelectedUser));				
    		if(ou != NULL) {
                UploadManager::getInstance()->reserveSlotWeek(ou->getUser());
            }
        }
	} else {
		ctrlUsers.forEachSelected(&UserInfoBase::grantSlotWeek);
	}
}

void HubFrame::onUnGrantSlot() {
	if(ChatCtrl::sSelectedUser.empty() == false) {
		int i = ctrlUsers.findItem(ChatCtrl::sSelectedUser);
		if(i != -1) {
			ctrlUsers.forEachAtPos(i, &UserInfoBase::ungrantSlot );
		} else {
            OnlineUser* ou = client->findUser(Text::fromT(ChatCtrl::sSelectedUser));				
    		if(ou != NULL) {
                UploadManager::getInstance()->unreserveSlot(ou->getUser());
            }
        }
	} else {
		ctrlUsers.forEachSelected(&UserInfoBase::ungrantSlot);
	}
}

void HubFrame::onSize(WPARAM wParam, LPARAM lParam) {
	if(wParam != SIZE_MINIMIZED) {
        UpdateLayout();

        if(HIWORD(lParam) > 0) {
            ctrlClient.GoToEnd();
        }
	}
}

HWND HubFrame::GetWindowHandle() {
    return m_hWnd;
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void HubFrame::UpdateSplitterParts() {
    int iFontHeight = WinUtil::fontHeight + 4;
    if(bSplitterDisabled == false) {
        ::SetWindowPos(ctrlClient.m_hWnd, NULL, rcSplitter.left, rcSplitter.top, iSplitterPos-2, (rcSplitter.bottom-rcSplitter.top)-((2 * iFontHeight) + 4), SWP_NOZORDER);
        ::SetWindowPos(hWndMessage, NULL, rcSplitter.left, rcSplitter.bottom-(2 * iFontHeight), iSplitterPos-2, 2 * iFontHeight, SWP_NOZORDER);

        ::SetWindowPos(ctrlUsers.m_hWnd, NULL, rcSplitter.left+(iSplitterPos+2), rcSplitter.top, (rcSplitter.right-rcSplitter.left)-(iSplitterPos+2),
            (rcSplitter.bottom-rcSplitter.top)-WinUtil::fontHeight-10, SWP_NOZORDER);
        ::SetWindowPos(hWndFilter, NULL, rcSplitter.left+(iSplitterPos+2), rcSplitter.bottom-iFontHeight-5, (rcSplitter.right-rcSplitter.left)-(iSplitterPos+2)-99,
            iFontHeight+5, SWP_NOZORDER);
        ::SetWindowPos(hWndFilterSel, NULL, rcSplitter.right - 97, rcSplitter.bottom-iFontHeight-5, 95, iFontHeight+5, SWP_NOZORDER);
    } else {
        ::SetWindowPos(ctrlClient.m_hWnd, NULL, rcSplitter.left, rcSplitter.top, rcSplitter.right-rcSplitter.left, (rcSplitter.bottom-rcSplitter.top)-((2 * iFontHeight) + 4), SWP_NOZORDER);
        ::SetWindowPos(hWndMessage, NULL, rcSplitter.left, rcSplitter.bottom-(2 * iFontHeight), rcSplitter.right-rcSplitter.left, 2 * iFontHeight, SWP_NOZORDER);
    }
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

/*
 * Some parts Copyright (C) 2002-2011 Petr Kozelka, PPK at czdc dot org
 * Licensed under GNU General Public License version 3.
 */
