/*
 * Copyright (C) 2001-2011 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef MAIN_FRM_H
#define MAIN_FRM_H

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "../client/TimerManager.h"
#include "../client/HttpConnection.h"
#include "../client/FavoriteManager.h"
#include "../client/QueueManagerListener.h"
#include "../client/Util.h"
#include "../client/LogManager.h"
#include "../client/version.h"
#include "../client/Client.h"
#include "../client/ShareManager.h"

#include "FlatTabCtrl.h"
#include "SingleInstance.h"
#include "TransferView.h"
#include "WinUtil.h"
#include "WinApiUtil.h"
#include "BasicSplitter.h"

class MainFrame : private BasicSplitter, private TimerManagerListener, private HttpConnectionListener, private QueueManagerListener,
    private LogManagerListener, private SettingsManagerListener {
public:
	MainFrame();
	~MainFrame();

	enum {
		DOWNLOAD_LISTING,
		STATS,
		AUTO_CONNECT,
		PARSE_COMMAND_LINE,
		VIEW_FILE_AND_DELETE,
		STATUS_MESSAGE,
		SET_NORMAL_TRAY_ICON,
		SET_PM_TRAY_ICON
	};

	BOOL PreTranslateMessage(MSG* pMsg) {
		if((pMsg->message >= WM_MOUSEFIRST) && (pMsg->message <= WM_MOUSELAST)) {
			::SendMessage(hwndLastLines, TTM_RELAYEVENT, 0, (LPARAM)pMsg);
        }

		if(TranslateMDISysAccel(WinUtil::mdiClient, pMsg) || TranslateAccelerator(m_hWnd, m_hAccel, pMsg)) {
			return TRUE;
        }

		HWND hWnd = (HWND)::SendMessage(WinUtil::mdiClient, WM_MDIGETACTIVE, 0, (LPARAM)NULL);
		if(hWnd != NULL) {
			return (BOOL)::SendMessage(hWnd, WM_FORWARDMSG, 0, (LPARAM)pMsg);
		}
		
		return FALSE;
	}

	void onSize(WPARAM wParam);
	void onSpeaker(WPARAM wParam, LPARAM lParam);
	bool OnClose();
	void onEndSession();
	void OnCreate();
	void OnFileSettings();
	void onMatchAll();
	void OnAppAbout();
	void onLink(WORD wID);
	void onOpenFileList();
	void onOpenOwnList();
	void onTrayIcon(LPARAM lParam);
	void OnViewStatusBar();
	void OnViewToolBar();
	void OnViewTransferView();
	void onGetToolTip(int idCtrl, LPNMHDR pnmh);
	void onCopyData(LPARAM lParam);
	void onRefreshFileList();
	void onQuickConnect();
	void onActivateApp(WPARAM wParam);
	bool onAppCommand(LPARAM lParam);
	void onToolbarAway();
	void onCloseWindows(WORD wID);
	void onOpenWindows(WORD wID);

	static DWORD WINAPI stopper(void *);
	void UpdateLayout(bool bResizeBars = true);
	void parseCommandLine(const tstring& cmdLine);

	void StartUPnP();
	void StopUPnP();

	void onTray() {
		bTrayIcon = false;
		updateTray(true); 
	}

	void onRowsChanged() {
		UpdateLayout();
		::InvalidateRect(m_hWnd, NULL, TRUE);
	}

	void onSelected(WPARAM wParam) {
		HWND hWnd = (HWND)wParam;
		if((HWND)::SendMessage(WinUtil::mdiClient, WM_MDIGETACTIVE, 0, (LPARAM)NULL) != hWnd) {
			::SendMessage(WinUtil::mdiClient, WM_MDIACTIVATE, wParam, 0);
		} else if(BOOLSETTING(TOGGLE_ACTIVE_WINDOW)) {
			::SetWindowPos(hWnd, HWND_BOTTOM, 0, 0, 0, 0, SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOSIZE);
			::SendMessage(WinUtil::mdiClient, WM_MDINEXT, wParam, (LPARAM)FALSE);
			hWnd = (HWND)::SendMessage(WinUtil::mdiClient, WM_MDIGETACTIVE, 0, (LPARAM)NULL);
		}

		if(::IsIconic(hWnd)) {
			::ShowWindow(hWnd, SW_RESTORE);
			::SendMessage(WinUtil::mdiClient, WM_MDIACTIVATE, (WPARAM)hWnd, 0);
		}
	}

	void onDestroy();

    void onSetFocus() {
		if(WinUtil::mdiClient != NULL) {
			::SetFocus(WinUtil::mdiClient);
        }
    }

	void onWindowMinimizeAll() {
		HWND tmpWnd = ::GetWindow(WinUtil::mdiClient, GW_CHILD); //getting first child window
		while(tmpWnd != NULL) {
			::CloseWindow(tmpWnd);
			tmpWnd = ::GetWindow(tmpWnd, GW_HWNDNEXT);
		}
	}

	void onWindowRestoreAll() {
        BOOL bmax = FALSE;
		HWND tmpWnd = ::GetWindow(WinUtil::mdiClient, GW_CHILD); //getting first child window
		while(tmpWnd != NULL) {
			::ShowWindow(tmpWnd, SW_RESTORE);
			::SendMessage(WinUtil::mdiClient, WM_MDIGETACTIVE, NULL, (LPARAM)&bmax);
			if(bmax == TRUE) {
                break; //bmax will be TRUE if active child window is maximized, so if bmax then break
            }
			tmpWnd = ::GetWindow(tmpWnd, GW_HWNDNEXT);
		}
	}

	void onAlternateLimitTray() {
		if(alternatelimitingfromtray == true) {
			alternatelimitingfromtray = false;
			::CheckMenuItem(trayMenu, IDC_ALTERNATE_LIMIT_TRAY, MF_UNCHECKED | MF_BYCOMMAND);
		} else {
			alternatelimitingfromtray = true;
			::CheckMenuItem(trayMenu, IDC_ALTERNATE_LIMIT_TRAY, MF_CHECKED | MF_BYCOMMAND);
		}
	}

	void onMuteSounds() {
		if(BOOLSETTING(MUTE_SOUND) == true) {
			SettingsManager::getInstance()->set(SettingsManager::MUTE_SOUND, false);
			::CheckMenuItem(trayMenu, IDC_MUTE_SOUNDS_TRAY, MF_UNCHECKED | MF_BYCOMMAND);
			if(::SendMessage(hwndToolbar, TB_ISBUTTONCHECKED, IDC_MUTE_SOUNDS_TRAY, 0) != 0) {
				::SendMessage(hwndToolbar, TB_CHECKBUTTON, IDC_MUTE_SOUNDS_TRAY, MAKELPARAM(FALSE, 0));
            }
		} else {
			SettingsManager::getInstance()->set(SettingsManager::MUTE_SOUND, true);
			::CheckMenuItem(trayMenu, IDC_MUTE_SOUNDS_TRAY, MF_CHECKED | MF_BYCOMMAND);
			if(::SendMessage(hwndToolbar, TB_ISBUTTONCHECKED, IDC_MUTE_SOUNDS_TRAY, 0) == 0) {
				::SendMessage(hwndToolbar, TB_CHECKBUTTON, IDC_MUTE_SOUNDS_TRAY, MAKELPARAM(TRUE, 0));
            }
		}
	}

	static void setShutDown(bool b) {
		bShutdown = b;
		if(b) iCurrentShutdownTime = (time_t) (GET_TICK() / 1000);
	}
	static bool getShutDown() {
		return bShutdown;
	}

	HWND CreateEx();

	HIMAGELIST largeImages, largeImagesHot;
	static bool closing;

    static HWND m_hWnd;
	HACCEL m_hAccel;
	static MainFrame* pMainFrame;

	static LRESULT CALLBACK StaticWindowProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);

    HWND hwndToolbar;
private:
	LRESULT MainWindowProc(UINT uMsg, WPARAM wParam, LPARAM lParam);

	NOTIFYICONDATA normalicon;
	NOTIFYICONDATA pmicon;

	class DirectoryListInfo {
	public:
		DirectoryListInfo(const UserPtr& aUser, const tstring& aFile, int64_t aSpeed) : user(aUser), file(aFile), speed(aSpeed) { }
		UserPtr user;
		tstring file;
		int64_t speed;
	};
	
	TransferView transferView;

	enum { MAX_CLIENT_LINES = 10 };
	TStringList lastLinesList;
	tstring lastLines;
	HWND hwndLastLines;

	FlatTabCtrl ctrlTab;
	HttpConnection* c;
	string versionInfo;
	HIMAGELIST images;
	HBITMAP m_MenuBitmaps[22];
		
	bool awaybyminimize;
	bool alternatelimitingfromtray;
	bool bTrayIcon;
	bool bAppMinimized;
	bool bIsPM;
	
	static bool bShutdown;
	static time_t iCurrentShutdownTime;
	static bool bIsShuttingDown;
	HICON hShutdownIcon;
	static bool isShutdownStatus;

	HMENU trayMenu;

	UINT trayMessage;
	/** Was the window maximized when minimizing it? */
	bool maximized;
	uint64_t lastMove;
	uint64_t lastUpdate;
	uint64_t lastUp;
	uint64_t lastDown;
	uint64_t lastSave;

	bool oldshutdown;
	bool tabsontop;

	int lastUpload;

	int statusSizes[10];

	HANDLE stopperThread;

	bool missedAutoConnect;
	void createToolbar();
	void updateTray(bool add = true);
	void setNormalTrayIcon() {
		::PostMessage(m_hWnd, WM_SPEAKER, SET_NORMAL_TRAY_ICON, 0);
	}

	void onAppShow() {
		if(::IsIconic(m_hWnd)) {
			::ShowWindow(m_hWnd, SW_SHOW);
			::ShowWindow(m_hWnd, maximized == true ? SW_MAXIMIZE : SW_RESTORE);
		}
	}

	void autoConnect(const FavoriteHubEntry::List& fl);
	void startSocket();
	bool shutDown();
	void SetCheck(const int &iID, const bool &bChecked);

	MainFrame(const MainFrame&) { dcassert(0); }

	// LogManagerListener
	void on(LogManagerListener::Message, time_t t, const string& m) throw() {
		::PostMessage(m_hWnd, WM_SPEAKER, STATUS_MESSAGE, (LPARAM)new pair<time_t, tstring>(t, tstring(Text::toT(m))));
	}

	// TimerManagerListener
	void on(TimerManagerListener::Second type, uint64_t aTick) throw();
	void on(TimerManagerListener::Minute type, uint64_t aTick) throw();

	// HttpConnectionListener
	void on(HttpConnectionListener::Complete, HttpConnection* conn, string const& /*aLine*/) throw();
	void on(HttpConnectionListener::Data, HttpConnection* /*conn*/, const uint8_t* buf, size_t len) throw();	

	// QueueManagerListener
	void on(QueueManagerListener::Finished, QueueItem* qi, int64_t speed) throw();
	
	// SettingsManagerListener
	void on(SettingsManagerListener::Save, SimpleXML& /*xml*/) throw();

    HWND m_hWndToolBar;
    HWND m_hWndStatusBar;

    void OnChevronPushed(LPNMREBARCHEVRON nmrcRebarChevron);

    HWND GetWindowHandle();
    void UpdateSplitterParts();
};

#endif // MAIN_FRM_H

/*
 * Some parts Copyright (C) 2002-2011 Petr Kozelka, PPK at czdc dot org
 * Licensed under GNU General Public License version 3.
 */
