/* 
 * Copyright (C) 2003 Twink,  spm7@waikato.ac.nz
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "stdafx.h"
#include "../client/DCPlusPlus.h"
#include "../client/SettingsManager.h"
#include "../Client/FavoriteManager.h"

#include "PreviewPage.h"
#include "PreviewDlg.h"

PreviewPage * pPreviewPage = NULL;

PreviewPage::PreviewPage(SettingsManager *s) : PropPage(s, IDD_PREVIEWPAGE) {
    pPreviewPage = this;
	title = _tcsdup((TSTRING(SETTINGS_ADVANCED) + _T('\\') + TSTRING(SETTINGS_PREVIEW)).c_str());
	SetTitle(title);
	m_psp.pfnDlgProc = (DLGPROC)PreviewPageProc;
}

PreviewPage::~PreviewPage() {
    pPreviewPage = NULL;
	ctrlCommands.m_hWnd = NULL;
	free(title);
}

BOOL CALLBACK PreviewPage::PreviewPageProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    switch(uMsg) {
        case WM_INITDIALOG:
            pPreviewPage->m_hWnd = hWnd;
            pPreviewPage->onInitDialog();
            return TRUE;
        case WM_COMMAND:
            switch(LOWORD(wParam)) {
                case IDC_ADD_MENU:
                    pPreviewPage->onAddMenu();
                    return TRUE;
                case IDC_REMOVE_MENU:
                    pPreviewPage->onRemoveMenu();
                    return TRUE;
                case IDC_CHANGE_MENU:
                    pPreviewPage->onChangeMenu();
                    return TRUE;
            }

            break;
        case WM_NOTIFY:
            if(((LPNMHDR)lParam)->idFrom == IDC_MENU_ITEMS) {
                switch(((LPNMHDR)lParam)->code) {
                    case BN_DOUBLECLICKED:
                        pPreviewPage->onChangeMenu();
                        return TRUE;
                    case LVN_ITEMCHANGED:
                        pPreviewPage->onItemchanged((LPNMHDR)lParam);
                        return TRUE;
                    case LVN_KEYDOWN:
                        if(pPreviewPage->onKeyDown((LPNMHDR)lParam) == true) {
                            return TRUE;
                        }
                }
            }

            break;
    }

	return FALSE;
}

PropPage::TextItem PreviewPage::texts[] = {
	{ IDC_ADD_MENU, ResourceManager::ADD },
	{ IDC_CHANGE_MENU, ResourceManager::SETTINGS_CHANGE },
	{ IDC_REMOVE_MENU, ResourceManager::REMOVE },
	{ 0, ResourceManager::SETTINGS_AUTO_AWAY }
};


void PreviewPage::onInitDialog(){
	PropPage::translate(m_hWnd, texts);

	RECT rc;

	ctrlCommands.m_hWnd = ::GetDlgItem(m_hWnd, IDC_MENU_ITEMS);
	::GetClientRect(ctrlCommands.m_hWnd, &rc);

	ListViewInsertColumn(ctrlCommands.m_hWnd, 0, CTSTRING(SETTINGS_NAME), LVCFMT_LEFT, (rc.right - rc.left) / 5, 0);
	ListViewInsertColumn(ctrlCommands.m_hWnd, 1, CTSTRING(SETTINGS_COMMAND), LVCFMT_LEFT, (rc.right - rc.left) * 2 / 5, 1);
	ListViewInsertColumn(ctrlCommands.m_hWnd, 2, CTSTRING(SETTINGS_ARGUMENT), LVCFMT_LEFT, (rc.right - rc.left) / 5, 2);
	ListViewInsertColumn(ctrlCommands.m_hWnd, 3, CTSTRING(SETTINGS_EXTENSIONS), LVCFMT_LEFT, (rc.right - rc.left) / 5, 3);

	::SendMessage(ctrlCommands.m_hWnd, LVM_SETEXTENDEDLISTVIEWSTYLE, 0, LVS_EX_FULLROWSELECT);

	// Do specialized reading here
	PreviewApplication::List lst = FavoriteManager::getInstance()->getPreviewApps();
	for(PreviewApplication::Iter i = lst.begin(); i != lst.end(); ++i) {
		PreviewApplication * pa = *i;	
		addEntry(pa, (int)::SendMessage(ctrlCommands.m_hWnd, LVM_GETITEMCOUNT, 0, 0));
	}
}

void PreviewPage::addEntry(PreviewApplication * pa, int pos) {
	TStringList lst;
	lst.push_back(Text::toT(pa->getName()));
	lst.push_back(Text::toT(pa->getApplication())); 
	lst.push_back(Text::toT(pa->getArguments()));
	lst.push_back(Text::toT(pa->getExtension()));
	ctrlCommands.insert(pos, lst, 0, 0);
}


void PreviewPage::onAddMenu() {
	PreviewDlg dlg;
	if(dlg.DoModal(m_hWnd) == IDOK){
		addEntry(FavoriteManager::getInstance()->addPreviewApp(Text::fromT(dlg.name), Text::fromT(dlg.application), 
			Text::fromT(dlg.argument), Text::fromT(dlg.extensions)), (int)::SendMessage(ctrlCommands.m_hWnd, LVM_GETITEMCOUNT, 0, 0));
	}
}

void PreviewPage::onChangeMenu(){
	if((UINT)::SendMessage(ctrlCommands.m_hWnd, LVM_GETSELECTEDCOUNT, 0, 0) == 1) {
		int sel = (int)::SendMessage(ctrlCommands.m_hWnd, LVM_GETNEXTITEM, (WPARAM)-1, LVNI_ALL | LVNI_SELECTED);
		PreviewApplication pa;
		FavoriteManager::getInstance()->getPreviewApp(sel, pa);

		PreviewDlg dlg;
		dlg.name = Text::toT(pa.getName());
		dlg.application = Text::toT(pa.getApplication());
		dlg.argument = Text::toT(pa.getArguments());
		dlg.extensions = Text::toT(pa.getExtension());

		if(dlg.DoModal(m_hWnd) == IDOK) {
			pa.setName(Text::fromT(dlg.name));
			pa.setApplication(Text::fromT(dlg.application));
			pa.setArguments(Text::fromT(dlg.argument));
			pa.setExtension(Text::fromT(dlg.extensions));

			FavoriteManager::getInstance()->updatePreviewApp(sel, pa);

			ListViewSetItem(ctrlCommands.m_hWnd, sel, 0, LVIF_TEXT, dlg.name.c_str(), 0);
			ListViewSetItem(ctrlCommands.m_hWnd, sel, 1, LVIF_TEXT, dlg.application.c_str(), 0);
			ListViewSetItem(ctrlCommands.m_hWnd, sel, 2, LVIF_TEXT, dlg.argument.c_str(), 0);
			ListViewSetItem(ctrlCommands.m_hWnd, sel, 3, LVIF_TEXT, dlg.extensions.c_str(), 0);
		}
	}
}

void PreviewPage::onRemoveMenu() {
	if((UINT)::SendMessage(ctrlCommands.m_hWnd, LVM_GETSELECTEDCOUNT, 0, 0) == 1) {
		int sel = (int)::SendMessage(ctrlCommands.m_hWnd, LVM_GETNEXTITEM, (WPARAM)-1, LVNI_ALL | LVNI_SELECTED);
		FavoriteManager::getInstance()->removePreviewApp(sel);
		::SendMessage(ctrlCommands.m_hWnd, LVM_DELETEITEM, sel, 0);
	}
}

void PreviewPage::onItemchanged(LPNMHDR pnmh) {
	NM_LISTVIEW* lv = (NM_LISTVIEW*) pnmh;
	::EnableWindow(::GetDlgItem(m_hWnd, IDC_CHANGE_MENU), (lv->uNewState & LVIS_FOCUSED));
	::EnableWindow(::GetDlgItem(m_hWnd, IDC_REMOVE_MENU), (lv->uNewState & LVIS_FOCUSED));
}

bool PreviewPage::onKeyDown(LPNMHDR pnmh) {
	NMLVKEYDOWN* kd = (NMLVKEYDOWN*) pnmh;
	switch(kd->wVKey) {
    	case VK_INSERT:
			::PostMessage(m_hWnd, WM_COMMAND, IDC_ADD_MENU, 0);
    		return true;
    	case VK_DELETE:
			::PostMessage(m_hWnd, WM_COMMAND, IDC_REMOVE_MENU, 0);
    		return true;
    	default:
    		return false;
	}
}

/*
 * Some parts Copyright (C) 2002-2011 Petr Kozelka, PPK at czdc dot org
 * Licensed under GNU General Public License version 3.
 */
