/*
 * Copyright (C) 2001-2011 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "stdafx.h"
#include "../client/DCPlusPlus.h"
#include "Resource.h"

#include "PropPage.h"

#include "../client/SettingsManager.h"
#include "WinApiUtil.h"
#include "WinUtil.h"

#define SETTING_STR_MAXLEN 1024

void PropPage::read(HWND page, Item const* items, ListItem* listItems /* = NULL */, HWND hWndList /* = NULL */) {
	dcassert(page != NULL);

	bool const useDef = true;
	for(Item const* i = items; i->type != T_END; i++) {
		switch(i->type) {
		case T_STR:
			if(::GetDlgItem(page, i->itemID) == NULL) {
				// Control not exist ? Why ??
				throw;
			}
			::SetDlgItemText(page, i->itemID,
				Text::toT(settings->get((SettingsManager::StrSetting)i->setting, useDef)).c_str());
			break;
		case T_INT:
			if(((SettingsManager::IntSetting)i->setting) != SettingsManager::TCP_PORT &&
				((SettingsManager::IntSetting)i->setting) != SettingsManager::UDP_PORT) {
					if(::GetDlgItem(page, i->itemID) == NULL) {
					// Control not exist ? Why ??
					throw;
				}
				::SetDlgItemInt(page, i->itemID,
					settings->get((SettingsManager::IntSetting)i->setting, useDef), FALSE);
			} else {
				if(!SettingsManager::getInstance()->isDefault(i->setting)) {
					::SetDlgItemInt(page, i->itemID,
						settings->get((SettingsManager::IntSetting)i->setting, useDef), FALSE);
				}
			}
			break;
		case T_BOOL:
			if(::GetDlgItem(page, i->itemID) == NULL) {
				// Control not exist ? Why ??
				throw;
			}
			if(settings->getBool((SettingsManager::IntSetting)i->setting, useDef))
				::CheckDlgButton(page, i->itemID, BST_CHECKED);
			else
				::CheckDlgButton(page, i->itemID, BST_UNCHECKED);
		}
	}

	if(listItems != NULL) {
		::SendMessage(hWndList, LVM_SETEXTENDEDLISTVIEWSTYLE, 0, LVS_EX_LABELTIP | LVS_EX_CHECKBOXES | LVS_EX_FULLROWSELECT);

		ListViewInsertDummyColumn(hWndList);

		LVITEM lvi;
		lvi.mask = LVIF_TEXT;
		lvi.iSubItem = 0;

		for(int i = 0; listItems[i].setting != 0; i++) {
			lvi.iItem = i;
			lvi.pszText = const_cast<TCHAR*>(CTSTRING_I(listItems[i].desc));
			::SendMessage(hWndList, LVM_INSERTITEM, 0, (LPARAM)&lvi);

			ListView_SetItemState(hWndList, i,
				INDEXTOSTATEIMAGEMASK(SettingsManager::getInstance()->getBool(SettingsManager::IntSetting(listItems[i].setting), true) ? 2 : 1), 
				LVIS_STATEIMAGEMASK);
		}

		::SendMessage(hWndList, LVM_SETCOLUMNWIDTH, 0, LVSCW_AUTOSIZE);
	}
}

void PropPage::write(HWND page, Item const* items, ListItem* listItems /* = NULL */, HWND hWndList /* = NULL */) {
	dcassert(page != NULL);

	AutoArray<TCHAR> buf(SETTING_STR_MAXLEN);
	for(Item const* i = items; i->type != T_END; i++) {
		switch(i->type) {
		case T_STR:
			if(::GetDlgItem(page, i->itemID) == NULL) {
				// Control not exist ? Why ??
				throw;
			}
			::GetDlgItemText(page, i->itemID, buf, SETTING_STR_MAXLEN);
			settings->set((SettingsManager::StrSetting)i->setting, Text::fromT(tstring(buf)));

			break;
		case T_INT:
			if(::GetDlgItem(page, i->itemID) == NULL) {
				// Control not exist ? Why ??
				throw;
			}
			::GetDlgItemText(page, i->itemID, buf, SETTING_STR_MAXLEN);
			settings->set((SettingsManager::IntSetting)i->setting, Text::fromT(tstring(buf)));

			break;
		case T_BOOL:
			if(::GetDlgItem(page, i->itemID) == NULL) {
				// Control not exist ? Why ??
				throw;
			}
			if(::IsDlgButtonChecked(page, i->itemID) == BST_CHECKED)
				settings->set((SettingsManager::IntSetting)i->setting, true);
			else
				settings->set((SettingsManager::IntSetting)i->setting, false);
		}
	}

	if(listItems != NULL) {
		for(int i = 0; listItems[i].setting != 0; i++) {
			SettingsManager::getInstance()->set(SettingsManager::IntSetting(listItems[i].setting), 
                (int)(((UINT)::SendMessage(hWndList, LVM_GETITEMSTATE, i, LVIS_STATEIMAGEMASK) >> 12) - 1));
		}
	}
}

void PropPage::translate(HWND page, TextItem* textItems) {
	if(textItems != NULL) {
		for(int i = 0; textItems[i].itemID != 0; i++) {
			::SetDlgItemText(page, textItems[i].itemID,
				CTSTRING_I(textItems[i].translatedString));
		}
	}
}

void PropPage::SetTitle(const TCHAR * title) {
	m_psp.pszTitle = title;
	m_psp.dwFlags |= PSP_USETITLE;
}

/*
 * Some parts Copyright (C) 2002-2011 Petr Kozelka, PPK at czdc dot org
 * Licensed under GNU General Public License version 3.
 */
