/*
 * CzDC - client for Direct Connect peer to peer network.

 * Copyright (C) 2002-2003 Trpaslik
 * Copyright (C) 2002-2011 Petr Kozelka, PPK at CzDC dot org

 * CzDC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3
 * as published by the Free Software Foundation.

 * CzDC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with CzDC. If not, see <http://www.gnu.org/licenses/>.
 */

//----------------------------------------------------------------------------------------------------------------------------------------------------
#include "stdafx.h"
//----------------------------------------------------------------------------------------------------------------------------------------------------
#include "../client/DCPlusPlus.h"
//----------------------------------------------------------------------------------------------------------------------------------------------------
#include "Resource.h"
//----------------------------------------------------------------------------------------------------------------------------------------------------
#include "PropPageTextStyles.h"
//----------------------------------------------------------------------------------------------------------------------------------------------------
#include "../client/SettingsManager.h"
//----------------------------------------------------------------------------------------------------------------------------------------------------
#include "WinApiUtil.h"
//----------------------------------------------------------------------------------------------------------------------------------------------------

PropPageTextStyles * pPropPageTextStyles = NULL;
//----------------------------------------------------------------------------------------------------------------------------------------------------

PropPageTextStyles::PropPageTextStyles(SettingsManager *s) : PropPage(s, IDD_TEXT_STYLES) {
    pPropPageTextStyles = this;
	fg = 0;
	bg = 0;
	title = _tcsdup((TSTRING(SETTINGS_APPEARANCE) + _T('\\') + TSTRING(SETTINGS_TEXT_STYLES)).c_str());
	SetTitle(title);
	m_psp.pfnDlgProc = (DLGPROC)TextStylesPageProc;
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

PropPageTextStyles::~PropPageTextStyles() {
    pPropPageTextStyles = NULL;
	free(title);
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

BOOL CALLBACK PropPageTextStyles::TextStylesPageProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM /*lParam*/) {
    switch(uMsg) {
        case WM_INITDIALOG:
            pPropPageTextStyles->m_hWnd = hWnd;
            pPropPageTextStyles->onInitDialog();
            return TRUE;
        case WM_DESTROY:
            pPropPageTextStyles->OnDestroy();
            return TRUE;
        case WM_COMMAND:
            if(LOWORD(wParam) == IDC_SELTEXT) {
                pPropPageTextStyles->onClickedText();
                return TRUE;
            } else if(HIWORD(wParam) == BN_CLICKED) {
                switch(LOWORD(wParam)) {
                    case IDC_BACK_COLOR:
                        pPropPageTextStyles->onEditBackColor();
                        return TRUE;
                    case IDC_TEXT_COLOR:
                        pPropPageTextStyles->onEditForeColor();
                        return TRUE;
                    case IDC_TEXT_STYLE:
                        pPropPageTextStyles->onEditTextStyle();
                        return TRUE;
                    case IDC_DEFAULT_STYLES:
                        pPropPageTextStyles->onDefaultStyles();
                        return TRUE;
                    case IDC_BLACK_AND_WHITE:
                        pPropPageTextStyles->onBlackAndWhite();
                        return TRUE;
                    case IDC_SELWINCOLOR:
                        pPropPageTextStyles->onEditBackground();
                        return TRUE;
                    case IDC_ERROR_COLOR:
                        pPropPageTextStyles->onEditError();
                        return TRUE;
                    case IDC_PAUSED_COLOR:
                        pPropPageTextStyles->onEditPaused();
                        return TRUE;
                }
            }

            break;
    }

	return FALSE;
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

PropPage::TextItem PropPageTextStyles::texts[] = {
	{ IDC_CZDC_AVAILABLE_STYLES, ResourceManager::SETCZDC_STYLES },
	{ IDC_BACK_COLOR, ResourceManager::SETCZDC_BACK_COLOR },
	{ IDC_TEXT_COLOR, ResourceManager::SETCZDC_TEXT_COLOR },
	{ IDC_TEXT_STYLE, ResourceManager::SETCZDC_TEXT_STYLE },
	{ IDC_DEFAULT_STYLES, ResourceManager::SETCZDC_DEFAULT_STYLE },
	{ IDC_BLACK_AND_WHITE, ResourceManager::SETCZDC_BLACK_WHITE },
	{ IDC_BOLD_AUTHOR_MESS, ResourceManager::SETCZDC_BOLD },
	{ IDC_CZDC_PREVIEW, ResourceManager::SETCZDC_PREVIEW },
	{ IDC_SELWINCOLOR, ResourceManager::SETTINGS_SELECT_WINDOW_COLOR },
	{ IDC_SELTEXT, ResourceManager::SETTINGS_SELECT_TEXT_FACE },
	{ IDC_ERROR_COLOR, ResourceManager::SETCZDC_ERROR_COLOR },
	{ IDC_PAUSED_COLOR, ResourceManager::PAUSED_COLOR },
	{ 0, ResourceManager::SETTINGS_AUTO_AWAY }
}; 
//----------------------------------------------------------------------------------------------------------------------------------------------------

PropPage::Item PropPageTextStyles::items[] = {
	{ IDC_BOLD_AUTHOR_MESS, SettingsManager::BOLD_AUTHOR_MESS, PropPage::T_BOOL },
	{ 0, 0, PropPage::T_END }
};
//----------------------------------------------------------------------------------------------------------------------------------------------------

void PropPageTextStyles::onInitDialog() {
	PropPage::translate(m_hWnd, texts);
	PropPage::read(m_hWnd, items);

	m_Preview.m_hWnd = ::GetDlgItem(m_hWnd, IDC_PREVIEW);
	::SendMessage(m_Preview.m_hWnd, EM_SETLANGOPTIONS, 0, 0);

	WinUtil::decodeFont(Text::toT(SETTING(TEXT_FONT)), m_Font);
	m_BackColor = SETTING(BACKGROUND_COLOR);
	m_ForeColor = SETTING(TEXT_COLOR);

	fg = SETTING(TEXT_COLOR);
	bg = SETTING(BACKGROUND_COLOR);
	err = SETTING(ERROR_COLOR);
	paused = SETTING(PAUSED_COLOR);
	TextStyles[ TS_GENERAL ].Init(m_hWnd, settings, "General text", "ABCD EFGH | General chat text",
	SettingsManager::TEXT_GENERAL_BACK_COLOR, SettingsManager::TEXT_GENERAL_FORE_COLOR, 
	SettingsManager::TEXT_GENERAL_BOLD, SettingsManager::TEXT_GENERAL_ITALIC );

	TextStyles[ TS_MYNICK ].Init(m_hWnd, settings, "My nick", "It's me | My nick",
	SettingsManager::TEXT_MYNICK_BACK_COLOR, SettingsManager::TEXT_MYNICK_FORE_COLOR, 
	SettingsManager::TEXT_MYNICK_BOLD, SettingsManager::TEXT_MYNICK_ITALIC );

	TextStyles[ TS_MYMSG ].Init(m_hWnd, settings, "My own message", "Hello world ! | My own message",
	SettingsManager::TEXT_MYOWN_BACK_COLOR, SettingsManager::TEXT_MYOWN_FORE_COLOR, 
	SettingsManager::TEXT_MYOWN_BOLD, SettingsManager::TEXT_MYOWN_ITALIC );

	TextStyles[ TS_PRIVATE ].Init(m_hWnd, settings, "Private message", "Could you give me a slot ? | Private message",
	SettingsManager::TEXT_PRIVATE_BACK_COLOR, SettingsManager::TEXT_PRIVATE_FORE_COLOR, 
	SettingsManager::TEXT_PRIVATE_BOLD, SettingsManager::TEXT_PRIVATE_ITALIC );

	TextStyles[ TS_SYSTEM ].Init(m_hWnd, settings, "System message", "File list refreshed | System message ",
	SettingsManager::TEXT_SYSTEM_BACK_COLOR, SettingsManager::TEXT_SYSTEM_FORE_COLOR, 
	SettingsManager::TEXT_SYSTEM_BOLD, SettingsManager::TEXT_SYSTEM_ITALIC);

	TextStyles[ TS_SERVER ].Init(m_hWnd, settings, "Server message", "*** Connecting to hub XYZ ...| Server message",
	SettingsManager::TEXT_SERVER_BACK_COLOR, SettingsManager::TEXT_SERVER_FORE_COLOR, 
	SettingsManager::TEXT_SERVER_BOLD, SettingsManager::TEXT_SERVER_ITALIC );

	TextStyles[ TS_TIMESTAMP ].Init(m_hWnd, settings, "Timestamp", "01.02.2345 12:34 | The style for timestamp",
	SettingsManager::TEXT_TIMESTAMP_BACK_COLOR, SettingsManager::TEXT_TIMESTAMP_FORE_COLOR, 
	SettingsManager::TEXT_TIMESTAMP_BOLD, SettingsManager::TEXT_TIMESTAMP_ITALIC );

	TextStyles[ TS_URL ].Init(m_hWnd, settings, "URL (http, ftp, ...)", "http://www.czdc.org | URL",
	SettingsManager::TEXT_URL_BACK_COLOR, SettingsManager::TEXT_URL_FORE_COLOR, 
	SettingsManager::TEXT_URL_BOLD, SettingsManager::TEXT_URL_ITALIC );

	TextStyles[ TS_FAVORITE ].Init(m_hWnd, settings, "Favorite user", "Popeye the Sailor Man | Favorite user",
	SettingsManager::TEXT_FAV_BACK_COLOR, SettingsManager::TEXT_FAV_FORE_COLOR, 
	SettingsManager::TEXT_FAV_BOLD, SettingsManager::TEXT_FAV_ITALIC );

	TextStyles[ TS_OP ].Init(m_hWnd, settings, "OP", "For ones GOD for others EVIL | OP",
	SettingsManager::TEXT_OP_BACK_COLOR, SettingsManager::TEXT_OP_FORE_COLOR, 
	SettingsManager::TEXT_OP_BOLD, SettingsManager::TEXT_OP_ITALIC );

	for(int i = 0; i < TS_LAST; i++) {
		TextStyles[ i ].LoadSettings();
		_tcscpy_s(TextStyles[i].szFaceName, m_Font.lfFaceName );
		TextStyles[ i ].bCharSet = m_Font.lfCharSet;
		TextStyles[ i ].yHeight = m_Font.lfHeight;
		::SendMessage(::GetDlgItem(m_hWnd, IDC_TEXT_STYLES), LB_ADDSTRING, 0, (LPARAM)Text::toT(TextStyles[i].m_sText).c_str());
	}

	::SendMessage(::GetDlgItem(m_hWnd, IDC_TEXT_STYLES), LB_SETCURSEL, 0, 0);

	RefreshPreview();
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void PropPageTextStyles::write() {
	PropPage::write(m_hWnd, items);

	tstring f = WinUtil::encodeFont(m_Font);
	settings->set(SettingsManager::TEXT_FONT, Text::fromT(f));

	m_BackColor = TextStyles[ TS_GENERAL ].crBackColor;
	m_ForeColor = TextStyles[ TS_GENERAL ].crTextColor;

	settings->set(SettingsManager::TEXT_COLOR, (int)fg);
	settings->set(SettingsManager::BACKGROUND_COLOR, (int)bg);
	settings->set(SettingsManager::ERROR_COLOR, (int)err);
	settings->set(SettingsManager::PAUSED_COLOR, (int)paused);

	for ( int i = 0; i < TS_LAST; i++ ) {
		TextStyles[ i ].SaveSettings();
	}
	WinUtil::initColors();
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void PropPageTextStyles::onEditBackColor() {
	int iNdx = (int)::SendMessage(::GetDlgItem(m_hWnd, IDC_TEXT_STYLES), LB_GETCURSEL, 0, 0);
	TextStyles[iNdx].EditBackColor();
	RefreshPreview();
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void PropPageTextStyles::onEditForeColor() {
	int iNdx = (int)::SendMessage(::GetDlgItem(m_hWnd, IDC_TEXT_STYLES), LB_GETCURSEL, 0, 0);
	TextStyles[iNdx].EditForeColor();
	RefreshPreview();
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void PropPageTextStyles::onEditTextStyle() {
	int iNdx = (int)::SendMessage(::GetDlgItem(m_hWnd, IDC_TEXT_STYLES), LB_GETCURSEL, 0, 0);
	TextStyles[iNdx].EditTextStyle();

	_tcscpy_s(m_Font.lfFaceName, TextStyles[iNdx].szFaceName);
	m_Font.lfCharSet = TextStyles[iNdx].bCharSet;
	m_Font.lfHeight = TextStyles[iNdx].yHeight;

	if(iNdx == TS_GENERAL) {
		if(( TextStyles[iNdx].dwEffects & CFE_ITALIC) == CFE_ITALIC) {
			m_Font.lfItalic = true;
        }
		if((TextStyles[iNdx].dwEffects & CFE_BOLD) == CFE_BOLD) {
			m_Font.lfWeight = FW_BOLD;
        }
	}

	for(int i = 0; i < TS_LAST; i++) {
		_tcscpy_s(TextStyles[iNdx].szFaceName, m_Font.lfFaceName);
		TextStyles[i].bCharSet = m_Font.lfCharSet;
		TextStyles[i].yHeight = m_Font.lfHeight;
		m_Preview.AppendText(_T("My nick"), _T("12:34 "), Text::toT(TextStyles[i].m_sPreviewText).c_str(), TextStyles[i]);
	}

	RefreshPreview();
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void PropPageTextStyles::RefreshPreview() {
	::SendMessage(m_Preview.m_hWnd, EM_SETBKGNDCOLOR, 0, bg);
	m_Preview.SetTextStyleMyNick( TextStyles[ TS_MYNICK ] );
	::SetWindowText(m_Preview.m_hWnd, _T(""));

	string sText;
	for ( int i = 0; i < TS_LAST; i++ ) {
		m_Preview.AppendText(_T("My nick"), _T("12:34 "), Text::toT(TextStyles[i].m_sPreviewText).c_str(), 
		TextStyles[i] );
	}
	::InvalidateRect(m_Preview.m_hWnd, NULL, TRUE);
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void PropPageTextStyles::onDefaultStyles() {
	bg = RGB(0, 40, 80);
	fg = RGB(255, 255, 255);
	TextStyles[TS_GENERAL].crBackColor = RGB(0, 40, 80);
	TextStyles[TS_GENERAL].crTextColor = RGB(255, 255, 255);
	TextStyles[TS_GENERAL].dwEffects = 0;

	TextStyles[TS_MYNICK].crBackColor = RGB(0, 0, 0);
	TextStyles[TS_MYNICK].crTextColor = RGB(255, 255, 0);
	TextStyles[TS_MYNICK].dwEffects = CFE_BOLD;

	TextStyles[TS_MYMSG].crBackColor = RGB(0, 0, 0);
	TextStyles[TS_MYMSG].crTextColor = RGB(0, 255, 0);
	TextStyles[TS_MYMSG].dwEffects = 0;

	TextStyles[TS_PRIVATE].crBackColor = RGB(128, 0, 255);
	TextStyles[TS_PRIVATE].crTextColor = RGB(255, 255, 255);
	TextStyles[TS_PRIVATE].dwEffects = CFE_BOLD;

	TextStyles[TS_SYSTEM].crBackColor = RGB(0, 128, 0);
	TextStyles[TS_SYSTEM].crTextColor = RGB(255, 255, 255);
	TextStyles[TS_SYSTEM].dwEffects = CFE_BOLD | CFE_ITALIC;

	TextStyles[TS_SERVER].crBackColor = RGB(0, 0, 0);
	TextStyles[TS_SERVER].crTextColor = RGB(0, 225, 225);
	TextStyles[TS_SERVER].dwEffects = CFE_BOLD;

	TextStyles[TS_TIMESTAMP].crBackColor = RGB(255, 0, 0);
	TextStyles[TS_TIMESTAMP].crTextColor = RGB(255, 255, 255);
	TextStyles[TS_TIMESTAMP].dwEffects = 0;

	TextStyles[TS_URL].crBackColor = RGB(0, 0, 255);
	TextStyles[TS_URL].crTextColor = RGB(255, 255, 255);
	TextStyles[TS_URL].dwEffects = CFE_BOLD;

	TextStyles[TS_FAVORITE].crBackColor = RGB(0, 0, 150);
	TextStyles[TS_FAVORITE].crTextColor = RGB(128, 255, 255);
	TextStyles[TS_FAVORITE].dwEffects = CFE_BOLD;

	TextStyles[TS_OP].crBackColor = RGB(0, 0, 0);
	TextStyles[TS_OP].crTextColor = RGB(0, 255, 0);
	TextStyles[TS_OP].dwEffects = CFE_BOLD;

	RefreshPreview();
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void PropPageTextStyles::onBlackAndWhite() {
	bg = RGB(255, 255, 255);
	fg = RGB(0, 0, 0);

	for(int i = 0; i < TS_LAST; i++) {
        TextStyles[i].crBackColor = RGB(255, 255, 255);
        TextStyles[i].crTextColor = RGB(0, 0, 0);
        TextStyles[i].dwEffects = 0;
	}

	TextStyles[TS_OP].dwEffects = CFE_BOLD;

	RefreshPreview();
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void PropPageTextStyles::TextStyleSettings::Init(HWND hWndParent, SettingsManager *pSM, LPCSTR sText, LPCSTR sPreviewText,
	SettingsManager::IntSetting iBack, SettingsManager::IntSetting iFore, 
	SettingsManager::IntSetting iBold, SettingsManager::IntSetting iItalic) {

	cbSize = sizeof(CHARFORMAT2);
	dwMask = CFM_COLOR | CFM_BOLD | CFM_ITALIC | CFM_BACKCOLOR;
	dwReserved = 0;
  
	m_hWndParent = hWndParent;
	settings = pSM;      
	m_sText = sText;
	m_sPreviewText = sPreviewText;
	m_iBackColor = iBack;
	m_iForeColor = iFore;
	m_iBold = iBold;
	m_iItalic = iItalic;
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void PropPageTextStyles::TextStyleSettings::LoadSettings() {
	dwEffects = 0;
	crBackColor = settings->get( m_iBackColor );
	crTextColor = settings->get( m_iForeColor );
	if ( settings->get( m_iBold ) ) dwEffects |= CFE_BOLD;
	if ( settings->get( m_iItalic) ) dwEffects |= CFE_ITALIC;
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void PropPageTextStyles::TextStyleSettings::SaveSettings() {
	settings->set( m_iBackColor, (int) crBackColor);
	settings->set( m_iForeColor, (int) crTextColor);
	BOOL boBold = ( ( dwEffects & CFE_BOLD ) == CFE_BOLD );
	settings->set( m_iBold, (int) boBold);
	BOOL boItalic = ( ( dwEffects & CFE_ITALIC ) == CFE_ITALIC );
	settings->set( m_iItalic, (int) boItalic);
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void PropPageTextStyles::TextStyleSettings::EditBackColor() {
    CHOOSECOLOR cc;
    InitializeChooseColor(cc, crBackColor, m_hWndParent);

	if(::ChooseColor(&cc) == TRUE) {
		crBackColor = cc.rgbResult;
	}
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void PropPageTextStyles::TextStyleSettings::EditForeColor() {
    CHOOSECOLOR cc;
    InitializeChooseColor(cc, crTextColor, m_hWndParent);

	if(::ChooseColor(&cc) == TRUE) {
		crTextColor = cc.rgbResult;
	}
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void PropPageTextStyles::TextStyleSettings::EditTextStyle() {
	LOGFONT font;
	WinUtil::decodeFont(Text::toT(SETTING(TEXT_FONT)), font);

	_tcscpy_s(font.lfFaceName, szFaceName);
	font.lfCharSet = bCharSet;
	font.lfHeight = yHeight;

	if(dwEffects & CFE_BOLD) {
		font.lfWeight = FW_BOLD;
	} else {
		font.lfWeight = FW_REGULAR;
    }

	if(dwEffects & CFE_ITALIC) {
		font.lfItalic = true;
	} else {
		font.lfItalic = false;
    }

    CHOOSEFONT cf;
    memset(&cf, 0, sizeof(cf));
	cf.lStructSize = sizeof(cf);
	cf.hwndOwner = m_hWndParent;
	cf.rgbColors = crTextColor;
	cf.lpLogFont = &font;
	cf.Flags = CF_SCREENFONTS | CF_FORCEFONTEXIST | CF_INITTOLOGFONTSTRUCT;

    if(::ChooseFont(&cf) == TRUE) {
        _tcscpy_s(szFaceName, cf.lpLogFont->lfFaceName);
        bCharSet = cf.lpLogFont->lfCharSet;
        yHeight = cf.lpLogFont->lfHeight;

        crTextColor = cf.rgbColors;
        if(cf.lpLogFont->lfWeight == FW_BOLD) {
            dwEffects |= CFE_BOLD;
        } else {
            dwEffects &= ~CFE_BOLD;
        }

        if(cf.lpLogFont->lfItalic) {
            dwEffects |= CFE_ITALIC;
        } else {
            dwEffects &= ~CFE_ITALIC;
        }
    }
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void PropPageTextStyles::OnDestroy() {
	if(m_Preview.m_hWnd != NULL) {
		m_Preview.m_hWnd = NULL;
	}
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void PropPageTextStyles::onEditBackground() {
    CHOOSECOLOR cc;
    InitializeChooseColor(cc, bg, m_hWnd);

	if(::ChooseColor(&cc) == TRUE) {
		bg = cc.rgbResult;
	}

	RefreshPreview();
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void PropPageTextStyles::onClickedText() {
	LOGFONT tmp = m_Font;

    CHOOSEFONT cf;
    memset(&cf, 0, sizeof(cf));
	cf.lStructSize = sizeof(cf);
	cf.hwndOwner = m_hWnd;
	cf.rgbColors = fg;
	cf.lpLogFont = &tmp;
	cf.Flags = CF_EFFECTS | CF_SCREENFONTS | CF_FORCEFONTEXIST | CF_INITTOLOGFONTSTRUCT;

    if(::ChooseFont(&cf) == TRUE) {
		m_Font = tmp;
		fg = cf.rgbColors;
    }

	RefreshPreview();
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void PropPageTextStyles::onEditError() {
    CHOOSECOLOR cc;
    InitializeChooseColor(cc, err, m_hWnd);

	if(::ChooseColor(&cc) == TRUE) {
		err = cc.rgbResult;
	}
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void PropPageTextStyles::onEditPaused() {
    CHOOSECOLOR cc;
    InitializeChooseColor(cc, paused, m_hWnd);

	if(::ChooseColor(&cc) == TRUE) {
		paused = cc.rgbResult;
	}
}
//----------------------------------------------------------------------------------------------------------------------------------------------------
