/*
 * Copyright (C) 2001-2011 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "stdafx.h"
#include "../client/DCPlusPlus.h"
#include "Resource.h"

#include "PropertiesDlg.h"

#include "GeneralPage.h"
#include "DownloadPage.h"
#include "UploadPage.h"
#include "AppearancePage.h"
#include "AdvancedPage.h"
#include "LogPage.h"
#include "UCPage.h"
#include "FavoriteDirsPage.h"
#include "WindowsPage.h"
#include "QueuePage.h"
#include "CzDCPage.h"
#include "LimitPage.h"
#include "PropPageTextStyles.h"
#include "ToolbarPage.h"
#include "PreviewPage.h"
#include "ThemesPage.h"
#include "ExpertPage.h"

static const UINT WM_USER_INITDIALOG = WM_APP + 501;
static const TCHAR SEPARATOR = _T('\\');

static PropertiesDlg * pPropertiesDlg = NULL;

PropertiesDlg::PropertiesDlg(HWND hWndParent, SettingsManager *s) {
    pPropertiesDlg = this;

    m_hWnd = NULL;

	int n = 0;

	pages[n++] = new GeneralPage(s);
	pages[n++] = new DownloadPage(s);
	pages[n++] = new FavoriteDirsPage(s);
	pages[n++] = new QueuePage(s);
	pages[n++] = new UploadPage(s);
	pages[n++] = new AppearancePage(s);
	pages[n++] = new PropPageTextStyles(s);
	pages[n++] = new ThemesPage(s);
	pages[n++] = new ToolbarPage(s);
	pages[n++] = new WindowsPage(s);
	pages[n++] = new AdvancedPage(s);
	pages[n++] = new ExpertPage(s);
	pages[n++] = new LimitPage(s);
	pages[n++] = new LogPage(s);
	pages[n++] = new PreviewPage(s);
	pages[n++] = new UCPage(s);
	pages[n++] = new CzDCPage(s);

	memset(&m_psh, 0, sizeof(PROPSHEETHEADER));
	m_psh.dwSize = sizeof(PROPSHEETHEADER);
	m_psh.dwFlags = PSH_USECALLBACK | PSH_RTLREADING | PSH_NOAPPLYNOW | PSH_NOCONTEXTHELP;
	m_psh.hInstance = g_hInstance;
	m_psh.pszCaption = CTSTRING(SETTINGS);
	m_psh.nStartPage = 0;
	m_psh.hwndParent = hWndParent;
	m_psh.pfnCallback = &PropDlgProc;
}

PropertiesDlg::~PropertiesDlg() {
    pPropertiesDlg = NULL;

	for(int i = 0; i < numPages; i++) {
		delete pages[i];
	}
}

void PropertiesDlg::onInitDialog() {
	if(ResourceManager::getInstance()->isRTL()) {
		::SetWindowLong(m_hWnd, GWL_EXSTYLE, ::GetWindowLong(m_hWnd, GWL_EXSTYLE) | WS_EX_LAYOUTRTL);
	}

	hideTab();
	addTree();
	fillTree();
}

void PropertiesDlg::onSetCurSel(WPARAM wParam) {
	::SendMessage(hWndTree, TVM_SELECTITEM, TVGN_CARET, (LPARAM)findItem((int)wParam,
        (HTREEITEM)::SendMessage(hWndTree, TVM_GETNEXTITEM, TVGN_ROOT, 0)));
}

void PropertiesDlg::onSelChanged(LPNMHDR pnmh) {
	NMTREEVIEW* nmtv = (NMTREEVIEW*)pnmh;
	int page = (int)nmtv->itemNew.lParam;
	if(page == -1) {
		HTREEITEM next = (HTREEITEM)::SendMessage(hWndTree, TVM_GETNEXTITEM, TVGN_CHILD, (LPARAM)nmtv->itemNew.hItem);
		if(next == NULL) {
			next = (HTREEITEM)::SendMessage(hWndTree, TVM_GETNEXTITEM, TVGN_NEXT, (LPARAM)nmtv->itemNew.hItem);
			if(next == NULL) {
				next = (HTREEITEM)::SendMessage(hWndTree, TVM_GETNEXTITEM, TVGN_PARENT, (LPARAM)nmtv->itemNew.hItem);
				if(next != NULL)
					next = (HTREEITEM)::SendMessage(hWndTree, TVM_GETNEXTITEM, TVGN_NEXT, (LPARAM)next);
			}
		}
		if(next != NULL) {
			::SendMessage(hWndTree, TVM_SELECTITEM, TVGN_CARET, (LPARAM)next);
		}
	} else {
		if((int)::SendMessage(m_hWnd, PSM_HWNDTOINDEX, (WPARAM)::SendMessage(m_hWnd, PSM_GETCURRENTPAGEHWND, 0, 0), 0) != page) {
    		if(m_hWnd == NULL) {
    			m_psh.nStartPage = page;
    		} else {
        		::SendMessage(m_hWnd, PSM_SETCURSEL, page, 0);
            }
        }
	}
}

void PropertiesDlg::write() {
	for(int i = 0; i < numPages; i++) {
		// Check HWND of page to see if it has been created
		const HWND page = PropSheet_IndexToHwnd(m_hWnd, i);

		if(page != NULL)
			pages[i]->write();
	}
}

INT_PTR PropertiesDlg::DoModal(HWND hWndParent/* = ::GetActiveWindow()*/) {
	if(m_psh.hwndParent == NULL) {
		m_psh.hwndParent = hWndParent;
    }

    HPROPSHEETPAGE hpsPages[numPages];

	for(int i = 0; i < numPages; i++) {
        HPROPSHEETPAGE hPage = ::CreatePropertySheetPage(pages[i]->getPSP());
        hpsPages[i] = hPage;
	}

	m_psh.phpage = (HPROPSHEETPAGE*)&hpsPages;
	m_psh.nPages = numPages;

	INT_PTR nRet = ::PropertySheet(&m_psh);

	m_psh.nPages = 0;
	m_psh.phpage = NULL;

	return nRet;
}

void PropertiesDlg::hideTab() {
	RECT rcClient, rcTab, rcPage, rcWindow;
	HWND hWndTab = (HWND)::SendMessage(m_hWnd, PSM_GETTABCONTROL, 0, 0);
	HWND hWndPage = (HWND)::SendMessage(m_hWnd, PSM_INDEXTOHWND, 0, 0);

	::GetClientRect(m_hWnd, &rcClient);

	::GetWindowRect(hWndTab, &rcTab);
	::GetClientRect(hWndPage, &rcPage);
	::MapWindowPoints(hWndPage, m_hWnd, (LPPOINT)&rcPage, 2);
	::GetWindowRect(m_hWnd, &rcWindow);
	::MapWindowPoints(NULL, m_hWnd, (LPPOINT)&rcTab, 2);

	::ScrollWindow(m_hWnd, SPACE_LEFT + TREE_WIDTH + SPACE_MID - rcPage.left, SPACE_TOP - rcPage.top, NULL, NULL);
	rcWindow.right += SPACE_LEFT + TREE_WIDTH + SPACE_MID - rcPage.left - ((rcClient.right - rcClient.left)-rcTab.right) + SPACE_RIGHT;
	rcWindow.bottom += SPACE_TOP - rcPage.top;

	::ShowWindow(hWndTab, SW_HIDE);

	::MoveWindow(m_hWnd, rcWindow.left, rcWindow.top, rcWindow.right - rcWindow.left, rcWindow.bottom - rcWindow.top, TRUE);

    ::SetWindowLongPtr(hWndTab, GWLP_USERDATA, (LONG_PTR)this);
    OldTabControlProc = (WNDPROC)::SetWindowLongPtr(hWndTab, GWLP_WNDPROC, (LONG_PTR)TabWndProc);
}

void PropertiesDlg::addTree() {
	// Insert the space to the left
	RECT rcPage;

	HWND page = (HWND)::SendMessage(m_hWnd, PSM_INDEXTOHWND, 0, 0);
	::GetWindowRect(page, &rcPage);
	::MapWindowPoints(NULL, m_hWnd, (LPPOINT)&rcPage, 2);

	hWndTree = CreateWindowEx(WS_EX_CLIENTEDGE, WC_TREEVIEW, _T(""), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | TVS_HASBUTTONS | TVS_HASLINES | TVS_LINESATROOT | TVS_SHOWSELALWAYS |
        TVS_DISABLEDRAGDROP, SPACE_LEFT, rcPage.top, TREE_WIDTH - SPACE_LEFT, rcPage.bottom - rcPage.top, m_hWnd, (HMENU)IDC_PAGE, g_hInstance, NULL);
}

void PropertiesDlg::fillTree() {
	HWND tab = (HWND)::SendMessage(m_hWnd, PSM_GETTABCONTROL, 0, 0);
	int pages = (int)::SendMessage(tab, TCM_GETITEMCOUNT, 0, 0);

	TCHAR buf[MAX_NAME_LENGTH];
	TCITEM item;
	item.mask = TCIF_TEXT;
	item.pszText = buf;
	item.cchTextMax = MAX_NAME_LENGTH - 1;

	HTREEITEM first = NULL;
	for(int i = 0; i < pages; ++i) {
		::SendMessage(tab, TCM_GETITEM, i, (LPARAM)&item);
		if(i == 0)
			first = createTree(buf, TVI_ROOT, i);
		else
			createTree(buf, TVI_ROOT, i);
	}

	::SendMessage(hWndTree, TVM_SELECTITEM, TVGN_CARET, (LPARAM)first);
}

HTREEITEM PropertiesDlg::createTree(const tstring& str, HTREEITEM parent, int page) {
	TVINSERTSTRUCT tvi;
	tvi.hInsertAfter = TVI_LAST;
	tvi.hParent = parent;

	HTREEITEM first = (parent == TVI_ROOT) ? (HTREEITEM)::SendMessage(hWndTree, TVM_GETNEXTITEM, TVGN_ROOT, 0) :
        (HTREEITEM)::SendMessage(hWndTree, TVM_GETNEXTITEM, TVGN_CHILD, (LPARAM)parent);

	string::size_type i = str.find(SEPARATOR);
	if(i == string::npos) {
		// Last dir, the actual page
		HTREEITEM item = findItem(str, first);
		if(item == NULL) {
			// Doesn't exist, add
			tvi.item.mask = TVIF_PARAM | TVIF_TEXT;
			tvi.item.pszText = const_cast<LPTSTR>(str.c_str());
			tvi.item.lParam = page;
			item = (HTREEITEM)::SendMessage(hWndTree, TVM_INSERTITEM, 0, (LPARAM)&tvi);
			::SendMessage(hWndTree, TVM_EXPAND, TVE_EXPAND, (LPARAM)parent);
			return item;
		} else {
			// Update page
			if(TreeGetItemData(hWndTree, item) == -1) {
				TreeSetItem(hWndTree, item, TVIF_PARAM, NULL, 0, 0, 0, 0, (LPARAM)page);
            }
			return item;
		}
	} else {
		tstring name = str.substr(0, i);
		HTREEITEM item = findItem(name, first);
		if(item == NULL) {
			// Doesn't exist, add...
			tvi.item.mask = TVIF_PARAM | TVIF_TEXT;
			tvi.item.lParam = -1;
			tvi.item.pszText = const_cast<LPTSTR>(name.c_str());
			item = (HTREEITEM)::SendMessage(hWndTree, TVM_INSERTITEM, 0, (LPARAM)&tvi);
		}
		::SendMessage(hWndTree, TVM_EXPAND, TVE_EXPAND, (LPARAM)parent);
		// Recurse...
		return createTree(str.substr(i+1), item, page);
	}
}

HTREEITEM PropertiesDlg::findItem(const tstring& str, HTREEITEM start) {
	TCHAR buf[MAX_NAME_LENGTH];

	while(start != NULL) {
		TreeGetItemText(hWndTree, start, buf, MAX_NAME_LENGTH-1);
		if(str == buf) {
			return start;
		}
		start = (HTREEITEM)::SendMessage(hWndTree, TVM_GETNEXTITEM, TVGN_NEXT, (LPARAM)start);
	}
	return start;
}

HTREEITEM PropertiesDlg::findItem(int page, HTREEITEM start) {
	while(start != NULL) {
		if(((int)TreeGetItemData(hWndTree, start)) == page)
			return start;
		HTREEITEM ret = findItem(page, (HTREEITEM)::SendMessage(hWndTree, TVM_GETNEXTITEM, TVGN_CHILD, (LPARAM)start));
		if(ret != NULL)
			return ret;
		start = (HTREEITEM)::SendMessage(hWndTree, TVM_GETNEXTITEM, TVGN_NEXT, (LPARAM)start);
	}
	return NULL;
}

int PropertiesDlg::PropDlgProc(HWND hwndDlg, UINT uMsg, LPARAM /*lParam*/) {
	if(uMsg == PSCB_INITIALIZED) {
        pPropertiesDlg->m_hWnd = hwndDlg;

        ::SetWindowLongPtr(hwndDlg, GWLP_USERDATA, (LONG_PTR)pPropertiesDlg);
        OldPropertySheetProc = (WNDPROC)::SetWindowLongPtr(hwndDlg, GWLP_WNDPROC, (LONG_PTR)PropDlgWndProc);

        ::PostMessage(hwndDlg, WM_USER_INITDIALOG, 0, 0);
	}

	return 0;
}


LRESULT CALLBACK PropertiesDlg::PropDlgWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
	PropertiesDlg * pParent = (PropertiesDlg *)::GetWindowLongPtr(hWnd, GWLP_USERDATA);
	if(pParent != NULL) {
        switch(uMsg) {
            case WM_USER_INITDIALOG:
                pParent->onInitDialog();
                return 0;
            case WM_NOTIFYFORMAT:
                return NFR_UNICODE;
            case WM_NOTIFY:
                if(((LPNMHDR)lParam)->idFrom == IDC_PAGE && ((LPNMHDR)lParam)->code == TVN_SELCHANGED) {
                    pParent->onSelChanged((LPNMHDR)lParam);
                    return 0;
                }

                break;
            case WM_COMMAND:
                if(LOWORD(wParam) == IDOK) {
                    pParent->write();
                }

                break;
        }
    }

    return ::CallWindowProc(OldPropertySheetProc, hWnd, uMsg, wParam, lParam);
}

LRESULT CALLBACK PropertiesDlg::TabWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    if(uMsg == TCM_SETCURSEL) {
		PropertiesDlg * pParent = (PropertiesDlg *)::GetWindowLongPtr(hWnd, GWLP_USERDATA);
		if(pParent != NULL) {
			pParent->onSetCurSel(wParam);
        }
    }

    return ::CallWindowProc(OldTabControlProc, hWnd, uMsg, wParam, lParam);
}

/*
 * Some parts Copyright (C) 2002-2011 Petr Kozelka, PPK at czdc dot org
 * Licensed under GNU General Public License version 3.
 */
