/*
 * Copyright (C) 2001-2011 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "stdafx.h"
#include "../client/DCPlusPlus.h"
#include "Resource.h"

#include "WinUtil.h"
#include "QueueFrame.h"
#include "SearchFrame.h"
#include "PrivateFrame.h"
#include "LineDlg.h"
#include "FlatTabCtrl.h"

#include "../client/StringTokenizer.h"
#include "../client/ShareManager.h"
#include "../client/ClientManager.h"
#include "../client/version.h"

#define FILE_LIST_NAME _T("File Lists")

int QueueFrame::columnIndexes[] = { COLUMN_TARGET, COLUMN_STATUS, COLUMN_SIZE, COLUMN_DOWNLOADED, COLUMN_PRIORITY, COLUMN_USERS, COLUMN_PATH, COLUMN_EXACT_SIZE, COLUMN_ERRORS, COLUMN_ADDED,
    COLUMN_TTH, COLUMN_TYPE
};
int QueueFrame::columnSizes[] = { 200, 300, 75, 110, 75, 200, 200, 75, 200, 100, 125, 75 };
static ResourceManager::Strings columnNames[] = { ResourceManager::FILENAME, ResourceManager::STATUS, ResourceManager::SIZE, ResourceManager::DOWNLOADED, ResourceManager::PRIORITY,
    ResourceManager::USERS, ResourceManager::PATH, ResourceManager::EXACT_SIZE, ResourceManager::ERRORS, ResourceManager::ADDED, ResourceManager::TTH_ROOT, ResourceManager::TYPE
};

ATOM QueueFrame::m_aAtom = 0;
UINT QueueFrame::m_uiResourceID = IDR_QUEUE;

QueueFrame::QueueFrame() : menuItems(0), queueSize(0), queueItems(0), spoken(false), dirty(false),
	usingDirMenu(false),  readdItems(0), fileLists(NULL), showTree(true), closed(false), PreviewAppsSize(0),
	singleMenu(NULL), multiMenu(NULL), browseMenu(NULL), removeMenu(NULL), removeAllMenu(NULL), pmMenu(NULL),
    readdMenu(NULL), dirMenu(NULL), previewMenu(NULL), copyMenu(NULL) {
    RegisterWindowClass(m_aAtom, QueueFrameWndProc, _T("QueueFrame"), m_uiResourceID);
}

QueueFrame::~QueueFrame() {
    if(singleMenu != NULL) {
        ::DestroyMenu(singleMenu);
    }

    if(multiMenu != NULL) {
        ::DestroyMenu(multiMenu);
    }

    if(dirMenu != NULL) {
        ::DestroyMenu(dirMenu);
    }

	TaskQueue::List t;
	tasks.get(t);

	for(TaskQueue::Iter i = t.begin(); i != t.end(); ++i) {
		if(i->first == ADD_ITEM) {
			QueueItemInfoTask * qiit = (QueueItemInfoTask *)i->second;
			delete qiit;
		} else if(i->first == REMOVE_ITEM) {
			StringTask * st = (StringTask *)i->second;
			delete st;
		} else if(i->first == UPDATE_ITEM) {
			UpdateTask * ut = (UpdateTask *)i->second;
			delete ut;
		}
    }
}

LRESULT CALLBACK QueueFrame::QueueFrameWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
	QueueFrame * pParent;

	if(uMsg == WM_NCCREATE) {
		pParent = (QueueFrame *)((LPMDICREATESTRUCT)((LPCREATESTRUCT)lParam)->lpCreateParams)->lParam;
		SetWindowLongPtr(hWnd, GWLP_USERDATA, (LONG_PTR)pParent);

		pParent->m_hWnd = hWnd;
	} else {
		pParent = (QueueFrame *)GetWindowLongPtr(hWnd, GWLP_USERDATA);
		if(pParent == NULL) {
			return ::DefMDIChildProc(hWnd, uMsg, wParam, lParam);
		}
	}

	return pParent->QueueFrameProc(hWnd, uMsg, wParam, lParam);
}

LRESULT QueueFrame::QueueFrameProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    switch(uMsg) {
        case WM_CREATE:
            OnCreate();
            return 0;
        case WM_CLOSE:
            if(onClose() == true) {
                return 0;
            }
            break;
        case WM_SPEAKER:
            onSpeaker();
            return 0;
        case WM_SIZE:
            if(wParam != SIZE_MINIMIZED) {
                UpdateLayout();
            }
            break;
        case WM_CONTEXTMENU:
            if(onContextMenu(wParam, lParam) == true) {
                return 0;
            }
            break;
        case WM_SETFOCUS:
            onSetFocus();
            break;
        case WM_REFRESH_SETTINGS:
            onRefreshSettings();
            return 0;
        case WM_COMMAND:
            switch(LOWORD(wParam)) {
                case IDC_SEARCH_ALTERNATES:
                    onSearchAlternates();
                    return 0;
                case IDC_REMOVE:
                    onRemove();
                    return 0;
                case IDC_REMOVE_OFFLINE:
                    onRemoveOffline();
                    return 0;
                case IDC_MOVE:
                    onMove();
                    return 0;
                case IDC_CLEAR_ERRORS:
                    onClearErrors();
                    return 0;
                case IDC_COPY_FILENAME:
                case IDC_COPY_MAGNET:
                case IDC_COPY_TTH:
                    onCopy(LOWORD(wParam));
                    return 0;
            }

            if(LOWORD(wParam) >= IDC_PRIORITY_PAUSED && LOWORD(wParam) <= IDC_PRIORITY_HIGHEST) {
                onPriority(LOWORD(wParam));
                return 0;
            } else if(LOWORD(wParam) >= IDC_BROWSELIST && LOWORD(wParam) <= IDC_BROWSELIST + menuItems) {
                onBrowseList(LOWORD(wParam));
                return 0;
            } else if(LOWORD(wParam) >= IDC_REMOVE_SOURCE && LOWORD(wParam) <= IDC_REMOVE_SOURCE + menuItems) {
                onRemoveSource(LOWORD(wParam));
                return 0;
            } else if(LOWORD(wParam) >= IDC_REMOVE_SOURCES && LOWORD(wParam) <= IDC_REMOVE_SOURCES + 1 + menuItems) {
                onRemoveSources(LOWORD(wParam));
                return 0;
            } else if(LOWORD(wParam) >= IDC_PM && LOWORD(wParam) <= IDC_PM + menuItems) {
                onPM(LOWORD(wParam));
                return 0;
            } else if(LOWORD(wParam) >= IDC_READD && LOWORD(wParam) <= IDC_READD + 1 + readdItems) {
                onReadd(LOWORD(wParam));
                return 0;
            } else if(LOWORD(wParam) >= IDC_PREVIEW_APP && LOWORD(wParam) <= IDC_PREVIEW_APP + PreviewAppsSize) {
                onPreviewCommand(LOWORD(wParam));
                return 0;
            }

            break;
        case WM_NOTIFY:
            if(((LPNMHDR)lParam)->idFrom == IDC_QUEUE) {
                switch(((LPNMHDR)lParam)->code) {
                    case LVN_GETDISPINFO: {
                        ctrlQueue.onGetDispInfo((LPNMHDR)lParam);
                        return 0;
                    }
                    case LVN_COLUMNCLICK: {
                        ctrlQueue.onColumnClick((LPNMHDR)lParam);
                        return 0;
                    }
                    case LVN_KEYDOWN:
                        if(onKeyDown((LPNMHDR)lParam) == true) {
                            return 0;
                        }
                        break;
                    case LVN_ITEMCHANGED:
                        onItemChangedQueue((LPNMHDR)lParam);
                        return 0;
                    case NM_CUSTOMDRAW: {
                        bool bHandled = true;
                        LRESULT lResult = onCustomDraw((LPNMHDR)lParam, bHandled);
                        if(bHandled == true) {
                            return lResult;
                        }
                        break;
                    }
                }
            } else if(((LPNMHDR)lParam)->idFrom == IDC_DIRECTORIES) {
                if(((LPNMHDR)lParam)->code == TVN_SELCHANGED) {
                    updateQueue();
                    return 0;
                } else if(((LPNMHDR)lParam)->code == TVN_KEYDOWN) {
                    if(onKeyDownDirs((LPNMHDR)lParam) == true) {
                        return 0;
                    }
                }
            }

            break;
        case WM_NCDESTROY:
            delete this;
            return ::DefMDIChildProc(hWnd, uMsg, wParam, lParam);
    }

    LRESULT lResult = 0;
    if(MDIWindowProc(hWnd, uMsg, wParam, lParam, lResult) == true) {
		return lResult;
    }

    if(uMsg == WM_MOUSEACTIVATE) {
        lResult = ::DefMDIChildProc(m_hWnd, uMsg, wParam, lParam);
    }

    if(BasicSplitterProc(uMsg, wParam, lParam) == true) {
		return lResult;
    }

	return ::DefMDIChildProc(hWnd, uMsg, wParam, lParam);
}

LRESULT CALLBACK QueueFrame::ShowTreeWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    if(uMsg == BM_SETCHECK) {
		QueueFrame * pParent = (QueueFrame *)::GetWindowLongPtr(hWnd, GWLP_USERDATA);
		if(pParent != NULL) {
			pParent->onShowTree(wParam);
        }
    }

    return ::CallWindowProc(OldButtonProc, hWnd, uMsg, wParam, lParam);
}

void QueueFrame::OnCreate() {
    WinUtil::tabCtrl->addTab(m_hWnd, RGB(0, 0, 0), IDR_QUEUE);
    created = true;

	showTree = BOOLSETTING(QUEUEFRAME_SHOW_TREE);

    m_hWndStatusBar = CreateWindowEx(0, STATUSCLASSNAME, _T(""), WS_CHILD | WS_VISIBLE | WS_CLIPCHILDREN | WS_CLIPSIBLINGS | SBARS_SIZEGRIP, 0, 0, 0, 0, m_hWnd, 0, g_hInstance, NULL);

	ctrlQueue.Create(m_hWnd, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | WS_HSCROLL | WS_VSCROLL | LVS_REPORT | LVS_SHOWSELALWAYS | LVS_SHAREIMAGELISTS, WS_EX_CLIENTEDGE,
        IDC_QUEUE, LVS_EX_LABELTIP | LVS_EX_HEADERDRAGDROP | LVS_EX_FULLROWSELECT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT);

    hWndDirs = CreateWindowEx(WS_EX_CLIENTEDGE, WC_TREEVIEW, _T(""), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | TVS_HASBUTTONS | TVS_LINESATROOT | TVS_HASLINES | TVS_SHOWSELALWAYS |
        TVS_DISABLEDRAGDROP, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, m_hWnd, (HMENU)IDC_DIRECTORIES, g_hInstance, NULL);

	::SendMessage(hWndDirs, TVM_SETIMAGELIST, (WPARAM)TVSIL_NORMAL, (LPARAM)WinUtil::fileImages);
	::SendMessage(ctrlQueue.m_hWnd, LVM_SETIMAGELIST, LVSIL_SMALL, (LPARAM)WinUtil::fileImages);

	iProportionalPos = SETTING(QUEUEFRAME_SPLITTER);

	// Create listview columns
	WinUtil::splitTokens(columnIndexes, SETTING(QUEUEFRAME_ORDER), COLUMN_LAST);
	WinUtil::splitTokens(columnSizes, SETTING(QUEUEFRAME_WIDTHS), COLUMN_LAST);

	for(uint8_t ui8i = 0; ui8i < (sizeof(columnSizes) / sizeof(columnSizes[0])); ui8i++) {
		int fmt = (ui8i == COLUMN_SIZE || ui8i == COLUMN_DOWNLOADED || ui8i == COLUMN_EXACT_SIZE) ? LVCFMT_RIGHT : LVCFMT_LEFT;
		ctrlQueue.InsertColumn(ui8i, CTSTRING_I(columnNames[ui8i]), fmt, columnSizes[ui8i], ui8i);
	}

	ctrlQueue.setColumnOrderArray(COLUMN_LAST, columnIndexes);
	ctrlQueue.setSortColumn(COLUMN_TARGET);
	ctrlQueue.setVisible(SETTING(QUEUEFRAME_VISIBLE));

	::SendMessage(hWndDirs, TVM_SETBKCOLOR, 0, (LPARAM)WinUtil::bgColor);
	::SendMessage(hWndDirs, TVM_SETTEXTCOLOR, 0, (LPARAM)WinUtil::textColor);

    hWndShowTree = CreateWindowEx(0, WC_BUTTON, _T(""), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | BS_AUTOCHECKBOX, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
        m_hWndStatusBar, 0, g_hInstance, NULL);

    ::SetWindowLongPtr(hWndShowTree, GWLP_USERDATA, (LONG_PTR)this);
    OldButtonProc = (WNDPROC)::SetWindowLongPtr(hWndShowTree, GWLP_WNDPROC, (LONG_PTR)ShowTreeWndProc);
	::SendMessage(hWndShowTree, BM_SETCHECK, (showTree == true ? BST_CHECKED : BST_UNCHECKED), 0);

    copyMenu = ::CreatePopupMenu();
    ::AppendMenu(copyMenu, MF_STRING, IDC_COPY_FILENAME, CTSTRING(FILENAME));
	::AppendMenu(copyMenu, MF_STRING, IDC_COPY_TTH, CTSTRING(TTH_ROOT));
	::AppendMenu(copyMenu, MF_STRING, IDC_COPY_MAGNET, CTSTRING(MAGNET_LINK));

    browseMenu = ::CreatePopupMenu();
    removeAllMenu = ::CreatePopupMenu();
    pmMenu = ::CreatePopupMenu();
    previewMenu = ::CreatePopupMenu();

    removeMenu = ::CreatePopupMenu();
    ::AppendMenu(removeMenu, MF_STRING, IDC_REMOVE_SOURCE, CTSTRING(ALL));
    ::AppendMenu(removeMenu, MF_SEPARATOR, 0, NULL);

    readdMenu = ::CreatePopupMenu();
    ::AppendMenu(readdMenu, MF_STRING, IDC_READD, CTSTRING(ALL));
	::AppendMenu(readdMenu, MF_SEPARATOR, 0, NULL);

    singleMenu = ::CreatePopupMenu();
    ::AppendMenu(singleMenu, MF_STRING, IDC_SEARCH_ALTERNATES, CTSTRING(SEARCH_FOR_ALTERNATES));
    ::AppendMenu(singleMenu, MF_POPUP, (UINT_PTR)copyMenu, CTSTRING(COPY));
	::AppendMenu(singleMenu, MF_POPUP, (UINT_PTR)previewMenu, CTSTRING(PREVIEW_MENU));
    ::AppendMenu(singleMenu, MF_STRING, IDC_MOVE, CTSTRING(MOVE));
	::AppendMenu(singleMenu, MF_STRING, IDC_CLEAR_ERRORS, CTSTRING(CLEAR_ERRORS));

    WinUtil::AddPrioritySubMenu(singleMenu, TSTRING(SET_PRIORITY));

	::AppendMenu(singleMenu, MF_POPUP, (UINT_PTR)browseMenu, CTSTRING(GET_FILE_LIST));
	::AppendMenu(singleMenu, MF_POPUP, (UINT_PTR)pmMenu, CTSTRING(SEND_PRIVATE_MESSAGE));
	::AppendMenu(singleMenu, MF_POPUP, (UINT_PTR)readdMenu, CTSTRING(READD_SOURCE));
	::AppendMenu(singleMenu, MF_SEPARATOR, 0, NULL);
	::AppendMenu(singleMenu, MF_POPUP, (UINT_PTR)removeMenu, CTSTRING(REMOVE_SOURCE));
	::AppendMenu(singleMenu, MF_POPUP, (UINT_PTR)removeAllMenu, CTSTRING(REMOVE_FROM_ALL));
    ::AppendMenu(singleMenu, MF_STRING, IDC_REMOVE_OFFLINE, CTSTRING(REMOVE_OFFLINE));
	::AppendMenu(singleMenu, MF_STRING, IDC_REMOVE, CTSTRING(REMOVE));

    multiMenu = ::CreatePopupMenu();

    WinUtil::AddPrioritySubMenu(multiMenu, TSTRING(SET_PRIORITY));

    ::AppendMenu(multiMenu, MF_STRING, IDC_MOVE, CTSTRING(MOVE));
	::AppendMenu(multiMenu, MF_STRING, IDC_CLEAR_ERRORS, CTSTRING(CLEAR_ERRORS));
	::AppendMenu(multiMenu, MF_SEPARATOR, 0, NULL);
	::AppendMenu(multiMenu, MF_STRING, IDC_REMOVE, CTSTRING(REMOVE));

    dirMenu = ::CreatePopupMenu();

    WinUtil::AddPrioritySubMenu(dirMenu, TSTRING(SET_PRIORITY));

    ::AppendMenu(dirMenu, MF_STRING, IDC_MOVE, CTSTRING(MOVE));
	::AppendMenu(dirMenu, MF_STRING, IDC_CLEAR_ERRORS, CTSTRING(CLEAR_ERRORS));
	::AppendMenu(dirMenu, MF_SEPARATOR, 0, NULL);
	::AppendMenu(dirMenu, MF_STRING, IDC_REMOVE, CTSTRING(REMOVE));

	addQueueList(QueueManager::getInstance()->lockQueue());
	QueueManager::getInstance()->unlockQueue();
	QueueManager::getInstance()->addListener(this);
	SettingsManager::getInstance()->addListener(this);

	hIcPaused = (HICON)LoadImage((HINSTANCE)::GetWindowLongPtr(::GetParent(m_hWnd), GWLP_HINSTANCE), MAKEINTRESOURCE(IDR_PAUSED), IMAGE_ICON, 16, 16, LR_DEFAULTSIZE);
	hIcRunning = (HICON)LoadImage((HINSTANCE)::GetWindowLongPtr(::GetParent(m_hWnd), GWLP_HINSTANCE), MAKEINTRESOURCE(IDR_RUNNING), IMAGE_ICON, 16, 16, LR_DEFAULTSIZE);
	hIcUserOffline = (HICON)LoadImage((HINSTANCE)::GetWindowLongPtr(::GetParent(m_hWnd), GWLP_HINSTANCE), MAKEINTRESOURCE(IDR_USER_OFFLINE), IMAGE_ICON, 16, 16, LR_DEFAULTSIZE);
	hIcUsersOffline = (HICON)LoadImage((HINSTANCE)::GetWindowLongPtr(::GetParent(m_hWnd), GWLP_HINSTANCE), MAKEINTRESOURCE(IDR_USERS_OFFLINE), IMAGE_ICON, 16, 16, LR_DEFAULTSIZE);
	hIcNoUser = (HICON)LoadImage((HINSTANCE)::GetWindowLongPtr(::GetParent(m_hWnd), GWLP_HINSTANCE), MAKEINTRESOURCE(IDR_NOUSER), IMAGE_ICON, 16, 16, LR_DEFAULTSIZE);
	hIcWaiting = (HICON)LoadImage((HINSTANCE)::GetWindowLongPtr(::GetParent(m_hWnd), GWLP_HINSTANCE), MAKEINTRESOURCE(IDR_WAITING), IMAGE_ICON, 16, 16, LR_DEFAULTSIZE);

	memset(statusSizes, 0, sizeof(statusSizes));
	statusSizes[0] = 16;
	::SendMessage(m_hWndStatusBar, SB_SETPARTS, 6, (LPARAM)statusSizes);
	updateStatus();
}

void QueueFrame::QueueItemInfo::update() {
	if(display != NULL) {
		uint16_t colMask = updateMask;
		updateMask = 0;

		if(colMask & MASK_TARGET) {
			display->columns[COLUMN_TARGET] = Text::toT(Util::getFileName(getTarget()));
		}
		int online = 0;
		if(colMask & MASK_USERS || colMask & MASK_STATUS) {
			tstring tmp;

			for(QueueItem::SourceIter j = getSources().begin(); j != getSources().end(); ++j) {
				tstring nick = WinUtil::getNicks(j->getUser());
    			if(nick.empty() == false) {
    				if(tmp.size() > 0)
    					tmp += _T(", ");
    					
    				tmp += nick;
					pair<tstring, bool> hubs = WinUtil::getHubNames(j->getUser());
                	tmp += _T(" (") + hubs.first + _T(")");
    			}

				if(j->getUser()->isOnline())
                    online++;    			
			}
			display->columns[COLUMN_USERS] = tmp.empty() ? TSTRING(NO_USERS) : tmp;
		}
		if(colMask & MASK_STATUS) {
			if(getStatus() == QueueItem::STATUS_WAITING) {

				TCHAR buf[64];
				if(online > 0) {
					if(getSources().size() == 1) {
						display->columns[COLUMN_STATUS] = TSTRING(WAITING_USER_ONLINE);
					} else {
						_stprintf_s(buf, 64, CTSTRING(WAITING_USERS_ONLINE), online, getSources().size());
						display->columns[COLUMN_STATUS] = buf;
					}
				} else {
					if(getSources().size() == 0) {
						display->columns[COLUMN_STATUS] = TSTRING(NO_USERS_TO_DOWNLOAD_FROM);
					} else if(getSources().size() == 1) {
						display->columns[COLUMN_STATUS] = TSTRING(USER_OFFLINE);
					} else if(getSources().size() == 2) {
						display->columns[COLUMN_STATUS] = TSTRING(BOTH_USERS_OFFLINE);
					} else if(getSources().size() == 3) {
						display->columns[COLUMN_STATUS] = TSTRING(ALL_3_USERS_OFFLINE);
					} else if(getSources().size() == 4) {
						display->columns[COLUMN_STATUS] = TSTRING(ALL_4_USERS_OFFLINE);
					} else {
						_stprintf_s(buf, 64, CTSTRING(ALL_USERS_OFFLINE), getSources().size());
						display->columns[COLUMN_STATUS] = buf;
					}
				}
			} else if(getStatus() == QueueItem::STATUS_RUNNING) {
				TCHAR buf[64];
				if(online > 0) {
					if(getSources().size() == 1) {
						display->columns[COLUMN_STATUS] = TSTRING(RUNNING_USER_ONLINE);
					} else {
						_stprintf_s(buf, 64, CTSTRING(RUNNING_USERS_ONLINE), online, getSources().size());
						display->columns[COLUMN_STATUS] = buf;
					}
				} else {
					display->columns[COLUMN_STATUS] = TSTRING(RUNNING);
				}
			} 
		}
		if(colMask & MASK_SIZE) {
			display->columns[COLUMN_SIZE] = (getSize() == -1) ? TSTRING(UNKNOWN) : Text::toT(Util::formatBytes(getSize()));
			display->columns[COLUMN_EXACT_SIZE] = (getSize() == -1) ? TSTRING(UNKNOWN) : Text::toT(Util::formatExactSize(getSize()));
		}
		if(colMask & MASK_DOWNLOADED) {
				if(getSize() > 0)
				display->columns[COLUMN_DOWNLOADED] = Text::toT(Util::formatBytes(getDownloadedBytes()) + " (" + Util::toString((double)getDownloadedBytes()*100.0/(double)getSize()) + "%)");
				else
					display->columns[COLUMN_DOWNLOADED].clear();
		}
		if(colMask & MASK_PRIORITY) {
			switch(getPriority()) {
				case QueueItem::PAUSED: display->columns[COLUMN_PRIORITY] = TSTRING(PAUSED); break;
				case QueueItem::LOWEST: display->columns[COLUMN_PRIORITY] = TSTRING(LOWEST); break;
				case QueueItem::LOW: display->columns[COLUMN_PRIORITY] = TSTRING(LOW); break;
				case QueueItem::NORMAL: display->columns[COLUMN_PRIORITY] = TSTRING(NORMAL); break;
				case QueueItem::HIGH: display->columns[COLUMN_PRIORITY] = TSTRING(HIGH); break;
				case QueueItem::HIGHEST: display->columns[COLUMN_PRIORITY] = TSTRING(HIGHEST); break;
				case QueueItem::FILELIST: display->columns[COLUMN_PRIORITY] = TSTRING(FILELIST); break;
				default: dcasserta(0); break;
			}
		}

		if(colMask & MASK_PATH) {
			display->columns[COLUMN_PATH] = Text::toT(getPath());
		}

		if(colMask & MASK_ERRORS) {
			tstring tmp;
			for(QueueItem::SourceIter j = getBadSources().begin(); j != getBadSources().end(); ++j) {
				if(!j->isSet(QueueItem::Source::FLAG_REMOVED)) {
					if(tmp.size() > 0)
						tmp += _T(", ");
					tmp += WinUtil::getNicks(j->getUser());
					tmp += _T(" (");
					if(j->isSet(QueueItem::Source::FLAG_FILE_NOT_AVAILABLE)) {
						tmp += TSTRING(FILE_NOT_AVAILABLE);
					} else if(j->isSet(QueueItem::Source::FLAG_PASSIVE)) {
						tmp += TSTRING(PASSIVE_USER);
					} else if(j->isSet(QueueItem::Source::FLAG_ROLLBACK_INCONSISTENCY)) {
						tmp += TSTRING(ROLLBACK_INCONSISTENCY);
					} else if(j->isSet(QueueItem::Source::FLAG_CRC_FAILED)) {
						tmp += TSTRING(SFV_INCONSISTENCY);
					} else if(j->isSet(QueueItem::Source::FLAG_BAD_TREE)) {
						tmp += TSTRING(INVALID_TREE);
					} else if(j->isSet(QueueItem::Source::FLAG_SLOW_SOURCE)) {
						tmp += TSTRING(SOURCE_TOO_SLOW);
					}
					tmp += _T(')');
				}
			}
			display->columns[COLUMN_ERRORS] = tmp.empty() ? TSTRING(NO_ERRORS) : tmp;
		}

		if(colMask & MASK_ADDED) {
			display->columns[COLUMN_ADDED] = Text::toT(Util::formatTime("%Y-%m-%d %H:%M", getAdded()));
		}
		if(colMask & MASK_TTH && getTTH() != NULL) {
			display->columns[COLUMN_TTH] = Text::toT(getTTH()->toBase32());
		}
		if(colMask & MASK_TYPE) {
			display->columns[COLUMN_TYPE] = Text::toT(Util::getFileExt(getTarget()));
			if(display->columns[COLUMN_TYPE].size() > 0 && display->columns[COLUMN_TYPE][0] == '.')
				display->columns[COLUMN_TYPE].erase(0, 1);
		}
	}
}

void QueueFrame::on(QueueManagerListener::Added, QueueItem* aQI) {
	QueueItemInfo* ii = new QueueItemInfo(*aQI);

	speak(ADD_ITEM,	new QueueItemInfoTask(ii));
}

void QueueFrame::addQueueItem(QueueItemInfo* ii, bool noSort) {
	if(!ii->isSet(QueueItem::FLAG_USER_LIST)) {
		queueSize+=ii->getSize();
	}
	queueItems++;
	dirty = true;

	const string& dir = ii->getPath();

	bool updateDir = (directories.find(dir) == directories.end());
	directories.insert(make_pair(dir, ii));

	if(updateDir) {
		addDirectory(dir, ii->isSet(QueueItem::FLAG_USER_LIST));
	}
	if(!showTree || isCurDir(dir)) {
		ii->update();
		if(noSort)
			ctrlQueue.insertItem((int)::SendMessage(ctrlQueue.m_hWnd, LVM_GETITEMCOUNT, 0, 0), ii, WinUtil::getIconIndex(Text::toT(ii->getTarget())));
		else
			ctrlQueue.insertItem(ii, WinUtil::getIconIndex(Text::toT(ii->getTarget())));
	}
}

QueueFrame::QueueItemInfo* QueueFrame::getItemInfo(const string& target) {
	string path = Util::getFilePath(target);
	DirectoryPair items = directories.equal_range(path);
	for(DirectoryIter i = items.first; i != items.second; ++i) {
		if(i->second->getTarget() == target) {
			return i->second;
		}
	}
	return 0;
}

void QueueFrame::addQueueList(const QueueItem::StringMap& li) {
	::SendMessage(ctrlQueue.m_hWnd, WM_SETREDRAW, (WPARAM)FALSE, 0);
	::SendMessage(hWndDirs, WM_SETREDRAW, (WPARAM)FALSE, 0);
	for(QueueItem::StringMap::const_iterator j = li.begin(); j != li.end(); ++j) {
		QueueItem* aQI = j->second;
		QueueItemInfo* ii = new QueueItemInfo(*aQI);
		addQueueItem(ii, true);
	}
	ctrlQueue.resort();
	::SendMessage(ctrlQueue.m_hWnd, WM_SETREDRAW, (WPARAM)TRUE, 0);
	::SendMessage(hWndDirs, WM_SETREDRAW, (WPARAM)TRUE, 0);
	::InvalidateRect(hWndDirs, NULL, TRUE);
}

bool QueueFrame::onKeyDown(LPNMHDR pnmh) {
	NMLVKEYDOWN* kd = (NMLVKEYDOWN*) pnmh;
	if(kd->wVKey == VK_DELETE) {
		if(BOOLSETTING(CONFIRM_ITEM_REMOVAL) == false ||
			::MessageBox(m_hWnd, CTSTRING(CONFIRM_REMOVE), _T(CzDC_VERSION_STRING), MB_YESNO | MB_ICONQUESTION | MB_DEFBUTTON2) == IDYES) {
			removeSelected();
        }
        return true;
	} else if(kd->wVKey == VK_ADD){
		// Increase Item priority
		changePriority(true);
		return true;
	} else if(kd->wVKey == VK_SUBTRACT){
		// Decrease item priority
		changePriority(false);
		return true;
	} else if(kd->wVKey == VK_TAB) {
		onTab();
		return true;
	}

	return false;
}

HTREEITEM QueueFrame::addDirectory(const string& dir, bool isFileList /* = false */, HTREEITEM startAt /* = NULL */) {
	TVINSERTSTRUCT tvi;
	tvi.hInsertAfter = TVI_SORT;
	tvi.item.mask = TVIF_IMAGE | TVIF_PARAM | TVIF_SELECTEDIMAGE | TVIF_TEXT;
	tvi.item.iImage = tvi.item.iSelectedImage = WinUtil::getDirIconIndex();

	if(BOOLSETTING(EXPAND_QUEUE)) {
		tvi.itemex.mask |= TVIF_STATE;
		tvi.itemex.state = TVIS_EXPANDED;
		tvi.itemex.stateMask = TVIS_EXPANDED;
	}

	if(isFileList) {
		// We assume we haven't added it yet, and that all filelists go to the same
		// directory...
		dcassert(fileLists == NULL);
		tvi.hParent = NULL;
		tvi.item.pszText = FILE_LIST_NAME;
		tvi.item.lParam = (LPARAM) new string(dir);
		fileLists = (HTREEITEM)::SendMessage(hWndDirs, TVM_INSERTITEM, 0, (LPARAM)&tvi);
		return fileLists;
	}

	// More complicated, we have to find the last available tree item and then see...
	string::size_type i = 0;
	string::size_type j;

	HTREEITEM next = NULL;
	HTREEITEM parent = NULL;

	if(startAt == NULL) {
		// First find the correct drive letter
		dcassert(dir[1] == ':');
		dcassert(dir[2] == '\\');

		next = (HTREEITEM)::SendMessage(hWndDirs, TVM_GETNEXTITEM, TVGN_ROOT, 0);

		while(next != NULL) {
			if(next != fileLists) {
				string* stmp = (string*)TreeGetItemData(hWndDirs, next);
				if(Util::strnicmp(*stmp, dir, 3) == 0)
					break;
			}
			next = (HTREEITEM)::SendMessage(hWndDirs, TVM_GETNEXTITEM, TVGN_NEXT, (LPARAM)next);
		}

		if(next == NULL) {
			// First addition, set commonStart to the dir minus the last part...
			i = dir.rfind('\\', dir.length()-2);
			if(i != string::npos) {
				tstring name = Text::toT(dir.substr(0, i));
				tvi.hParent = NULL;
				tvi.item.pszText = const_cast<TCHAR*>(name.c_str());
				tvi.item.lParam = (LPARAM)new string(dir.substr(0, i+1));
				next = (HTREEITEM)::SendMessage(hWndDirs, TVM_INSERTITEM, 0, (LPARAM)&tvi);
			} else {
				dcassert(dir.length() == 3);
				tstring name = Text::toT(dir);
				tvi.hParent = NULL;
				tvi.item.pszText = const_cast<TCHAR*>(name.c_str());
				tvi.item.lParam = (LPARAM)new string(dir);
				next = (HTREEITEM)::SendMessage(hWndDirs, TVM_INSERTITEM, 0, (LPARAM)&tvi);
			}
		}

		// Ok, next now points to x:\... find how much is common

		string* rootStr = (string*)TreeGetItemData(hWndDirs, next);

		i = 0;

		for(;;) {
			j = dir.find('\\', i);
			if(j == string::npos)
				break;
			if(Util::strnicmp(dir.c_str() + i, rootStr->c_str() + i, j - i + 1) != 0)
				break;
			i = j + 1;
		}

		if(i < rootStr->length()) {
			HTREEITEM oldRoot = next;

			// Create a new root
			tstring name = Text::toT(rootStr->substr(0, i-1));
			tvi.hParent = NULL;
			tvi.item.pszText = const_cast<TCHAR*>(name.c_str());
			tvi.item.lParam = (LPARAM)new string(rootStr->substr(0, i));
			HTREEITEM newRoot = (HTREEITEM)::SendMessage(hWndDirs, TVM_INSERTITEM, 0, (LPARAM)&tvi);

			parent = addDirectory(*rootStr, false, newRoot);

			next = (HTREEITEM)::SendMessage(hWndDirs, TVM_GETNEXTITEM, TVGN_CHILD, (LPARAM)oldRoot);
			while(next != NULL) {
				moveNode(next, parent);
				next = (HTREEITEM)::SendMessage(hWndDirs, TVM_GETNEXTITEM, TVGN_CHILD, (LPARAM)oldRoot);
			}
			delete rootStr;
			::SendMessage(hWndDirs, TVM_DELETEITEM, 0, (LPARAM)oldRoot);
			parent = newRoot;
		} else {
			// Use this root as parent
			parent = next;
			next = (HTREEITEM)::SendMessage(hWndDirs, TVM_GETNEXTITEM, TVGN_CHILD, (LPARAM)parent);
		}
	} else {
		parent = startAt;
		next = (HTREEITEM)::SendMessage(hWndDirs, TVM_GETNEXTITEM, TVGN_CHILD, (LPARAM)parent);
		i = getDir(parent).length();
		dcassert(Util::strnicmp(getDir(parent), dir, getDir(parent).length()) == 0);
	}

	while( i < dir.length() ) {
		while(next != NULL) {
			if(next != fileLists) {
			const string& n = getDir(next);
			if(Util::strnicmp(n.c_str()+i, dir.c_str()+i, n.length()-i) == 0) {
				// Found a part, we assume it's the best one we can find...
				i = n.length();

				parent = next;
				next = (HTREEITEM)::SendMessage(hWndDirs, TVM_GETNEXTITEM, TVGN_CHILD, (LPARAM)next);
				break;
			}
			}
			next = (HTREEITEM)::SendMessage(hWndDirs, TVM_GETNEXTITEM, TVGN_NEXT, (LPARAM)next);
		}

		if(next == NULL) {
			// We didn't find it, add...
			j = dir.find('\\', i);
			dcassert(j != string::npos);
			tstring name = Text::toT(dir.substr(i, j-i));
			tvi.hParent = parent;
			tvi.item.pszText = const_cast<TCHAR*>(name.c_str());
			tvi.item.lParam = (LPARAM) new string(dir.substr(0, j+1));

			parent = (HTREEITEM)::SendMessage(hWndDirs, TVM_INSERTITEM, 0, (LPARAM)&tvi);

			i = j + 1;
		}
	}

	return parent;
}

void QueueFrame::removeDirectory(const string& dir, bool isFileList /* = false */) {

	// First, find the last name available
	string::size_type i = 0;

	HTREEITEM next = (HTREEITEM)::SendMessage(hWndDirs, TVM_GETNEXTITEM, TVGN_ROOT, 0);
	HTREEITEM parent = NULL;

	if(isFileList == true) {
		dcassert(fileLists != NULL);
        delete (string*)TreeGetItemData(hWndDirs, fileLists);
		::SendMessage(hWndDirs, TVM_DELETEITEM, 0, (LPARAM)fileLists);
		fileLists = NULL;
		return;
	} else {
		while(i < dir.length()) {
			while(next != NULL) {
				if(next != fileLists) {
				const string& n = getDir(next);
				if(Util::strnicmp(n.c_str()+i, dir.c_str()+i, n.length()-i) == 0) {
					// Match!
					parent = next;
					next = (HTREEITEM)::SendMessage(hWndDirs, TVM_GETNEXTITEM, TVGN_CHILD, (LPARAM)next);
					i = n.length();
					break;
				}
				}
				next = (HTREEITEM)::SendMessage(hWndDirs, TVM_GETNEXTITEM, TVGN_NEXT, (LPARAM)next);
			}
			if(next == NULL)
				break;
		}
	}

	next = parent;

	while(((HTREEITEM)::SendMessage(hWndDirs, TVM_GETNEXTITEM, TVGN_CHILD, (LPARAM)next) == NULL) &&
        (directories.find(getDir(next)) == directories.end())) {
        delete (string*)TreeGetItemData(hWndDirs, next);
		parent = (HTREEITEM)::SendMessage(hWndDirs, TVM_GETNEXTITEM, TVGN_PARENT, (LPARAM)next);
		
		::SendMessage(hWndDirs, TVM_DELETEITEM, 0, (LPARAM)next);
		if(parent == NULL)
			break;
		next = parent;
	}
}

void QueueFrame::removeDirectories(HTREEITEM ht) {
	HTREEITEM next = (HTREEITEM)::SendMessage(hWndDirs, TVM_GETNEXTITEM, TVGN_CHILD, (LPARAM)ht);
	while(next != NULL) {
		removeDirectories(next);
		next = (HTREEITEM)::SendMessage(hWndDirs, TVM_GETNEXTITEM, TVGN_NEXT, (LPARAM)ht);
	}
    delete (string*)TreeGetItemData(hWndDirs, ht);
	::SendMessage(hWndDirs, TVM_DELETEITEM, 0, (LPARAM)ht);
}

void QueueFrame::on(QueueManagerListener::Removed, QueueItem* aQI) {
	speak(REMOVE_ITEM, new StringTask(aQI->getTarget()));
}

void QueueFrame::on(QueueManagerListener::Moved, QueueItem* aQI, const string& oldTarget) {
	speak(REMOVE_ITEM, new StringTask(oldTarget));
	speak(ADD_ITEM,	new QueueItemInfoTask(new QueueItemInfo(*aQI)));
}

void QueueFrame::on(QueueManagerListener::SourcesUpdated, QueueItem* aQI) {
	speak(UPDATE_ITEM, new UpdateTask(*aQI));
}

void QueueFrame::onSpeaker() {
	TaskQueue::List t;

	tasks.get(t);
	spoken = false;

	for(TaskQueue::Iter ti = t.begin(); ti != t.end(); ++ti) {
		if(ti->first == ADD_ITEM) {
			QueueItemInfoTask * qiit = (QueueItemInfoTask *)ti->second;
			
			dcassert(ctrlQueue.findItem(qiit->ii) == -1);
			addQueueItem(qiit->ii, false);
			updateStatus();
			delete qiit;
		} else if(ti->first == REMOVE_ITEM) {
			StringTask * st = (StringTask *)ti->second;
			QueueItemInfo* ii = getItemInfo(st->str);
			if(ii == NULL) {
				dcassert(ii);
				delete st;
				continue;
			}

			if(!showTree || isCurDir(ii->getPath()) ) {
				dcassert(ctrlQueue.findItem(ii) != -1);
				ctrlQueue.deleteItem(ii);
			}

			if(!ii->isSet(QueueItem::FLAG_USER_LIST)) {
				queueSize-=ii->getSize();
				dcassert(queueSize >= 0);
			}
			queueItems--;
			dcassert(queueItems >= 0);

			pair<DirectoryIter, DirectoryIter> i = directories.equal_range(ii->getPath());
			DirectoryIter j;
			for(j = i.first; j != i.second; ++j) {
				if(j->second == ii)
					break;
			}
			dcassert(j != i.second);
			directories.erase(j);
			if(directories.count(ii->getPath()) == 0) {
				removeDirectory(ii->getPath(), ii->isSet(QueueItem::FLAG_USER_LIST));
				if(isCurDir(ii->getPath()))
					curDir.clear();
			}

			delete ii;
			updateStatus();
			if(BOOLSETTING(BOLD_QUEUE)) {
				setDirty();
			}
			dirty = true;
			delete st;
		} else if(ti->first == UPDATE_ITEM) {
			UpdateTask * ut = (UpdateTask *)ti->second;
            QueueItemInfo* ii = getItemInfo(ut->target);

			ii->setPriority(ut->priority);
			ii->setStatus(ut->status);
			ii->setDownloadedBytes(ut->downloadedBytes);
			ii->setSources(ut->sources);
			ii->setBadSources(ut->badSources);

			ii->updateMask |= QueueItemInfo::MASK_PRIORITY | QueueItemInfo::MASK_USERS | QueueItemInfo::MASK_ERRORS | QueueItemInfo::MASK_STATUS | QueueItemInfo::MASK_DOWNLOADED;

			if(!showTree || isCurDir(ii->getPath())) {
				dcassert(ctrlQueue.findItem(ii) != -1);
				ii->update();
				ctrlQueue.updateItem(ii);
			}
			delete ut;
		}
	}
}

void QueueFrame::removeSelected() {
	ctrlQueue.forEachSelected(&QueueItemInfo::remove);
}

void QueueFrame::removeSelectedDir() {
	removeDir((HTREEITEM)::SendMessage(hWndDirs, TVM_GETNEXTITEM, TVGN_CARET, 0));
}

void QueueFrame::moveSelected() {
	int n = (UINT)::SendMessage(ctrlQueue.m_hWnd, LVM_GETSELECTEDCOUNT, 0, 0);
	
	if(n == 1) {
		// Single file, get the full filename and move...
		QueueItemInfo* ii = ctrlQueue.getItemData((int)::SendMessage(ctrlQueue.m_hWnd, LVM_GETNEXTITEM, (WPARAM)-1, LVNI_SELECTED));
		tstring target = Text::toT(ii->getTarget());
		tstring ext = Util::getFileExt(target);
		tstring ext2;
		if (!ext.empty())
		{
			ext = ext.substr(1); // remove leading dot so default extension works when browsing for file
			ext2 = _T("*.") + ext;
			ext2 += (TCHAR)0;
			ext2 += _T("*.") + ext;
		}
		ext2 += _T("*.*");
		ext2 += (TCHAR)0;
		ext2 += _T("*.*");
		ext2 += (TCHAR)0;

		tstring path = Text::toT(ii->getPath());
		if(WinUtil::browseFile(target, m_hWnd, true, path, ext2.c_str(), ext.empty() ? NULL : ext.c_str())) {
			QueueManager::getInstance()->move(ii->getTarget(), Text::fromT(target));
		}
	} else if(n > 1) {
		tstring name;
		if(showTree) {
			name = Text::toT(curDir);
		}

		if(WinUtil::browseDirectory(name, m_hWnd)) {
			int i = -1;
			while( (i = (int)::SendMessage(ctrlQueue.m_hWnd, LVM_GETNEXTITEM, i, LVNI_SELECTED)) != -1) {
				QueueItemInfo* ii = ctrlQueue.getItemData(i);
				QueueManager::getInstance()->move(ii->getTarget(), Text::fromT(name) + Util::getFileName(ii->getTarget()));
			}
		}
	}
}

void QueueFrame::moveSelectedDir() {
	if((HTREEITEM)::SendMessage(hWndDirs, TVM_GETNEXTITEM, TVGN_CARET, 0) == NULL)
		return;

	dcassert(!curDir.empty());
	tstring name = Text::toT(curDir);

	if(WinUtil::browseDirectory(name, m_hWnd)) {
		moveDir((HTREEITEM)::SendMessage(hWndDirs, TVM_GETNEXTITEM, TVGN_CARET, 0), Text::fromT(name));
	}
}

void QueueFrame::moveDir(HTREEITEM ht, const string& target) {
	HTREEITEM next = (HTREEITEM)::SendMessage(hWndDirs, TVM_GETNEXTITEM, TVGN_CHILD, (LPARAM)ht);
	while(next != NULL) {
		moveDir(next, target + Util::getLastDir(getDir(next)));
		next = (HTREEITEM)::SendMessage(hWndDirs, TVM_GETNEXTITEM, TVGN_NEXT, (LPARAM)next);
	}
	string* s = (string*)TreeGetItemData(hWndDirs, ht);

	DirectoryPair p = directories.equal_range(*s);

	for(DirectoryIter i = p.first; i != p.second; ++i) {
		QueueItemInfo* ii = i->second;
		QueueManager::getInstance()->move(ii->getTarget(), target + Util::getFileName(ii->getTarget()));
	}
}

bool QueueFrame::onContextMenu(WPARAM wParam, LPARAM lParam) {
	if((HWND)wParam == ctrlQueue.m_hWnd) {
        int iX = GET_X_LPARAM(lParam),
            iY = GET_Y_LPARAM(lParam);

        {
            RECT rc;
            ::GetWindowRect((HWND)::SendMessage(ctrlQueue.m_hWnd, LVM_GETHEADER, 0, 0), &rc);

            POINT pt = { iX, iY };

            if(PtInRect(&rc, pt)){
                ctrlQueue.showMenu(pt);
                return true;
            }
        }

		if(iX == -1 && iY == -1) {
			ListViewGetContextMenuPos(ctrlQueue.m_hWnd, iX, iY);
		}

		if((UINT)::SendMessage(ctrlQueue.m_hWnd, LVM_GETSELECTEDCOUNT, 0, 0) > 0) {
			usingDirMenu = false;

			while(::GetMenuItemCount(browseMenu) > 0) {
                ::RemoveMenu(browseMenu, 0, MF_BYPOSITION);
			}
			while(::GetMenuItemCount(removeMenu) > 2) {
                ::RemoveMenu(removeMenu, 2, MF_BYPOSITION);
			}
			while(::GetMenuItemCount(removeAllMenu) > 0) {
                ::RemoveMenu(removeAllMenu, 0, MF_BYPOSITION);
			}
			while(::GetMenuItemCount(pmMenu) > 0) {
                ::RemoveMenu(pmMenu, 0, MF_BYPOSITION);
			}
			while(::GetMenuItemCount(readdMenu) > 2) {
                ::RemoveMenu(readdMenu, 2, MF_BYPOSITION);
			}
	
			while(::GetMenuItemCount(previewMenu) > 0) {
				::RemoveMenu(previewMenu, 0, MF_BYPOSITION);
			}

			if((UINT)::SendMessage(ctrlQueue.m_hWnd, LVM_GETSELECTEDCOUNT, 0, 0) == 1) {
				QueueItemInfo* ii = ctrlQueue.getItemData((int)::SendMessage(ctrlQueue.m_hWnd, LVM_GETNEXTITEM, (WPARAM)-1, LVNI_SELECTED));

				if((ii->isSet(QueueItem::FLAG_USER_LIST)) == false) {
					string ext = Util::getFileExt(Text::fromT(ii->getTargetFileName()));
					if(ext.size()>1) ext = ext.substr(1);
					PreviewAppsSize = WinUtil::SetupPreviewMenu(previewMenu, ext);
					if(::GetMenuItemCount(previewMenu) > 0) {
                        ::EnableMenuItem(singleMenu, (UINT)previewMenu, MF_BYCOMMAND | MFS_ENABLED);
					} else {
                        ::EnableMenuItem(singleMenu, (UINT)previewMenu, MF_BYCOMMAND | MFS_DISABLED);
					}
				}

				menuItems = 0;
				int pmItems = 0;

				MENUITEMINFO mi;
				memset(&mi, 0, sizeof(MENUITEMINFO));
				mi.cbSize = sizeof(MENUITEMINFO);

				for(QueueItem::SourceIter i = ii->getSources().begin(); i != ii->getSources().end(); ++i) {
				    tstring nick = WinUtil::escapeMenu(WinUtil::getNicks(i->getUser()));
				    if(nick.empty()) continue;
					mi.fMask = MIIM_ID | MIIM_TYPE | MIIM_DATA;
					mi.fType = MFT_STRING;
					mi.dwTypeData = (LPTSTR)nick.c_str();
					mi.dwItemData = (ULONG_PTR)&(*i);
					mi.wID = IDC_BROWSELIST + menuItems;
					::InsertMenuItem(browseMenu, menuItems, TRUE, &mi);
					mi.wID = IDC_REMOVE_SOURCE + 1 + menuItems; // "All" is before sources
					::InsertMenuItem(removeMenu, menuItems + 2, TRUE, &mi); // "All" and separator come first
					mi.wID = IDC_REMOVE_SOURCES + menuItems;
					::InsertMenuItem(removeAllMenu, menuItems, TRUE, &mi);
					if(i->getUser()->isOnline()) {
						mi.wID = IDC_PM + menuItems;
						::InsertMenuItem(pmMenu, menuItems, TRUE, &mi);
						pmItems++;
					}
					menuItems++;
				}
				if(ii->getBadSources().size() != 0) {
					memset(&mi, 0, sizeof(MENUITEMINFO));
					mi.cbSize = sizeof(MENUITEMINFO);
                    //at least one bad source exists
                    readdItems = 0;
					for(QueueItem::SourceIter i = ii->getBadSources().begin(); i != ii->getBadSources().end(); ++i) {
						tstring nick = WinUtil::getNicks(i->getUser());
						if(i->isSet(QueueItem::Source::FLAG_FILE_NOT_AVAILABLE)) {
							nick += _T(" (") + TSTRING(FILE_NOT_AVAILABLE) + _T(")");
						} else if(i->isSet(QueueItem::Source::FLAG_PASSIVE)) {
							nick += _T(" (") + TSTRING(PASSIVE_USER) + _T(")");
						} else if(i->isSet(QueueItem::Source::FLAG_ROLLBACK_INCONSISTENCY)) {
							nick += _T(" (") + TSTRING(ROLLBACK_INCONSISTENCY) + _T(")");
						} else if(i->isSet(QueueItem::Source::FLAG_CRC_FAILED)) {
							nick += _T(" (") + TSTRING(SFV_INCONSISTENCY) + _T(")");
						} else if(i->isSet(QueueItem::Source::FLAG_BAD_TREE)) {
							nick += _T(" (") + TSTRING(INVALID_TREE) + _T(")");
						} else if(i->isSet(QueueItem::Source::FLAG_SLOW_SOURCE)) {
							nick += _T(" (") + TSTRING(SLOW_USER) + _T(")");
						}
						mi.fMask = MIIM_ID | MIIM_TYPE | MIIM_DATA;
						mi.fType = MFT_STRING;
						mi.dwTypeData = (LPTSTR)nick.c_str();
						mi.dwItemData = (ULONG_PTR)&(*i);
						mi.wID = IDC_READD + 1 + readdItems;  // "All" is before sources
						::InsertMenuItem(readdMenu, readdItems + 2, TRUE, &mi); // "All" and separator come first
						readdItems++;
					}
					::EnableMenuItem(singleMenu, (UINT)readdMenu, MF_BYCOMMAND | MFS_ENABLED);
					::EnableMenuItem(singleMenu, IDC_CLEAR_ERRORS, MF_BYCOMMAND | MFS_ENABLED);
				} else {
                    //no bad sources for this item
                    readdItems = 0;
					::EnableMenuItem(singleMenu, (UINT)readdMenu, MF_BYCOMMAND | MFS_DISABLED);
					::EnableMenuItem(singleMenu, IDC_CLEAR_ERRORS, MF_BYCOMMAND | MFS_DISABLED);
				}

				if(menuItems == 0) {
                    ::EnableMenuItem(singleMenu, (UINT)browseMenu, MF_BYCOMMAND | MFS_DISABLED);
                    ::EnableMenuItem(singleMenu, (UINT)removeMenu, MF_BYCOMMAND | MFS_DISABLED);
                    ::EnableMenuItem(singleMenu, (UINT)removeAllMenu, MF_BYCOMMAND | MFS_DISABLED);
				} else {
                    ::EnableMenuItem(singleMenu, (UINT)browseMenu, MF_BYCOMMAND | MFS_ENABLED);
                    ::EnableMenuItem(singleMenu, (UINT)removeMenu, MF_BYCOMMAND | MFS_ENABLED);
                    ::EnableMenuItem(singleMenu, (UINT)removeAllMenu, MF_BYCOMMAND | MFS_ENABLED);
				}

				if(pmItems == 0) {
                    ::EnableMenuItem(singleMenu, (UINT)pmMenu, MF_BYCOMMAND | MFS_DISABLED);
				} else {
                    ::EnableMenuItem(singleMenu, (UINT)pmMenu, MF_BYCOMMAND | MFS_ENABLED);
				}

				if(ii->getTTH() != NULL) {
                    ::EnableMenuItem(singleMenu, IDC_SEARCH_ALTERNATES, MF_BYCOMMAND | MFS_ENABLED);
                    ::EnableMenuItem(copyMenu, IDC_COPY_TTH, MF_BYCOMMAND | MFS_ENABLED);
	                ::EnableMenuItem(copyMenu, IDC_COPY_MAGNET, MF_BYCOMMAND | MFS_ENABLED);
	            } else {
                    ::EnableMenuItem(singleMenu, IDC_SEARCH_ALTERNATES, MF_BYCOMMAND | MFS_DISABLED);
                    ::EnableMenuItem(copyMenu, IDC_COPY_TTH, MF_BYCOMMAND | MFS_DISABLED);
	                ::EnableMenuItem(copyMenu, IDC_COPY_MAGNET, MF_BYCOMMAND | MFS_DISABLED);
				}

                ::TrackPopupMenuEx(singleMenu, TPM_LEFTALIGN | TPM_RIGHTBUTTON, iX, iY, m_hWnd, NULL);
			} else {
                ::TrackPopupMenuEx(multiMenu, TPM_LEFTALIGN | TPM_RIGHTBUTTON, iX, iY, m_hWnd, NULL);
			}

			return true;
		}
	} else if((HWND)wParam == hWndDirs && (HTREEITEM)::SendMessage(hWndDirs, TVM_GETNEXTITEM, TVGN_CARET, 0) != NULL) {
        usingDirMenu = true;

        int iX = GET_X_LPARAM(lParam),
            iY = GET_Y_LPARAM(lParam);

        TreeGetContextMenuPos(hWndDirs, iX, iY);

        ::TrackPopupMenuEx(dirMenu, TPM_LEFTALIGN | TPM_RIGHTBUTTON, iX, iY, m_hWnd, NULL);

		return true;
	}

	return false;
}

void QueueFrame::onSearchAlternates() {
	if((UINT)::SendMessage(ctrlQueue.m_hWnd, LVM_GETSELECTEDCOUNT, 0, 0) == 1) {
		int i = (int)::SendMessage(ctrlQueue.m_hWnd, LVM_GETNEXTITEM, (WPARAM)-1, LVNI_SELECTED);
		QueueItemInfo* ii = ctrlQueue.getItemData(i);

		if(ii->getTTH() != NULL) {
			SearchFrame::OpenWindow(Text::toT(ii->getTTH()->toBase32()), 0, SearchManager::SIZE_DONTCARE, SearchManager::TYPE_TTH);
		}
	} 
}

void QueueFrame::onCopy(WORD wID) {
	if((UINT)::SendMessage(ctrlQueue.m_hWnd, LVM_GETSELECTEDCOUNT, 0, 0) == 1) {
    	int i = (int)::SendMessage(ctrlQueue.m_hWnd, LVM_GETNEXTITEM, (WPARAM)-1, LVNI_SELECTED);
    	if(i != -1) {
    		QueueItemInfo* ii = ctrlQueue.getItemData(i);
    		switch (wID) {
    			case IDC_COPY_FILENAME:
    				WinUtil::setClipboard(Text::toT(Util::getFileName(ii->getTarget())));
    				break;
    			case IDC_COPY_TTH:
    				if(ii->getTTH()) {
    					WinUtil::setClipboard(Text::toT(ii->getTTH()->toBase32()));
                    }
    				break;
    			case IDC_COPY_MAGNET:
    				if(ii->getTTH()) {
    					WinUtil::copyMagnet(ii->getTTH(), ii->getSize(), Text::toT(Util::getFileName(ii->getTarget())));
                    }
    				break;
    			default:
    				break;
    		}
    	}
    }
}

void QueueFrame::onBrowseList(WORD wID) {
	if((UINT)::SendMessage(ctrlQueue.m_hWnd, LVM_GETSELECTEDCOUNT, 0, 0) == 1) {
		MENUITEMINFO mi;
		memset(&mi, 0, sizeof(MENUITEMINFO));
		mi.cbSize = sizeof(MENUITEMINFO);
		mi.fMask = MIIM_DATA;

		::GetMenuItemInfo(browseMenu, wID, FALSE, &mi);
		QueueItem::Source* s = (QueueItem::Source*)mi.dwItemData;
		try {
			QueueManager::getInstance()->addList(s->getUser(), QueueItem::FLAG_CLIENT_VIEW);
		} catch(const Exception&) {
            // ...
		}
	}
}

void QueueFrame::onReadd(WORD wID) {
	if((UINT)::SendMessage(ctrlQueue.m_hWnd, LVM_GETSELECTEDCOUNT, 0, 0) == 1) {
		int i = (int)::SendMessage(ctrlQueue.m_hWnd, LVM_GETNEXTITEM, (WPARAM)-1, LVNI_SELECTED);
		QueueItemInfo* ii = ctrlQueue.getItemData(i);

		MENUITEMINFO mi;
		memset(&mi, 0, sizeof(MENUITEMINFO));
		mi.cbSize = sizeof(MENUITEMINFO);
		mi.fMask = MIIM_DATA;

        ::GetMenuItemInfo(readdMenu, wID, FALSE, &mi);
		if(wID == IDC_READD) {
			// re-add all sources
			QueueManager::getInstance()->readdall(ii->getTarget());
		} else {
			QueueItem::Source* s = (QueueItem::Source*)mi.dwItemData;
			try {
				QueueManager::getInstance()->readd(ii->getTarget(), s->getUser());
			} catch(const Exception& e) {
				::SendMessage(m_hWndStatusBar, SB_SETTEXT, (0 | 0), (LPARAM)Text::toT(e.getError()).c_str());
			}
		}
	}
}

void QueueFrame::onRemoveSource(WORD wID) {
	if((UINT)::SendMessage(ctrlQueue.m_hWnd, LVM_GETSELECTEDCOUNT, 0, 0) == 1) {
		int i = (int)::SendMessage(ctrlQueue.m_hWnd, LVM_GETNEXTITEM, (WPARAM)-1, LVNI_SELECTED);
		QueueItemInfo* ii = ctrlQueue.getItemData(i);
		if(wID == IDC_REMOVE_SOURCE) {
			QueueManager::getInstance()->removeAllSources(ii->getTarget());
		} else {
			MENUITEMINFO mi;
			memset(&mi, 0, sizeof(MENUITEMINFO));
			mi.cbSize = sizeof(MENUITEMINFO);
			mi.fMask = MIIM_DATA;

            ::GetMenuItemInfo(removeMenu, wID, FALSE, &mi);
			QueueItem::Source* s = (QueueItem::Source*)mi.dwItemData;
			QueueManager::getInstance()->removeSource(ii->getTarget(), s->getUser(), QueueItem::Source::FLAG_REMOVED);
		}
	}
}

void QueueFrame::onRemoveSources(WORD wID) {
	MENUITEMINFO mi;
	memset(&mi, 0, sizeof(MENUITEMINFO));
	mi.cbSize = sizeof(MENUITEMINFO);
	mi.fMask = MIIM_DATA;
	::GetMenuItemInfo(removeAllMenu, wID, FALSE, &mi);
	QueueItem::Source* s = (QueueItem::Source*)mi.dwItemData;
	QueueManager::getInstance()->removeSource(s->getUser(), QueueItem::Source::FLAG_REMOVED);
}

void QueueFrame::onPM(WORD wID) {
	if((UINT)::SendMessage(ctrlQueue.m_hWnd, LVM_GETSELECTEDCOUNT, 0, 0) == 1) {
		MENUITEMINFO mi;
		memset(&mi, 0, sizeof(MENUITEMINFO));
		mi.cbSize = sizeof(MENUITEMINFO);
		mi.fMask = MIIM_DATA;

        ::GetMenuItemInfo(pmMenu, wID, FALSE, &mi);
		QueueItem::Source* s = (QueueItem::Source*)mi.dwItemData;
		PrivateFrame::openWindow(s->getUser());
	}
}

void QueueFrame::onPriority(WORD wID) {
	QueueItem::Priority p;

	switch(wID) {
		case IDC_PRIORITY_PAUSED:
            p = QueueItem::PAUSED;
            break;
		case IDC_PRIORITY_LOWEST:
            p = QueueItem::LOWEST;
            break;
		case IDC_PRIORITY_LOW:
            p = QueueItem::LOW;
            break;
		case IDC_PRIORITY_NORMAL:
            p = QueueItem::NORMAL;
            break;
		case IDC_PRIORITY_HIGH:
            p = QueueItem::HIGH;
            break;
		case IDC_PRIORITY_HIGHEST:
            p = QueueItem::HIGHEST;
            break;
		default:
            p = QueueItem::DEFAULT;
            break;
	}

	if(usingDirMenu == true) {
		setPriority((HTREEITEM)::SendMessage(hWndDirs, TVM_GETNEXTITEM, TVGN_CARET, 0), p);
	} else {
		int i = -1;
		while( (i = (int)::SendMessage(ctrlQueue.m_hWnd, LVM_GETNEXTITEM, i, LVNI_SELECTED)) != -1) {
			QueueManager::getInstance()->setPriority(ctrlQueue.getItemData(i)->getTarget(), p);
		}
	}
}

void QueueFrame::removeDir(HTREEITEM ht) {
	if(ht == NULL)
		return;
	HTREEITEM child = (HTREEITEM)::SendMessage(hWndDirs, TVM_GETNEXTITEM, TVGN_CHILD, (LPARAM)ht);
	while(child) {
		removeDir(child);
		child = (HTREEITEM)::SendMessage(hWndDirs, TVM_GETNEXTITEM, TVGN_NEXT, (LPARAM)child);
	}
	const string& name = getDir(ht);
	DirectoryPair dp = directories.equal_range(name);
	for(DirectoryIter i = dp.first; i != dp.second; ++i) {
		QueueManager::getInstance()->remove(i->second->getTarget());
	}
}

/*
 * @param inc True = increase, False = decrease
 */
void QueueFrame::changePriority(bool inc){
	int i = -1;
	while( (i = (int)::SendMessage(ctrlQueue.m_hWnd, LVM_GETNEXTITEM, i, LVNI_SELECTED)) != -1){
		QueueItem::Priority p = ctrlQueue.getItemData(i)->getPriority();

		if ((inc && p == QueueItem::HIGHEST) || (!inc && p == QueueItem::PAUSED)){
			// Trying to go higher than HIGHEST or lower than PAUSED
			// so do nothing
			continue;
		}

		switch(p){
			case QueueItem::HIGHEST: p = QueueItem::HIGH; break;
			case QueueItem::HIGH:    p = inc ? QueueItem::HIGHEST : QueueItem::NORMAL; break;
			case QueueItem::NORMAL:  p = inc ? QueueItem::HIGH    : QueueItem::LOW; break;
			case QueueItem::LOW:     p = inc ? QueueItem::NORMAL  : QueueItem::LOWEST; break;
			case QueueItem::LOWEST:  p = inc ? QueueItem::LOW     : QueueItem::PAUSED; break;
			case QueueItem::PAUSED:  p = QueueItem::LOWEST; break;
		}

		QueueManager::getInstance()->setPriority(ctrlQueue.getItemData(i)->getTarget(), p);
	}
}

void QueueFrame::setPriority(HTREEITEM ht, const QueueItem::Priority& p) {
	if(ht == NULL)
		return;
	HTREEITEM child = (HTREEITEM)::SendMessage(hWndDirs, TVM_GETNEXTITEM, TVGN_CHILD, (LPARAM)ht);
	while(child) {
		setPriority(child, p);
		child = (HTREEITEM)::SendMessage(hWndDirs, TVM_GETNEXTITEM, TVGN_NEXT, (LPARAM)child);
	}
	const string& name = getDir(ht);
	DirectoryPair dp = directories.equal_range(name);
	for(DirectoryIter i = dp.first; i != dp.second; ++i) {
		QueueManager::getInstance()->setPriority(i->second->getTarget(), p);
	}
}

void QueueFrame::updateStatus() {
	if(::IsWindow(m_hWndStatusBar)) {
		int64_t total = 0;
		int cnt = (UINT)::SendMessage(ctrlQueue.m_hWnd, LVM_GETSELECTEDCOUNT, 0, 0);
		if(cnt == 0) {
			cnt = (int)::SendMessage(ctrlQueue.m_hWnd, LVM_GETITEMCOUNT, 0, 0);
			if(showTree) {
				for(int i = 0; i < cnt; ++i) {
					QueueItemInfo* ii = ctrlQueue.getItemData(i);
					total += (ii->getSize() > 0) ? ii->getSize() : 0;
				}
			} else {
				total = queueSize;
			}
		} else {
			int i = -1;
			while( (i = (int)::SendMessage(ctrlQueue.m_hWnd, LVM_GETNEXTITEM, i, LVNI_SELECTED)) != -1) {
				QueueItemInfo* ii = ctrlQueue.getItemData(i);
				total += (ii->getSize() > 0) ? ii->getSize() : 0;
			}

		}

		tstring tmp1 = Text::toT(STRING(ITEMS) + ": " + Util::toString(cnt));
		tstring tmp2 = Text::toT(STRING(SIZE) + ": " + Util::formatBytes(total));
		bool u = false;

		int w = WinUtil::getTextWidth(tmp1, m_hWndStatusBar);
		if(statusSizes[1] < w) {
			statusSizes[1] = w;
			u = true;
		}
		::SendMessage(m_hWndStatusBar, SB_SETTEXT, (2 | 0), (LPARAM)tmp1.c_str());
		w = WinUtil::getTextWidth(tmp2, m_hWndStatusBar);
		if(statusSizes[2] < w) {
			statusSizes[2] = w;
			u = true;
		}
		::SendMessage(m_hWndStatusBar, SB_SETTEXT, (3 | 0), (LPARAM)tmp2.c_str());

		if(dirty) {
			tmp1 = Text::toT(STRING(FILES) + ": " + Util::toString(queueItems));
			tmp2 = Text::toT(STRING(SIZE) + ": " + Util::formatBytes(queueSize));

			w = WinUtil::getTextWidth(tmp1, m_hWndStatusBar);
			if(statusSizes[3] < w) {
				statusSizes[3] = w;
				u = true;
			}
			::SendMessage(m_hWndStatusBar, SB_SETTEXT, (4 | 0), (LPARAM)tmp1.c_str());

			w = WinUtil::getTextWidth(tmp2, m_hWndStatusBar);
			if(statusSizes[4] < w) {
				statusSizes[4] = w;
				u = true;
			}
			::SendMessage(m_hWndStatusBar, SB_SETTEXT, (5 | 0), (LPARAM)tmp2.c_str());

			dirty = false;
		}

		if(u)
			UpdateLayout(true);
	}
}

void QueueFrame::UpdateLayout(bool bResizeStatusBar /* = true */) {
	RECT rect;
	::GetClientRect(m_hWnd, &rect);

	UpdateStatusBar(rect, bResizeStatusBar);

	if(::IsWindow(m_hWndStatusBar)) {
		RECT sr;
		int w[6];
		::GetClientRect(m_hWndStatusBar, &sr);
		w[5] = sr.right - 16;
#define setw(x) w[x] = max(w[x+1] - statusSizes[x], 0)
		setw(4); setw(3); setw(2); setw(1);

		w[0] = 16;

		::SendMessage(m_hWndStatusBar, SB_SETPARTS, 6, (LPARAM)w);

		::SendMessage(m_hWndStatusBar, SB_GETRECT, 0, (LPARAM)&sr);
		::MoveWindow(hWndShowTree, sr.left, sr.top, sr.right - sr.left, sr.bottom - sr.top, TRUE);
	}

	SetSplitterRect(&rect);
}

bool QueueFrame::onClose() {
	if(!closed) {
		QueueManager::getInstance()->removeListener(this);
		SettingsManager::getInstance()->removeListener(this);
		closed = true;
		::PostMessage(m_hWnd, WM_CLOSE, 0, 0);
		return true;
	} else {
		HTREEITEM ht = (HTREEITEM)::SendMessage(hWndDirs, TVM_GETNEXTITEM, TVGN_ROOT, 0);
		while(ht != NULL) {
			clearTree(ht);
			ht = (HTREEITEM)::SendMessage(hWndDirs, TVM_GETNEXTITEM, TVGN_NEXT, (LPARAM)ht);
		}

		DestroyIcon(hIcPaused);
		DestroyIcon(hIcRunning);
		DestroyIcon(hIcUserOffline);
		DestroyIcon(hIcUsersOffline);
		DestroyIcon(hIcNoUser);
		DestroyIcon(hIcWaiting);

		SettingsManager::getInstance()->set(SettingsManager::QUEUEFRAME_SHOW_TREE, (int)::SendMessage(hWndShowTree, BM_GETCHECK, 0, 0) == BST_CHECKED);
		SettingsManager::getInstance()->set(SettingsManager::QUEUEFRAME_SPLITTER, iProportionalPos);
		for(DirectoryIter i = directories.begin(); i != directories.end(); ++i) {
			delete i->second;
		}
		directories.clear();
		::SendMessage(ctrlQueue.m_hWnd, LVM_DELETEALLITEMS, 0, 0);

		ctrlQueue.saveHeaderOrder(SettingsManager::QUEUEFRAME_ORDER, 
			SettingsManager::QUEUEFRAME_WIDTHS, SettingsManager::QUEUEFRAME_VISIBLE);

        return false;
	}
}

void QueueFrame::onTab() {
	if(showTree) {
		HWND focus = ::GetFocus();
		if(focus == hWndDirs) {
			::SetFocus(ctrlQueue.m_hWnd);
		} else if(focus == ctrlQueue.m_hWnd) {
			::SetFocus(hWndDirs);
		}
	}
}

void QueueFrame::updateQueue() {
	::SendMessage(ctrlQueue.m_hWnd, LVM_DELETEALLITEMS, 0, 0);
	pair<DirectoryIter, DirectoryIter> i;
	if(showTree) {
		i = directories.equal_range(getSelectedDir());
	} else {
		i.first = directories.begin();
		i.second = directories.end();
	}

	::SendMessage(ctrlQueue.m_hWnd, WM_SETREDRAW, (WPARAM)FALSE, 0);
	for(DirectoryIter j = i.first; j != i.second; ++j) {
		QueueItemInfo* ii = j->second;
		ii->update();
		ctrlQueue.insertItem((int)::SendMessage(ctrlQueue.m_hWnd, LVM_GETITEMCOUNT, 0, 0), ii, WinUtil::getIconIndex(Text::toT(ii->getTarget())));
	}
	ctrlQueue.resort();
	::SendMessage(ctrlQueue.m_hWnd, WM_SETREDRAW, (WPARAM)TRUE, 0);
	curDir = getSelectedDir();
	updateStatus();
}

void QueueFrame::clearTree(HTREEITEM item) {
	HTREEITEM next = (HTREEITEM)::SendMessage(hWndDirs, TVM_GETNEXTITEM, TVGN_CHILD, (LPARAM)item);
	while(next != NULL) {
		clearTree(next);
		next = (HTREEITEM)::SendMessage(hWndDirs, TVM_GETNEXTITEM, TVGN_NEXT, (LPARAM)next);
	}
	delete (string*)TreeGetItemData(hWndDirs, item);
}

// Put it here to avoid a copy for each recursion...
static TCHAR tmpBuf[1024];
void QueueFrame::moveNode(HTREEITEM item, HTREEITEM parent) {
	TVINSERTSTRUCT tvis;
	memset(&tvis, 0, sizeof(tvis));
	tvis.itemex.hItem = item;
	tvis.itemex.mask = TVIF_CHILDREN | TVIF_HANDLE | TVIF_IMAGE | TVIF_INTEGRAL | TVIF_PARAM |
		TVIF_SELECTEDIMAGE | TVIF_STATE | TVIF_TEXT;
	tvis.itemex.pszText = tmpBuf;
	tvis.itemex.cchTextMax = 1024;
	::SendMessage(hWndDirs, TVM_GETITEM, 0, (LPARAM)(TVITEM*)&tvis.itemex);
	tvis.hInsertAfter =	TVI_SORT;
	tvis.hParent = parent;
	HTREEITEM ht = (HTREEITEM)::SendMessage(hWndDirs, TVM_INSERTITEM, 0, (LPARAM)&tvis);
	HTREEITEM next = (HTREEITEM)::SendMessage(hWndDirs, TVM_GETNEXTITEM, TVGN_CHILD, (LPARAM)item);
	while(next != NULL) {
		moveNode(next, ht);
		next = (HTREEITEM)::SendMessage(hWndDirs, TVM_GETNEXTITEM, TVGN_CHILD, (LPARAM)item);
	}
	::SendMessage(hWndDirs, TVM_DELETEITEM, 0, (LPARAM)item);
}

LRESULT QueueFrame::onCustomDraw(LPNMHDR pnmh, bool& bHandled) {
	LPNMLVCUSTOMDRAW cd = (LPNMLVCUSTOMDRAW)pnmh;

	if(!BOOLSETTING(SHOW_PROGRESS_BARS)) {
		if (cd->nmcd.dwDrawStage != (CDDS_ITEMPREPAINT)) {
			bHandled = false;
			return 0;
		}
	}

	switch(cd->nmcd.dwDrawStage) {
		case CDDS_PREPAINT:
			return CDRF_NOTIFYITEMDRAW;
		case CDDS_ITEMPREPAINT: {
			QueueItemInfo *ii = (QueueItemInfo*)cd->nmcd.lItemlParam;
			if(ii->getErrTxt() != (TSTRING(NO_ERRORS))) {
				cd->clrText = SETTING(ERROR_COLOR);
				return CDRF_NEWFONT | CDRF_NOTIFYSUBITEMDRAW;
			} else if(ii->getPriority() == QueueItem::PAUSED) {
				cd->clrText = SETTING(PAUSED_COLOR);
				return CDRF_NEWFONT | CDRF_NOTIFYSUBITEMDRAW;
			}
			return CDRF_NOTIFYSUBITEMDRAW;
		}
		case CDDS_SUBITEM | CDDS_ITEMPREPAINT: {
			// Let's draw a box if needed...
			RECT rc;
			int colIndex = ctrlQueue.findColumn(cd->iSubItem);
			if(colIndex == COLUMN_DOWNLOADED) {
				QueueItemInfo *qi = (QueueItemInfo*)cd->nmcd.lItemlParam;
				// draw something nice...
				TCHAR buf[256];
				COLORREF barBase = SETTING(DOWNLOAD_BAR_COLOR);
				COLORREF bgBase = WinUtil::bgColor;
				int mod = (HLS_L(RGB2HLS(bgBase)) >= 128) ? -30 : 30;
				COLORREF barPal[3] = { HLS_TRANSFORM(barBase, -40, 50), barBase, HLS_TRANSFORM(barBase, 40, -30) };
				COLORREF bgPal[2] = { HLS_TRANSFORM(bgBase, mod, 0), HLS_TRANSFORM(bgBase, mod/2, 0) };


				ListViewGetItemText(ctrlQueue.m_hWnd, (int)cd->nmcd.dwItemSpec, cd->iSubItem, buf, 255);
				buf[255] = 0;

				rc.top = cd->iSubItem;
				rc.left = LVIR_BOUNDS;
				::SendMessage(ctrlQueue.m_hWnd, LVM_GETSUBITEMRECT, (int)cd->nmcd.dwItemSpec, (LPARAM)&rc);

				RECT rc2 = rc;
				rc2.left += 6;

				// draw background
				HGDIOBJ oldpen = ::SelectObject(cd->nmcd.hdc, CreatePen(PS_SOLID,0,bgPal[0]));
				HGDIOBJ oldbr = ::SelectObject(cd->nmcd.hdc, CreateSolidBrush(bgPal[1]));
				::Rectangle(cd->nmcd.hdc, rc.left, rc.top - 1, rc.right, rc.bottom);			

				rc.left += 1;
				rc.right -= 1;
				rc.bottom -= 1;

				LONG left = rc.left;
				int64_t w = rc.right - rc.left;
				// draw start part
				double percent = (qi->getSize() > 0) ? (double)((double)qi->getDownloadedBytes()) / ((double)qi->getSize()) : 0;
				rc.right = left + (int) (w * percent);
				DeleteObject(SelectObject(cd->nmcd.hdc, CreateSolidBrush(barPal[0])));
				DeleteObject(SelectObject(cd->nmcd.hdc, CreatePen(PS_SOLID,0,barPal[0])));

				::Rectangle(cd->nmcd.hdc, rc.left, rc.top, rc.right, rc.bottom);
					
				// draw progressbar highlight
				if(rc.right - rc.left > 2) {
					DeleteObject(SelectObject(cd->nmcd.hdc, CreatePen(PS_SOLID,1,barPal[2])));

					rc.top += 2;
					::MoveToEx(cd->nmcd.hdc,rc.left+1,rc.top,(LPPOINT)NULL);
					::LineTo(cd->nmcd.hdc,rc.right-2,rc.top);
				};

				// draw status text
				DeleteObject(::SelectObject(cd->nmcd.hdc, oldpen));
				DeleteObject(::SelectObject(cd->nmcd.hdc, oldbr));

				LONG right = rc2.right;
				left = rc2.left;
				rc2.right = rc.right;
				LONG top = rc2.top + ((rc2.bottom - rc2.top) - WinUtil::getTextHeight(cd->nmcd.hdc) - 1)/2;
				int textcolor = SETTING(PROGRESS_TEXT_COLOR_DOWN);
				SetBkMode(cd->nmcd.hdc, TRANSPARENT);
				SetTextColor(cd->nmcd.hdc, textcolor);
				::ExtTextOut(cd->nmcd.hdc, left, top, ETO_CLIPPED, &rc2, buf, (uint32_t)_tcslen(buf), NULL);

				rc2.left = rc2.right;
				rc2.right = right;

				SetTextColor(cd->nmcd.hdc, WinUtil::textColor);
				::ExtTextOut(cd->nmcd.hdc, left, top, ETO_CLIPPED, &rc2, buf, (uint32_t)_tcslen(buf), NULL);

				return CDRF_SKIPDEFAULT;
			} else if(colIndex == COLUMN_STATUS) {
				QueueItemInfo *qi = (QueueItemInfo*)cd->nmcd.lItemlParam;
				rc.top = cd->iSubItem;
				rc.left = LVIR_BOUNDS;
				::SendMessage(ctrlQueue.m_hWnd, LVM_GETSUBITEMRECT, (int)cd->nmcd.dwItemSpec, (LPARAM)&rc);
				COLORREF color;
				if((UINT)::SendMessage(ctrlQueue.m_hWnd, LVM_GETITEMSTATE, (int)cd->nmcd.dwItemSpec, LVIS_SELECTED) & LVIS_SELECTED) {
					if(ctrlQueue.m_hWnd == ::GetFocus()) {
						color = GetSysColor(COLOR_HIGHLIGHT);
						SetBkColor(cd->nmcd.hdc, GetSysColor(COLOR_HIGHLIGHT));
						SetTextColor(cd->nmcd.hdc, GetSysColor(COLOR_HIGHLIGHTTEXT));
					} else {
						color = GetBkColor(cd->nmcd.hdc);
						SetBkColor(cd->nmcd.hdc, color);
					}
				} else {
					color = WinUtil::bgColor;
					SetBkColor(cd->nmcd.hdc, WinUtil::bgColor);
					SetTextColor(cd->nmcd.hdc, WinUtil::textColor);
				}

				RECT rc2 = rc;
				rc2.left += 2;
				HGDIOBJ oldpen = ::SelectObject(cd->nmcd.hdc, CreatePen(PS_SOLID,0, color));
				HGDIOBJ oldbr = ::SelectObject(cd->nmcd.hdc, CreateSolidBrush(color));
				Rectangle(cd->nmcd.hdc,rc.left, rc.top, rc.right, rc.bottom);
				DeleteObject(::SelectObject(cd->nmcd.hdc, oldpen));
				DeleteObject(::SelectObject(cd->nmcd.hdc, oldbr));
				TCHAR buf[256];
				ListViewGetItemText(ctrlQueue.m_hWnd, (int)cd->nmcd.dwItemSpec, cd->iSubItem, buf, 255);
				buf[255] = 0;

				if(qi != NULL) {
					if(qi->getStatus() == QueueItem::STATUS_WAITING) {
						int iOnline = 0;
						for(QueueItem::SourceIter j = qi->getSources().begin(); j != qi->getSources().end(); ++j) {	
							if(j->getUser()->isOnline())
								iOnline++;
						}

						if(qi->getPriority() == QueueItem::PAUSED) {
							DrawIconEx(cd->nmcd.hdc, rc.left, rc.top, hIcPaused, 16, 16, NULL, NULL, DI_NORMAL | DI_COMPAT);
						} else if(iOnline > 0) {
							DrawIconEx(cd->nmcd.hdc, rc.left, rc.top, hIcWaiting, 16, 16, NULL, NULL, DI_NORMAL | DI_COMPAT);
						} else if(qi->getSources().size() > 1) {
							DrawIconEx(cd->nmcd.hdc, rc.left, rc.top, hIcUsersOffline, 16, 16, NULL, NULL, DI_NORMAL | DI_COMPAT);
						} else if(qi->getSources().size() == 1) {
							DrawIconEx(cd->nmcd.hdc, rc.left, rc.top, hIcUserOffline, 16, 16, NULL, NULL, DI_NORMAL | DI_COMPAT);
						} else {
							DrawIconEx(cd->nmcd.hdc, rc.left, rc.top, hIcNoUser, 16, 16, NULL, NULL, DI_NORMAL | DI_COMPAT);
						}
					} else if(qi->getStatus() == QueueItem::STATUS_RUNNING) {
						DrawIconEx(cd->nmcd.hdc, rc.left, rc.top, hIcRunning, 16, 16, NULL, NULL, DI_NORMAL | DI_COMPAT);
					}
				}

				LONG top = rc2.top + ((rc2.bottom - rc2.top) - WinUtil::getTextHeight(cd->nmcd.hdc) - 1)/2;
				::ExtTextOut(cd->nmcd.hdc, rc2.left + 20, top, ETO_CLIPPED, &rc2, buf, (uint32_t)_tcslen(buf), NULL);

				return CDRF_NEWFONT | CDRF_SKIPDEFAULT;
			}
		}
		default:
			return CDRF_DODEFAULT;
	}
}

void QueueFrame::onPreviewCommand(WORD wID) {
	if((UINT)::SendMessage(ctrlQueue.m_hWnd, LVM_GETSELECTEDCOUNT, 0, 0) == 1) {
		string sTempTarget = QueueManager::getInstance()->getTempTarget(ctrlQueue.getItemData((int)::SendMessage(ctrlQueue.m_hWnd, LVM_GETNEXTITEM, (WPARAM)-1, LVNI_SELECTED))->getTarget());
		if(sTempTarget.empty() == false) {
			WinUtil::RunPreviewCommand(wID - IDC_PREVIEW_APP, sTempTarget);
		}
	}
}

void QueueFrame::onRemoveOffline() {
	if(usingDirMenu == false) {
        int i = -1;
		while((i = (int)::SendMessage(ctrlQueue.m_hWnd, LVM_GETNEXTITEM, i, LVNI_SELECTED)) != -1) {
            QueueManager::getInstance()->removeOffline(ctrlQueue.getItemData(i)->getTarget());
        }
	}
}

void QueueFrame::on(SettingsManagerListener::Save, SimpleXML& /*xml*/) throw() {
	::PostMessage(m_hWnd, WM_REFRESH_SETTINGS, 0, 0);
}

void QueueFrame::onRefreshSettings() {
	bool refresh = false;

	if((COLORREF)::SendMessage(ctrlQueue.m_hWnd, LVM_GETBKCOLOR, 0, 0) != WinUtil::bgColor) {
		::SendMessage(ctrlQueue.m_hWnd, LVM_SETBKCOLOR, 0, WinUtil::bgColor);
		::SendMessage(ctrlQueue.m_hWnd, LVM_SETTEXTBKCOLOR, 0, WinUtil::bgColor);
		::SendMessage(hWndDirs, TVM_SETBKCOLOR, 0, (LPARAM)WinUtil::bgColor);
		refresh = true;
	}
	if((COLORREF)::SendMessage(ctrlQueue.m_hWnd, LVM_GETTEXTCOLOR, 0, 0) != WinUtil::textColor) {
		::SendMessage(ctrlQueue.m_hWnd, LVM_SETTEXTCOLOR, 0, WinUtil::textColor);
		::SendMessage(hWndDirs, TVM_SETTEXTCOLOR, 0, (LPARAM)WinUtil::textColor);
		refresh = true;
	}

	if(refresh == true) {
		::RedrawWindow(m_hWnd, NULL, NULL, RDW_ERASE | RDW_INVALIDATE | RDW_UPDATENOW | RDW_ALLCHILDREN);
	}
}

void QueueFrame::onClearErrors() {
	if(usingDirMenu == true) {
        clearErrors((HTREEITEM)::SendMessage(hWndDirs, TVM_GETNEXTITEM, TVGN_CARET, 0));
	} else {
		int i = -1;
		while((i = (int)::SendMessage(ctrlQueue.m_hWnd, LVM_GETNEXTITEM, i, LVNI_SELECTED)) != -1) {
		    QueueManager::getInstance()->clearErrors(ctrlQueue.getItemData(i)->getTarget());
		}
	}
}

void QueueFrame::clearErrors(HTREEITEM ht) {
	if(ht == NULL)
		return;

	HTREEITEM child = (HTREEITEM)::SendMessage(hWndDirs, TVM_GETNEXTITEM, TVGN_CHILD, (LPARAM)ht);
	while(child) {
		clearErrors(child);
		child = (HTREEITEM)::SendMessage(hWndDirs, TVM_GETNEXTITEM, TVGN_NEXT, (LPARAM)child);
	}
	const string& name = getDir(ht);
	DirectoryPair dp = directories.equal_range(name);
	for(DirectoryIter i = dp.first; i != dp.second; ++i) {
		QueueManager::getInstance()->clearErrors(i->second->getTarget());
	}
}

void QueueFrame::onShowTree(WPARAM wParam) {
	showTree = (wParam == BST_CHECKED);

    if(showTree == true) {
        bSplitterDisabled = false;
        ::ShowWindow(hWndDirs, SW_SHOW);
        ::EnableWindow(hWndDirs, TRUE);
    } else {
        bSplitterDisabled = true;
        ::ShowWindow(hWndDirs, SW_HIDE);
        ::EnableWindow(hWndDirs, FALSE);
    }

    UpdateSplitterParts();

    updateQueue();
}

HWND QueueFrame::GetWindowHandle() {
    return m_hWnd;
}
//--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

void QueueFrame::UpdateSplitterParts() {
    if(bSplitterDisabled == false) {
        ::SetWindowPos(hWndDirs, NULL, rcSplitter.left, rcSplitter.top, iSplitterPos-2, rcSplitter.bottom-rcSplitter.top, SWP_NOZORDER);
        ::SetWindowPos(ctrlQueue.m_hWnd, NULL, rcSplitter.left+(iSplitterPos+2), rcSplitter.top, (rcSplitter.right-rcSplitter.left)-(iSplitterPos+2), rcSplitter.bottom-rcSplitter.top,
            SWP_NOZORDER);
    } else {
        ::SetWindowPos(ctrlQueue.m_hWnd, NULL, rcSplitter.left, rcSplitter.top, rcSplitter.right-rcSplitter.left, rcSplitter.bottom-rcSplitter.top, SWP_NOZORDER);
    }
}
//--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

/*
 * Some parts Copyright (C) 2002-2011 Petr Kozelka, PPK at czdc dot org
 * Licensed under GNU General Public License version 3.
 */
