/*
 * CzDC - client for Direct Connect peer to peer network.

 * Copyright (C) 2002-2011 Petr Kozelka, PPK at CzDC dot org

 * CzDC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3
 * as published by the Free Software Foundation.

 * CzDC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with CzDC. If not, see <http://www.gnu.org/licenses/>.
 */

//----------------------------------------------------------------------------------------------------------------------------------------------------
#ifndef SEARCH_FRAME_H
#define SEARCH_FRAME_H
//----------------------------------------------------------------------------------------------------------------------------------------------------
#pragma once
//----------------------------------------------------------------------------------------------------------------------------------------------------
#include "../client/Client.h"
#include "../client/ClientManagerListener.h"
#include "../client/SearchManager.h"
#include "../client/QueueItem.h"
//----------------------------------------------------------------------------------------------------------------------------------------------------
#include "MDIChildWindow.h"
#include "SearchInfo.h"
#include "TreeListViewCtrl.h"
#include "TypedListViewCtrl.h"
#include "UCHandler.h"
//----------------------------------------------------------------------------------------------------------------------------------------------------

class SearchFrame : private MDITabChildWindow, private UCHandler<SearchFrame>, private ClientManagerListener, private SearchManagerListener, private SettingsManagerListener,
    private TimerManagerListener {
public:
    HWND hWindowItems[17];

    SearchFrame();
    ~SearchFrame();

    LRESULT SearchFrameWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);

    static void OpenWindow(tstring tsInitialString = Util::emptyStringT, uint64_t ui64InitialSize = 0, SearchManager::SizeModes smInitialMode = SearchManager::SIZE_ATLEAST,
        SearchManager::TypeModes tmInitialType = SearchManager::TYPE_ANY);
    static void CloseAllWindows();

    void runUserCommand(UserCommand & uc);
private:
    uint64_t ui64ResultsCount, ui64ResultsDropped, ui64ResultsPaused;

    UINT_PTR uipTimer;

    TreeListViewCtrl<SearchInfo> tlvResults;

	HMENU hResultsMenu;
	HMENU hTargetMenu;
	HMENU hTargetDirMenu;

    TStringList SearchWordsList;

    StringList sTargetsList;

    SearchInfo * pPausedResults, * pNewResults;

    SearchFrame * pPrevSearchFrame, * pNextSearchFrame;

    SearchManager::TypeModes smtmInitialType;

    CriticalSection csSearch;

	static int iColumnIndexes[];

    bool bClosed, bExpandGroups, bOnlyFreeSlots, bSearchingHash, bSearchOnlyTTH, bSearchPaused, bShowSearchUI, bUserWarned;

    enum {
        GB_SEARCH_FOR,
        CB_SEARCH_STRING,
        GB_FILE_TYPE,
        CB_FILE_TYPE,
        GB_SIZE,
        CB_SIZE_MODE,
        EDT_SIZE,
        CB_SIZE_UNITS,
        BTN_SEARCH,
        BTN_PAUSE_SEARCH,
        GB_SEARCH_OPTIONS,
        CHB_HIDE_OPTIONS,
        CHB_ONLY_FREE_SLOTS,
        CHB_ONLY_WITH_HASH,
        CHB_EXPAND_GROUPS,
        GB_HUBS,
        CHB_SHOW_SEARCH_OPTIONS
    };

    enum {
        ADD_SEARCH_RESULTS,
        SEARCH_RESULTS_FILTERED,
        ADD_HUB,
        CHANGE_HUB,
        REMOVE_HUB
    };

    struct HubInfo {
        tstring tsHubUrl;
        tstring tsHubName;

        bool bOperator;

        HubInfo(const tstring & tsUrl, const tstring & tsName, const bool & bOP) : tsHubUrl(tsUrl), tsHubName(tsName), bOperator(bOP) { }
        ~HubInfo() { }

        const TCHAR * getText(const int & iColumn, TCHAR * /*tpBuf*/) const {
            return (iColumn == 0) ? tsHubName.c_str() : Util::emptyStringT.c_str();
        }

        uint8_t imageIndex() const {
            return 0;
        }

        static int compareItems(const HubInfo * pFirst, const HubInfo * pSecond, const int & iColumn) {
            return (iColumn == 0) ? _tcsicmp(pFirst->tsHubName.c_str(), pSecond->tsHubName.c_str()) : 0;
        }
    };

	TypedListViewCtrl<HubInfo, IDC_HUB> lvHubs;

    struct ResultsCheck {
        int64_t i64Size;
        string sHub;
        tstring tsExtension, tsHash;
        bool bFirstHash, bHasHash, bOneHub, bOperator;


        ResultsCheck() : i64Size(-1), bOperator(true), bOneHub(true), bHasHash(false), bFirstHash(true) { }
        ~ResultsCheck() { }
    };

    static LRESULT CALLBACK FrameWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
    static LRESULT CALLBACK KeyPressCheckWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
    static LRESULT CALLBACK SearchUIWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);

    void OnCreate();
    bool OnClose();
    void OnTimer();
    void OnSize();
    void UpdateSearchUI();

    bool OnChar(UINT uMsg, WPARAM wParam);
    bool OnContextMenu(WPARAM wParam, LPARAM lParam);
    void OnDoubleClickResults(LPARAM lParam);
    void OnPauseButton();

    LRESULT OnCtlColor(WPARAM wParam, LPARAM lParam);
    LRESULT OnCustomDraw(LPARAM lParam);

    void OnSpeaker(WPARAM wParam, LPARAM lParam);
    void OnRefreshSettings();

    void OnCopy(WORD wID);
    void OnGetFileList();
    void OnMatchQueue();
    void OnSearchForAlternates();
    void OnViewAsText();

    void OnDownloadBrowseTo();
    void OnDownloadToFavoriteDir(WORD wID);
    void OnDownloadToTarget(WORD wID);
    void OnDownloadWithPriority(WORD wID);
    void OnDownloadWholeBrowseTo();
    void OnDownloadWholeToFavoriteDir(WORD wID);
    void OnDownloadWholeToTarget(WORD wID);

    void AddDownload(SearchInfo * si, const tstring& tsDir, const bool &bWhole, QueueItem::Priority Priority = QueueItem::DEFAULT);
    void AddResult(SearchInfo * si);

    void CheckSelectedResults(ResultsCheck * rc);
    void CheckResult(ResultsCheck * rc, SearchInfo * si);

	void DownloadSelected(const tstring& tsDir, const bool &bWhole, QueueItem::Priority Priority = QueueItem::DEFAULT);
	void DownloadWholeSelected(const tstring& tsDir);

    void OnEnter();
    void OnTab(const bool & bShift);

    void runUserCommand(UserCommand& uc, SearchResult * sr, StringMap &ucParams, set<CID> &users);

    void AddHub(HubInfo * pInfo);
    void ChangeHub(HubInfo * pInfo);
    void RemoveHub(HubInfo * pInfo);

    void OnHubItemChanged(LPARAM lParam);

    void on(ClientManagerListener::ClientConnected, Client * pClient) throw();
    void on(ClientManagerListener::ClientUpdated, Client * pClient) throw();
    void on(ClientManagerListener::ClientDisconnected, Client * pClient) throw();

    void on(SearchManagerListener::SR, SearchResult * pResult) throw();

    void on(SettingsManagerListener::Save, SimpleXML&) throw();

    void on(TimerManagerListener::Second, uint64_t /*aTick*/) throw();

    bool IsDupeResult(SearchInfo * si);
    void UpdateStatusBarResultsCount();
};
//----------------------------------------------------------------------------------------------------------------------------------------------------

#endif
