/* 
 * Copyright (C) 2003 Twink,  spm7@waikato.ac.nz
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "stdafx.h"
#include "../client/DCPlusPlus.h"
#include "Resource.h"

#include "ToolbarPage.h"
#include "../client/SettingsManager.h"
#include "../client/StringTokenizer.h"
#include "../client/Util.h"
#include "WinUtil.h"
#include "MainFrm.h"

ToolbarPage * pToolbarPage = NULL;

ToolbarPage::ToolbarPage(SettingsManager *s) : PropPage(s, IDD_TOOLBARPAGE) {
    pToolbarPage = this;
	title = _tcsdup((TSTRING(SETTINGS_APPEARANCE) + _T('\\') + TSTRING(SETTINGS_TOOLBAR)).c_str());
	SetTitle(title);
	m_psp.pfnDlgProc = (DLGPROC)ToolbarPageProc;
}

ToolbarPage::~ToolbarPage() {
    pToolbarPage = NULL;
	free(title);
}

BOOL CALLBACK ToolbarPage::ToolbarPageProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM /*lParam*/) {
    if(uMsg == WM_INITDIALOG) {
        pToolbarPage->m_hWnd = hWnd;
        pToolbarPage->onInitDialog();
        return TRUE;
    } else if(uMsg == WM_COMMAND) {
        if(HIWORD(wParam) == BN_CLICKED) {
            if(LOWORD(wParam) == IDC_TOOLBAR_ADD) {
                pToolbarPage->onAdd();
                return TRUE;
            } else if(LOWORD(wParam) == IDC_TOOLBAR_REMOVE) {
                pToolbarPage->onRemove();
                return TRUE;
            }
        }
    }

	return FALSE;
}		

PropPage::TextItem ToolbarPage::texts[] = {
	{ IDC_TOOLBAR_ADD, ResourceManager::SETTINGS_TOOLBAR_ADD },
	{ IDC_TOOLBAR_REMOVE, ResourceManager::SETTINGS_TOOLBAR_REMOVE },
	{ IDC_TOOLBAR_AVAILABLE, ResourceManager::SETTINGS_TOOLBAR_AVAILABLE },
	{ IDC_TOOLBAR_CURRENT, ResourceManager::SETTINGS_TOOLBAR_CURRENT },
	{ 0, ResourceManager::SETTINGS_AUTO_AWAY }
};

PropPage::Item ToolbarPage::items[] = {
	{ 0, 0, PropPage::T_END }
};


string ToolbarPage::filter(string s){
	s = Util::replace(s, "&","");
	s = s.substr(0,s.find("\t"));
	return s;
}

void ToolbarPage::onInitDialog() {
	PropPage::translate(m_hWnd, texts);
	PropPage::read(m_hWnd, items);

	ListViewInsertDummyColumn(::GetDlgItem(m_hWnd, IDC_TOOLBAR_POSSIBLE));
	ListViewInsertDummyColumn(::GetDlgItem(m_hWnd, IDC_TOOLBAR_ACTUAL));

	::SendMessage(::GetDlgItem(m_hWnd, IDC_TOOLBAR_POSSIBLE), LVM_SETIMAGELIST, LVSIL_SMALL, (LPARAM)MainFrame::pMainFrame->largeImagesHot);
	::SendMessage(::GetDlgItem(m_hWnd, IDC_TOOLBAR_ACTUAL), LVM_SETIMAGELIST, LVSIL_SMALL, (LPARAM)MainFrame::pMainFrame->largeImagesHot);

	LVITEM lvi;
	lvi.mask = LVIF_TEXT | LVIF_IMAGE;
	lvi.iSubItem = 0;

	for(int i = -1; i < 0 || WinUtil::ToolbarButtons[i].id != 0; i++) {
		makeItem(&lvi, i);
		lvi.iItem = i+1;
		::SendMessage(::GetDlgItem(m_hWnd, IDC_TOOLBAR_POSSIBLE), LVM_INSERTITEM, 0, (LPARAM)&lvi);

		ListViewSetItem(::GetDlgItem(m_hWnd, IDC_TOOLBAR_POSSIBLE), lvi.iItem, 0, LVIF_PARAM, NULL, 0, i);
	}

	::SendMessage(::GetDlgItem(m_hWnd, IDC_TOOLBAR_POSSIBLE), LVM_SETCOLUMNWIDTH, 0, LVSCW_AUTOSIZE);
	
	StringTokenizer<string> t(SETTING(TOOLBAR), ',');
	StringList& l = t.getTokens();

	int n = 0;
	for(StringList::const_iterator k = l.begin(); k != l.end(); ++k) {
		int i = Util::toInt(*k);
		makeItem(&lvi, i);
		lvi.iItem = n++;
		::SendMessage(::GetDlgItem(m_hWnd, IDC_TOOLBAR_ACTUAL), LVM_INSERTITEM, 0, (LPARAM)&lvi);

		ListViewSetItem(::GetDlgItem(m_hWnd, IDC_TOOLBAR_ACTUAL), lvi.iItem, 0, LVIF_PARAM, NULL, 0, i);
	}

	::SendMessage(::GetDlgItem(m_hWnd, IDC_TOOLBAR_ACTUAL), LVM_SETCOLUMNWIDTH, 0, LVSCW_AUTOSIZE);
}

void ToolbarPage::write() {
	PropPage::write(m_hWnd, items);
	string toolbar;
	for(int i = 0; i < (int)::SendMessage(::GetDlgItem(m_hWnd, IDC_TOOLBAR_ACTUAL), LVM_GETITEMCOUNT, 0, 0); i++) {
		if(i != 0) toolbar += ",";

		int j = (int)ListViewGetItem(::GetDlgItem(m_hWnd, IDC_TOOLBAR_ACTUAL), i);

		toolbar += Util::toString(j);
	}
	if(toolbar != settings->get(SettingsManager::TOOLBAR)) {
		settings->set(SettingsManager::TOOLBAR, toolbar);
		::SendMessage(WinUtil::mainWnd, IDC_REBUILD_TOOLBAR, 0, 0);
	}
}

void ToolbarPage::makeItem(LPLVITEM lvi, int item){
	if(item > -1 && item < 22) {
		lvi->iImage = WinUtil::ToolbarButtons[item].image;
		name = Text::toT(filter(ResourceManager::getInstance()->getString(WinUtil::ToolbarButtons[item].tooltip)));			
	} else {
		name = TSTRING(SEPARATOR);
		lvi->iImage = -1;
	}	
	lvi->pszText = const_cast<TCHAR*>(name.c_str());	
}

void ToolbarPage::onAdd() {
	if((UINT)::SendMessage(::GetDlgItem(m_hWnd, IDC_TOOLBAR_POSSIBLE), LVM_GETSELECTEDCOUNT, 0, 0) == 1) {
		LVITEM lvi = { 0 };
		lvi.mask = LVIF_TEXT | LVIF_IMAGE;
		lvi.iSubItem = 0;
		int i = (int)ListViewGetItem(::GetDlgItem(m_hWnd, IDC_TOOLBAR_POSSIBLE),
			(int)::SendMessage(::GetDlgItem(m_hWnd, IDC_TOOLBAR_POSSIBLE), LVM_GETNEXTITEM, (WPARAM)-1, LVNI_ALL | LVNI_SELECTED));
		makeItem(&lvi, i);
		lvi.iItem = (int)::SendMessage(::GetDlgItem(m_hWnd, IDC_TOOLBAR_ACTUAL), LVM_GETNEXTITEM, (WPARAM)-1, LVNI_ALL | LVNI_SELECTED) + 1;
		::SendMessage(::GetDlgItem(m_hWnd, IDC_TOOLBAR_ACTUAL), LVM_INSERTITEM, 0, (LPARAM)&lvi);

		ListViewSetItem(::GetDlgItem(m_hWnd, IDC_TOOLBAR_ACTUAL), lvi.iItem, 0, LVIF_PARAM, NULL, 0, i);
	}
}

void ToolbarPage::onRemove() {
	if((UINT)::SendMessage(::GetDlgItem(m_hWnd, IDC_TOOLBAR_ACTUAL), LVM_GETSELECTEDCOUNT, 0, 0) == 1) {
		int sel = (int)::SendMessage(::GetDlgItem(m_hWnd, IDC_TOOLBAR_ACTUAL), LVM_GETNEXTITEM, (WPARAM)-1, LVNI_ALL | LVNI_SELECTED);
		::SendMessage(::GetDlgItem(m_hWnd, IDC_TOOLBAR_ACTUAL), LVM_DELETEITEM, sel, 0);

		ListViewSelectItem(::GetDlgItem(m_hWnd, IDC_TOOLBAR_ACTUAL), max(sel-1, 0));
	}
}

/*
 * Some parts Copyright (C) 2004-2011 Petr Kozelka, PPK at czdc dot org
 * Licensed under GNU General Public License version 3.
 */
