/*
 * Copyright (C) 2001-2011 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef UC_HANDLER_H
#define UC_HANDLER_H

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "../client/FavoriteManager.h"
#include "../client/StringTokenizer.h"

template<class T>
class UCHandler {
public:
	UCHandler() : menuPos(0) { }

    bool UCHandlerProc(UINT uMsg, WPARAM wParam) {
        if(uMsg == WM_COMMAND && LOWORD(wParam) >= IDC_USER_COMMAND  && LOWORD(wParam) <= IDC_USER_COMMAND + userCommands.size()) {
            onUserCommand(LOWORD(wParam));
            return true;
        }

    	return false;
    }

	void onUserCommand(WORD wID) {
		dcassert(wID >= IDC_USER_COMMAND);
		size_t n = (size_t)wID - IDC_USER_COMMAND;
		dcassert(n < userCommands.size());

		UserCommand& uc = userCommands[n];

		T* t = static_cast<T*>(this);

		t->runUserCommand(uc);
	}

	void prepareMenu(HMENU menu, int ctx, const string& server, bool op) {
		userCommands = FavoriteManager::getInstance()->getUserCommands(ctx, server, op);
		int n = 0;

		menuPos = ::GetMenuItemCount(menu);
		if(!userCommands.empty()) {
            ::AppendMenu(menu, MF_SEPARATOR, 0, NULL);
			HMENU cur = menu;
			for(UserCommand::List::iterator ui = userCommands.begin(); ui != userCommands.end(); ++ui) {
				UserCommand& uc = *ui;
				if(uc.getType() == UserCommand::TYPE_SEPARATOR) {
					// Avoid double separators...
					if(::GetMenuItemCount(cur) >= 1 &&
						!(::GetMenuState(cur, ::GetMenuItemCount(cur)-1, MF_BYPOSITION) & MF_SEPARATOR)) {
						::AppendMenu(cur, MF_SEPARATOR, 0, NULL);
					}
				} else if(uc.getType() == UserCommand::TYPE_RAW || uc.getType() == UserCommand::TYPE_RAW_ONCE) {
					cur = menu;
					StringTokenizer<tstring> t(Text::toT(uc.getName()), _T('\\'));
					for(TStringIter i = t.getTokens().begin(); i != t.getTokens().end(); ++i) {
						if(i+1 == t.getTokens().end()) {
                            ::AppendMenu(cur, MF_STRING, IDC_USER_COMMAND + n, i->c_str());
						} else {
							bool found = false;
							TCHAR buf[1024];
							// Let's see if we find an existing item...
							for(int k = 0; k < ::GetMenuItemCount(cur); k++) {
								if(::GetMenuState(cur, k, MF_BYPOSITION) & MF_POPUP) {
                                    ::GetMenuString(cur, k, buf, 1024, MF_BYPOSITION);
									if(Util::stricmp(buf, i->c_str()) == 0) {
										found = true;
										cur = ::GetSubMenu(cur, k);
									}
								}
							}
							if(!found) {
								HMENU m = ::CreatePopupMenu();
								::AppendMenu(cur, MF_POPUP, (UINT_PTR)m, i->c_str());
								cur = m;
							}
						}
					}
				} else {
					dcasserta(0);
				}
				n++;
			}
		}
	}
	void cleanMenu(HMENU menu) {
		if(!userCommands.empty()) {
			for(size_t i = 0; i < userCommands.size()+1; ++i) {
                ::DeleteMenu(menu, menuPos, MF_BYPOSITION);
			}
		}
	}
private:
	UserCommand::List userCommands;
	int menuPos;
};

#endif // UC_HANDLER_H

/*
 * Some parts Copyright (C) 2002-2011 Petr Kozelka, PPK at czdc dot org
 * Licensed under GNU General Public License version 3.
 */
