/*
 * Copyright (C) 2001-2011 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "stdafx.h"
#include "../client/DCPlusPlus.h"
#include "Resource.h"

#include "UCPage.h"
#include "CommandDlg.h"

#include "../client/SettingsManager.h"
#include "../client/FavoriteManager.h"
#include "WinUtil.h"

UCPage * pUCPage = NULL;

UCPage::UCPage(SettingsManager *s) : PropPage(s, IDD_UCPAGE) {
    pUCPage = this;
	title = _tcsdup((TSTRING(SETTINGS_ADVANCED) + _T('\\') + TSTRING(SETTINGS_USER_COMMANDS)).c_str());
	SetTitle(title);
	m_psp.pfnDlgProc = (DLGPROC)UCPageProc;
}

UCPage::~UCPage() {
    pUCPage = NULL;
	ctrlCommands.m_hWnd = NULL;
	free(title);
}

BOOL CALLBACK UCPage::UCPageProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    switch(uMsg) {
        case WM_INITDIALOG:
            pUCPage->m_hWnd = hWnd;
            pUCPage->onInitDialog();
            return TRUE;
        case WM_COMMAND:
            switch(LOWORD(wParam)) {
                case IDC_ADD_MENU:
                    pUCPage->onAddMenu();
                    return TRUE;
                case IDC_REMOVE_MENU:
                    pUCPage->onRemoveMenu();
                    return TRUE;
                case IDC_CHANGE_MENU:
                    pUCPage->onChangeMenu();
                    return TRUE;
                case IDC_MOVE_UP:
                    pUCPage->onMoveUp();
                    return TRUE;
                case IDC_MOVE_DOWN:
                    pUCPage->onMoveDown();
                    return TRUE;
            }

            break;
        case WM_NOTIFY:
            if(((LPNMHDR)lParam)->idFrom == IDC_MENU_ITEMS) {
                switch(((LPNMHDR)lParam)->code) {
                    case LVN_KEYDOWN:
                        if(pUCPage->onKeyDown((LPNMHDR)lParam) == true) {
                            return TRUE;
                        }

                        return FALSE;
                    case NM_DBLCLK:
                        pUCPage->onDoubleClick((LPNMHDR)lParam);
                        return TRUE;
                    case LVN_ITEMCHANGED:
                        pUCPage->onItemchanged((LPNMHDR)lParam);
                        return TRUE;
                }
            }

            break;
    }

	return FALSE;
}

PropPage::TextItem UCPage::texts[] = {
	{ IDC_MOVE_UP, ResourceManager::MOVE_UP },
	{ IDC_MOVE_DOWN, ResourceManager::MOVE_DOWN },
	{ IDC_ADD_MENU, ResourceManager::ADD },
	{ IDC_CHANGE_MENU, ResourceManager::SETTINGS_CHANGE },
	{ IDC_REMOVE_MENU, ResourceManager::REMOVE },
	{ 0, ResourceManager::SETTINGS_AUTO_AWAY }
};

void UCPage::onInitDialog() {
	PropPage::translate(m_hWnd, texts);

	RECT rc;

	ctrlCommands.m_hWnd = ::GetDlgItem(m_hWnd, IDC_MENU_ITEMS);
	::GetClientRect(ctrlCommands.m_hWnd, &rc);

	ListViewInsertColumn(ctrlCommands.m_hWnd, 0, CTSTRING(SETTINGS_NAME), LVCFMT_LEFT, (rc.right - rc.left) / 4, 0);
	ListViewInsertColumn(ctrlCommands.m_hWnd, 1, CTSTRING(SETTINGS_COMMAND), LVCFMT_LEFT, (rc.right - rc.left)*2 / 4, 1);
	ListViewInsertColumn(ctrlCommands.m_hWnd, 2, CTSTRING(HUB), LVCFMT_LEFT, (rc.right - rc.left) / 4, 2);

	::SendMessage(ctrlCommands.m_hWnd, LVM_SETEXTENDEDLISTVIEWSTYLE, 0, LVS_EX_LABELTIP | LVS_EX_FULLROWSELECT);

	// Do specialized reading here
	UserCommand::List lst = FavoriteManager::getInstance()->getUserCommands();
	for(UserCommand::List::iterator i = lst.begin(); i != lst.end(); ++i) {
		UserCommand& uc = *i;
		if(!uc.isSet(UserCommand::FLAG_NOSAVE)) {
			addEntry(uc, (int)::SendMessage(ctrlCommands.m_hWnd, LVM_GETITEMCOUNT, 0, 0));
		}
	}
}

void UCPage::onAddMenu() {
	CommandDlg dlg;

	if(dlg.DoModal(m_hWnd) == IDOK) {
		addEntry(FavoriteManager::getInstance()->addUserCommand(dlg.type, dlg.ctx,
			0, Text::fromT(dlg.name), Text::fromT(dlg.command), Text::fromT(dlg.hub)), (int)::SendMessage(ctrlCommands.m_hWnd, LVM_GETITEMCOUNT, 0, 0));
	}
}

void UCPage::onChangeMenu() {
	if((UINT)::SendMessage(ctrlCommands.m_hWnd, LVM_GETSELECTEDCOUNT, 0, 0) == 1) {
		int sel = (int)::SendMessage(ctrlCommands.m_hWnd, LVM_GETNEXTITEM, (WPARAM)-1, LVNI_ALL | LVNI_SELECTED);
		UserCommand uc;
		FavoriteManager::getInstance()->getUserCommand((int)ListViewGetItem(ctrlCommands.m_hWnd, sel), uc);
		
		CommandDlg dlg;
		dlg.type = uc.getType();
		dlg.ctx = uc.getCtx();
		dlg.name = Text::toT(uc.getName());
		dlg.command = Text::toT(uc.getCommand());
		dlg.hub = Text::toT(uc.getHub());

		if(dlg.DoModal(m_hWnd) == IDOK) {
			if(dlg.type == UserCommand::TYPE_SEPARATOR) {
                ListViewSetItem(ctrlCommands.m_hWnd, sel, 0, LVIF_TEXT, CTSTRING(SEPARATOR), 0);
			} else {
                ListViewSetItem(ctrlCommands.m_hWnd, sel, 0, LVIF_TEXT, dlg.name.c_str(), 0);
			}
			ListViewSetItem(ctrlCommands.m_hWnd, sel, 1, LVIF_TEXT, dlg.command.c_str(), 0);
            ListViewSetItem(ctrlCommands.m_hWnd, sel, 2, LVIF_TEXT, dlg.hub.c_str(), 0);
			uc.setName(Text::fromT(dlg.name));
			uc.setCommand(Text::fromT(dlg.command));
			uc.setHub(Text::fromT(dlg.hub));
			uc.setType(dlg.type);
			uc.setCtx(dlg.ctx);
			FavoriteManager::getInstance()->updateUserCommand(uc);
		}
	}
}

void UCPage::onRemoveMenu() {
	if((UINT)::SendMessage(ctrlCommands.m_hWnd, LVM_GETSELECTEDCOUNT, 0, 0) == 1) {
		int i = (int)::SendMessage(ctrlCommands.m_hWnd, LVM_GETNEXTITEM, (WPARAM)-1, LVNI_SELECTED);
		FavoriteManager::getInstance()->removeUserCommand((int)ListViewGetItem(ctrlCommands.m_hWnd, i));
		::SendMessage(ctrlCommands.m_hWnd, LVM_DELETEITEM, i, 0);
	}
}

void UCPage::onMoveUp() {
	int i = (int)::SendMessage(ctrlCommands.m_hWnd, LVM_GETNEXTITEM, (WPARAM)-1, LVNI_ALL | LVNI_SELECTED);
	if(i != -1 && i != 0) {
		DWORD_PTR n = ListViewGetItem(ctrlCommands.m_hWnd, i);
		FavoriteManager::getInstance()->moveUserCommand((int)n, -1);
		::SendMessage(ctrlCommands.m_hWnd, WM_SETREDRAW, (WPARAM)FALSE, 0);
		::SendMessage(ctrlCommands.m_hWnd, LVM_DELETEITEM, i, 0);
		UserCommand uc;
		FavoriteManager::getInstance()->getUserCommand((int)n, uc);
		addEntry(uc, i-1);
		ListViewSelectItem(ctrlCommands.m_hWnd, i-1);
		::SendMessage(ctrlCommands.m_hWnd, LVM_ENSUREVISIBLE, i-1, FALSE);
		::SendMessage(ctrlCommands.m_hWnd, WM_SETREDRAW, (WPARAM)TRUE, 0);
	}
}

void UCPage::onMoveDown() {
	int i = (int)::SendMessage(ctrlCommands.m_hWnd, LVM_GETNEXTITEM, (WPARAM)-1, LVNI_ALL | LVNI_SELECTED);
	if(i != -1 && i != ((int)::SendMessage(ctrlCommands.m_hWnd, LVM_GETITEMCOUNT, 0, 0)-1) ) {
		DWORD_PTR n = ListViewGetItem(ctrlCommands.m_hWnd, i);
		FavoriteManager::getInstance()->moveUserCommand((int)n, 1);
		::SendMessage(ctrlCommands.m_hWnd, WM_SETREDRAW, (WPARAM)FALSE, 0);
		::SendMessage(ctrlCommands.m_hWnd, LVM_DELETEITEM, i, 0);
		UserCommand uc;
		FavoriteManager::getInstance()->getUserCommand((int)n, uc);
		addEntry(uc, i+1);
		ListViewSelectItem(ctrlCommands.m_hWnd, i+1);
		::SendMessage(ctrlCommands.m_hWnd, LVM_ENSUREVISIBLE, i+1, FALSE);
		::SendMessage(ctrlCommands.m_hWnd, WM_SETREDRAW, (WPARAM)TRUE, 0);
	}
}

void UCPage::onItemchanged(LPNMHDR pnmh) {
	NM_LISTVIEW* lv = (NM_LISTVIEW*) pnmh;
	::EnableWindow(::GetDlgItem(m_hWnd, IDC_MOVE_UP), (lv->uNewState & LVIS_FOCUSED));
	::EnableWindow(::GetDlgItem(m_hWnd, IDC_MOVE_DOWN), (lv->uNewState & LVIS_FOCUSED));
	::EnableWindow(::GetDlgItem(m_hWnd, IDC_CHANGE_MENU), (lv->uNewState & LVIS_FOCUSED));
	::EnableWindow(::GetDlgItem(m_hWnd, IDC_REMOVE_MENU), (lv->uNewState & LVIS_FOCUSED));
}

bool UCPage::onKeyDown(LPNMHDR pnmh) {
	NMLVKEYDOWN* kd = (NMLVKEYDOWN*) pnmh;
	switch(kd->wVKey) {
        case VK_INSERT:
			::PostMessage(m_hWnd, WM_COMMAND, IDC_ADD_MENU, 0);
            return true;
        case VK_DELETE:
			::PostMessage(m_hWnd, WM_COMMAND, IDC_REMOVE_MENU, 0);
            return true;
        default:
            return false;
	}
}

void UCPage::onDoubleClick(LPNMHDR pnmh) {
	NMITEMACTIVATE* item = (NMITEMACTIVATE*)pnmh;
	if(item->iItem >= 0) {
		::PostMessage(m_hWnd, WM_COMMAND, IDC_CHANGE_MENU, 0);
	} else if(item->iItem == -1) {
		::PostMessage(m_hWnd, WM_COMMAND, IDC_ADD_MENU, 0);
	}
}

void UCPage::addEntry(const UserCommand& uc, int pos) {
	TStringList lst;
	if(uc.getType() == UserCommand::TYPE_SEPARATOR)
		lst.push_back(TSTRING(SEPARATOR));
	else
		lst.push_back(Text::toT(uc.getName()));
	lst.push_back(Text::toT(uc.getCommand()));
	lst.push_back(Text::toT(uc.getHub()));
	ctrlCommands.insert(pos, lst, 0, (LPARAM)uc.getId());
}

/*
 * Some parts Copyright (C) 2002-2011 Petr Kozelka, PPK at czdc dot org
 * Licensed under GNU General Public License version 3.
 */
