/*
 * Copyright (C) 2001-2011 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "stdafx.h"
#include "../client/DCPlusPlus.h"
#include "Resource.h"

#include "UploadPage.h"
#include "WinUtil.h"
#include "HashProgressDlg.h"
#include "LineDlg.h"

#include "../client/Util.h"
#include "../client/ShareManager.h"
#include "../client/SettingsManager.h"
#include "../client/version.h"

UploadPage * pUploadPage = NULL;

UploadPage::UploadPage(SettingsManager *s) : PropPage(s, IDD_UPLOADPAGE) {
    pUploadPage = this;
	SetTitle(CTSTRING(SETTINGS_UPLOADS));
	m_psp.pfnDlgProc = (DLGPROC)UploadPageProc;
}

UploadPage::~UploadPage() {
    pUploadPage = NULL;
	ctrlDirectories.m_hWnd = NULL;
}

BOOL CALLBACK UploadPage::UploadPageProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    switch(uMsg) {
        case WM_INITDIALOG:
            pUploadPage->m_hWnd = hWnd;
            pUploadPage->onInitDialog();
            return TRUE;
        case WM_DROPFILES:
            pUploadPage->onDropFiles(wParam);
            return TRUE;
        case WM_COMMAND:
            switch(LOWORD(wParam)) {
                case IDC_ADD:
                    pUploadPage->onClickedAdd();
                    return TRUE;
                case IDC_REMOVE:
                    pUploadPage->onClickedRemove();
                    return TRUE;
                case IDC_RENAME:
                    pUploadPage->onClickedRename();
                    return TRUE;
                case IDC_SHAREHIDDEN:
                    pUploadPage->onClickedShareHidden();
                    return TRUE;
            }

            break;
        case WM_NOTIFY:
            if(((LPNMHDR)lParam)->idFrom == IDC_DIRECTORIES) {
                switch(((LPNMHDR)lParam)->code) {
                    case LVN_ITEMCHANGED:
                        pUploadPage->onItemchangedDirectories((LPNMHDR)lParam);
                        return TRUE;
                    case LVN_KEYDOWN:
                        if(pUploadPage->onKeyDown((LPNMHDR)lParam) == true) {
                            return TRUE;
                        }

                        return FALSE;
                    case NM_DBLCLK:
                        pUploadPage->onDoubleClick((LPNMHDR)lParam);
                        return TRUE;
                }
            }

            break;
    }

	return FALSE;
}

PropPage::TextItem UploadPage::texts[] = {
	{ IDC_SETTINGS_SHARED_DIRECTORIES, ResourceManager::SETTINGS_SHARED_DIRECTORIES },
	{ IDC_SETTINGS_SHARE_SIZE, ResourceManager::SETTINGS_SHARE_SIZE },
	{ IDC_SHAREHIDDEN, ResourceManager::SETTINGS_SHARE_HIDDEN },
	{ IDC_REMOVE, ResourceManager::REMOVE },
	{ IDC_ADD, ResourceManager::SETTINGS_ADD_FOLDER },
	{ IDC_RENAME, ResourceManager::SETTINGS_RENAME_FOLDER },
	{ IDC_SETTINGS_UPLOADS_MIN_SPEED, ResourceManager::SETTINGS_UPLOADS_MIN_SPEED },
	{ IDC_SETTINGS_KBPS, ResourceManager::KBPS },
	{ IDC_SETTINGS_UPLOADS_SLOTS, ResourceManager::SETTINGS_UPLOADS_SLOTS },
	{ IDC_SETTINGS_ONLY_HASHED, ResourceManager::SETTINGS_ONLY_HASHED },
	{ IDC_CZDC_SMALL_SLOTS, ResourceManager::SETCZDC_SMALL_UP_SLOTS },
	{ IDC_CZDC_SMALL_SIZE, ResourceManager::SETCZDC_SMALL_FILES },
	{ IDC_CZDC_NOTE_SMALL, ResourceManager::SETCZDC_NOTE_SMALL_UP },
	{ 0, ResourceManager::SETTINGS_AUTO_AWAY }
};

PropPage::Item UploadPage::items[] = {
	{ IDC_SLOTS, SettingsManager::SLOTS, PropPage::T_INT },
	{ IDC_SHAREHIDDEN, SettingsManager::SHARE_HIDDEN, PropPage::T_BOOL },
	{ IDC_MIN_UPLOAD_SPEED, SettingsManager::MIN_UPLOAD_SPEED, PropPage::T_INT },
	{ IDC_EXTRA_SLOTS, SettingsManager::EXTRA_SLOTS, PropPage::T_INT },
	{ IDC_SMALL_FILE_SIZE, SettingsManager::SMALL_FILE_SIZE, PropPage::T_INT },
	{ 0, 0, PropPage::T_END }
};

void UploadPage::onInitDialog() {
	PropPage::translate(m_hWnd, texts);
	ctrlDirectories.m_hWnd = ::GetDlgItem(m_hWnd, IDC_DIRECTORIES);
	::SendMessage(ctrlDirectories.m_hWnd, LVM_SETEXTENDEDLISTVIEWSTYLE, 0, LVS_EX_LABELTIP | LVS_EX_FULLROWSELECT);

	PropPage::read(m_hWnd, items);

	// Prepare shared dir list
	ListViewInsertColumn(ctrlDirectories.m_hWnd, 0, CTSTRING(VIRTUAL_NAME), LVCFMT_LEFT, 80, 0);
	ListViewInsertColumn(ctrlDirectories.m_hWnd, 1, CTSTRING(DIRECTORY), LVCFMT_LEFT, 197, 1);
	ListViewInsertColumn(ctrlDirectories.m_hWnd, 2, CTSTRING(SIZE), LVCFMT_RIGHT, 90, 2);
	StringPairList directories = ShareManager::getInstance()->getDirectories();
	for(StringPairIter j = directories.begin(); j != directories.end(); j++) {
		int i = ctrlDirectories.insert((int)::SendMessage(ctrlDirectories.m_hWnd, LVM_GETITEMCOUNT, 0, 0), Text::toT(j->first));
		ListViewSetItem(ctrlDirectories.m_hWnd, i, 1, LVIF_TEXT, Text::toT(j->second).c_str(), 0);
        ListViewSetItem(ctrlDirectories.m_hWnd, i, 2, LVIF_TEXT, Text::toT(Util::formatBytes(ShareManager::getInstance()->getShareSize(j->second))).c_str(), 0);
	}

	::SetWindowText(::GetDlgItem(m_hWnd, IDC_TOTAL), Text::toT(Util::formatBytes(ShareManager::getInstance()->getShareSize())).c_str());

	::SendMessage(::GetDlgItem(m_hWnd, IDC_SLOTSPIN), UDM_SETRANGE, 0, MAKELPARAM(100, 1));
	::SendMessage(::GetDlgItem(m_hWnd, IDC_MIN_UPLOAD_SPIN), UDM_SETRANGE32, 0, 30000);
	::SendMessage(::GetDlgItem(m_hWnd, IDC_EXTRA_SLOTS_SPIN), UDM_SETRANGE, 0, MAKELPARAM(100, 3));
	::SendMessage(::GetDlgItem(m_hWnd, IDC_SMALL_FILE_SIZE_SPIN), UDM_SETRANGE32, 64, 30000);
}

void UploadPage::onDropFiles(WPARAM wParam) {
	HDROP drop = (HDROP)wParam;
	TCHAR buf[MAX_PATH];
	UINT nrFiles;

	nrFiles = DragQueryFile(drop, (UINT)-1, NULL, 0);

	for(UINT i = 0; i < nrFiles; ++i){
		if(DragQueryFile(drop, i, buf, MAX_PATH)){
			if(PathIsDirectory(buf))
				addDirectory(buf);
		}
	}

	DragFinish(drop);
}

void UploadPage::write() {
	PropPage::write(m_hWnd, items);

	if(SETTING(SLOTS) < 1)
		settings->set(SettingsManager::SLOTS, 1);

	// Do specialized writing here
	if(SETTING(EXTRA_SLOTS) < 3)
		settings->set(SettingsManager::EXTRA_SLOTS, 3);

	if(SETTING(SMALL_FILE_SIZE) < 64)
		settings->set(SettingsManager::SMALL_FILE_SIZE, 64);

	if(SETTING(MAX_UPLOAD_SPEED_LIMIT_NORMAL) > 0) {
		if( SETTING(MAX_UPLOAD_SPEED_LIMIT_NORMAL) < ((2 * Util::getSlots()) + 3) ) {
			settings->set(SettingsManager::MAX_UPLOAD_SPEED_LIMIT_NORMAL, ((2 * Util::getSlots()) + 3) );
		}
		if ( (SETTING(MAX_DOWNLOAD_SPEED_LIMIT_NORMAL) > ( SETTING(MAX_UPLOAD_SPEED_LIMIT_NORMAL) * 7)) || ( SETTING(MAX_DOWNLOAD_SPEED_LIMIT_NORMAL) == 0) ) {
			settings->set(SettingsManager::MAX_DOWNLOAD_SPEED_LIMIT_NORMAL, (SETTING(MAX_UPLOAD_SPEED_LIMIT_NORMAL)*7) );
		}
	}

	if(SETTING(MAX_UPLOAD_SPEED_LIMIT_TIME) > 0) {
		if( SETTING(MAX_UPLOAD_SPEED_LIMIT_TIME) < ((2 * Util::getSlots()) + 3) ) {
			settings->set(SettingsManager::MAX_UPLOAD_SPEED_LIMIT_TIME, ((2 * Util::getSlots()) + 3) );
		}
		if ( (SETTING(MAX_DOWNLOAD_SPEED_LIMIT_TIME) > ( SETTING(MAX_UPLOAD_SPEED_LIMIT_TIME) * 7)) || ( SETTING(MAX_DOWNLOAD_SPEED_LIMIT_TIME) == 0) ) {
			settings->set(SettingsManager::MAX_DOWNLOAD_SPEED_LIMIT_TIME, (SETTING(MAX_UPLOAD_SPEED_LIMIT_TIME)*7) );
		}
	}
	Util::setSlots(SETTING(SLOTS));
	ShareManager::getInstance()->refresh();
}

void UploadPage::onItemchangedDirectories(LPNMHDR pnmh) {
	NM_LISTVIEW* lv = (NM_LISTVIEW*) pnmh;
	::EnableWindow(::GetDlgItem(m_hWnd, IDC_REMOVE), (lv->uNewState & LVIS_FOCUSED));
	::EnableWindow(::GetDlgItem(m_hWnd, IDC_RENAME), (lv->uNewState & LVIS_FOCUSED));
}

bool UploadPage::onKeyDown(LPNMHDR pnmh) {
	NMLVKEYDOWN* kd = (NMLVKEYDOWN*) pnmh;
	switch(kd->wVKey) {
        case VK_INSERT:
			::PostMessage(m_hWnd, WM_COMMAND, IDC_ADD, 0);
            return true;
        case VK_DELETE:
			::PostMessage(m_hWnd, WM_COMMAND, IDC_REMOVE, 0);
            return true;
        default:
            return false;
	}
}

void UploadPage::onDoubleClick(LPNMHDR pnmh) {
	NMITEMACTIVATE* item = (NMITEMACTIVATE*)pnmh;
	if(item->iItem >= 0) {
		::PostMessage(m_hWnd, WM_COMMAND, IDC_RENAME, 0);
	} else if(item->iItem == -1) {
		::PostMessage(m_hWnd, WM_COMMAND, IDC_ADD, 0);
	}
}

void UploadPage::onClickedAdd() {
	tstring target;
	if(WinUtil::browseDirectory(target, m_hWnd)) {
		addDirectory(target);
		HashProgressDlg(true).DoModal(m_hWnd);
	}
}

void UploadPage::onClickedRemove() {
	TCHAR buf[MAX_PATH];
	LVITEM item;
	::ZeroMemory(&item, sizeof(item));
	item.mask = LVIF_TEXT;
	item.cchTextMax = sizeof(buf);
	item.pszText = buf;

	int i = -1;
	while((i = (int)::SendMessage(ctrlDirectories.m_hWnd, LVM_GETNEXTITEM, (WPARAM)-1, LVNI_SELECTED)) != -1) {
		item.iItem = i;
		item.iSubItem = 1;
		::SendMessage(ctrlDirectories.m_hWnd, LVM_GETITEM, 0, (LPARAM)&item);
		ShareManager::getInstance()->removeDirectory(Text::fromT(buf));
		::SetWindowText(::GetDlgItem(m_hWnd, IDC_TOTAL), Text::toT(Util::formatBytes(ShareManager::getInstance()->getShareSize())).c_str());
		::SendMessage(ctrlDirectories.m_hWnd, LVM_DELETEITEM, i, 0);
	}
}

void UploadPage::onClickedRename() {
	TCHAR buf[MAX_PATH];
	LVITEM item;
	::ZeroMemory(&item, sizeof(item));
	item.mask = LVIF_TEXT;
	item.cchTextMax = sizeof(buf);
	item.pszText = buf;

	bool setDirty = false;

	int i = -1;
	while((i = (int)::SendMessage(ctrlDirectories.m_hWnd, LVM_GETNEXTITEM, i, LVNI_SELECTED)) != -1) {
		item.iItem = i;
		item.iSubItem = 0;
		::SendMessage(ctrlDirectories.m_hWnd, LVM_GETITEM, 0, (LPARAM)&item);
		tstring vName = buf;
		item.iSubItem = 1;
		::SendMessage(ctrlDirectories.m_hWnd, LVM_GETITEM, 0, (LPARAM)&item);
		tstring rPath = buf;

		LineDlg virt;
		virt.title = TSTRING(VIRTUAL_NAME);
		virt.description = TSTRING(VIRTUAL_NAME_LONG);
		virt.line = vName;
		if(virt.DoModal(m_hWnd) == IDOK) {
			if (Util::stricmp(buf, virt.line) != 0) {
				ShareManager::getInstance()->renameDirectory(Text::fromT(rPath), Text::fromT(virt.line));
				ListViewSetItem(ctrlDirectories.m_hWnd, i, 0, LVIF_TEXT, virt.line.c_str(), 0);

				setDirty = true;
			} else {
				::MessageBox(m_hWnd, CTSTRING(SKIP_RENAME), _T(CzDC_VERSION_STRING), MB_ICONINFORMATION | MB_OK);
			}
		}
	}

	if(setDirty) {
		ShareManager::getInstance()->setDirty();
    }
}

void UploadPage::onClickedShareHidden() {
	// Save the checkbox state so that ShareManager knows to include/exclude hidden files
	Item i = items[1]; // The checkbox. Explicit index used - bad!
	if(::IsDlgButtonChecked(m_hWnd, i.itemID) == BST_CHECKED){
		settings->set((SettingsManager::IntSetting)i.setting, true);
	} else {
		settings->set((SettingsManager::IntSetting)i.setting, false);
	}

	// Refresh the share. This is a blocking refresh. Might cause problems?
	// Hopefully people won't click the checkbox enough for it to be an issue. :-)
	ShareManager::getInstance()->setDirty();
	ShareManager::getInstance()->refresh(true, false, true);

	// Clear the GUI list, for insertion of updated shares
	::SendMessage(ctrlDirectories.m_hWnd, LVM_DELETEALLITEMS, 0, 0);
	StringPairList directories = ShareManager::getInstance()->getDirectories();
	for(StringPairIter j = directories.begin(); j != directories.end(); j++) {
		int i = ctrlDirectories.insert((int)::SendMessage(ctrlDirectories.m_hWnd, LVM_GETITEMCOUNT, 0, 0), Text::toT(j->first));
		ListViewSetItem(ctrlDirectories.m_hWnd, i, 1, LVIF_TEXT, Text::toT(j->second).c_str(), 0);
        ListViewSetItem(ctrlDirectories.m_hWnd, i, 2, LVIF_TEXT, Text::toT(Util::formatBytes(ShareManager::getInstance()->getShareSize(j->second))).c_str(), 0);
	}

	// Display the new total share size
	::SetWindowText(::GetDlgItem(m_hWnd, IDC_TOTAL), Text::toT(Util::formatBytes(ShareManager::getInstance()->getShareSize())).c_str());
}

void UploadPage::addDirectory(const tstring& aPath){
	tstring path = aPath;
	if( path[ path.length() -1 ] != _T('\\') )
		path += _T('\\');
	try {
		LineDlg virt;
		virt.title = TSTRING(VIRTUAL_NAME);
		virt.description = TSTRING(VIRTUAL_NAME_LONG);
		virt.line = Text::toT(ShareManager::getInstance()->validateVirtual(
			Util::getLastDir(Text::fromT(path))));
		if(virt.DoModal(m_hWnd) == IDOK) {
			ShareManager::getInstance()->addDirectory(Text::fromT(path), Text::fromT(virt.line));
			int i = ctrlDirectories.insert((int)::SendMessage(ctrlDirectories.m_hWnd, LVM_GETITEMCOUNT, 0, 0), virt.line );
			ListViewSetItem(ctrlDirectories.m_hWnd, i, 1, LVIF_TEXT, path.c_str(), 0);
			ListViewSetItem(ctrlDirectories.m_hWnd, i, 2, LVIF_TEXT, Text::toT(Util::formatBytes(ShareManager::getInstance()->getShareSize(Text::fromT(path)))).c_str(), 0);
			::SetWindowText(::GetDlgItem(m_hWnd, IDC_TOTAL), Text::toT(Util::formatBytes(ShareManager::getInstance()->getShareSize())).c_str());
		}
	} catch(const ShareException& e) {
		::MessageBox(m_hWnd, Text::toT(e.getError()).c_str(), _T(CzDC_VERSION_STRING), MB_ICONSTOP | MB_OK);
	}
}

/*
 * Some parts Copyright (C) 2002-2011 Petr Kozelka, PPK at czdc dot org
 * Licensed under GNU General Public License version 3.
 */
