/*
 * Copyright (C) 2001-2011 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "stdafx.h"
#include "../client/DCPlusPlus.h"
#include "Resource.h"

#include "UsersFrame.h"

#include "../client/ClientManager.h"

#include "LineDlg.h"
#include "../client/version.h"

#include "HubFrame.h"

int UsersFrame::columnIndexes[] = { COLUMN_NICK, COLUMN_STATUS, COLUMN_HUB, COLUMN_SEEN, COLUMN_DESCRIPTION };
int UsersFrame::columnSizes[] = { 200, 150, 300, 150, 200 };
static ResourceManager::Strings columnNames[] = { ResourceManager::AUTO_GRANT, ResourceManager::STATUS, ResourceManager::LAST_HUB, ResourceManager::LAST_SEEN, ResourceManager::DESCRIPTION };
int UsersFrame::columnIndexesIgn[] = { COLUMN_NICK, COLUMN_STATUS, COLUMN_HUB, COLUMN_SEEN, COLUMN_DESCRIPTION };
int UsersFrame::columnSizesIgn[] = { 200, 150, 300, 150, 200 };

ATOM UsersFrame::m_aAtom = 0;
UINT UsersFrame::m_uiResourceID = IDR_USERS;

UsersFrame::UsersFrame() : closed(false), startup(true), bFavorite(true), usersMenu(NULL) {
    RegisterWindowClass(m_aAtom, UsersFrameWndProc, _T("UsersFrame"), m_uiResourceID);
}

UsersFrame::~UsersFrame() {
    if(usersMenu != NULL) {
        ::DestroyMenu(usersMenu);
    }
}

LRESULT CALLBACK UsersFrame::UsersFrameWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
	UsersFrame * pParent;

	if(uMsg == WM_NCCREATE) {
		pParent = (UsersFrame *)((LPMDICREATESTRUCT)((LPCREATESTRUCT)lParam)->lpCreateParams)->lParam;
		SetWindowLongPtr(hWnd, GWLP_USERDATA, (LONG_PTR)pParent);

		pParent->m_hWnd = hWnd;
	} else {
		pParent = (UsersFrame *)GetWindowLongPtr(hWnd, GWLP_USERDATA);
		if(pParent == NULL) {
			return ::DefMDIChildProc(hWnd, uMsg, wParam, lParam);
		}
	}

	return pParent->UsersFrameProc(hWnd, uMsg, wParam, lParam);
}

LRESULT UsersFrame::UsersFrameProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    switch(uMsg) {
        case WM_CREATE:
            onCreate();
            return 0;
        case WM_CLOSE:
            if(onClose() == true) {
                return 0;
            }
            break;
        case WM_CONTEXTMENU:
            if(onContextMenu(wParam, lParam) == true) {
                return 0;
            }
            break;
        case WM_SPEAKER:
            onSpeaker(wParam, lParam);
            return 0;
        case WM_SIZE:
            if(wParam != SIZE_MINIMIZED) {
                UpdateLayout();
            }
            break;
        case WM_SETFOCUS:
            onSetFocus();
            break;
        case WM_REFRESH_SETTINGS:
            onRefreshSettings();
            return 0;
        case WM_COMMAND:
            switch(LOWORD(wParam)) {
                case IDC_REMOVE:
                    onRemove();
                    return 0;
                case IDC_EDIT:
                    onEdit();
                    return 0;
                case IDC_CONNECT:
                    onConnect();
                    return 0;
                case IDC_OPEN_USER_LOG:
                    onOpenUserLog();
                    return 0;
            }

            break;
        case WM_NOTIFY:
            if(((LPNMHDR)lParam)->idFrom == IDC_USERS) {
                switch(((LPNMHDR)lParam)->code) {
                    case LVN_GETDISPINFO: {
                        ctrlUsers.onGetDispInfo((LPNMHDR)lParam);
                        return 0;
                    }
                    case LVN_COLUMNCLICK: {
                        ctrlUsers.onColumnClick((LPNMHDR)lParam);
                        return 0;
                    }
                    case LVN_ITEMCHANGED:
                        onItemChanged((LPNMHDR)lParam);
                        return 0;
                    case LVN_KEYDOWN:
                        if(onKeyDown((LPNMHDR)lParam) == true) {
                            return 0;
                        }
                        break;
                    case NM_DBLCLK:
                        if(onDoubleClick((LPNMHDR)lParam) == true) {
                            return 0;
                        }
                        break;
                    case NM_CUSTOMDRAW:
                        return onCustomDraw((LPNMHDR)lParam);
                }
            } else if(((LPNMHDR)lParam)->idFrom == IDC_IGNORED_USERS) {
                switch(((LPNMHDR)lParam)->code) {
                    case LVN_GETDISPINFO: {
                        ctrlIgnUsers.onGetDispInfo((LPNMHDR)lParam);
                        return 0;
                    }
                    case LVN_COLUMNCLICK: {
                        ctrlIgnUsers.onColumnClick((LPNMHDR)lParam);
                        return 0;
                    }
                    case LVN_KEYDOWN:
                        if(onIgnKeyDown((LPNMHDR)lParam) == true) {
                              return 0;
                        }
                        break;
                    case NM_DBLCLK:
                        if(onDoubleClick((LPNMHDR)lParam) == true) {
                            return 0;
                        }
                        break;
                    case NM_CUSTOMDRAW:
                        return onCustomDraw((LPNMHDR)lParam);
                }
            }

            break;
        case WM_NCDESTROY:
            delete this;
            return ::DefMDIChildProc(hWnd, uMsg, wParam, lParam);
    }

	LRESULT lResult = 0;
	if(UserInfoBaseHandlerProc(uMsg, wParam) == true) {
		return 0;
    } else if(MDIWindowProc(hWnd, uMsg, wParam, lParam, lResult) == true) {
		return lResult;
    }

	return ::DefMDIChildProc(m_hWnd, uMsg, wParam, lParam);
}

void UsersFrame::onCreate() {
    WinUtil::tabCtrl->addTab(m_hWnd, RGB(0, 0, 0), IDR_USERS);
    created = true;

	ctrlUsers.Create(m_hWnd, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | WS_HSCROLL | WS_VSCROLL | LVS_REPORT | LVS_SHOWSELALWAYS | LVS_SHAREIMAGELISTS, WS_EX_CLIENTEDGE,
        IDC_USERS, LVS_EX_LABELTIP | LVS_EX_HEADERDRAGDROP | LVS_EX_CHECKBOXES | LVS_EX_FULLROWSELECT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT);
	::SendMessage(ctrlUsers.m_hWnd, LVM_SETIMAGELIST, LVSIL_SMALL, (LPARAM)WinUtil::userImages);

	// Create listview columns
	WinUtil::splitTokens(columnIndexes, SETTING(USERSFRAME_ORDER), COLUMN_LAST);
	WinUtil::splitTokens(columnSizes, SETTING(USERSFRAME_WIDTHS), COLUMN_LAST);

	for(uint8_t ui8i = 0; ui8i < (sizeof(columnSizes) / sizeof(columnSizes[0])); ui8i++) {
		ctrlUsers.InsertColumn(ui8i, CTSTRING_I(columnNames[ui8i]), LVCFMT_LEFT, columnSizes[ui8i], ui8i);
	}

	ctrlUsers.setColumnOrderArray(COLUMN_LAST, columnIndexes);
	ctrlUsers.setSortColumn(COLUMN_NICK);

	ctrlIgnUsers.Create(m_hWnd, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | WS_HSCROLL | WS_VSCROLL | LVS_REPORT | LVS_SHOWSELALWAYS | LVS_SHAREIMAGELISTS, WS_EX_CLIENTEDGE,
        IDC_IGNORED_USERS, LVS_EX_LABELTIP | LVS_EX_HEADERDRAGDROP | LVS_EX_CHECKBOXES | LVS_EX_FULLROWSELECT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT);
	::SendMessage(ctrlIgnUsers.m_hWnd, LVM_SETIMAGELIST, LVSIL_SMALL, (LPARAM)WinUtil::userImages);

	// Create listview columns
	WinUtil::splitTokens(columnIndexesIgn, SETTING(IGNUSERSFRAME_ORDER), COLUMN_LAST);
	WinUtil::splitTokens(columnSizesIgn, SETTING(IGNUSERSFRAME_WIDTHS), COLUMN_LAST);

	for(uint8_t ui8i = 0; ui8i < (sizeof(columnSizesIgn) / sizeof(columnSizesIgn[0])); ui8i++) {
		ctrlIgnUsers.InsertColumn(ui8i, CTSTRING_I(columnNames[ui8i]), LVCFMT_LEFT, columnSizesIgn[ui8i], ui8i);
	}

	ctrlIgnUsers.setColumnOrderArray(COLUMN_LAST, columnIndexesIgn);
	ctrlIgnUsers.setSortColumn(COLUMN_NICK);

	usersMenu = ::CreatePopupMenu();
	::AppendMenu(usersMenu, MF_STRING, IDC_EDIT, CTSTRING(PROPERTIES));
	::AppendMenu(usersMenu, MF_STRING, IDC_OPEN_USER_LOG, CTSTRING(OPEN_USER_LOG));
	::AppendMenu(usersMenu, MF_SEPARATOR, 0, NULL);
    ::AppendMenu(usersMenu, MF_STRING, IDC_GETLIST, CTSTRING(GET_FILE_LIST));
	::AppendMenu(usersMenu, MF_STRING, IDC_MATCH_QUEUE, CTSTRING(MATCH_QUEUE));
	::AppendMenu(usersMenu, MF_STRING, IDC_PRIVATEMESSAGE, CTSTRING(SEND_PRIVATE_MESSAGE));
	::AppendMenu(usersMenu, MF_STRING, IDC_ADD_TO_FAVORITES, CTSTRING(ADD_TO_FAVORITES));

    WinUtil::AddGrantSubMenu(usersMenu);

	::AppendMenu(usersMenu, MF_STRING, IDC_REMOVEALL, CTSTRING(REMOVE_FROM_ALL));
	::AppendMenu(usersMenu, MF_SEPARATOR, 0, NULL);
	::AppendMenu(usersMenu, MF_STRING, IDC_REMOVE, CTSTRING(REMOVE));

	hWndFavUsers = CreateWindowEx(WS_EX_TRANSPARENT, WC_BUTTON, CTSTRING(FAVORITE_USERS), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | BS_GROUPBOX,
        CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, m_hWnd, 0, g_hInstance, NULL);
	::SendMessage(hWndFavUsers, WM_SETFONT, (WPARAM)WinUtil::systemFont, MAKELPARAM(TRUE, 0));

	hWndIgnUsers = CreateWindowEx(WS_EX_TRANSPARENT, WC_BUTTON, CTSTRING(IGNORED_USERS), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | BS_GROUPBOX,
        CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, m_hWnd, 0, g_hInstance, NULL);
	::SendMessage(hWndIgnUsers, WM_SETFONT, (WPARAM)WinUtil::systemFont, MAKELPARAM(TRUE, 0));

	FavoriteManager::getInstance()->addListener(this);
	SettingsManager::getInstance()->addListener(this);

	FavoriteManager::FavoriteMap ul = FavoriteManager::getInstance()->getFavoriteUsers();
	::SendMessage(ctrlUsers.m_hWnd, WM_SETREDRAW, (WPARAM)FALSE, 0);
	for(FavoriteManager::FavoriteMap::iterator i = ul.begin(); i != ul.end(); ++i) {
		addUser(i->second);
	}
	::SendMessage(ctrlUsers.m_hWnd, WM_SETREDRAW, (WPARAM)TRUE, 0);

	FavoriteManager::IgnoredMap iu = FavoriteManager::getInstance()->getIgnoredUsers();
	::SendMessage(ctrlIgnUsers.m_hWnd, WM_SETREDRAW, (WPARAM)FALSE, 0);
	for(FavoriteManager::IgnoredMap::iterator i = iu.begin(); i != iu.end(); ++i) {
		addUser(i->second);
	}
	::SendMessage(ctrlIgnUsers.m_hWnd, WM_SETREDRAW, (WPARAM)TRUE, 0);

	startup = false;
}

bool UsersFrame::onContextMenu(WPARAM wParam, LPARAM lParam) {
    int iX = GET_X_LPARAM(lParam),
        iY = GET_Y_LPARAM(lParam);

	if((HWND)wParam == ctrlUsers.m_hWnd && (UINT)::SendMessage(ctrlUsers.m_hWnd, LVM_GETSELECTEDCOUNT, 0, 0) != 0) {
        bFavorite = true;

        if(iX == -1 && iY == -1) {
            ListViewGetContextMenuPos(ctrlUsers.m_hWnd, iX, iY);
	   }
	} else if((HWND)wParam == ctrlIgnUsers.m_hWnd && (UINT)::SendMessage(ctrlIgnUsers.m_hWnd, LVM_GETSELECTEDCOUNT, 0, 0) != 0) {
        bFavorite = false;

        if(iX == -1 && iY == -1) {
            ListViewGetContextMenuPos(ctrlIgnUsers.m_hWnd, iX, iY);
	   }
	} else {
        return false;
    }

    ::TrackPopupMenuEx(usersMenu, TPM_LEFTALIGN | TPM_RIGHTBUTTON, iX, iY, m_hWnd, NULL);

	return true;
}

void UsersFrame::UpdateLayout() {
	RECT rect;
	::GetClientRect(m_hWnd, &rect);

	RECT rc = rect;
	rc.top += 2;
	rc.bottom -= (rect.bottom / 2) - 3;
	rc.left += 2;
	rc.right -= 2;
	::MoveWindow(hWndFavUsers, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, TRUE);

	rc.top += 16;
	rc.bottom -= 8;
	rc.left += 8;
	rc.right -= 8;
	::MoveWindow(ctrlUsers.m_hWnd, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, TRUE);

	rc = rect;
	rc.top += (rect.bottom / 2) + 3;
	rc.bottom -= 2;
	rc.left += 2;
	rc.right -= 2;
	::MoveWindow(hWndIgnUsers, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, TRUE);

	rc.top += 16;
	rc.bottom -= 8;
	rc.left += 8;
	rc.right -= 8;
	::MoveWindow(ctrlIgnUsers.m_hWnd, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, TRUE);
}

void UsersFrame::onRemove() {
    if(!BOOLSETTING(CONFIRM_ITEM_REMOVAL) || 
		::MessageBox(m_hWnd, CTSTRING(CONFIRM_REMOVE), _T(CzDC_VERSION_STRING), MB_YESNO | MB_ICONQUESTION | MB_DEFBUTTON2) == IDYES) {
        int i = -1;
        if(bFavorite == true) {
    		while((i = (int)::SendMessage(ctrlUsers.m_hWnd, LVM_GETNEXTITEM, (WPARAM)-1, LVNI_SELECTED)) != -1) {
    			ctrlUsers.getItemData(i)->remove();
    		}
    	} else {
    		while((i = (int)::SendMessage(ctrlIgnUsers.m_hWnd, LVM_GETNEXTITEM, (WPARAM)-1, LVNI_SELECTED)) != -1) {
    			ctrlIgnUsers.getItemData(i)->remove();
    		}
        }
    }
}

void UsersFrame::onEdit() {
	if(bFavorite == true && (UINT)::SendMessage(ctrlUsers.m_hWnd, LVM_GETSELECTEDCOUNT, 0, 0) == 1) {
		int i = (int)::SendMessage(ctrlUsers.m_hWnd, LVM_GETNEXTITEM, (WPARAM)-1, LVNI_SELECTED);
		UserInfo* ui = ctrlUsers.getItemData(i);

		LineDlg dlg;
		dlg.description = TSTRING(DESCRIPTION);
		dlg.title = ui->columns[COLUMN_NICK];
		dlg.line = ui->columns[COLUMN_DESCRIPTION];
		if(dlg.DoModal(m_hWnd)) {
			FavoriteManager::getInstance()->setUserDescription(ui->user, Text::fromT(dlg.line));
			ui->columns[COLUMN_DESCRIPTION] = dlg.line;
			ctrlUsers.updateItem(i);
		}
	} else if(bFavorite == false && (UINT)::SendMessage(ctrlIgnUsers.m_hWnd, LVM_GETSELECTEDCOUNT, 0, 0) == 1) {
		int i = (int)::SendMessage(ctrlIgnUsers.m_hWnd, LVM_GETNEXTITEM, (WPARAM)-1, LVNI_SELECTED);
		UserInfo* ui = ctrlIgnUsers.getItemData(i);

		LineDlg dlg;
		dlg.description = TSTRING(DESCRIPTION);
		dlg.title = ui->columns[COLUMN_NICK];
		dlg.line = ui->columns[COLUMN_DESCRIPTION];
		if(dlg.DoModal(m_hWnd)) {
			FavoriteManager::getInstance()->setIgnUserDescription(ui->user, Text::fromT(dlg.line));
			ui->columns[COLUMN_DESCRIPTION] = dlg.line;
			ctrlIgnUsers.updateItem(i);
		}
	}
}

void UsersFrame::onItemChanged(LPNMHDR pnmh) {
	NMITEMACTIVATE* l = (NMITEMACTIVATE*)pnmh;
	if(!startup && l->iItem != -1 && ((l->uNewState & LVIS_STATEIMAGEMASK) != (l->uOldState & LVIS_STATEIMAGEMASK))) {
		FavoriteManager::getInstance()->setAutoGrant(ctrlUsers.getItemData(l->iItem)->user, (BOOL)(((UINT)::SendMessage(ctrlUsers.m_hWnd, LVM_GETITEMSTATE, l->iItem, LVIS_STATEIMAGEMASK) >> 12) - 1) != FALSE);
	}
}

bool UsersFrame::onDoubleClick(LPNMHDR pnmh) {
	NMITEMACTIVATE* item = (NMITEMACTIVATE*)pnmh;

	if(item->iItem != -1) {
		::PostMessage(m_hWnd, WM_COMMAND, IDC_GETLIST, 0);
		return true;
	} else {
		return false;
	}
}

bool UsersFrame::onKeyDown(LPNMHDR pnmh) {
    bFavorite = true;

	NMLVKEYDOWN* kd = (NMLVKEYDOWN*) pnmh;
	switch(kd->wVKey) {
    	case VK_DELETE:
			::PostMessage(m_hWnd, WM_COMMAND, IDC_REMOVE, 0);
    		return true;
    	case VK_RETURN:
			::PostMessage(m_hWnd, WM_COMMAND, IDC_EDIT, 0);
    		return true;
    	default:
    		break;
	}

	return false;
}

bool UsersFrame::onIgnKeyDown(LPNMHDR pnmh) {
    bFavorite = false;

	NMLVKEYDOWN* kd = (NMLVKEYDOWN*) pnmh;
	switch(kd->wVKey) {
    	case VK_DELETE:
			::PostMessage(m_hWnd, WM_COMMAND, IDC_REMOVE, 0);
    		return true;
    	case VK_RETURN:
			::PostMessage(m_hWnd, WM_COMMAND, IDC_EDIT, 0);
    		return true;
    	default:
    		break;
	}

    return false;
}

void UsersFrame::onConnect() {
    if(bFavorite == true) {
    	for(int i = 0; i < (int)::SendMessage(ctrlUsers.m_hWnd, LVM_GETITEMCOUNT, 0, 0); ++i) {
    		UserInfo *ui = ctrlUsers.getItemData(i);
    		FavoriteManager::FavoriteMap favUsers = FavoriteManager::getInstance()->getFavoriteUsers();
    		const FavoriteUser u = favUsers.find(ui->user->getCID())->second;
    		if(u.getUrl().length() > 0) {
    			HubFrame::openWindow(Text::toT(u.getUrl()));
    		}
    	}
    } else {
    	for(int i = 0; i < (int)::SendMessage(ctrlIgnUsers.m_hWnd, LVM_GETITEMCOUNT, 0, 0); ++i) {
    		UserInfo *ui = ctrlIgnUsers.getItemData(i);
    		FavoriteManager::IgnoredMap ignUsers = FavoriteManager::getInstance()->getIgnoredUsers();
    		const FavoriteUser u = ignUsers.find(ui->user->getCID())->second;
    		if(u.getUrl().length() > 0) {
    			HubFrame::openWindow(Text::toT(u.getUrl()));
    		}
    	}
    }
}

void UsersFrame::addUser(const FavoriteUser& aUser) {
	int i = -1;
	Identity UserIdentity = ClientManager::getInstance()->GetOnlineUserIdentity(aUser.getUser());
	if(aUser.isSet(FavoriteUser::FLAG_IS_FAVORITE)) {
		i = ctrlUsers.insertItem(new UserInfo(aUser), UserIdentity.getcType() == 11 ? 23 : WinUtil::getImage(UserIdentity));
    	bool b = aUser.isSet(FavoriteUser::FLAG_GRANTSLOT);
		ListView_SetItemState(ctrlUsers.m_hWnd, i, INDEXTOSTATEIMAGEMASK(b ? 2 : 1), LVIS_STATEIMAGEMASK);
    } else {
    	i = ctrlIgnUsers.insertItem(new UserInfo(aUser), UserIdentity.getcType() == 11 ? 23 : WinUtil::getImage(UserIdentity));
    }
}

void UsersFrame::updateUser(const UserPtr& aUser) {
	for(int i = 0; i < (int)::SendMessage(ctrlUsers.m_hWnd, LVM_GETITEMCOUNT, 0, 0); ++i) {
		UserInfo *ui = ctrlUsers.getItemData(i);
		if(ui->user == aUser) {
			Identity UserIdentity = ClientManager::getInstance()->GetOnlineUserIdentity(aUser);
			ListViewSetItem(ctrlUsers.m_hWnd, i, 0, LVIF_IMAGE, NULL, UserIdentity.getcType() == 11 ? 23 : WinUtil::getImage(UserIdentity));

			if(aUser->isOnline() == true) {
				ui->columns[COLUMN_STATUS] = TSTRING(ONLINE);
				ui->columns[COLUMN_HUB] = WinUtil::getHubNames(aUser).first;
				ui->columns[COLUMN_SEEN] = TSTRING(ONLINE);
			} else {
				ui->columns[COLUMN_STATUS] = TSTRING(OFFLINE);
				FavoriteManager::FavoriteMap favUsers = FavoriteManager::getInstance()->getFavoriteUsers();
				const FavoriteUser u = favUsers.find(ui->user->getCID())->second;
				ui->columns[COLUMN_HUB] = Text::toT(u.getUrl());
				ui->columns[COLUMN_SEEN] = Text::toT(Util::formatTime("%Y-%m-%d %H:%M", FavoriteManager::getInstance()->getIgnLastSeen(aUser)));
			}
			ctrlUsers.updateItem(i);
		}
	}

	for(int i = 0; i < (int)::SendMessage(ctrlIgnUsers.m_hWnd, LVM_GETITEMCOUNT, 0, 0); ++i) {
		UserInfo *ui = ctrlIgnUsers.getItemData(i);
		if(ui->user == aUser) {
			Identity UserIdentity = ClientManager::getInstance()->GetOnlineUserIdentity(aUser);
			ListViewSetItem(ctrlIgnUsers.m_hWnd, i, 0, LVIF_IMAGE, NULL, UserIdentity.getcType() == 11 ? 23 : WinUtil::getImage(UserIdentity));

			if(aUser->isOnline() == true) {
				ui->columns[COLUMN_STATUS] = TSTRING(ONLINE);
				ui->columns[COLUMN_HUB] = WinUtil::getHubNames(aUser).first;
				ui->columns[COLUMN_SEEN] = TSTRING(ONLINE);
			} else {
				ui->columns[COLUMN_STATUS] = TSTRING(OFFLINE);
				FavoriteManager::IgnoredMap ignUsers = FavoriteManager::getInstance()->getIgnoredUsers();
				const FavoriteUser u = ignUsers.find(ui->user->getCID())->second;
				ui->columns[COLUMN_HUB] = Text::toT(u.getUrl());
				ui->columns[COLUMN_SEEN] = Text::toT(Util::formatTime("%Y-%m-%d %H:%M", FavoriteManager::getInstance()->getIgnLastSeen(aUser)));
			}
			ctrlIgnUsers.updateItem(i);
		}
	}
}

void UsersFrame::removeUser(const FavoriteUser& aUser) {
    if(aUser.isSet(FavoriteUser::FLAG_IS_FAVORITE) == true) {
    	for(int i = 0; i < (int)::SendMessage(ctrlUsers.m_hWnd, LVM_GETITEMCOUNT, 0, 0); ++i) {
    		UserInfo *ui = ctrlUsers.getItemData(i);
    		if(ui->user == aUser.getUser()) {
				::SendMessage(ctrlUsers.m_hWnd, LVM_DELETEITEM, i, 0);
    			delete ui;
    			return;
    		}
    	}
    } else {
    	for(int i = 0; i < (int)::SendMessage(ctrlIgnUsers.m_hWnd, LVM_GETITEMCOUNT, 0, 0); ++i) {
    		UserInfo *ui = ctrlIgnUsers.getItemData(i);
    		if(ui->user == aUser.getUser()) {
				::SendMessage(ctrlIgnUsers.m_hWnd, LVM_DELETEITEM, i, 0);
    			delete ui;
    			return;
    		}
    	}
    }
}

bool UsersFrame::onClose() {
	if(!closed) {
		FavoriteManager::getInstance()->removeListener(this);
		SettingsManager::getInstance()->removeListener(this);
		closed = true;
		::PostMessage(m_hWnd, WM_CLOSE, 0, 0);
		return true;
	} else {
		WinUtil::saveHeaderOrder(ctrlUsers.m_hWnd, SettingsManager::USERSFRAME_ORDER,
			SettingsManager::USERSFRAME_WIDTHS, COLUMN_LAST, columnIndexes, columnSizes);

		WinUtil::saveHeaderOrder(ctrlIgnUsers.m_hWnd, SettingsManager::IGNUSERSFRAME_ORDER,
			SettingsManager::IGNUSERSFRAME_WIDTHS, COLUMN_LAST, columnIndexesIgn, columnSizesIgn);
	
		for(int i = 0; i < (int)::SendMessage(ctrlUsers.m_hWnd, LVM_GETITEMCOUNT, 0, 0); ++i) {
			delete ctrlUsers.getItemData(i);
		}

		::SendMessage(ctrlUsers.m_hWnd, LVM_DELETEALLITEMS, 0, 0);

		for(int i = 0; i < (int)::SendMessage(ctrlIgnUsers.m_hWnd, LVM_GETITEMCOUNT, 0, 0); ++i) {
			delete ctrlIgnUsers.getItemData(i);
		}

		::SendMessage(ctrlIgnUsers.m_hWnd, LVM_DELETEALLITEMS, 0, 0);

		return false;
	}
}

void UsersFrame::UserInfo::update(const FavoriteUser& u) {
	columns[COLUMN_NICK] = Text::toT(u.getNick());
	columns[COLUMN_STATUS] = u.getUser()->isOnline() ? TSTRING(ONLINE) : TSTRING(OFFLINE);
	columns[COLUMN_HUB] = user->isOnline() ? WinUtil::getHubNames(u.getUser()).first : Text::toT(u.getUrl());
	columns[COLUMN_SEEN] = user->isOnline() ? TSTRING(ONLINE) : Text::toT(Util::formatTime("%Y-%m-%d %H:%M", u.getLastSeen()));
	columns[COLUMN_DESCRIPTION] = Text::toT(u.getDescription());
}

void UsersFrame::onSpeaker(WPARAM wParam, LPARAM lParam) {
	if(wParam == USER_UPDATED) {
    	UserInfoBase* uib = (UserInfoBase*)lParam;
    	updateUser(uib->user);
    	delete uib;
    }
}

LRESULT UsersFrame::onCustomDraw(LPNMHDR pnmh) {
    LPNMLVCUSTOMDRAW cd = (LPNMLVCUSTOMDRAW)pnmh;

    switch(cd->nmcd.dwDrawStage) {
        case CDDS_PREPAINT:
            return CDRF_NOTIFYITEMDRAW;
        case CDDS_ITEMPREPAINT:
            return CDRF_NOTIFYSUBITEMDRAW;
        case CDDS_SUBITEM | CDDS_ITEMPREPAINT:
            // Let's draw a box if needed...
            if(cd->iSubItem == COLUMN_STATUS) {
                UserInfo *ui = (UserInfo*)cd->nmcd.lItemlParam;
                if(ui && (ui->user != (UserPtr)NULL) && ui->user->isOnline()) {
                    cd->clrText = RGB(0, 255, 0);
                } else {
                    cd->clrText = RGB(255, 0, 0);
                }
            } else {
                cd->clrText = WinUtil::textColor;
            }
            return CDRF_NEWFONT;
        default:
            return CDRF_DODEFAULT;
	}
}

void UsersFrame::onOpenUserLog() {
    UserInfo* ui = NULL;
	if(bFavorite == true && (UINT)::SendMessage(ctrlUsers.m_hWnd, LVM_GETSELECTEDCOUNT, 0, 0) == 1) {
		ui = ctrlUsers.getItemData((int)::SendMessage(ctrlUsers.m_hWnd, LVM_GETNEXTITEM, (WPARAM)-1, LVNI_SELECTED));
	} else if(bFavorite == false && (UINT)::SendMessage(ctrlIgnUsers.m_hWnd, LVM_GETSELECTEDCOUNT, 0, 0) == 1) {
		ui = ctrlIgnUsers.getItemData((int)::SendMessage(ctrlIgnUsers.m_hWnd, LVM_GETNEXTITEM, (WPARAM)-1, LVNI_SELECTED));
	}

    if(ui != NULL) {
		StringMap params;
		params["userNI"] = ui->getUser()->getFirstNick();
		params["userCID"] = ui->user->getCID().toBase32();
		params["hubNI"] = Util::toString(ClientManager::getInstance()->getHubNames(ui->getUser()->getCID()));
		params["hubURL"] = Util::toString(ClientManager::getInstance()->getHubs(ui->getUser()->getCID()));
		params["myCID"] = ClientManager::getInstance()->getMe()->getCID().toBase32();
		string file = Util::validateFileName(SETTING(LOG_DIRECTORY) + Util::formatParams(SETTING(LOG_FILE_PRIVATE_CHAT), params, true));
		if(File::FileExist(file)) {
			ShellExecute(NULL, NULL, Text::toT(file).c_str(), NULL, NULL, SW_SHOWNORMAL);
		} else {
			::MessageBox(m_hWnd, CTSTRING(NO_LOG_FOR_USER),CTSTRING(NO_LOG_FOR_USER), MB_OK);	  
		}	
    }
}

void UsersFrame::on(SettingsManagerListener::Save, SimpleXML& /*xml*/) throw() {
	::PostMessage(m_hWnd, WM_REFRESH_SETTINGS, 0, 0);
}

void UsersFrame::onRefreshSettings() {
    bool refresh = false;

    if((COLORREF)::SendMessage(ctrlUsers.m_hWnd, LVM_GETBKCOLOR, 0, 0) != WinUtil::bgColor) {
		::SendMessage(ctrlUsers.m_hWnd, LVM_SETBKCOLOR, 0, WinUtil::bgColor);
		::SendMessage(ctrlUsers.m_hWnd, LVM_SETTEXTBKCOLOR, 0, WinUtil::bgColor);
        refresh = true;
    }

    if((COLORREF)::SendMessage(ctrlUsers.m_hWnd, LVM_GETTEXTCOLOR, 0, 0) != WinUtil::textColor) {
        ::SendMessage(ctrlUsers.m_hWnd, LVM_SETTEXTCOLOR, 0, WinUtil::textColor);
        refresh = true;
    }

    if((COLORREF)::SendMessage(ctrlIgnUsers.m_hWnd, LVM_GETBKCOLOR, 0, 0) != WinUtil::bgColor) {
		::SendMessage(ctrlIgnUsers.m_hWnd, LVM_SETBKCOLOR, 0, WinUtil::bgColor);
		::SendMessage(ctrlIgnUsers.m_hWnd, LVM_SETTEXTBKCOLOR, 0, WinUtil::bgColor);
        refresh = true;
    }

    if((COLORREF)::SendMessage(ctrlIgnUsers.m_hWnd, LVM_GETTEXTCOLOR, 0, 0) != WinUtil::textColor) {
        ::SendMessage(ctrlIgnUsers.m_hWnd, LVM_SETTEXTCOLOR, 0, WinUtil::textColor);
        refresh = true;
    }

    if(refresh == true) {
		::RedrawWindow(m_hWnd, NULL, NULL, RDW_ERASE | RDW_INVALIDATE | RDW_UPDATENOW | RDW_ALLCHILDREN);
    }
}

/*
 * Some parts Copyright (C) 2002-2011 Petr Kozelka, PPK at czdc dot org
 * Licensed under GNU General Public License version 3.
 */
