/*
 * CzDC - client for Direct Connect peer to peer network.

 * Copyright (C) 2002-2011 Petr Kozelka, PPK at CzDC dot org

 * CzDC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3
 * as published by the Free Software Foundation.

 * CzDC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with CzDC. If not, see <http://www.gnu.org/licenses/>.
 */

//----------------------------------------------------------------------------------------------------------------------------------------------------
#ifndef WAITING_QUEUE_FRAME_H
#define WAITING_QUEUE_FRAME_H
//----------------------------------------------------------------------------------------------------------------------------------------------------
#pragma once
//----------------------------------------------------------------------------------------------------------------------------------------------------
#include "../client/TimerManager.h"
#include "../client/UploadManager.h"
//----------------------------------------------------------------------------------------------------------------------------------------------------
#include "MDIChildWindow.h"
#include "WinApiUtil.h"
#include "TypedListViewCtrl.h"
#include "WinUtil.h"
#include "BasicSplitter.h"
//----------------------------------------------------------------------------------------------------------------------------------------------------

class WaitingUsersFrame : public MDITabChildWindow, public StaticFrame<WaitingUsersFrame, ResourceManager::UPLOAD_QUEUE, IDC_UPLOAD_QUEUE>,
	private UploadManagerListener, private TimerManagerListener, private BasicSplitter, private SettingsManagerListener {
public:
	WaitingUsersFrame();
	~WaitingUsersFrame();

    static ATOM m_aAtom;
    static UINT m_uiResourceID;

private:
	enum {
		ADD_ITEM,
		REMOVE,
		REMOVE_ITEM,
		UPDATE_ITEMS
	};

	enum {
		COLUMN_FIRST,
		COLUMN_FILE = COLUMN_FIRST,
		COLUMN_PATH,
		COLUMN_NICK,
		COLUMN_HUB,
		COLUMN_TRANSFERRED,
		COLUMN_SIZE,
		COLUMN_ADDED,
		COLUMN_WAITING,
		COLUMN_LAST
	};

	struct UserItem {
		UserPtr u;
		UserItem(UserPtr u) : u(u) { }
	};

    bool showTree, closed, usingUserMenu;

	static int columnSizes[COLUMN_LAST];
	static int columnIndexes[COLUMN_LAST];

    int statusSizes[4];

	HWND hWndShowTree, hWndQueued;

	HMENU contextMenu;

	UserList UQFUsers;

	TypedListViewCtrl<UploadQueueItem, IDC_UPLOAD_QUEUE> ctrlList;

    static LRESULT CALLBACK WaitingUsersFrameWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
	LRESULT WaitingUsersFrameProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
	static LRESULT CALLBACK ShowTreeWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);

	void onCreate();
	bool onClose();
	void onGetList();
	void onRemove();
	void onItemChanged();
	bool onContextMenu(WPARAM wParam, LPARAM lParam);
	void onPrivateMessage();
	void onGrantSlot();
	void onGrantSlotHour();
	void onGrantSlotDay();
	void onGrantSlotWeek();
	void onUnGrantSlot();
	void onAddToFavorites();
	void onSpeaker(WPARAM wParam, LPARAM lParam);
	void onRefreshSettings();
	void UpdateLayout(bool bResizeStatusBar = true);

	UserPtr getSelectedUser();
	bool onKeyDown(LPNMHDR pnmh);
	bool onKeyDownDirs(LPNMHDR pnmh);
	void onShowTree(WPARAM wParam);
	void removeSelected();
	void removeSelectedUser();

	void LoadAll();
	void UpdateSearch(int index, BOOL doDelete = TRUE);

	HTREEITEM GetParentItem();

	void AddFile(UploadQueueItem* aUQI);
	void RemoveUser(const UserPtr& aUser);
	void addAllFiles(Upload * /*aUser*/);
	void updateStatus();

	void on(UploadManagerListener::QueueAdd, UploadQueueItem* aUQI) throw();
	void on(UploadManagerListener::QueueRemove, const UserPtr& aUser) throw();
	void on(UploadManagerListener::QueueItemRemove, UploadQueueItem* aUQI) throw();
	void on(TimerManagerListener::Second, uint64_t /*aTick*/) throw();
	void on(SettingsManagerListener::Save, SimpleXML& /*xml*/) throw();

    HWND GetWindowHandle();
    void UpdateSplitterParts();
};
//----------------------------------------------------------------------------------------------------------------------------------------------------

#endif
