/*
 * Copyright (C) 2001-2011 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "stdafx.h"

#include "../client/DCPlusPlus.h"
#include "Resource.h"

#include "MainFrm.h"
#include "ExceptionHandling.h"
#include "WinUtil.h"
#include "SingleInstance.h"

#include "../client/MerkleTree.h"

#include <delayimp.h>
HINSTANCE g_hInstance;

static void sendCmdLine(HWND hOther, LPTSTR lpstrCmdLine) {
	COPYDATASTRUCT cpd;
	cpd.dwData = 0;
	cpd.cbData = (DWORD)(sizeof(TCHAR)*(_tcslen(lpstrCmdLine) + 1));
	cpd.lpData = lpstrCmdLine;

	::SendMessage(hOther, WM_COPYDATA, NULL, (LPARAM)&cpd);
}

BOOL CALLBACK searchOtherInstance(HWND hWnd, LPARAM /*lParam*/) {
	DWORD_PTR result;
	LRESULT ok = SendMessageTimeout(hWnd, WMU_WHERE_ARE_YOU, 0, 0, SMTO_BLOCK | SMTO_ABORTIFHUNG, 5000, &result);

	if(ok == 0) {
		return TRUE;
    }

	if(result == WMU_WHERE_ARE_YOU) {
		return FALSE;
	}

	return TRUE;
}

static tstring sSubTitle = _T("");

void callBack(const tstring& a, HWND hWndSplash) {
    if(a.size() > 0) {
	   sSubTitle = _T("(") + a + _T(")");
    }

	::RedrawWindow(hWndSplash, NULL, NULL, RDW_UPDATENOW | RDW_INTERNALPAINT);
}

LRESULT CALLBACK splashCallback(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
	if(uMsg == WM_PAINT) {
		// Get some information
		HDC dc = ::GetDC(hwnd);
		RECT rc;
		::GetWindowRect(hwnd, &rc);
		::OffsetRect(&rc, -rc.left, -rc.top);

		// Draw the icon
		HICON hi = (HICON)::LoadImage(g_hInstance, MAKEINTRESOURCE(IDR_SPLASH), IMAGE_ICON, 250, 210, LR_SHARED);
		::DrawIconEx(dc, rc.left, rc.top, hi, 0, 0, 0,NULL, DI_NORMAL);
		::DestroyIcon(hi);

		rc.top = rc.bottom - 40;
		rc.bottom = rc.bottom - 10;

		::SetBkColor(dc, RGB(255, 255, 255));
		::SetTextColor(dc, RGB(0, 40, 80));

		LOGFONT logFont;
		::GetObject(GetStockObject(DEFAULT_GUI_FONT), sizeof(logFont), &logFont);
		logFont.lfHeight = 15;
		logFont.lfWeight = 700;
		HFONT hFont = ::CreateFontIndirect(&logFont);
		::SelectObject(dc, hFont);

		tstring sTitle = TSTRING(LOADING) + _T(" ") + _T(CzDC_VERSION_STRING) + _T(", ") + TSTRING(PLEASE_WAIT);

		::DrawText(dc, sTitle.c_str(), (int)sTitle.length(), &rc, DT_SINGLELINE | DT_TOP | DT_CENTER);
		::DeleteObject(hFont);

		logFont.lfHeight = 14;
		logFont.lfWeight = 400;
		hFont = ::CreateFontIndirect(&logFont);
		::SelectObject(dc, hFont);
		::DrawText(dc, sSubTitle.c_str(), (int)sSubTitle.length(), &rc, DT_SINGLELINE | DT_BOTTOM | DT_CENTER);
		::DeleteObject(hFont);

		::ReleaseDC(hwnd, dc);

		sSubTitle.clear();
	}

	return ::DefWindowProc(hwnd, uMsg, wParam, lParam);
}

static int Run(LPTSTR /*lpstrCmdLine*/ = NULL, int nCmdShow = SW_SHOWDEFAULT) {
	RECT rc;
	rc.bottom = GetSystemMetrics(SM_CYFULLSCREEN);
	rc.top = (rc.bottom / 2) - 105;
	rc.bottom = rc.top + 210;

	rc.right = GetSystemMetrics(SM_CXFULLSCREEN);
	rc.left = (rc.right / 2) - 125;
	rc.right = rc.left + 250;

    HWND hWndSplash = ::CreateWindowEx(0, WC_STATIC, _T(CzDC_VERSION_STRING), WS_POPUP | WS_VISIBLE | SS_USERITEM | WS_EX_TOOLWINDOW,
        CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, ::GetDesktopWindow(), 0, g_hInstance, NULL);
	::SendMessage(hWndSplash, WM_SETFONT, (WPARAM)(HFONT)GetStockObject(DEFAULT_GUI_FONT), MAKELPARAM(TRUE, 0));

	::HideCaret(hWndSplash);
	::SetWindowPos(hWndSplash, NULL, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, SWP_SHOWWINDOW);
	::SetWindowLongPtr(hWndSplash, GWLP_WNDPROC, (LONG_PTR)splashCallback);

	::SetFocus(hWndSplash);

	startup(callBack, hWndSplash);

	::DestroyWindow(hWndSplash);

	if(ResourceManager::getInstance()->isRTL()) {
		::SetProcessDefaultLayout(LAYOUT_RTL);
	}

	MainFrame wndMain;

	if(wndMain.CreateEx() == NULL) {
		return 0;
	}

	if(BOOLSETTING(MINIMIZE_ON_STARTUP)) {
		::ShowWindow(wndMain.m_hWnd, SW_SHOWMINIMIZED);
	} else {
		::ShowWindow(wndMain.m_hWnd, ((nCmdShow == SW_SHOWDEFAULT) || (nCmdShow == SW_SHOWNORMAL)) ? SETTING(MAIN_WINDOW_STATE) : nCmdShow);
	}

	MSG msg;
	BOOL bRet;

    // Main message loop
	while((bRet = GetMessage(&msg, NULL, 0, 0)) != 0) { 
	    if(bRet == -1) {
	        continue;
	    }

		if(wndMain.PreTranslateMessage(&msg) == FALSE) {
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}

	int nRet = (int)msg.wParam;

	shutdown();

	return nRet;
}

int WINAPI _tWinMain(HINSTANCE hInstance, HINSTANCE /*hPrevInstance*/, LPTSTR lpstrCmdLine, int nCmdShow) {
    ::SetDllDirectory(_T(""));

#ifndef _WIN64
    HINSTANCE hKernel32 = ::LoadLibrary(_T("Kernel32.dll"));

    typedef BOOL (WINAPI * SPDEPP)(DWORD);
    SPDEPP pSPDEPP = (SPDEPP)::GetProcAddress(hKernel32, "SetProcessDEPPolicy");

    if(pSPDEPP != NULL) {
        pSPDEPP(0x00000001);
    }
#endif


#ifndef _DEBUG
	SingleInstance dcapp(_T("{CZDC-AEE8350A-B49A-4753-AB4B-E55479A48351}"));
#else
	SingleInstance dcapp(_T("{CZDC-AEE8350A-B49A-4753-AB4B-E55479A48350}"));
#endif

	if(dcapp.IsAnotherInstanceRunning()) { // Another instance is running, we want to allow to run another one.
        bool multiple = false;

		if(_tcslen(lpstrCmdLine) == 0) { // We want to run another instance only when we don't have any command line parameters.
            TCHAR tcBuf[MAX_PATH+1];
            ::GetModuleFileName(NULL, tcBuf, MAX_PATH);
            TCHAR * tcPath = _tcsrchr(tcBuf, '\\');
            tstring tsPath = (tcPath != NULL ? tstring(tcBuf, (tcPath-tcBuf)+1) : tcBuf);

            if(File::FileExist(tsPath + _T("CzDC.xml"))) { // We want to run another instance only when we have CzDC.xml in app directory.
                if(MessageBox(NULL, _T("There is already an instance of CzDC running.\nDo you want to launch another instance anyway?"),
                    _T(CzDC_VERSION_STRING), MB_YESNO | MB_ICONQUESTION | MB_DEFBUTTON2 | MB_TOPMOST) == IDYES) {
                    multiple = true;
                }
            }
        }

		if(multiple == false) {
			HWND hOther = NULL;
			EnumWindows(searchOtherInstance, (LPARAM)&hOther);

			if(hOther != NULL) {
				// pop up
				SetForegroundWindow(hOther);

				if(IsIconic(hOther)) {
					// restore
					ShowWindow(hOther, SW_RESTORE);
				}

				sendCmdLine(hOther, lpstrCmdLine);
			}
            return FALSE;
		}
	}

	::CoInitializeEx(NULL, COINIT_APARTMENTTHREADED);

    ExceptionHandlingInitialize();

	WSADATA wsaData;
	WSAStartup(MAKEWORD(2, 2), &wsaData);

	INITCOMMONCONTROLSEX iccx = { sizeof(INITCOMMONCONTROLSEX), ICC_COOL_CLASSES | ICC_BAR_CLASSES | ICC_LISTVIEW_CLASSES | ICC_TREEVIEW_CLASSES | 
        ICC_PROGRESS_CLASS | ICC_STANDARD_CLASSES | ICC_TAB_CLASSES | ICC_UPDOWN_CLASS | ICC_USEREX_CLASSES }; // add flags to support other controls
	InitCommonControlsEx(&iccx);

    g_hInstance = hInstance;

	try {
        enum { DEBUG_BUFSIZE = 8192 };
        char buf[DEBUG_BUFSIZE];
		File f(WinUtil::getAppName(), File::READ, File::OPEN);
		TigerTree tth(TigerTree::calcBlockSize(f.getSize(), 1));
		size_t n = 0;
		size_t n2 = DEBUG_BUFSIZE;
		while( (n = f.read(buf, n2)) > 0) {
			tth.update(buf, n);
			n2 = DEBUG_BUFSIZE;
		}
		tth.finalize();
		strcpy(sExceptionTTH, tth.getRoot().toBase32().c_str());
		WinUtil::tth = Text::toT(sExceptionTTH);
	} catch(const FileException&) {
		dcdebug("Failed reading exe\n");
	}

	HINSTANCE hInstRich = ::LoadLibrary(_T("RICHED20.DLL"));

	int nRet = Run(lpstrCmdLine, nCmdShow);

	if(hInstRich != NULL) {
		::FreeLibrary(hInstRich);
	}

	WSACleanup();

    ExceptionHandlingUnitialize();

	CoUninitialize();

	return nRet;
}

/*
 * Some parts Copyright (C) 2002-2011 Petr Kozelka, PPK at czdc dot org
 * Licensed under GNU General Public License version 3.
 */
