#ifndef _MEMDC_H_
#define _MEMDC_H_

//////////////////////////////////////////////////
// CMemDC - memory DC
//
// Author: Keith Rule
// Email:  keithr@europa.com
// Copyright 1996-2002, Keith Rule
//
// You may freely use or modify this code provided this
// Copyright is included in all derived versions.
//
// History - 10/3/97 Fixed scrolling bug.
//                   Added print support. - KR
//
//           11/3/99 Fixed most common complaint. Added
//                   background color fill. - KR
//
//           11/3/99 Added support for mapping modes other than
//                   MM_TEXT as suggested by Lee Sang Hun. - KR
//
//           02/11/02 Added support for CScrollView as supplied
//                    by Gary Kirkham. - KR
//
// This class implements a memory Device Context which allows
// flicker free drawing.
//
// Modified by Trem to work with wtl
// Removed the print support

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "stdafx.h"
#include "../client/DCPlusPlus.h"

class CMemDC {
private:
	HBITMAP		m_bitmap;		// Offscreen bitmap
	HBITMAP		m_oldBitmap;	// bitmap originally found in CMemDC
	HDC*		m_pDC;			// Saves HDC passed in constructor
	RECT		m_rect;			// Rectangle of drawing area.

	void Init(const LPCRECT pRect) {
		// Get the rectangle to draw
		if (pRect == NULL) {
			::GetClipBox(*m_pDC, &m_rect);
		} else {
			m_rect = *pRect;
		}

		// Create a Memory DC
		m_hDC = ::CreateCompatibleDC(*m_pDC);
		::LPtoDP(*m_pDC, (LPPOINT)&m_rect, 2);

		m_bitmap = ::CreateCompatibleBitmap(*m_pDC, m_rect.right - m_rect.left, m_rect.bottom - m_rect.top);
		m_oldBitmap = (HBITMAP)::SelectObject(m_hDC, m_bitmap);

		::SetMapMode(m_hDC, ::GetMapMode(*m_pDC));

		::SelectObject(m_hDC, ::GetCurrentObject(*m_pDC, OBJ_FONT));

		SIZE size = {0, 0};
		if( 0 != ::GetWindowExtEx(*m_pDC, &size))
			::SetWindowExtEx(m_hDC, size.cx, size.cy, NULL);

		if( 0 != ::GetViewportExtEx(*m_pDC, &size))
			::SetViewportExtEx(m_hDC, size.cx, size.cy, NULL);

		::DPtoLP(*m_pDC, (LPPOINT)&m_rect, 2);

		::SetWindowOrgEx(m_hDC, m_rect.left, m_rect.top, NULL);

		// Fill background 
		COLORREF clrOld = ::SetBkColor(m_hDC, ::GetBkColor(*m_pDC));
		::ExtTextOut(m_hDC, 0, 0, ETO_OPAQUE, &m_rect, NULL, 0, NULL);
		::SetBkColor(m_hDC, clrOld);
	}
public:
	CMemDC(HDC& dc, const LPCRECT pRect = NULL) {
		dcassert(dc != NULL); 

		// Some initialization
		m_hDC = NULL;
		m_pDC = &dc;
		m_oldBitmap = NULL;

		Init(pRect);
	}

	//Do not attempt to use an CMemDC object after calling Paint
	void Paint() {
		if(m_pDC == NULL)
			return;

		// Copy the offscreen bitmap onto the screen.
		::BitBlt(*m_pDC, m_rect.left, m_rect.top, m_rect.right - m_rect.left, m_rect.bottom - m_rect.top,
			m_hDC, m_rect.left, m_rect.top, SRCCOPY);

		//Swap back the original bitmap.
		::SelectObject(m_hDC, m_oldBitmap);
		::DeleteObject(m_bitmap);

		m_pDC = NULL;
	}

	~CMemDC() {
		Paint();
		::DeleteDC(m_hDC);
	}

	HDC m_hDC;
};

#endif

/*
 * Some parts Copyright (C) 2002-2011 Petr Kozelka, PPK at czdc dot org
 * Licensed under GNU General Public License version 3.
 */
